/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.resource.sample;

import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeCategory;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.scope.AttributeScope;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.PoolModule;
import com.ibm.rpm.resource.scope.PoolScope;

/**
 * The aim of this sample is to add 6 custom attributes to a Resource Pool and
 * group them together under one category and classification. Also, each custom
 * attribute will have a distinct input type.
 * 
 * @desc Group 6 new custom attributes of a Pool under one category.
 * 
 */
public class PoolAttributeSample extends SampleTestController
{
    private final int NB_ATTRIBUTES = 6;

    public final String poolName = "POOL_123";

    public final String catName = "CATEGORY";

    public final String catDesc = "ABCDEF_123";

    public final String className = "CLASSIFICATION";

    public final String classDesc = "ABCDEF_123";

    public final String attName = "ATTRIBUTE";

    public final String attDesc = "ABCDEF_123";

    /*
     * These InputTypes represent the types of input for each of the
     * NB_ATTRIBUTES we associate with a specfic Pool. They will appear in the
     * Attributes portlet of POOL_123.
     */
    static InputType[] attType = new InputType[]
    {
            InputType.Date, InputType.None, InputType.Numeric,
            InputType.RiskAppetite, InputType.RiskTolerance, InputType.Text
    };

    protected void runSample() throws Exception
    {

        /*
         * Indicate that we want to modify the custom attributes of the pool
         * we're loading.
         */
        PoolScope pScope = new PoolScope();
        pScope.setAttributeAssignments(true);
        Pool pool = (Pool) application.loadFromXpath(sessionID,
            "/Pool[name='" + poolName + "']", pScope).getRpmObjectList()[0];

        PoolModule poolModule = new PoolModule();

        /*
         * Create the custom attribute we would like to associate to the pool we
         * have just loaded. The following classes correspond to the
         * "Attributes" portlet under the "Resource Management" category
         */
        AttributeScope attScope = new AttributeScope();
        AttributeCategory attCategory = new AttributeCategory();
        AttributeClassification attClassification = new AttributeClassification();

        attCategory.setName(catName);
        attCategory.setDescription(catDesc);
        attCategory.setParent(poolModule);

        /*
         * We set the parent of the Classification field to the Category we've
         * just created
         */
        attClassification.setName(className);
        attClassification.setDescription(classDesc);
        attClassification.setParent(attCategory);

        attCategory.setClassifications(new AttributeClassification[]
        {
            attClassification
        });
        attScope.setClassifications(new AttributeScope());
        attScope.getClassifications().setAttributes(new AttributeScope());

        attClassification.setAttributes(new Attribute[NB_ATTRIBUTES]);

        /*
         * This loop is a short-hand for creating all the attributes which
         * appear in the Attributes portlet of the pool we're modifying.
         */
        for (int i = 0; i < attType.length; i++)
        {
            Attribute att = new Attribute();

            att.setDescription(attDesc);
            att.setParent(attClassification);

            att.setName(attName + i);
            att.setInputType(attType[i]);
            attClassification.getAttributes()[i] = att;
        }

        /*
         * Save the attribute category in the RPM repository
         */
        SaveResult saveResult = application.save(sessionID, attCategory,
            attScope, ReloadType.ReloadResult);

        /*
         * Print any errors that may have been created
         */
        printRPMExceptions(saveResult);

        attCategory = (AttributeCategory) saveResult.getRpmObject();

        if (attCategory.getClassifications()[0].getAttributes() == null)
        {
            System.err
                .println("*** Fatal : The Attributes were not saved successfully");
        }

        attScope = new AttributeScope();
        attScope.setAttributes(new AttributeScope());

        pool.setAttributeAssignments(new AttributeAssignment[NB_ATTRIBUTES]);

        /*
         * For each of the attributes, associate them to the pool using the
         * AttributeAssignment class as a proxy
         */
        for (int i = 0; i < NB_ATTRIBUTES; i++)
        {
            AttributeAssignment attrAssignment = new AttributeAssignment();

            attrAssignment.setAttribute(attCategory.getClassifications()[0]
                .getAttributes()[i]);
            attrAssignment.setParent(pool);

            pool.getAttributeAssignments()[i] = attrAssignment;
        }

        /*
         * Store the pool in the RPM repository. Here, a scope is necessary
         * because we also want to store the pool's attributes
         */
        SaveResult result = application.save(sessionID, pool, pScope,
            ReloadType.ReloadResult);

        printRPMExceptions(result);

        if (((Pool) result.getRpmObject()).getAttributeAssignments() == null)
        {
            System.err
                .println("*** Fatal : Failure saving the pool into the repository");
        }

    }

    public static void main(String[] args)
    {
        new PoolAttributeSample();
    }
}
