/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.timesheet.sample;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.LoadResult;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;
import com.ibm.rpm.wbs.types.ResourceType;

public class TimesheetSampleSetUp extends SampleTestController
{

    public static String projectName = "TIMESHEET_SAMPLE_PROJECT";

    public static String taskName = "TIMESHEET_SAMPLE_TASK";

    public static String resourceName = "TIMESHEET_SAMPLE_RES";

    public static String poolName = "TIMESHEET_SAMPLE_POOL";

    public static Project project = null;

    public static Task task = null;

    public static Resource resource = null;

    public static void main(String[] args) throws Exception
    {
        new TimesheetSampleSetUp();
    }

    protected void runSample() throws Exception
    {
        setUp();
    }

    public static void setUp()
    {
        try
        {

            // delete objets created by this code sample
            clean();

            createResource();

            createTask();

            assignResourceToTask();

            System.out.println("Setup Done");
        }

        catch (Exception e)
        {
            System.out.println(e);
        }
    }

    public static void createResource() throws Exception
    {

        System.out.println("Create and save a Resource under a Pool");

        Pool pool = new Pool();
        pool.setName(poolName);

        resource = new Resource();
        resource.setFullName(resourceName);

        // Load the default calendar
        LoadResult result1 = application.loadFromXpath(sessionID,
            "/RPMCalendar[name = 'Standard (Default)']", null);

        RPMCalendar calendar = null;
        if (result1.getRpmObjectList() != null)
        {
            calendar = (RPMCalendar) result1.getRpmObjectList()[0];
        }
        else
        {
            throw new Exception("Cannot load the default calendar");
        }

        // Resource calendar is mandatory to assign a resource
        resource.setCalendar(calendar);

        pool.setResources(new Resource[]
        {
            resource
        });

        PoolScope poolScope = new PoolScope();
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        poolScope.setResources(resourceScope);

        SingleResult result2 = application.save(sessionID, pool, poolScope,
            ReloadType.ReloadResult);

        printRPMExceptions(result2);

        if (result2.isSuccessful())
        {
            pool = (Pool) result2.getRpmObject();
            resource = pool.getResources()[0];
        }

    }

    public static void createTask() throws Exception
    {

        System.out.println("Create and save a Task under a Project");

        project = new Project();
        project.setName(projectName);

        // Publish project to team (mandatory for timesheet creation)
        project.setPublished(ProjectPublishedType.ToTeam);

        task = new Task();
        task.setName(taskName);

        project.setChildren(new Task[]
        {
            task
        });

        // Create projectScope
        WorkElementScope projectScope = new WorkElementScope();
        WorkElementScope taskScope = new WorkElementScope();

        // Contaning project is mandatory to create a ResourceTaskAssignment
        taskScope.setContainingProject(new WorkElementScope());
        projectScope.setChildren(taskScope);

        SingleResult result1 = application.save(sessionID, project,
            projectScope, ReloadType.ReloadResult);

        printRPMExceptions(result1);

        if (result1.isSuccessful())
        {

            project = (Project) result1.getRpmObject();
            task = (Task) project.getChildren()[0];

        }

    }

    public static void assignResourceToTask() throws Exception
    {

        System.out.println("Assign Resource to Task with a Security role");

        // Create resourceTaskAssignment
        ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();

        // Load the security role
        LoadResult result1 = application.loadFromXpath(sessionID,
            "/SecurityRole[name = 'Task Responsible']", null);

        SecurityRole securityRole = null;
        if (result1.getRpmObjectList() != null)
        {
            securityRole = (SecurityRole) result1.getRpmObjectList()[0];
        }
        else
        {
            throw new Exception("Cannot load the security role");
        }
        resourceTaskAssignment.setSecurityRole(securityRole);
        resourceTaskAssignment.setResource(resource);
        resourceTaskAssignment.setResourceType(ResourceType.Resource);

        // Create TaskAssignment
        TaskAssignment taskAssignment = new TaskAssignment();
        resourceTaskAssignment.setTaskAssignment(taskAssignment);
        task.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            resourceTaskAssignment
        });

        // Create task scope
        WorkElementScope taskScope = new WorkElementScope();

        ResourceTaskAssignmentScope rtaScope = new ResourceTaskAssignmentScope();
        rtaScope.setResource(new ResourceScope());
        TaskAssignmentScope taScope = new TaskAssignmentScope();
        rtaScope.setTaskAssignment(taScope);
        taskScope.setResourceTaskAssignments(rtaScope);

        // Save Task
        SingleResult result2 = application.save(sessionID, task, taskScope,
            ReloadType.None);

        printRPMExceptions(result2);

        // Copy Project proposed dates to plan dates (mandatory for timesheet
        // creation)

        SingleResult result3 = application.copyProposedToPlan(sessionID,
            project, null);

        printRPMExceptions(result3);
    }

    private static void clean() throws Exception
    {
        // delete projects
        deleteObjects("/Project[name = '" + projectName + "']");

        // delete pools
        deleteObjects("/Pool[name = '" + poolName + "']");
    }

    private static void deleteObjects(String xpath) throws Exception
    {

        ArrayResult result = application.loadFromXpath(sessionID, xpath, null);

        if (result.getRpmObjectList() != null)
        {
            RPMObject[] objects = result.getRpmObjectList();
            for (int i = 0; i < objects.length; i++)
            {
                application.delete(sessionID, objects[0]);
            }
        }
    }
}
