/*
 * Licensed Materials - Use restricted, please refer to the "Sample Code" terms 
 * and conditions in the IBM International Program License Agreement (IPLA).
 * 
 *  Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 */

package com.ibm.rpm.wbs.sample;

import java.util.Calendar;

import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SampleTestController;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ContourType;

/**
 * This sample assign a resource as work using contours to a task. 
 * The type of Contour used is Percentile Day.
 * Specifications of this contour are as follows:
 * 
 * 			1. Contour Type -- PercentileDay
 * 			2. Start Date --> mandatory
 * 			3. Either Effort or FinishDate or Occurrences is required. But not any two or all three.
 * 			4. Work - How much percentage of a day can the resource dedicate on this task
 * 			
 * This contour is used to affect a resource to a task. This resource will be
 * dedicating the specified percentage of a day from the start date on this task
 * till the finish date or total effort is completed or till the number of occurrences.  

 * @preconditions : There exists a Task by the name of SAMPLE_TASK
 * 
 * 					There exists an active resource (Calendar assigned) by the name of SAMPLE_USER
 * 
 * 					There exists a Task Responsible SecurityRole by the name SAMPLE_TASK_SECURITY_ROLE
 * 
 * @postconditions : A Resource is assigned as work using PercentileDay Contour to this task.
 * 
 */

public class WBSPercentileDayContouredRTASample extends SampleTestController {
	
	public static final String taskName = "SAMPLE_TASK";

	public static final String resourceName = "SAMPLE_USER";

	public static final String taskSecurityRoleName = "SAMPLE_TASK_SECURITY_ROLE";

	protected void runSample() throws Exception {
		/**
		 * encapsulates the returned data
		 */
		SaveResult ret = null;

		/**
		 * Explicit what we want loaded from the RPM Database
		 */
		WorkElementScope taskScope = new WorkElementScope();
		taskScope.setContainingProject(new WorkElementScope());

		/**
		 * Load the specific task from the database
		 */
		Task task = (Task) application.loadFromXpath(sessionID,
				"/Task[name='" + taskName + "']", taskScope).getRpmObjectList()[0];

		/**
		 * Load the specific resource from the database
		 */
		Resource resource = (Resource) application.loadFromXpath(sessionID,
				"/Resource[username = '" + resourceName + "']", null).getRpmObjectList()[0];

		/**
		 * Load the specific task security role from the database
		 */
		TaskSecurityRole taskSecurityRole = (TaskSecurityRole) application.loadFromXpath(sessionID,
				"/TaskSecurityRole[name = '" + taskSecurityRoleName +"']", null).getRpmObjectList()[0];

		ResourceTaskAssignment resourceTaskAssignment = new ResourceTaskAssignment();
		resourceTaskAssignment.setParent(task);
		resourceTaskAssignment.setResource(resource);
		resourceTaskAssignment.setSecurityRole(taskSecurityRole);
		resourceTaskAssignment.setTaskAssignment(new TaskAssignment());

		Contour contour = new Contour();
		contour.setContourType(ContourType.PercentilePerDay);
		contour.setStartDate(Calendar.getInstance());
		contour.setOccurrences(new Integer(4));
		contour.setWork(new Double(50)); // The affected resource has to work only half a day for 4 days (4H) on this task

		resourceTaskAssignment.setContour(contour);

		/**
		 * Save the resource task assignment in the RPM database and reload what the database
		 * returned. Thi save will assign the resource as work using the specified contour to
		 * the task.
		 */
		ret = application.save(sessionID, resourceTaskAssignment, null, ReloadType.ReloadResult);
		printRPMExceptions(ret);
		resourceTaskAssignment = (ResourceTaskAssignment) ret.getRpmObject();
	}

	public static void main(String[] args)
	{
		new WBSPercentileDayContouredRTASample();
	}

}
