/*
 * Created on Sep 19, 2005
 */
package com.ibm.rpm.applicationadministration.checkpoints;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import com.ibm.rpm.applicationadministration.constants.ValidationConstants;
import com.ibm.rpm.applicationadministration.containers.Attribute;
import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.AttributeClassification;
import com.ibm.rpm.applicationadministration.types.InputType;
import com.ibm.rpm.applicationadministration.types.SelectType;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScopeTask;
import com.ibm.rpm.wbs.containers.WorkElement;

public class AttributeAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final List ATTRIBUTE_PARENTS = new ArrayList();

    private static final Class ATTRIBUTE_CLASSIFICATION_PARENTS = Resource.class;

    static
    {
        ATTRIBUTE_PARENTS.add(AbstractAggregateScope.class);
        ATTRIBUTE_PARENTS.add(AbstractScopeTask.class);
        ATTRIBUTE_PARENTS.add(Asset.class);
        ATTRIBUTE_PARENTS.add(ClientCostCenter.class);
        ATTRIBUTE_PARENTS.add(GenericDocument.class);
        ATTRIBUTE_PARENTS.add(Pool.class);
        ATTRIBUTE_PARENTS.add(Resource.class);
        ATTRIBUTE_PARENTS.add(WorkElement.class);
    }

    protected static AttributeAssignmentCheckpoint instance = new AttributeAssignmentCheckpoint();

    public static AttributeAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        
    	boolean valid = true;

        if (!GenericValidator.validateInstance(rpmObject,
            AttributeAssignment.class, msgCtx))
            return;

        // Initialize
        AttributeAssignment mainAA = (AttributeAssignment) rpmObject;

        // Parent is mandatory while creating or updating an attribute assignment
        // Only updating scenarion is validated here because validating the parent
        // during creation is done in validAttribute().
        if(mainAA.getID() != null)
        {
        	valid &= GenericValidator.validateMandatory(mainAA, ValidationConstants.PARENT_FIELD, 
        			mainAA.getParent(), msgCtx);
        }
        
        // Field validation: attribute
        valid &= GenericValidator.validateMandatoryID(mainAA, "attribute",
            mainAA.getAttribute(), msgCtx);

        if (valid)
        {
            if (mainAA.getAttribute() instanceof Attribute)
            {
                validAttribute(mainAA, msgCtx);
            }
            else if (mainAA.getAttribute() instanceof AttributeClassification)
            {
                validAttributeClassification(mainAA, msgCtx);
            }
            else
            {
                String[] errorParams =
                {
                        "AttributeAssignment attribute",
                        StringUtil.getShortClassName(mainAA.getAttribute()
                            .getClass()),
                        "Attribute or AttributeClassification"
                };
                RPMException e = new RPMException(ErrorCodes.INVALID_TYPE,
                    errorParams, mainAA.getClass().getName(), "attribute");
                addException(e, msgCtx);
            }
        }
    }
    
    public void validateDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {
    	super.validateDelete(rpmObject, messageContext);
    	
    	AttributeAssignment attributeAssignment = (AttributeAssignment)rpmObject;
    	
    	// Parent is mandatory while deleting an attribute assignment
    	GenericValidator.validateMandatory(attributeAssignment, ValidationConstants.PARENT_FIELD, 
    			attributeAssignment.getParent(), messageContext);
    }

    public boolean isValidSingleSelectRule(AttributeAssignment[] assignments,
            MessageContext msgCtx)
    {
        boolean valid = true;
        if (assignments != null && assignments.length > 1)
        {
            Attribute attribute = null;
            AttributeClassification classification = null;
            Collection singleSelectClassifications = new ArrayList();
            for (int i = 0; i < assignments.length && valid; i++)
            {
                if (assignments[i] != null
                        && assignments[i].getAttribute() != null
                        && assignments[i].getAttribute() instanceof Attribute
                        && assignments[i].getAttribute().getParent() != null
                        && assignments[i].getAttribute().getParent() instanceof AttributeClassification)
                {
                    attribute = (Attribute) assignments[i].getAttribute();
                    classification = (AttributeClassification) attribute
                        .getParent();
                    if (classification.getSelectType() == SelectType.Single)
                    {
                        valid &= updateAndValidateSingleSelectClassifications(
                            singleSelectClassifications, classification, msgCtx);
                    }
                }
            }
        }
        return valid;
    }

    private boolean updateAndValidateSingleSelectClassifications(
            Collection singleSelectClassifications,
            AttributeClassification classification, MessageContext msgCtx)
    {
        boolean valid = true;

        if (!singleSelectClassifications.contains(classification.getID()))
        {
            singleSelectClassifications.add(classification.getID());
        }
        else
        {
            String[] errorParams = new String[1];
            errorParams[0] = classification.getID();
            RPMException e = new RPMException(
                com.ibm.rpm.applicationadministration.constants.ErrorCodes.CANNOT_ASSIGN_MULTIPLE_ATTRIBUTES_FROM_A_SINGLE_SELECT_CLASSIFICATION,
                errorParams);
            addException(e, msgCtx);
            valid = false;
        }

        return valid;
    }

    private void validAttribute(AttributeAssignment attributeAssignment,
            MessageContext msgCtx)
    {
        if (!super.isValidParent(attributeAssignment, msgCtx, true,
            ATTRIBUTE_PARENTS))
        {
            return;
        }

        String[] errorParams =
        {
                "", "", "", ""
        };

        // Field validation: value
        Attribute mainAttribute = (Attribute) attributeAssignment
            .getAttribute();
        errorParams[0] = "AttributeAssignment value";

        //------------------------- InputType.None -------------------------//
        if (mainAttribute.getInputType() == InputType.None)
        {
            GenericValidator.validateMustBeNull(attributeAssignment, "value",
                attributeAssignment.getValue(), "attribute inputType is None",
                msgCtx);
        }
        else if (attributeAssignment.getValue() != null)
        {
            //--------- InputType.Numeric, RiskAppetite, RiskTolerance ----------//
        	if (mainAttribute.getInputType() == InputType.Numeric || 
                mainAttribute.getInputType() == InputType.RiskAppetite || 
                mainAttribute.getInputType() == InputType.RiskTolerance)
	        {
	            Double value = null;
	            // Convert value to double regardless of the format it was provided in
	            if (attributeAssignment.getValue() instanceof String)
	            {
	                try
	                {
	                    value = new Double(
	                        (String) (attributeAssignment.getValue()));
	                }
	                catch (Exception e2)
	                {
	                    RPMException e = new RPMException(
	                        com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_STRING_MUST_BE_VALID_NUMBER, 
	                        attributeAssignment.getClass().toString(), "value", attributeAssignment.getID());
	                    addException(e, msgCtx);                
	                }
	            }
	            else if (attributeAssignment.getValue() instanceof Number)
	            {
	                value = new Double(((Number) attributeAssignment.getValue()).doubleValue());
	                // WARNING - Will return a String and not a Number object
	                errorParams[0] = attributeAssignment.getValue().getClass().toString();
	                errorParams[1] = "String";
	                RPMException e = new RPMException(
	                    com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_WARNING_INPUT_NOT_THE_SAME_AS_OUTPUT,
	                    errorParams);
	                e.setSeverity(SeverityLevel.Warning);
	                addException(e, msgCtx);                
	            }
	            else
	            {
	                GenericValidator.validateType(attributeAssignment, new Class[]
	                {
	                        String.class, Integer.class, Double.class
	                }, "value", attributeAssignment.getValue(), msgCtx);
	                return;
	            }
	
	            if (GenericValidator.outOfRange(value,
	                ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MIN,
	                ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MAX))
	            {
	                errorParams[1] = String
	                    .valueOf(ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MIN);
	                errorParams[2] = String
	                    .valueOf(ValidationConstants.ATTRIBUTEASSIGNMENT_NUMERIC_MAX)
	                        + " when attribute inputType is \""
	                        + mainAttribute.getInputType().getValue() + "\"";
	                errorParams[3] = value.toString();
	
	                RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
	                    errorParams, attributeAssignment.getClass().getName(),
	                    "value", attributeAssignment.getID());
	                addException(e, msgCtx);
	                errorParams[2] = "";
	                errorParams[3] = "";
	            }
	        }
	
	        //------------------------- InputType.Text -------------------------//
	        else if (mainAttribute.getInputType() == InputType.Text)
	        {
	            if (!GenericValidator.validateType(attributeAssignment,
	                String.class, "value", attributeAssignment.getValue(), msgCtx))
	                return;
	
	            GenericValidator.validateMaxLength(attributeAssignment, "value",
	                (String) (attributeAssignment.getValue()),
	                ValidationConstants.ATTRIBUTEASSIGNMENT_TEXT_MAX, msgCtx);
	        }
	
	        //------------------------- InputType.Date -------------------------//
	        else if (mainAttribute.getInputType() == InputType.Date)
	        {
	            if (!GenericValidator.validateType(attributeAssignment, new Class[]
	            {
	                    Calendar.class, String.class
	            }, "value", attributeAssignment.getValue(), msgCtx))
	                return;
	
	            if (attributeAssignment.getValue() instanceof String)
	            {
	                Calendar dateValue = null;
	                try
	                {
	                    dateValue = DateUtil
	                        .convertToCalendarNoTime((String) attributeAssignment
	                            .getValue());
	                }
	                catch (Exception e)
	                {
	                }
	                if (dateValue == null)
	                {
	                    errorParams[0] = (String) attributeAssignment.getValue();
	                    RPMException e = new RPMException(
	                        com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_DATE_VALUE_WITH_WRONG_FORMAT,
	                        errorParams);
	                    addException(e, msgCtx);
	                }
	                
	                // WARNING - Will return a Calendar and not a String
	                errorParams[0] = "String";
	                errorParams[1] = "Calendar";
	                RPMException e = new RPMException(
	                    com.ibm.rpm.applicationadministration.constants.ErrorCodes.ATTRIBUTE_ASSIGNMENT_WARNING_INPUT_NOT_THE_SAME_AS_OUTPUT,
	                    errorParams);
	                e.setSeverity(SeverityLevel.Warning);
	                addException(e, msgCtx);                    
	            }
	        }
	        else
	        {
	            GenericValidator.validateType(attributeAssignment, new Class[]
	             {
	                     String.class, Integer.class, Double.class, Calendar.class
	             }, "value", attributeAssignment.getValue(), msgCtx);
	        }
        }
    }

    private void validAttributeClassification(
            AttributeAssignment attributeAssignment, MessageContext msgCtx)
    {
        super.isValidParent(attributeAssignment, msgCtx, true,
            ATTRIBUTE_CLASSIFICATION_PARENTS);

        GenericValidator.validateMustBeNull(attributeAssignment, "value",
            attributeAssignment.getValue(),
            "attribute is of type AttributeClassification", msgCtx);
    }
}
