package com.ibm.rpm.asset.managers;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.HashMap;
import java.util.HashSet;

import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.asset.containers.MaintenanceEstimates;
import com.ibm.rpm.framework.AbstractRPMObjectManager;
import com.ibm.rpm.framework.JoinCondition;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.FieldValueMap;
import com.ibm.rpm.framework.util.Manager;

/**
 * @author adler
 * 
 * Manages the Maintenance Estimates for assets. Manages table
 * ASSET_MAINTENANCE. The depreciation loading should always be done with the
 * parent asset because a delete requires the asset Id. The managed container is
 * MAINTENANCEESTIMATES
 */

public class AssetMaintenanceManager extends AbstractRPMObjectManager
{

    private static final HashMap FIELDPROPERTYMAP = new HashMap();

    private static final HashSet CONTAINERS = new HashSet();

    static
    {
        //The complete class name of all the containers that could be loaded by
        // this managers.
        CONTAINERS.add(MaintenanceEstimates.class.getName());

        String[] objs = new String[CONTAINERS.size()];
        CONTAINERS.toArray(objs);
        for (int i = 0; i < objs.length; i++)
        {
            RPMManagerFactory.getInstance().addContainerToMap(objs[i],
                    AssetMaintenanceManager.class);
        }
    }

    /**
     * Column ELEMENT_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ELEMENT_ID = 1;

    public static final int TYPE_ELEMENT_ID = Types.CHAR;

    public static final String NAME_ELEMENT_ID = "ASSET_MAINTENANCE.ELEMENT_ID";

    public static final String PROPERTY_ELEMENT_ID = "ID";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ELEMENT_ID, NAME_ELEMENT_ID);
    }

    /**
     * Column ASSET_ID of type Types.CHAR mapped to String.
     */
    public static final int ID_ASSET_ID = 2;

    public static final int TYPE_ASSET_ID = Types.CHAR;

    public static final String NAME_ASSET_ID = "ASSET_MAINTENANCE.ASSET_ID";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_ASSET_ID = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_ASSET_ID,NAME_ASSET_ID);}

    /**
     * Column MAINTENANCE_DATE of type Types.DATE mapped to java.util.Date.
     */
    public static final int ID_MAINTENANCE_DATE = 3;

    public static final int TYPE_MAINTENANCE_DATE = Types.DATE;

    public static final String NAME_MAINTENANCE_DATE = "ASSET_MAINTENANCE.MAINTENANCE_DATE";

    public static final String PROPERTY_MAINTENANCE_DATE = "SCHEDULEDATE";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_MAINTENANCE_DATE, NAME_MAINTENANCE_DATE);
    }

    /**
     * Column DESCRIPTION of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 4;

    public static final int TYPE_DESCRIPTION = Types.VARCHAR;

    public static final String NAME_DESCRIPTION = "ASSET_MAINTENANCE.DESCRIPTION";

    public static final String PROPERTY_DESCRIPTION = "DESCRIPTION";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_DESCRIPTION, NAME_DESCRIPTION);
    }

    /**
     * Column PERFORMED_BY of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PERFORMED_BY = 5;

    public static final int TYPE_PERFORMED_BY = Types.VARCHAR;

    public static final String NAME_PERFORMED_BY = "ASSET_MAINTENANCE.PERFORMED_BY";

    public static final String PROPERTY_PERFORMED_BY = "PERFORMEDBY";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_PERFORMED_BY, NAME_PERFORMED_BY);
    }

    /**
     * Column COST of type Types.DECIMAL mapped to java.math.BigDecimal.
     */
    public static final int ID_COST = 6;

    public static final int TYPE_COST = Types.DECIMAL;

    public static final String NAME_COST = "ASSET_MAINTENANCE.COST";

    public static final String PROPERTY_COST = "ACTUALCOST";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_COST, NAME_COST);
    }

    /**
     * Column REC_STATUS of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_STATUS = 7;

    public static final int TYPE_REC_STATUS = Types.CHAR;

    public static final String NAME_REC_STATUS = "ASSET_MAINTENANCE.REC_STATUS";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_STATUS = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_STATUS,NAME_REC_STATUS);}

    /**
     * Column REC_DATETIME of type Types.TIMESTAMP mapped to java.util.Date.
     */
    public static final int ID_REC_DATETIME = 8;

    public static final int TYPE_REC_DATETIME = Types.TIMESTAMP;

    public static final String NAME_REC_DATETIME = "ASSET_MAINTENANCE.REC_DATETIME";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_DATETIME = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_DATETIME,NAME_REC_DATETIME);}

    /**
     * Column REC_USER of type Types.CHAR mapped to String.
     */
    public static final int ID_REC_USER = 9;

    public static final int TYPE_REC_USER = Types.CHAR;

    public static final String NAME_REC_USER = "ASSET_MAINTENANCE.REC_USER";

    //uncomment and replace PROPERTYNAME with the container property loaded
    // from this column in upper case.
    //public static final String PROPERTY_REC_USER = "PROPERTYNAME";
    //static{ FIELDPROPERTYMAP.put(PROPERTY_REC_USER,NAME_REC_USER);}

    /**
     * Column ESTIMATED_COST of type Types.DECIMAL mapped to
     * java.math.BigDecimal.
     */
    public static final int ID_ESTIMATED_COST = 10;

    public static final int TYPE_ESTIMATED_COST = Types.DECIMAL;

    public static final String NAME_ESTIMATED_COST = "ASSET_MAINTENANCE.ESTIMATED_COST";

    public static final String PROPERTY_ESTIMATED_COST = "ESTIMATEDCOST";
    static
    {
        FIELDPROPERTYMAP.put(PROPERTY_ESTIMATED_COST, NAME_ESTIMATED_COST);
    }

    private static final String TABLE_NAME = "ASSET_MAINTENANCE";

    /**
     * Create an array of type string containing all the fields of the
     * ASSET_MAINTENANCE table.
     */
    private static final String[] FIELD_NAMES =
    {
            NAME_ELEMENT_ID, NAME_ASSET_ID, NAME_MAINTENANCE_DATE,
            NAME_DESCRIPTION, NAME_PERFORMED_BY, NAME_COST, NAME_REC_STATUS,
            NAME_REC_DATETIME, NAME_REC_USER, NAME_ESTIMATED_COST
    };

    private static final String ALL_FIELDS;
    static
    {
        StringBuffer fieldcat = new StringBuffer(FIELD_NAMES[0]);
        for (int i = 1; i < FIELD_NAMES.length; i++)
        {
            fieldcat.append(",");
            fieldcat.append(FIELD_NAMES[i]);
        }
        ALL_FIELDS = fieldcat.toString();
    };

    /**
     * Creates a new AssetMaintenanceManager instance.
     * 
     * @return AssetMaintenanceManager
     */
    public AssetMaintenanceManager()
    {
        super();
    }

    public String getTableName()
    {
        return TABLE_NAME;
    }

    protected String[] getFieldsNames()
    {
        return FIELD_NAMES;
    }

    protected String getAllFields()
    {
        return ALL_FIELDS;
    }

    public String getPrimaryKey()
    {
        return NAME_ELEMENT_ID;
    }

    protected Integer getContainerType(ResultSet resultSet)
            throws SQLException
    {
        //If container type doesn't apply to this manager
        return new Integer(0);
    }

    protected HashMap getPropertiesMap()
    {
        return FIELDPROPERTYMAP;
    }

    protected RPMObject createContainer(int typeId) throws RPMException
    {

        RPMObject object = new MaintenanceEstimates();
        return object;
    }

    /**
     * Create a record for Maintenance Estimates in DB and set its ID.
     */
    public void createObject(RPMObject rpmObject, MessageContext messageContext)
            throws SQLException, RPMException, ParseException
    {

        MaintenanceEstimates pObject = (MaintenanceEstimates) rpmObject;
        
        // Set defaults
        if (pObject.getActualCost() == null)
            pObject.setActualCost(new Double(0.00));
        
        if (pObject.getEstimatedCost() == null)
            pObject.setEstimatedCost(new Double(0.00));
        
        // Check validity
        pObject.setID(SP_I_ASSET_MAINTENANCE(pObject, messageContext));
    }

    /**
     * decode a row of Maintenance Estimate. this includes: 1. Scheduled
     * Estimation date 2. Description 3. Person that responsible for action
     * execution 4. Actual and estimared costs
     */

    protected RPMObject decodeRow(ResultSet rs, RPMObject rpmObject,
            boolean isDelta) throws RPMException, SQLException, ParseException
    {

        MaintenanceEstimates pObject = (MaintenanceEstimates) rpmObject;

        pObject.setID(rs.getString(ID_ELEMENT_ID));
        if (isDelta)
        {
            pObject.deltaScheduleDate(Manager.getCalendar(rs,
                    ID_MAINTENANCE_DATE));
            pObject.deltaDescription(rs.getString(ID_DESCRIPTION));
            pObject.deltaPerformedBy(rs.getString(ID_PERFORMED_BY));
            pObject.deltaActualCost(new Double(rs.getDouble(ID_COST)));
            pObject.deltaEstimatedCost(new Double(rs
                    .getDouble(ID_ESTIMATED_COST)));
        }
        else
        {
            pObject.setScheduleDate(Manager
                    .getCalendar(rs, ID_MAINTENANCE_DATE));
            pObject.setDescription(rs.getString(ID_DESCRIPTION));
            pObject.setPerformedBy(rs.getString(ID_PERFORMED_BY));
            pObject.setActualCost(new Double(rs.getDouble(ID_COST)));
            pObject
                    .setEstimatedCost(new Double(rs
                            .getDouble(ID_ESTIMATED_COST)));
        }

        return pObject;
    }

    /**
     * delete a Maintenance Estimate
     */
    protected void doDelete(RPMObject rpmObject, MessageContext messageContext)
            throws RPMException, SQLException, ParseException
    {
    	if (rpmObject != null)
        {
            SP_D_ASSET_MAINTENANCE((MaintenanceEstimates) rpmObject,
                    messageContext);
        }
    }

    /**
     * Returns the foreign key sql conditions of the asset that exists in the
     * Maintenance Estimate table
     */

    protected JoinCondition getForeignKeyCondition(RPMObjectManager manager,
            String extraWhere, String contextName) throws RPMException
    {

        JoinCondition joinCondition = null;
        if ((manager instanceof AssetManager))
        {
	        joinCondition = new JoinCondition();
	        joinCondition.setTableName(getTableName());
	
	        //where clause of the join condition
	        StringBuffer condition = new StringBuffer();
	        condition.append(NAME_ASSET_ID);
	        condition.append(" = ?");
	        if (extraWhere != null && extraWhere.length() > 0)
	        {
	            condition.append(" AND ");
	            condition.append(extraWhere);
	        }
	        // joinCondition.setTableName(getTableName());
	        joinCondition.setCondition(condition.toString());
        
    	}
		else
		{
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.FOREIGN_KEY);
		}
        
        return joinCondition;
    } // getForeignKeyCondition

    /**
     * Return JoinCondition object with condition input parameter
     */

    protected JoinCondition getJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {

        JoinCondition joinCondition = new JoinCondition();
        if (manager == null)
        {
            joinCondition.setTableName(getTableName());
            if (condition != null)
            {
                joinCondition.setCondition(condition);
            }
        }
        else
	    {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.JOIN);
        }

        return joinCondition;

    }
    
    protected JoinCondition getSubTableJoinCondition(RPMObjectManager manager,
            String condition, int level, String containerName,
            String contextName) throws RPMException
    {
		JoinCondition joinCondition = new JoinCondition();
        if (manager instanceof AssetManager)
        {
			// for XPath queries like /Asset[MaintenanceEstimates[<condition>]]
            StringBuffer where = new StringBuffer();
            where.append(AssetManager.NAME_ELEMENT_ID).append(" IN (");
            where.append(" SELECT ").append(NAME_ASSET_ID);
            where.append(" FROM ").append(getTableName());
            if (condition != null)
            {
                where.append(" WHERE ").append(condition);
            }
            joinCondition.setCondition(where.toString());
        }
        else
        {
            ExceptionUtil.handleUnsupportedJoinCondition(this, manager,
                    ExceptionUtil.SUB_TABLE_JOIN);
        	
        }

        return joinCondition;
    }
    
    /**
     * The only decoded related object is the parent asset
     */
    protected FieldValueMap decodeRelatedFields(ResultSet resultSet,
            FieldValueMap relatedFields, int index) throws RPMException,
            SQLException
    {
        String parentId = resultSet.getString(ID_ASSET_ID);
        relatedFields.put(index, ID_ASSET_ID, parentId);

        return relatedFields;
    }

    /**
     * The only related object is the parent asset
     */
    protected RPMObject loadRelatedObjects(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext,
            FieldValueMap relatedFields, int index, boolean isDelta)
            throws RPMException, SQLException, ParseException
    {
        if (rpmScope == null)
            return rpmObject;

        if (rpmScope.getParent() == null)
            return rpmObject;

        //Load the parent. For MaintenanceEstimates we always need to load the
        // asset because w/o we will not be able to delete the
        // MaintenanceEstimates object
        MaintenanceEstimates object = (MaintenanceEstimates) rpmObject;

        //ID of Related asset
        String assetId = (String) relatedFields.get(index, ID_ASSET_ID);

        if (assetId != null)
        {
            // Load the Asset Manager
            AssetManager assetManager = (AssetManager) RPMManagerFactory
                    .getInstance().getRPMObjectManager(Asset.class.getName());
            Asset asset = new Asset();
            // Set the Asset ID
            asset.setID(assetId);

            // Load the Asset record
            asset = (Asset) assetManager.loadByPrimaryKey(asset, rpmScope
                    .getParent(), messageContext, isDelta);
            if (isDelta)
            {
                object.deltaParent(asset);
            }
            else
            {
                object.setParent(asset);
            }
        }
        else
        {
            ExceptionUtil.handleNoId(this, object, ExceptionUtil.ASSET_ID);
        }

        return rpmObject;
    } // loadRelatedObjects

    /**
     * Update the Maintenance Estimates fields in DB
     */

    public RPMObject updateObject(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext) throws SQLException, RPMException,
            ParseException
    {
        MaintenanceEstimates estimates = (MaintenanceEstimates) rpmObject;
        if (estimates.testScheduleDateModified()
                || estimates.testDescriptionModified()
                || estimates.testActualCostModified()
                || estimates.testEstimatedCostModified()
                || estimates.testPerformedByModified())

            SP_U_ASSET_MAINTENANCE(estimates, messageContext);

        return estimates;
    }

    /**
     * Inserts an empty Maintenance Estimate for the asset
     * 
     * @param MaintenanceEstimates -
     *            Maintenance Estimate to save
     * @param messageContext -
     *            context
     * @return a String that represents the ID of the new created depreciation
     *         schedule
     * @throws SQLException
     * @throws RPMException
     */

    private String SP_I_ASSET_MAINTENANCE(MaintenanceEstimates estimates,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_I_ASSET_MAINTENANCE";
        Object args[] = new Object[4];
        args[0] = null;
        // estimates should include parent
        args[1] = estimates.getParent().getID();
        // Type id = 93
        args[2] = new Integer(93);
        args[3] = getUser(messageContext).getID();

        return executeProcedure(messageContext, name, args);
    }

    /**
     * Delete a Maintenance Estimate record from the DB
     * 
     * @param MaintenanceEstimates -
     *            Maintenance Estimate information
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_D_ASSET_MAINTENANCE(MaintenanceEstimates estimates,
            MessageContext messageContext) throws RPMException, SQLException
    {
        String name = "SP_D_ASSET_MAINTENANCE";
        Object args[] = new Object[4];
        args[0] = estimates.getID();
        // estimates has parent
        args[1] = estimates.getParent().getID();
        args[2] = "X";
        args[3] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }

    /**
     * Update a Maintenance Estimate record in DB
     * 
     * @param MaintenanceEstimates -
     *            Maintenance Estimate information
     * @param messageContext -
     *            context
     * @throws SQLException
     * @throws RPMException
     */
    private void SP_U_ASSET_MAINTENANCE(MaintenanceEstimates estimates,
            MessageContext messageContext) throws SQLException, RPMException
    {
        String name = "SP_U_ASSET_MAINTENANCE";
        java.sql.Date dt;
        Object args[] = new Object[7];
        args[0] = estimates.getID();
        args[1] = estimates.getDescription();
        if (estimates.getScheduleDate() != null)
        {
            dt = new java.sql.Date(estimates.getScheduleDate()
                    .getTimeInMillis());
            args[2] = dt.toString();
        }
        args[3] = estimates.getPerformedBy();
        args[4] = estimates.getActualCost();
        args[5] = estimates.getEstimatedCost();
        args[6] = getUser(messageContext).getID();

        executeProcedure(messageContext, name, args);
    }
}
