package com.ibm.rpm.client.junit;

import java.lang.reflect.Method;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.TimeZone;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.ArrayResult;
import com.ibm.rpm.framework.Property;
import com.ibm.rpm.framework.PropertyArray;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectArray;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.interfaces.Application;
import com.ibm.rpm.interfaces.Authenticate;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.MetadataStore;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author gosselin
 */
public class ClientTestController extends TestCase
{
    protected static Log logger = LogFactory.getLog(ClientTestController.class);

    public static final String DSN = "jdbc/RPMDATASOURCE";

    public static ClientTestController INSTANCE;

    public static SpecificTestController INTERFACE_PROVIDER;

    public static String sessionid;

    protected static ArrayList objectsToDelete = new ArrayList();
    
    private static final int MAX_OBJECT_DELETION = 200;

    public static ClientTestController initTestController()
    {
        if (ClientTestController.INSTANCE == null)
        {
            ClientTestController.INSTANCE = new ClientTestController();
        }
        return INSTANCE;
    }

    public ClientTestController()
    {
        INSTANCE = this;
    }

    public synchronized void addObjectToDelete(RPMObject rpmObject)
    {
        addObjectToDelete(rpmObject, false);
    }
    
    public synchronized void addObjectToDelete(RPMObject rpmObject,
            boolean deleteLast)
    {

        boolean delete = true;

        //AbstractModule is always added so we can delete its content.
        if (!(rpmObject instanceof AbstractModule))
        {
            if (rpmObject == null || rpmObject.getID() == null)
                return;

            MetadataStore store = MetadataStore.getInstance();
            Container container = store.getContainerByJavaClass(rpmObject
                .getClass());

            if (container.isDeletable())
            {
                for (Iterator i = objectsToDelete.iterator(); i.hasNext();)
                {
                    RPMObject object = (RPMObject) i.next();
                    if (object != null && object.getID() != null
                            && object.getClass().equals(rpmObject.getClass())
                            && object.getID().equals(rpmObject.getID()))
                    {
                        delete = false;
                    }

                }
            }
            else
            {
                delete = false;
            }
        }

        if (delete)
        {
            if (deleteLast)
            {
                objectsToDelete.add(0, rpmObject);
            }
            else
            {
                objectsToDelete.add(rpmObject);
            }
        }
    }

    /**
     * required by DocumentTransfer tests
     * 
     * @return
     */
    public String getSessionID()
    {
        return sessionid;
    }

    public synchronized void removeObjectToDelete(RPMObject obj)
    {
        objectsToDelete.remove(obj);
    }

    public void setUp2() throws Exception
    {
        setUp();
    }
    
    protected void setUp() throws Exception
    {
        super.setUp();
        
        // make sure the unit tests are using the GMT time zone
        TimeZone tz1 = TimeZone.getTimeZone("GMT");
        TimeZone.setDefault(tz1);

        createInterfaceProvider();
        
        // switch to WebServicesSecurity to false
        SpecificTestController.overrideOption("webServicesSecurityEnabled", Boolean.FALSE);        
        
        INTERFACE_PROVIDER.setUp();

        logger.info("=================");
        logger.info(this.getName());
        logger.info("=================");

        performLogin();
    }

    /**
     *  
     */
    protected void createInterfaceProvider()
    {
        if (INTERFACE_PROVIDER == null)
        {
            INTERFACE_PROVIDER = new SpecificTestController();
        }

    }

    public void callTearDown() throws Exception
    {
        tearDown();
    }

    public void callSetup() throws Exception
    {
        setUp();
    }

    protected void tearDown() throws Exception
    {
        // Clean up all the created objects if it's not already done
        logger.info("=================");
        logger.info("    cleanup");
        logger.info("=================");
        synchronized (objectsToDelete)
        {
            for (int i = objectsToDelete.size() - 1; i >= 0; i--)
            {
                RPMObject objectToDelete = (RPMObject) objectsToDelete.get(i);
                if (objectToDelete instanceof AbstractModule)
                {
                    delete((AbstractModule) objectToDelete);
                }
                else
                {
                    // Ignore exceptions.
                    Result result = doDelete(objectToDelete);
                    if(result.isSuccessful())
                    {
                        // objectsToDelete.remove(objectToDelete);
                    }
                }
            }
            objectsToDelete.clear();
        }

        try
        {
            getAuthenticateInterface().logout(sessionid);
        }
        catch (Exception e)
        {
            // Ignore exception.
        }

        if(INTERFACE_PROVIDER != null)
        {
            INTERFACE_PROVIDER.tearDown();
        }
        
        super.tearDown();

    }

    public Authenticate getAuthenticateInterface()
    {
        return INTERFACE_PROVIDER.authenticateInterface;
    }

    public Application getApplicationInterface()
    {
        return INTERFACE_PROVIDER.appInterface;
    }
  
    /**
     * Return true when the tests are executed remotely, through the Web Service
     * interface. Otherwise, returns false.
     * 
     * @return True when the tests are executed remotely.
     */
    public boolean isRemote()
    {
        return INTERFACE_PROVIDER.isRemote();
    }

    /**
     * Returns the URL where web services are located
     * 
     * @return
     */
    public String getRemoteURL()
    {
        return INTERFACE_PROVIDER.getRemoteURL();
    }

    /**
     * Return true when the tests are executed locally. Otherwise, returns
     * false.
     * 
     * @return True when the tests are executed locally.
     */
    public boolean isLocal()
    {
        return !INTERFACE_PROVIDER.isRemote();
    }

    public void performLogin() throws RemoteException, RemoteException,
            Exception
    {
        String username = RpmTestProperties.getProperty("rpm.test.user.name");
        String password = RpmTestProperties.getProperty("rpm.test.user.password");
        performLogin(username, password);
    }

    public void changePassword(String username, String oldPassword,
            String newPassword) throws Exception
    {
        getAuthenticateInterface().changePassword(username, oldPassword,
                newPassword, DSN);
    }

    public void performLogin(String username, String password) throws Exception
    {
        sessionid = getAuthenticateInterface().login(username, password, DSN);
        assertNotNull(sessionid);
    }

    public SingleResult save(RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws Exception
    {
        return save(rpmObject, rpmObjectScope, ReloadType.ReloadResult);
    }

    public SingleResult save(RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
            throws Exception
    {
        SingleResult result = saveWithoutDelete(rpmObject, rpmObjectScope,
                reloadType);

        rpmObject = result.getRpmObject();

        addObjectToDelete(rpmObject);

        return result;

    }

    public SingleResult saveWithoutDelete(RPMObject rpmObject,
            RPMObjectScope rpmObjectScope) throws Exception
    {
        return saveWithoutDelete(rpmObject, rpmObjectScope,
                ReloadType.ReloadResult);
    }

    public SingleResult saveWithoutDelete(RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
            throws Exception
    {
        SingleResult saveResult = doSave(rpmObject, rpmObjectScope, reloadType);
        logAndThrow(saveResult, "Save");
        return saveResult;
    }

    public SingleResult doSave(RPMObject rpmObject,
            RPMObjectScope rpmObjectScope, ReloadType reloadType)
            throws Exception
    {
        return getApplicationInterface().save(sessionid, rpmObject,
                rpmObjectScope, reloadType);
    }
    
    public Calendar getDatabaseTimestamp(String sessionID) throws Exception
    {
        return getApplicationInterface().getDatabaseTimestamp(sessionID);
    }

    public AssignedScorecard publishScorecard(
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws Exception
    {
        SingleResult result = getApplicationInterface().publishScorecard(
                sessionid, assignedScorecard, scorecardScope);
        logAndThrow(result, "publishScorecard");
        return (AssignedScorecard) result.getRpmObject();
    }

    public static void logAndThrow(Result result, String operationName)
            throws RPMException
    {
        if (result == null)
        {
            // This should never happen.
            Assert.fail(operationName + " result is null");
        }

        // Log errors, if required.
        String errorMessage = null;
        if (!result.isSuccessful())
        {
            errorMessage = logExceptions(true, result.getErrors(),
                    operationName);
        }

        // Log warnings, if required.
        if (!result.isWarningFree())
        {
            logExceptions(false, result.getWarnings(), operationName);
        }

        // Throws a clone of the last exception with a summary message,
        // if required.
        if (!result.isSuccessful())
        {
            RPMException[] exceptions = result.getErrors();
            RPMException lastException = exceptions[exceptions.length - 1];

            RPMException summaryException = new RPMException();
            summaryException.setRpmMessage(errorMessage);
            summaryException.setClassName(lastException.getClassName());
            summaryException.setContainerID(lastException.getContainerID());
            summaryException.setErrorID(lastException.getErrorID());
            summaryException.setFieldName(lastException.getFieldName());
            summaryException.setSecondaryFieldName(lastException
                    .getSecondaryFieldName());
            summaryException.setSeverity(lastException.getSeverity());
            summaryException.setStackTrace(lastException.getStackTrace());

            throw summaryException;
        }
    }

    private static String logExceptions(boolean isError,
            RPMException[] exceptions, String operationName)
    {
        String exceptionType = isError ? "error" : "warning";
        StringBuffer msg = new StringBuffer(operationName
                + " operation has generated " + exceptionType
                + "(s). See log for details.");
        for (int i = 0; i < exceptions.length; i++)
        {
            RPMException currentException = exceptions[i];
            String currentMessage = exceptionType + ' ' + i + ": "
                    + currentException.toString();
            msg.append("\n    ");
            msg.append(currentMessage);
        }
        if (isError)
            logger.error(msg);
        else
            logger.warn(msg);
        return msg.toString();
    }

    public SingleResult saveMustFail(RPMObject rpmObject,
            RPMObjectScope rpmObjectScope) throws Exception
    {
        addObjectToDelete(rpmObject);

        SingleResult saveResult = getApplicationInterface().save(sessionid,
                rpmObject, rpmObjectScope, ReloadType.ReloadResult);

        if (saveResult == null)
        {
            Assert.fail("Result is null");
        }

        return saveResult;
    }
    
    public Result deleteMustFail(RPMObject rpmObject) throws Exception
    {
        Result deleteResult = getApplicationInterface().delete(sessionid,
                rpmObject);

        if (deleteResult == null)
        {
            Assert.fail("Result is null");
        }
        else
        {
            if( deleteResult.isSuccessful() )
            {
                Assert.fail("Delete wasn't supposed to be successful.");
            }
        }

        return deleteResult;
    }

    public synchronized Result delete(RPMObject rpmObject) throws Exception
    {
        if (rpmObject == null)
            return null;

        Result result = doDelete(rpmObject);

        logAndThrow(result, "delete");

        for (Iterator i = objectsToDelete.iterator(); i.hasNext();)
        {
            RPMObject object = (RPMObject) i.next();
            String id = object.getID();
            if (object != null && id != null
                    && object.getClass() == rpmObject.getClass()
                    && id.equals(rpmObject.getID()))
            {
                i.remove();
            }
        }
        return result;
    }

    private Result doDelete(RPMObject rpmObject) throws Exception
    {
        return getApplicationInterface().delete(sessionid, rpmObject);
    }

    public void delete(RPMObject[] rpmObjects) throws Exception
    {
        if (rpmObjects == null)
        {
            return;
        }
        
        for (int i = 0; i < rpmObjects.length; i++)
        {
            delete(rpmObjects[i]);
        }
    }

    public void doDelete(RPMObject[] rpmObjects) throws Exception
    {
        if (rpmObjects == null)
        {
            return;
        }
        
        for (int i = 0; i < rpmObjects.length; i++)
        {
            RPMObject objectToDelete = rpmObjects[i];
            if( objectToDelete != null )
            {
                doDelete(objectToDelete);
            }
        }
    }
    
    public void delete(AbstractModule module)
    {
        if (module != null)
        {
            Class moduleClass = module.getClass();
            Method[] methods = moduleClass.getMethods();

            for (int i = 0; i < methods.length; i++)
            {
                Method method = methods[i];

                try
                {
                    if (method.getName().startsWith("get"))
                    {
                        Object invokeResult = method.invoke(
                                module, null);
                        
                        if ( invokeResult instanceof RPMObject[])
                        {
	                        RPMObject[] objects = (RPMObject[]) invokeResult;
	                        if (objects != null)
	                        {
	                            doDelete(objects);
	                        }
                        }
                    }
                }
                catch (Exception e)
                {
                    //continue...
                }
            }
        }
    }

    public RPMObject[] loadFromXpath(String xpath,
			RPMObjectScope scope, Integer idsType, String idsList,
			String attributesIdsList, Integer viewType, Integer inclusions,
			Integer startIndex, Integer endIndex, String extraWhere, String orderBy,
			boolean buildSecurity) throws Exception {
		logger.info("loadFromXpath: " + xpath);
		ArrayResult result = INTERFACE_PROVIDER.loadFromXpath(sessionid, xpath, scope, idsType,
				idsList, attributesIdsList, viewType, inclusions, startIndex,
				endIndex, extraWhere, orderBy, buildSecurity);
		logAndThrow(result, "loadFromXpath");
		return result.getRpmObjectList();
	}
    
    /**
     * Loads one object using <code>xpath</code> and fails if it matches zero
     * or more than one object(s). 
     * @param xpath XPath expression.
     * @param scope Scope of this load opperation. Nullable.
     * @return The object that matches <code>xpath</code>.
     */
    public final RPMObject loadOneFromXpath(String xpath, RPMObjectScope scope)
    	throws Exception
	{
		RPMObject[] result = loadFromXpath(xpath, scope);
		if (result == null || result.length == 0)
		{
		    throw new Exception("loadOneFromXpath returned no result - "
		            + xpath);
		}
		else if (result.length > 1)
		{
		    throw new Exception(
		            "loadOneFromXpath returned more than one result - " + xpath);
		}
		return result[0];
	}

    public RPMObject[] loadFromXpath(String xpath, RPMObjectScope scope)
            throws Exception
    {
        logger.info("loadFromXpath: " + xpath);
        ArrayResult result = getApplicationInterface().loadFromXpath(sessionid,
                xpath, scope);
        logAndThrow(result, "loadFromXpath");
        return result.getRpmObjectList();
    }

    public RPMObject[] loadFromXpathMustFail(String xpath, RPMObjectScope scope)
            throws Exception
    {
        logger.info("loadFromXpath: " + xpath);
        ArrayResult result = getApplicationInterface().loadFromXpath(sessionid,
                xpath, scope);

        if (!result.isSuccessful())
        {
            throw result.getErrors()[0];
        }

        return result.getRpmObjectList();
    }

    public RPMObject loadFromID(RPMObject rpmObject, RPMObjectScope scope)
            throws Exception
    {
        SingleResult result = getApplicationInterface().loadFromID(sessionid,
                rpmObject, scope);
        logAndThrow(result, "loadFromID");
        return result.getRpmObject();
    }

    public RPMObject[] loadArrayFromID(RPMObject[] rpmObjectArray,
            RPMObjectScope scope) throws Exception
    {
        RPMObjectArray arrayWrapper = new RPMObjectArray();
        arrayWrapper.setRPMObjectList(rpmObjectArray);
        ArrayResult result = getApplicationInterface().loadArrayFromID(
                sessionid, arrayWrapper, scope);
        logAndThrow(result, "loadArrayFromID");
        return result.getRpmObjectList();
    }

    public SingleResult copyProposedToPlan(WorkElement rpmObject, WorkElementScope scope)
            throws Exception
    {
        SingleResult result = getApplicationInterface().copyProposedToPlan(sessionid,
            rpmObject, scope);
        logAndThrow(result, "copyProposedToPlan");
        return result;
    }
    
    public Result transferUnderWBS(WorkElement rpmObject,
            AbstractScope abstractScope) throws Exception
    {
        Result result = getApplicationInterface().transferUnderWBS(sessionid,
            rpmObject, abstractScope);
        logAndThrow(result, "transferUnderWBS");
        return result;
    }
    
    public Result moveBeforeSibling(ScopeElement objectToMove,
            ScopeElement nextObject) throws Exception
    {
        Result result = getApplicationInterface().moveBeforeSibling(sessionid,
            objectToMove, nextObject);
        logAndThrow(result, "moveBeforeSibling");
        return result;
    }
    
    public Result moveBeforeSiblingMustFail(ScopeElement objectToMove,
            ScopeElement nextObject) throws Exception
    {
        Result result = getApplicationInterface().moveBeforeSibling(sessionid,
            objectToMove, nextObject);

        if (result == null)
        {
            Assert.fail("Result is null");
        }
        else
        {
            if (result.isSuccessful())
            {
                Assert
                    .fail("moveBeforeSibling wasn't supposed to be successful.");
            }
        }

        return result;
    }
    
    public SingleResult lockDocument(GenericDocument documentToLock) throws Exception
    {
    	SingleResult result = doLockDocument(documentToLock);
    	logAndThrow(result, "lockDocument");
    	return result;
    }
    
    public SingleResult unLockDocument(GenericDocument documentToUnlock) throws Exception
    {
    	SingleResult result = doUnLockDocument(documentToUnlock);
    	logAndThrow(result, "unLockDocument");
    	return result;
    }
    
    public SingleResult doLockDocument(GenericDocument documentToLock) throws Exception
    {
    	return getApplicationInterface().lockDocument(sessionid, documentToLock);
    }
    
    public SingleResult doUnLockDocument(GenericDocument documentToUnlock) throws Exception
    {
    	return getApplicationInterface().unLockDocument(sessionid, documentToUnlock);
    }
    
    public Property[] getSystemInfo() throws Exception
    {
        PropertyArray systemInfo = getApplicationInterface().getSystemInfo(sessionid);
        return systemInfo.getProperties();
    }

    /**
     * Fails if <code>expected.getID() != actual.getID()</code> but only if
     * expected is not null.
     * @param expected Expected object.
     * @param actual Actual object.
     */
    public static final void assertIdEqualsIfNotNull(RPMObject expected, RPMObject actual)
    {
        if (expected != null && expected.getID() != null)
        {
            assertNotNull(actual);
            assertEquals(expected.getID(), actual.getID());
        }
    }

    /**
     * Fails if either expected or actual is null but not the other
     * and fails if <code>expected.getID() != actual.getID()</code>.
     * @param expected Expected object.
     * @param actual Actual object.
     */
    public static final void assertIdEquals(RPMObject expected, RPMObject actual)
    {
        if (expected == null)
        {
            assertEquals(expected, actual);
        }
        else
        {
            assertNotNull(actual);
            assertEquals(expected.getID(), actual.getID());
        }
    }

    /**
     * Deletes all instances of <code>aClass</code> or its subclasses
     * that have a name "" (empty) or "A" or that starts with "TC" or "ABCDE".
     * Assumes that XPath queries on this class are implemented.
     * @param aClass Class of the objects to be deleted.
     */
    public void deleteAllTestObjects(Class aClass) throws Exception
    {
        deleteTestObjects(aClass, 
                HelperTestUtilities.unitTestPrefix, true);
        deleteTestObjects(aClass, "", false);
        deleteTestObjects(aClass, "ABCDE", true);
        deleteTestObjects(aClass, "A", false);
        //deleteTestObjects(aClass, "WS", true);
    }
    
    /**
     * Deletes all instances of <code>aClass</code> or its subclasses
     * that have a name or name prefix <code>objectName</code>, as 
     * specified by <code>isPrefix</code>.
     * @param aClass Class of the objects to be deleted.
     * @param objectName Name or name prefix of the objects to be deleted. 
     * @param isPrefix True if objectName specifies a prefix, false otherwise.
     */
    public void deleteTestObjects(Class aClass, String objectName, 
            boolean isPrefix) throws Exception
    {
        RPMObject[] objects = loadObjectsToDelete(aClass, objectName, 
                isPrefix);
        if (objects != null)
        {
            // Design note: avoid to delete too many objects in the same
            // run to avoid killing the database.
            int limit = Math.min(MAX_OBJECT_DELETION, objects.length);
	        for (int i = 0; i < limit; i++)
	        {
	            try
	            {
	                delete(objects[i]);
	            }
	            catch (RPMException x)
	            {
		            // Ignores non-SQL exceptions.
	                if (x.getErrorID() < 0)
	                    throw x;
	            }
	            catch (Exception x)
	            {
		            // Ignores most exceptions.
	            }
	        }
        }
    }

    private RPMObject[] loadObjectsToDelete(Class aClass, 
            String objectName, boolean isPrefix) throws Exception
    {
        String classShortName = StringUtil.getShortClassName(aClass);
        StringBuffer queryBuffer = new StringBuffer("/");
        queryBuffer.append(classShortName);
        queryBuffer.append('[');
        if (isPrefix)
        {
            queryBuffer.append("starts-with(name, '");
            queryBuffer.append(objectName);
            queryBuffer.append("')");
        }
        else
        {
            queryBuffer.append("name='");
            queryBuffer.append(objectName);
            queryBuffer.append('\'');
        }
        queryBuffer.append(']');
        final String query = queryBuffer.toString();
        RPMObject[] loaded = loadFromXpath(query, null);
        return loaded;
    }    
    
    public Result disableAutomaticCheckout() throws Exception
    {
        return getApplicationInterface().disableAutomaticCheckoutMode(sessionid);
    }
    
    public Result enableAutomaticCheckout() throws Exception
    {
        return getApplicationInterface().enableAutomaticCheckoutMode(sessionid);
    }
 
    public void checkIn(RPMObject rpmObject) throws Exception
    {
        Result result = getApplicationInterface().checkIn(sessionid, rpmObject);
        logAndThrow(result, "checkIn");
    }
    
    public void checkOut(RPMObject rpmObject)
            throws Exception
    {
        checkOut(rpmObject, false);
    }
    
    public void checkOut(RPMObject rpmObject, boolean isTimeless)
            throws Exception
    {
        Result result = getApplicationInterface().checkOut(sessionid,
            rpmObject, isTimeless);
        logAndThrow(result, "checkOut");
    }
    
    public boolean isCheckedOut(RPMObject rpmObject) throws Exception
    {
        AlreadyCheckedOutResult result = getApplicationInterface().isCheckedOut(sessionid,
            rpmObject);
        logAndThrow(result, "isCheckedOut");
        return result.isCheckedOut();
    }
    
    public void forceCheckinOnRPMObjectCheckOut(RPMObject rpmObject)
            throws Exception
    {
        Result result = getApplicationInterface().forceCheckinOnRPMObjectCheckOut(
            sessionid, rpmObject);
        logAndThrow(result, "forceUndoRPMObjectCheckOut");
    }
    
    public void forceCheckinOnUsersCheckOut(Resource user) throws Exception
    {
        Result result = getApplicationInterface().forceCheckinOnUsersCheckOut(
            sessionid, user);
        logAndThrow(result, "forceCheckinOnUsersCheckOut");
    }
    
    public void forceCheckinOnSessionsCheckOut(String targetSession)
            throws Exception
    {
        Result result = getApplicationInterface()
        .forceCheckinOnSessionsCheckOut(sessionid, targetSession);
        logAndThrow(result, "forceCheckinOnSessionsCheckOut");
    }
    
    public RPMObject[] searchTimeCodes(String workElementID,
            String scopeElementID, boolean isTimeCode1) throws Exception
    {
        ArrayResult result = INTERFACE_PROVIDER.internalServices
            .searchTimeCodes(sessionid, workElementID, scopeElementID,
                isTimeCode1);
        if (!result.isSuccessful())
        {
            throw result.getErrors()[0];
        }
        
        return result.getRpmObjectList();
    }   
    
    public Result executeProcedure(Class callerClass, String procedureName,
            Object[] args) throws Exception {
		logger.info("executeProcedure: " + procedureName);
		Result result = INTERFACE_PROVIDER.executeProcedure(sessionid, callerClass, procedureName, args);
		logAndThrow(result, "executeProcedure");
		return result;
	}
    
    public Result executeSqlUpdate(String updateString,
            Object[] args) throws Exception
    {
        logger.info("executeSqlUpdate: " + updateString);
        Result result = INTERFACE_PROVIDER.executeSqlUpdate(
            sessionid, updateString, args);
        logAndThrow(result, "executeSqlUpdate");
        return result;
    }
    
    public ArrayList executeSqlQuery(String query, Object[] args)
            throws Exception
    {
        logger.info("executeSqlQuery: " + query);
        ArrayList rowsList = INTERFACE_PROVIDER.executeSqlQuery(sessionid, query,
            args);
        return rowsList;
    }
}
