/*
 * Created on 5-Oct-2005
 */
package com.ibm.rpm.client.junit.communications.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.communications.fixtures.GenericCommunicationFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.communications.constants.ValidationConstants;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowAttachment;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.interfaces.wrapper.ResourceArray;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.wbs.containers.Project;

/**
 * @author dominiqu
 */
public class TestSendStaffingRequestValidation extends GenericValidation
{
    private StaffingRequest mainStaffingRequest;

    private GenericCommunicationScope fullScope = null;

    protected void setUp() throws Exception
    {
        super.setUp();

        fullScope = GenericCommunicationFixture.createFullScope(true);

        Project attachment = (Project) WorkElementFixture.createSaveFixture(
            new Project(), null, this);

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(attachment);

        mainStaffingRequest = (StaffingRequest) GenericCommunicationFixture.createFixture(
            workflowAttachment, new StaffingRequest(), fullScope, this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithNotExistingAttachment() throws AssertionError,
            Exception
    {
        Project wrongAttachment = new Project();
        wrongAttachment.setID("NOT_EXISTING_ID");

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(wrongAttachment);
        mainStaffingRequest.setWorkflowAttachment(workflowAttachment);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                ErrorCodes.NOT_EXISTING_ID,
                mainStaffingRequest.getWorkflowAttachment().getClass().getName(),
                ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                mainStaffingRequest.getWorkflowAttachment().getID())
        };

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(ErrorCodes.AM_INVALID_RPMOBJECTID,
                wrongAttachment.getClass().getName(), null,
                wrongAttachment.getID())
        };
        
        verifySendStaffingRequestMustFail(mainStaffingRequest, fullScope,
            expectedExceptions, expectedWarnings);
    }

    public void testFailureWithInvalidAttachmentType() throws AssertionError,
            Exception
    {
        ClientCostCenter wrongAttachment = new ClientCostCenter();
        wrongAttachment.setID("NOT_EXISTING_ID");

        WorkflowAttachment workflowAttachment = new WorkflowAttachment();
        workflowAttachment.setAttachment(wrongAttachment);
        mainStaffingRequest.setWorkflowAttachment(workflowAttachment);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                ErrorCodes.INVALID_TYPE,
                mainStaffingRequest.getWorkflowAttachment().getClass().getName(),
                ValidationConstants.WORKFLOWATTACHMENT_ATTACHMENT_FIELD,
                mainStaffingRequest.getWorkflowAttachment().getID())
        };

        verifySendStaffingRequestMustFail(mainStaffingRequest, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithInvalidResource() throws AssertionError,
            Exception
    {
        Resource notExistingResource = new Resource();
        notExistingResource.setID("NOT_EXISTING_RESOURCE");
        mainStaffingRequest.getResponses()[0].setResource(notExistingResource);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.NOT_EXISTING_ID,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_RESOURCE_FIELD, null)
        };

        verifySendStaffingRequestMustFail(mainStaffingRequest, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithDuplicateResources() throws AssertionError,
            Exception
    {
        mainStaffingRequest.getResponses()[1].setResource(mainStaffingRequest.getResponses()[0].getResource());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.communications.constants.ErrorCodes.NO_DUPLICATE_RECIPIENTS,
                StaffingRequest.class.getName(),
                ValidationConstants.GENERICCOMMUNICATION_REPLIES_FIELD, null)
        };

        verifySendStaffingRequestMustFail(mainStaffingRequest, fullScope,
            expectedExceptions, null);
    }

    public void testFailureWithInvalidResourceManager() throws AssertionError,
            Exception
    {
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setCalendar(new CalendarScope());
        Resource notResourceManager = ResourceFixture.createSaveFixture(
            resourceScope, this);
        mainStaffingRequest.getResponses()[0].setResource(notResourceManager);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.resource.constants.ErrorCodes.NOT_RESOURCE_MANAGER,
                WorkflowResponse.class.getName(),
                ValidationConstants.WORKFLOWRESPONSE_RESOURCE_FIELD, null)
        };

        verifySendStaffingRequestMustFail(mainStaffingRequest, fullScope,
            expectedExceptions, null);
    }

    public void testSuccessValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainStaffingRequest.setSubject(getTestString(ValidationConstants.GENERICCOMMUNICATION_SUBJECT_MAX));
        mainStaffingRequest.setComments(getTestString(ValidationConstants.GENERICCOMMUNICATION_COMMENTS_MAX));

        // Attempt to save the StaffingRequest object
        StaffingRequest saved = (StaffingRequest) sendStaffingRequestMustSucceed(
            mainStaffingRequest, StaffingRequest.class, fullScope);
        mainStaffingRequest.setID(saved.getID());

        // Load the inserted StaffingRequest from xpath
        RPMObject[] rpmObjects = loadFromXpath("/StaffingRequest[Id='"
                + mainStaffingRequest.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof StaffingRequest);

        // Check that the loaded StaffingRequest is identical to the one we
        // requested to store
        GenericCommunicationFixture.compare(mainStaffingRequest,
            (StaffingRequest) rpmObjects[0], null);
    }

    public void testFailureValidation() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainStaffingRequest.setSubject(getTestString(ValidationConstants.GENERICCOMMUNICATION_SUBJECT_MAX + 1));
        mainStaffingRequest.setComments(getTestString(ValidationConstants.GENERICCOMMUNICATION_COMMENTS_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH,
                    mainStaffingRequest.getClass().getName(),
                    ValidationConstants.GENERICCOMMUNICATION_SUBJECT_FIELD,
                    mainStaffingRequest.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH,
                    mainStaffingRequest.getClass().getName(),
                    ValidationConstants.GENERICCOMMUNICATION_COMMENTS_FIELD,
                    mainStaffingRequest.getID())
        };

        verifySendStaffingRequestMustFail(mainStaffingRequest, fullScope,
            expectedExceptions, null);
    }

    /**
     * Generic sendStaffingRequest function for sendStaffingRequest calls that
     * must succeed
     */
    protected RPMObject sendStaffingRequestMustSucceed(StaffingRequest obj,
            Class objType, GenericCommunicationScope scope)
            throws AssertionError, Exception
    {
        assertNotNull(obj);
        assertTrue(objType.isInstance(obj));

        addObjectToDelete(obj);

        // Retrieve the recipients
        Resource[] recipients = new Resource[obj.getResponses().length];
        for (int i = 0; i < recipients.length; ++i)
        {
            if (obj.getResponses()[i] != null)
            {
                recipients[i] = obj.getResponses()[i].getResource();
            }
        }

        // Attempt to save the object
        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        SingleResult result = getApplicationInterface().sendStaffingRequest(
            sessionid, scope, obj.getWorkflowAttachment().getAttachment(),
            resourceArray, obj.getSubject(), obj.getComments(),
            obj.getDueDate());

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(objType.isInstance(result.getRpmObject()));
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());

        obj = (StaffingRequest) result.getRpmObject();
        return obj;
    }

    /**
     * Generic function to verify that the expected exceptions are received from
     * the failed sendStaffingRequest
     */
    protected void verifySendStaffingRequestMustFail(StaffingRequest obj,
            GenericCommunicationScope scope,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings) throws AssertionError,
            Exception
    {
        addObjectToDelete(obj);

        // Retrieve the recipients
        Resource[] recipients = new Resource[obj.getResponses().length];
        for (int i = 0; i < recipients.length; ++i)
        {
            if (obj.getResponses()[i] != null)
            {
                recipients[i] = obj.getResponses()[i].getResource();
            }
        }

        // Attempt to save the object
        ResourceArray resourceArray = new ResourceArray();
        resourceArray.setResourceArray(recipients);
        SingleResult saveResult = getApplicationInterface().sendStaffingRequest(
            sessionid, scope, obj.getWorkflowAttachment().getAttachment(),
            resourceArray, obj.getSubject(), obj.getComments(),
            obj.getDueDate());

        if (saveResult == null)
        {
            Assert.fail("Result is null");
        }

        // Check that we received the right exceptions
        if (saveResult.getErrors() == null)
        {
            Assert.fail(StringUtil.getShortClassName(obj.getClass())
                    + " validation failed: expected exceptions");
        }

        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, saveResult);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}
