/*
 * Created on January 3, 2006
 * @author mmatwiy 
 * Test cases for creation of a ContractChargeSchedule.
 */
package com.ibm.rpm.client.junit.contract.testcases.validation;

import com.ibm.rpm.client.junit.contract.fixtures.ContractChargeScheduleFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.contract.constants.ValidationConstants;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractChargeSchedule;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

public class TestContractChargeScheduleValidation extends GenericValidation
{

    private ContractChargeSchedule contractChargeSchedule;

    public void setUp() throws Exception
    {
        super.setUp();

        contractChargeSchedule = new ContractChargeSchedule();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        ContractFolder parentContractFolder = ContractFolderFixture
            .createSaveFixture(1, this);
        contractChargeSchedule.setParent(parentContractFolder);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE,
                contractChargeSchedule.getClass().getName(),
                ValidationConstants.PARENT, contractChargeSchedule.getID())
        };

        verifySaveMustFail(contractChargeSchedule, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        contractChargeSchedule.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, contractChargeSchedule
                .getClass().getName(), ValidationConstants.PARENT,
                contractChargeSchedule.getID())
        };

        verifySaveMustFail(contractChargeSchedule, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        Contract parentContract = ContractFixture.createSaveFixture(this);

        contractChargeSchedule.setParent(parentContract);

        // Set field values with maxLength and verify that they are accepted
        contractChargeSchedule.setAmount(new Double(
            ValidationConstants.AMOUNT_MAX));

        // Attempt to save the ContractChargeSchedule object
        contractChargeSchedule = (ContractChargeSchedule) saveMustSucceed(
            contractChargeSchedule, ContractChargeSchedule.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractChargeSchedule[id='"
                + contractChargeSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractChargeSchedule);

        // Check that the loaded ContractChargeSchedule is identical to the one we
        // requested to store
        ContractChargeScheduleFixture.compare(contractChargeSchedule,
            (ContractChargeSchedule) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        Contract parentContract = ContractFixture.createSaveFixture(this);
        contractChargeSchedule.setParent(parentContract);

        // Set field values with maxLength + 1 and verify that they are not
        // accepted
        contractChargeSchedule.setAmount(new Double(
            ValidationConstants.AMOUNT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, contractChargeSchedule
                .getClass().getName(), ValidationConstants.AMOUNT_FIELD,
                contractChargeSchedule.getID())
        };

        verifySaveMustFail(contractChargeSchedule, null, expectedExceptions);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        Contract parentContract = ContractFixture.createSaveFixture(this);

        contractChargeSchedule.setParent(parentContract);

        // Set field values with minLength and verify that they are accepted
        contractChargeSchedule.setAmount(new Double(
            ValidationConstants.AMOUNT_MIN));

        // Attempt to save the ContractChargeSchedule object
        contractChargeSchedule = (ContractChargeSchedule) saveMustSucceed(
            contractChargeSchedule, ContractChargeSchedule.class,
            ContractFixture.createScope());

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractChargeSchedule[id='"
                + contractChargeSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractChargeSchedule);

        // Check that the loaded ContractChargeSchedule is identical to the one we
        // requested to store
        ContractChargeScheduleFixture.compare(contractChargeSchedule,
            (ContractChargeSchedule) rpmObjects[0], null);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        Contract parentContract = ContractFixture.createSaveFixture(this);
        contractChargeSchedule.setParent(parentContract);

        // Set field values with minLength -1 and verify that they are not
        // accepted        
        contractChargeSchedule.setAmount(new Double(
            ValidationConstants.AMOUNT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, contractChargeSchedule
                .getClass().getName(), ValidationConstants.AMOUNT_FIELD,
                contractChargeSchedule.getID())
        };

        verifySaveMustFail(contractChargeSchedule, null, expectedExceptions);
    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        Contract parentContract = ContractFixture.createSaveFixture(this);

        contractChargeSchedule.setParent(parentContract);

        // Leave default values as null and save
        contractChargeSchedule = (ContractChargeSchedule) saveMustSucceed(
            contractChargeSchedule, ContractChargeSchedule.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractChargeSchedule[id='"
                + contractChargeSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractChargeSchedule);

        // Check that the default values are now set
        ContractChargeSchedule savedContractChargeSchedule = (ContractChargeSchedule) rpmObjects[0];
        assertNotNull(savedContractChargeSchedule.getScheduleDate());
        assertNotNull(savedContractChargeSchedule.getAmount());

        // Save again to validate the default values
        savedContractChargeSchedule = (ContractChargeSchedule) saveMustSucceed(
            savedContractChargeSchedule, ContractChargeSchedule.class, null);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ContractChargeSchedule[id='"
                + savedContractChargeSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractChargeSchedule);

        // Check that the loaded ContractChargeSchedule is identical to the one we
        // requested to store
        ContractChargeScheduleFixture.compare(savedContractChargeSchedule,
            (ContractChargeSchedule) rpmObjects[0], null);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        Contract parentContract = ContractFixture.createSaveFixture(this);

        contractChargeSchedule.setParent(parentContract);

        // Leave default values as null and save
        contractChargeSchedule = (ContractChargeSchedule) saveMustSucceed(
            contractChargeSchedule, ContractChargeSchedule.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ContractChargeSchedule[id='"
                + contractChargeSchedule.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ContractChargeSchedule);

        // Check that the default values are now set
        ContractChargeSchedule savedContractChargeSchedule = (ContractChargeSchedule) rpmObjects[0];
        assertNotNull(savedContractChargeSchedule.getScheduleDate());
        assertNotNull(savedContractChargeSchedule.getAmount());

        // Update the default values to null
        savedContractChargeSchedule.setScheduleDate(null);
        savedContractChargeSchedule.setAmount(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedContractChargeSchedule.getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FIELD,
                    savedContractChargeSchedule.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedContractChargeSchedule.getClass().getName(),
                    ValidationConstants.AMOUNT_FIELD, savedContractChargeSchedule
                        .getID())
        };

        verifySaveMustFail(savedContractChargeSchedule, null, expectedExceptions);

    }

}
