/*
 * Created on January 3, 2006
 * @author mmatwiy 
 * Test cases for creation of a contract.
 */
package com.ibm.rpm.client.junit.contract.testcases.validation;

import com.ibm.rpm.applicationadministration.containers.ContractType;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFixture;
import com.ibm.rpm.client.junit.contract.fixtures.ContractFolderFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.contract.constants.ValidationConstants;
import com.ibm.rpm.contract.containers.Contract;
import com.ibm.rpm.contract.containers.ContractFolder;
import com.ibm.rpm.contract.scope.ContractScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 *  
 */
public class TestContractValidation extends GenericValidation
{

    private Contract contract;

    private ContractScope scope;

    public void setUp() throws Exception
    {
        super.setUp();

        scope = ContractFixture.createScope();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithParentContractFolder() throws AssertionError,
            Exception
    {
        contract = new Contract();
        contract.setName(getTestString(ValidationConstants.NAME_MAX));

        ContractFolder parentFolder = ContractFolderFixture.createSaveFixture(
            1, this);
        contract.setParent(parentFolder);

        // Attempt to save the object
        contract = (Contract) saveMustSucceed(contract, Contract.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Contract[id='"
                + contract.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the loaded Contract is identical to the one we
        // requested to store
        ContractFixture.compare(contract, (Contract) rpmObjects[0], scope);
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        contract = new Contract();

        // set mandatory field
        contract.setName("test");

        // create and set invalid parent
        Contract parentContract = ContractFixture.createSaveFixture(this);
        contract.setParent(parentContract);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, contract
                .getClass().getName(), ValidationConstants.PARENT, contract
                .getID())
        };

        verifySaveMustFail(contract, scope, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        contract = new Contract();

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, contract.getClass()
                .getName(), ValidationConstants.NAME_FIELD, contract.getID())
        };

        verifySaveMustFail(contract, scope, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        contract = new Contract();
        contract.setName(getTestString(ValidationConstants.NAME_MAX));

        // Attempt to save the Contract object
        contract = (Contract) saveMustSucceed(contract, Contract.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Contract[id='"
                + contract.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the loaded Contract is identical to the one we
        // requested to store
        ContractFixture.compare(contract, (Contract) rpmObjects[0], scope);
    }

    public void testFailureWithInvalidContractType() throws AssertionError, Exception
    {
        ContractType contractType = new ContractType();
        contractType.setValue("DDD");
        
        Contract contract = new Contract();
        contract.setName("CtrName");
        contract.setContractType(contractType);
        
        ContractScope scope = new ContractScope();
        scope.setContractType(true);
        
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.NOT_EXISTING_ID, contract.getClass()
                    .getName(), "contractType", contract
                    .getID())
        };

        verifySaveMustFailOneWay(contract, scope, expectedExceptions);
    }
    
    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        contract = new Contract();

        // Set field values with maxLength and verify that they are accepted
        contract.setName(getTestString(ValidationConstants.NAME_MAX));
        contract
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX));
        contract
            .setExternalReference(getTestString(ValidationConstants.EXTERNAL_REFERENCE_MAX));
        contract.setMaximumAmount(new Double(
            ValidationConstants.MAXIMUM_AMOUNT_MAX));

        // Attempt to save the Contract object
        contract = (Contract) saveMustSucceed(contract, Contract.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Contract[id='"
                + contract.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the loaded Contract is identical to the one we
        // requested to store
        ContractFixture.compare(contract, (Contract) rpmObjects[0], scope);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        contract = new Contract();

        // Set field values with maxLength+1 and verify that they are not
        // accepted
        contract.setName(getTestString(ValidationConstants.NAME_MAX + 1));
        contract
            .setDescription(getTestString(ValidationConstants.DESCRIPTION_MAX + 1));
        contract
            .setExternalReference(getTestString(ValidationConstants.EXTERNAL_REFERENCE_MAX + 1));
        contract.setMaximumAmount(new Double(
            ValidationConstants.MAXIMUM_AMOUNT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, contract.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, contract
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, contract.getClass()
                    .getName(), ValidationConstants.DESCRIPTION_FIELD, contract
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, contract.getClass()
                    .getName(), ValidationConstants.EXTERNAL_REFERENCE_FIELD,
                    contract.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, contract.getClass()
                    .getName(), ValidationConstants.MAXIMUM_AMOUNT_FIELD,
                    contract.getID())
        };

        verifySaveMustFail(contract, scope, expectedExceptions);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        contract = new Contract();
        contract.setName(getTestString(ValidationConstants.NAME_MAX));

        // Set field values with minLength and verify that they are accepted
        contract.setMaximumAmount(new Double(
            ValidationConstants.MAXIMUM_AMOUNT_MIN));

        // Attempt to save the Contract object
        contract = (Contract) saveMustSucceed(contract, Contract.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Contract[id='"
                + contract.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the loaded Contract is identical to the one we
        // requested to store
        ContractFixture.compare(contract, (Contract) rpmObjects[0], scope);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        contract = new Contract();

        // Set field values with minLength -1 and verify that they are not
        // accepted
        contract.setName("");
        contract.setMaximumAmount(new Double(
            ValidationConstants.MAXIMUM_AMOUNT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, contract.getClass()
                    .getName(), ValidationConstants.MAXIMUM_AMOUNT_FIELD,
                    contract.getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, contract.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, contract
                    .getID())
        };

        verifySaveMustFail(contract, scope, expectedExceptions);
    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        contract = new Contract();
        contract.setName(getTestString(ValidationConstants.NAME_MAX));

        // Leave default values as null and save
        contract = (Contract) saveMustSucceed(contract, Contract.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Contract[id='"
                + contract.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the default values are now set
        Contract savedContract = (Contract) rpmObjects[0];
        assertNotNull(savedContract.getContractType());

        // Save again to validate the default values
        savedContract = (Contract) saveMustSucceed(savedContract,
            Contract.class, scope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/Contract[id='" + savedContract.getID()
                + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the loaded Contract is identical to the one we
        // requested to store
        ContractFixture.compare(savedContract, (Contract) rpmObjects[0], scope);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        contract = new Contract();
        contract.setName(getTestString(ValidationConstants.NAME_MAX));

        // Leave default values as null and save
        contract = (Contract) saveMustSucceed(contract, Contract.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Contract[id='"
                + contract.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Contract);

        // Check that the default values are now set
        Contract savedContract = (Contract) rpmObjects[0];
        assertNotNull(savedContract.getContractType());

        // Update the default values to null
        savedContract.setContractType(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedContract
                .getClass().getName(), ValidationConstants.CONTRACT_TYPE_FIELD,
                savedContract.getID())
        };

        verifySaveMustFail(savedContract, scope, expectedExceptions);

    }
}
