package com.ibm.rpm.client.junit.financial.fixtures;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.Assert;

import com.ibm.rpm.applicationadministration.containers.ChargeCodeState;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.fixtures.RPMObjectFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;

/**
 * @author legaults
 */
public class ChargeCodeFixture extends RPMObjectFixture
{
    // ====================
    // CHARGE CODE DELETION
    // ====================

    public static void assertAllDeleted(ClientTestController controller)
            throws Exception
    {
        RPMObject[] loaded = loadNonDeleted(controller);
        Assert.assertTrue("All charge codes should be deleted", loaded == null
                || loaded.length == 0);
    }

    private static RPMObject[] loadNonDeleted(ClientTestController controller)
    	throws Exception
	{
        ArrayList list = new ArrayList();
        list.addAll(doLoadNonDeleted(controller, PREFIX, true));
        list.addAll(doLoadNonDeleted(controller, TEST_STRING, true));
        list.addAll(doLoadNonDeleted(controller, "", false));
        list.addAll(doLoadNonDeleted(controller, "A", false));
		
		RPMObject[] loaded = (RPMObject[]) listToArray(RPMObject.class, list);
		return loaded;
	}

    private static Object[] listToArray(Class type, ArrayList list)
    {
        Object[] array = null;
        if (list != null && !list.isEmpty())
        {
            array = (Object[]) Array.newInstance(type, list.size());
            array = list.toArray(array);
        }
        return array;
    }
    
    private static List doLoadNonDeleted(ClientTestController controller,
            String name, boolean isPrefix) throws Exception
    {
        // Build query.
        final String query;
        if (isPrefix)
            query = "/GenericChargeCode[starts-with(name, '" + name + "')]";
        else
            query = "/GenericChargeCode[name='" + name + "']";

        // Load the set of objects that match the specified property.
        ChargeCodeScope scope = createUniversalChargeCodeScope();
        RPMObject[] loaded = controller.loadFromXpath(query, scope);
        return loaded == null ? new ArrayList() : Arrays.asList(loaded);
    }

    /**
     * Deletes all saved charge codes that have a name that starts with
     * ChargeCodeFixture.PREFIX.
     * 
     * @param controller
     */
    public static void deleteAllChargeCodes(ClientTestController controller)
            throws Exception
    {
        RPMObject[] chargeCodes = loadNonDeleted(controller);
        if (chargeCodes == null)
            return;
        // Set all external ids to null.
        for (int i = 0; i < chargeCodes.length; i++)
        {
            doMakeDeletable(controller, (GenericChargeCode) chargeCodes[i]);
        }
        // Delete all charge codes.
        for (int i = 0; i < chargeCodes.length; i++)
        {
            try
            {
                controller.delete(chargeCodes[i]);
            }
            catch (Exception x)
            {
                // Charge code might have been deleted when ancestor
                // was deleted.
            }
        }
    }

    public static void makeDeletable(ClientTestController controller,
            GenericChargeCode code) throws Exception
    {
        if (isTestChargeCode(code))
        {
            doMakeDeletable(controller, code);
        }
    }

    private static boolean isTestChargeCode(GenericChargeCode code)
    {
        String name = code.getName();
        return (name == null)  || (name.length() == 0)
                || name.startsWith(PREFIX) || name.startsWith(TEST_STRING)
                || name.equals("A");
    }

    private static void doMakeDeletable(ClientTestController controller,
            GenericChargeCode code) throws Exception
    {
        if (code.getExternalID() != null)
        {
            code.setName(PREFIX + " to be deleted");
            code.setExternalID(null);
            controller.save(code, null, ReloadType.None);
        }
        if (code instanceof ChargeCodeCategory)
        {
            ChargeCodeCategory category = (ChargeCodeCategory) code;
            GenericChargeCode[] children = category.getChildren();
            if (children != null)
            {
                for (int i = 0; i < children.length; i++)
                {
                    doMakeDeletable(controller, children[i]);
                }
            }
        }
    }

    private static final String TEST_STRING = GenericValidation
            .createFixedLengthString(10);

    // =========================
    // SIMPLE CHARGECODE BUILDER
    // =========================

    public static ChargeCode createAndSaveSimpleChargeCode(
            ClientTestController controler) throws Exception
    {
        return createAndSaveSimpleChargeCode(controler, false);
    }

    public static ChargeCode createAndSaveSimpleChargeCode(
            ClientTestController controler, boolean hasExternalId)
            throws Exception
    {
        // Create a charge code with its default parent.
        ChargeCode code = createSimpleChargeCode(hasExternalId);
        ChargeCodeCategory parentCode = (ChargeCodeCategory) code.getParent();
        // Save the charge code.
        ChargeCodeScope scope = createUniversalChargeCodeScope();
        SingleResult result = controler.save(parentCode, scope);
        parentCode = (ChargeCodeCategory) result.getRpmObject();
        Assert.assertNotNull(parentCode);
        Assert.assertNotNull(parentCode.getChildren());
        Assert.assertEquals(1, parentCode.getChildren().length);
        code = (ChargeCode) parentCode.getChildren()[0];
        return code;
    }

    public static ChargeCode createSimpleChargeCode()
    {
        return createSimpleChargeCode(false);
    }

    public static ChargeCode createSimpleChargeCode(boolean hasExternalId)
    {
        // Create a charge code with its default parent.
        ChargeCodeCategory parentCC = (ChargeCodeCategory) createGenericChargeCode(
                2, 1, null, hasExternalId);
        ChargeCode cc = (ChargeCode) parentCC.getChildren()[0];
        return cc;
    }

    // ===================
    // CHARGECODE BUILDERS
    // ===================

    private static final int DEFAULT_BRANCH_FACTOR = 3;

    /**
     * Creates an unsaved, stub GenericChargeCode instance.
     * 
     * @param levelCount
     *            Specifies the number of levels in the tree of objects.
     * @param parent
     *            May be null.
     * @param hasExternalIDs
     * @return Unsaved, stub GenericChargeCode matching the scope.
     */
    // Design note: the following creation methods are recursive.
    public static GenericChargeCode createGenericChargeCode(int levelCount,
            int branchFactor, ChargeCodeCategory parent, boolean hasExternalIDs)
    {
        // Create the right type of object.
        GenericChargeCode thisObject;
        if (levelCount == 1)
            thisObject = createChargeCode(parent, hasExternalIDs);
        else
            thisObject = createChargeCodeCategory(levelCount, branchFactor,
                    parent, hasExternalIDs);

        // Return the result.
        return thisObject;
    }

    public static GenericChargeCode createGenericChargeCode(int levelCount,
            boolean hasExternalIDs)
    {
        return createGenericChargeCode(levelCount, DEFAULT_BRANCH_FACTOR, null,
                hasExternalIDs);
    }

    /**
     * The new instance will be a root if <code>parent</code> is
     * <code>null</code>.
     */
    private static ChargeCodeCategory createChargeCodeCategory(int levelCount,
            int branchFactor, ChargeCodeCategory parent, boolean hasExternalIDs)
    {
        // Create the charge code category.
        ChargeCodeCategory chargeCodeCategory = createChargeCodeCategory(
                parent, hasExternalIDs);

        // Create children GenericChargeCodes (using this chargeCodeCategory as
        // the parent).
        // Note that children will be automatically linked with their parent and
        // vice-versa.
        for (int i = 0; i < branchFactor; i++)
        {
            createGenericChargeCode(levelCount - 1, branchFactor,
                    chargeCodeCategory, hasExternalIDs);
        }

        // Return the result.
        return chargeCodeCategory;
    }

    public static ChargeCodeCategory createChargeCodeCategory(
            ChargeCodeCategory parent, boolean hasExternalID)
    {
        // Create the charge code category.
        ChargeCodeCategory chargeCodeCategory = new ChargeCodeCategory();

        // Associate the child and the parent.
        associateParentChild(parent, chargeCodeCategory);

        // Initialize the properties.
        initializeCommonProperties(chargeCodeCategory, hasExternalID);

        // Return the result.
        return chargeCodeCategory;
    }

    public static ChargeCodeCategory createAndSaveAssociatedChargeCodes(
            ClientTestController controller) throws Exception
    {
        ChargeCodeCategory root = (ChargeCodeCategory) createGenericChargeCode(
                2, 2, null, false);
        ChargeCode child1 = (ChargeCode) root.getChildren()[1];
        child1.setFinancialCategory(FinancialCategory.Benefit);
        ChargeCodeScope scope = createUniversalChargeCodeScope();
        SingleResult result = controller.save(root, scope);
        root = (ChargeCodeCategory) result.getRpmObject();
        ChargeCode child0 = (ChargeCode) root.getChildren()[0];
        child1 = (ChargeCode) root.getChildren()[1];
        child0.setAssociatedChargeCode(child1);
        child1.setAssociatedChargeCode(child0);
        result = controller.save(root, scope);
        root = (ChargeCodeCategory) result.getRpmObject();
        child0 = (ChargeCode) root.getChildren()[0];
        child1 = (ChargeCode) root.getChildren()[1];
        Assert.assertNotNull(child0.getAssociatedChargeCode());
        Assert.assertNotNull(child1.getAssociatedChargeCode());
        Assert.assertEquals(child0.getAssociatedChargeCode().getID(), child1
                .getID());
        Assert.assertEquals(child1.getAssociatedChargeCode().getID(), child0
                .getID());
        return root;
    }

    private static ChargeCode createChargeCode(ChargeCodeCategory parent,
            boolean hasExternalIDs)
    {
        Assert.assertNotNull(parent);

        // Create the charge code.
        ChargeCode chargeCode = new ChargeCode();

        // Associate the child and the parent.
        associateParentChild(parent, chargeCode);

        // Initialize the properties.
        initializeCommonProperties(chargeCode, hasExternalIDs);
        chargeCode.setBillable(Boolean.TRUE);
        chargeCode.setFinancialCategory(FinancialCategory.Expense);

        // Create State object.
        chargeCode.setState(getStubState1());

        // Return the result.
        return chargeCode;
    }

    /**
     * Initialize common properties.
     */
    private static void initializeCommonProperties(GenericChargeCode gcc,
            boolean hasExternalID)
    {
        // Initialize the properties.
        int no = getSequenceNo();
        gcc.setName(PREFIX + ((gcc instanceof ChargeCode) ? "code " : "cat. ")
                + "name " + no);
        gcc.setDescription("description " + no);
        String externalID = hasExternalID ? "external ID " + no : null;
        gcc.setExternalID(externalID);
    }

    public static final String PREFIX = HelperTestUtilities.unitTestPrefix;

    private static ChargeCodeState getStubState1()
    {
        return createChargeCodeStage(CHARGE_CODE_STAGE_ACTIVE_ID);
    }

    private static ChargeCodeState getStubState2()
    {
        return createChargeCodeStage(CHARGE_CODE_STAGE_NON_POSTING_ID);
    }

    private static ChargeCodeState createChargeCodeStage(String id)
    {
        ChargeCodeState state = new ChargeCodeState();
        state.setID(id);
        return state;
    }

    private static final String CHARGE_CODE_STAGE_ACTIVE_ID = "6801";

    private static final String CHARGE_CODE_STAGE_NON_POSTING_ID = "6802";

    private static void associateParentChild(ChargeCodeCategory parent,
            GenericChargeCode child)
    {
        Assert.assertNotNull(child);

        // Associate the parent to the child.
        child.setParent(parent);

        // Include the given child into the parent's children set.
        if (parent != null)
        {
            GenericChargeCode[] children = parent.getChildren();
            if (children == null)
            {
                children = new GenericChargeCode[1];
                children[0] = child;
            }
            else
            {
                Set childrenSet = new HashSet(Arrays.asList(children));
                childrenSet.add(child);
                children = new GenericChargeCode[childrenSet.size()];
                childrenSet.toArray(children);
            }
            parent.setChildren(children);
        }
    }

    // =========================
    // CHARGECODE SCOPE BUILDERS
    // =========================

    /**
     * Creates a ChargeCode tree scope, including all related objects.
     */
    public static ChargeCodeScope createChargeCodeTreeScope(int levelCount)
    {
        ChargeCodeScope rootScope = null;
        ChargeCodeScope previousScope = null;
        ChargeCodeScope currentScope = null;
        for (int i = 1; i <= levelCount; i++)
        {
            // Create the current level.
            if (i < levelCount)
                currentScope = new ChargeCodeScope();
            else
                currentScope = createChargeCodeScope();

            // Link the current level with the previous level;
            currentScope.setParent(previousScope);
            if (previousScope != null)
                previousScope.setChildren(currentScope);

            // Remember the root scope.
            if (rootScope == null)
                rootScope = currentScope;

            // Initialize next level.
            previousScope = currentScope;
        }
        return rootScope;
    }

    /**
     * Creates a single ChargeCode scope.
     */
    public static ChargeCodeScope createChargeCodeScope()
    {
        ChargeCodeScope scope = createSimpleScope();
        scope.setState(true);
        return scope;
    }

    /**
     * Creates a scope that refers to itself in all dimensions. It allows to
     * reach the transitive closure of a charge code (all other charge codes in
     * the same tree, wherever you start from). BE VERY CAUTIOUS WITH THIS.
     */
    public static ChargeCodeScope createUniversalChargeCodeScope()
    {
        ChargeCodeScope scope = createChargeCodeScope();
        scope.setAssociatedChargeCode(scope);
        scope.setChildren(scope);
        scope.setParent(scope);
        scope.setState(true);
        return scope;
    }

    public static ChargeCodeScope createSimpleScope()
    {
        ChargeCodeScope scope = new ChargeCodeScope();
        ChargeCodeScope parentScope = new ChargeCodeScope();
        scope.setParent(parentScope);
        parentScope.setChildren(scope);
        return scope;
    }

    // =============
    // MODIFICATIONS
    // =============

    private static final String UPDATE_SUFFIX = " upd";

    public static void modify(GenericChargeCode object, ChargeCodeScope scope,
            boolean allowDeletion)
    {
        if (object == null)
            return;
        object.setDescription(null);
        object.setName(object.getName().trim() + UPDATE_SUFFIX);
        if (object.getExternalID() != null)
            object.setExternalID(object.getExternalID() + UPDATE_SUFFIX);
        if (object instanceof ChargeCodeCategory)
            modifyChargeCodeCategory((ChargeCodeCategory) object, scope,
                    allowDeletion);
        else
            modifyChargeCode((ChargeCode) object, scope, allowDeletion);
    }

    private static void modifyChargeCode(ChargeCode object,
            ChargeCodeScope scope, boolean allowDeletion)
    {
        if (object == null)
            return;
        object.setBillable(Boolean.FALSE);
        if (object.getAssociatedChargeCode() != null
                && object.getAssociatedChargeCode().getFinancialCategory()
                        .equals(FinancialCategory.Benefit))
        {
            object.setFinancialCategory(FinancialCategory.Capital);
        }
        if (scope != null)
        {
            if (scope.isState())
            {
                object.setState(getStubState2());
            }
        }
    }

    private static void modifyChargeCodeCategory(ChargeCodeCategory object,
            ChargeCodeScope scope, boolean allowDeletion)
    {
        if (object == null || scope == null)
            return;
        ChargeCodeScope childrenScope = scope.getChildren();
        if (childrenScope == null)
            return;
        GenericChargeCode[] children = object.getChildren();
        // Modify existing children, if any.
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                if (allowDeletion)
                {
                    switch (i % 3)
                    {
                    case 0: // Child modification.
                        modify(children[i], childrenScope, allowDeletion);
                        break;
                    case 1: // Child deletion.
                        children[i].setDeleted(Boolean.TRUE);
                        break;
                    case 2: // Child ignored.
                        children[i] = null;
                        break;
                    }
                }
                else
                {
                    modify(children[i], childrenScope, allowDeletion);
                }
            }
        }
        // Add new children.
        int levelCount = 1;
        for (int i = 0; i < CREATION_COUNT; i++)
        {
            GenericChargeCode newChild = createGenericChargeCode(levelCount,
                    DEFAULT_BRANCH_FACTOR, object, false);
            associateParentChild(object, newChild);
        }
    }

    private static final int CREATION_COUNT = 1;

    // ==========
    // ASSERTIONS
    // ==========

    public static void assertEquals(GenericChargeCode first,
            GenericChargeCode second, ChargeCodeScope scope)
    {
        // Design note: parents are checked only from the start object and their
        // children scope is set to null to avoid infinite recursion.
        if (scope != null)
        {
            ChargeCodeScope parentScope = (ChargeCodeScope)scope.getParent();
            if (parentScope != null)
            {
                // Patch to avoid cycle.
                ChargeCodeScope savedScope = parentScope.getChildren();
                parentScope.setChildren(null);
                doAssertEquals((GenericChargeCode) first.getParent(),
                        (GenericChargeCode) second.getParent(), parentScope);
                parentScope.setChildren(savedScope);
            }
        }
        doAssertEquals(first, second, scope);
    }

    // Design note: this method calls itself recursively, directly and
    // indirectly.
    private static void doAssertEquals(GenericChargeCode first,
            GenericChargeCode second, ChargeCodeScope scope)
    {
        // When both are null, they are equal.
        if (first == null && second == null)
            return;

        // When one is null but not the other, they are not equal.
        if ((first == null && second != null)
                || (first != null && second == null))
            Assert
                    .fail("One of the GenericChargeCodes is null but not the other.");

        // Objects must be instances of the same class.
        Assert.assertEquals(
                "Compared objects must be instances of same class.", first
                        .getClass(), second.getClass());

        // Generic.
        // Assert that IDs are equal, if required.
        Assert.assertEquals("ID", first.getID(), second.getID());

        // Patch to avoid checking predefined code.
        //if (defaultParent.getID().equals(first.getID()))
        //    return;

        Assert.assertEquals("Description", first.getDescription(), second
                .getDescription());
        assertTrimmedEquals("Name", first.getName(), second.getName());
        Assert.assertEquals("ExternalID", first.getExternalID(), second
                .getExternalID());

        // ChargeCode-specific.
        if (first instanceof ChargeCode)
        {
            ChargeCode ccFirst = (ChargeCode) first;
            ChargeCode ccSecond = (ChargeCode) second;
            Assert.assertEquals("Billable", ccFirst.getBillable(), ccSecond
                    .getBillable());
            Assert.assertEquals("FinancialCategory", ccFirst
                    .getFinancialCategory(), ccSecond.getFinancialCategory());
            if (scope != null)
            {
                if (scope.isState())
                {
                    assertEqualIds("state", ccFirst.getState(), ccSecond
                            .getState());
                }
                if (scope.getAssociatedChargeCode() != null)
                {
                    assertEqualIds("associatedChargeCode", ccFirst
                            .getAssociatedChargeCode(), ccSecond
                            .getAssociatedChargeCode());
                }
            }
        }

        // ChargeCodeCategory-specific.
        if (first instanceof ChargeCodeCategory)
        {
            if (scope != null)
            {
                ChargeCodeCategory cccFirst = (ChargeCodeCategory) first;
                ChargeCodeCategory cccSecond = (ChargeCodeCategory) second;
                doAssertChildrenEquals(cccFirst.getChildren(), cccSecond
                        .getChildren(), scope.getChildren());
            }
        }
    }

    private static void assertTrimmedEquals(String message, String first,
            String second)
    {
        if (first != null)
        {
            Assert.assertNotNull(message, second);
            Assert.assertEquals(message, first.trim(), second.trim());
        }
    }

    private static void assertEqualIds(String message, RPMObject expected,
            RPMObject actual)
    {
        if (expected == null)
        {
            Assert.assertNull(message, actual);
        }
        else
        {
            Assert.assertNotNull(message, actual);
            Assert.assertEquals(message, expected.getID(), actual.getID());
        }
    }

    private static void doAssertChildrenEquals(
            GenericChargeCode[] firstChildren,
            GenericChargeCode[] secondChildren, ChargeCodeScope childrenScope)
    {
        if (childrenScope != null)
        {
            if (firstChildren != null && secondChildren != null)
            {
                Assert
                        .assertTrue(
                                "Must have at least the as many children charge codes.",
                                count(firstChildren) <= count(secondChildren));
                doAssertEqualsInDisorder(firstChildren, secondChildren,
                        childrenScope);
            }
        }
    }

    private static int count(GenericChargeCode[] children)
    {
        int count = 0;
        if (children != null)
        {
            for (int i = 0; i < children.length; i++)
            {
                GenericChargeCode child = children[i];
                if (child != null && !child.getDeleted().booleanValue())
                    count++;
            }
        }
        return count;
    }

    private static void doAssertEqualsInDisorder(
            GenericChargeCode[] firstChildren,
            GenericChargeCode[] secondChildren, ChargeCodeScope childrenScope)
    {
        for (int i = 0; i < firstChildren.length; i++)
        {
            GenericChargeCode firstChild = firstChildren[i];
            GenericChargeCode secondChild = findInArray(firstChild,
                    secondChildren);
            doAssertEquals(firstChild, secondChild, childrenScope);
        }
    }

    public static GenericChargeCode findInArray(GenericChargeCode object,
            Object[] genChargeCodes)
    {
        if (object != null && genChargeCodes != null)
        {
            String expectedId = object.getID();
            for (int i = 0; i < genChargeCodes.length; i++)
            {
                GenericChargeCode candidate = (GenericChargeCode) genChargeCodes[i];
                if (expectedId.equals(candidate.getID()))
                    return candidate;
            }
        }
        return null;
    }

    private static int getSequenceNo()
    {
        sequenceNo = (sequenceNo + 1) % 1000;
        return sequenceNo;
    }

    private static int sequenceNo = (int) (System.currentTimeMillis() % 1000L);
}
