package com.ibm.rpm.client.junit.financial.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.GenericChargeCode;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.util.StringUtil;

/**
 * @author legaults
 */
public class TestChargeCode extends ClientTestController
{
    // ==============================
    // INITIALIZATION AND TERMINATION
    // ==============================

    protected void setUp() throws Exception
    {
        super.setUp();
        logger.info("setUp");
    }

    public void tearDown() throws Exception
    {
        logger.info("tearDown");
        // Delete all saved generic charge codes.
        ChargeCodeFixture.deleteAllChargeCodes(this);
        super.tearDown();
    }

    // ====
    // LOAD
    // ====

    
    /**
     * When the associated charge code has been deleted, the load should not
     * fail but the corresponding reference should be set to null.
     */
    public void testReloadWithDeletedAssociatedChargeCode() throws Exception
    {
        ChargeCodeCategory root = ChargeCodeFixture
                .createAndSaveAssociatedChargeCodes(this);
        ChargeCode original = (ChargeCode) root.getChildren()[0];
        ChargeCodeScope scope = ChargeCodeFixture
        .createUniversalChargeCodeScope();
        ChargeCode reloaded = (ChargeCode) loadFromID(original, scope);
        assertNotNull("Associated charge code should be set.", reloaded
                .getAssociatedChargeCode());
        delete(original.getAssociatedChargeCode());
        ChargeCode updated = (ChargeCode) loadFromID(original, scope);
        assertNull("Dangling reference should be set to null.", updated
                .getAssociatedChargeCode());
    }

    // ======
    // CREATE
    // ======

    public void testCreateSimpleChargeCode() throws Exception
    {
        ChargeCode original = ChargeCodeFixture
                .createAndSaveSimpleChargeCode(this, true);
        ChargeCodeScope scope = ChargeCodeFixture.createSimpleScope();
        GenericChargeCode reloaded = this.doLoadFromId(original, scope);
        ChargeCodeFixture.assertEquals(original, reloaded, scope);
    }

    public void testCreateChargeCodeCategoryWithoutScope() throws Exception
    {
        ChargeCodeCategory root = ChargeCodeFixture.createChargeCodeCategory(
                null, false);
        doTestCreate(root, null);
    }

    public void testCreateChargeCodeCategory() throws Exception
    {
        ChargeCodeCategory root = ChargeCodeFixture.createChargeCodeCategory(
                null, false);
        doTestCreate(root);
    }

    public void testCreateAssociatedChargeCode() throws Exception
    {
        ChargeCodeFixture.createAndSaveAssociatedChargeCodes(this);
    }

    public void testCreateTree() throws Exception
    {
        doTestCreate(ChargeCodeFixture.createGenericChargeCode(3, false));
    }

    private GenericChargeCode doTestCreate(GenericChargeCode original)
            throws Exception
    {
        return doTestCreate(original, ChargeCodeFixture
                .createUniversalChargeCodeScope());
    }

    private GenericChargeCode doTestCreate(GenericChargeCode original,
            ChargeCodeScope scope) throws Exception
    {
        // Save the object.
        SingleResult result = save(original, scope);

        // Get the updated object from the result.
        RPMObject savedObject = result.getRpmObject();
        GenericChargeCode saved = (GenericChargeCode) savedObject;

        ChargeCodeFixture.assertEquals(original, saved, scope);
        return saved;
    }

    private GenericChargeCode doLoadFromId(GenericChargeCode original,
            ChargeCodeScope scope) throws Exception
    {
        // Load the object.
        RPMObject loadedObject = loadFromID(original, scope);

        // Convert to the right type.
        assertNotNull(loadedObject);
        assertTrue(loadedObject.getClass().equals(original.getClass()));
        GenericChargeCode loaded = (GenericChargeCode) loadedObject;

        assertTrue(
                "Loaded object must be a different instance from original object",
                original != loaded);

        // Return the loaded object.
        return loaded;
    }

    // ======
    // UPDATE
    // ======

    public void testUpdateWithoutScope() throws Exception
    {
        ChargeCodeCategory original = ChargeCodeFixture
                .createChargeCodeCategory(null, false);
        doTestUpdate(original, true);
    }

    public void testUpdateChargeCode() throws Exception
    {
        doTestUpdate(ChargeCodeFixture.createAndSaveSimpleChargeCode(this, true),
                false);
    }

    public void testUpdateAssociatedChargeCodes() throws Exception
    {
        doTestUpdate(
                ChargeCodeFixture.createAndSaveAssociatedChargeCodes(this),
                false);
    }

    public void testUpdateTree() throws Exception
    {
        doTestUpdate(ChargeCodeFixture.createGenericChargeCode(3, false), true);
    }

    public void testUpdateTreeWithExternalIDs() throws Exception
    {
        doTestUpdate(ChargeCodeFixture.createGenericChargeCode(3, true), false);
    }

    private void doTestUpdate(GenericChargeCode original, boolean allowDeletion)
            throws Exception
    {
        doTestUpdate(original, ChargeCodeFixture
                .createUniversalChargeCodeScope(), allowDeletion);
    }

    private void doTestUpdate(GenericChargeCode original,
            ChargeCodeScope scope, boolean allowDeletion) throws Exception
    {
        // Save the objects.
        original = doTestCreate(original, scope);
        // Modify the objects.
        ChargeCodeFixture.modify(original, scope, allowDeletion);
        // Save the modified objects.
        save(original, scope, ReloadType.None);
        // Reload a copy of the modified objects.
        GenericChargeCode modified = new GenericChargeCode();
        modified.setID(original.getID());
        modified = (GenericChargeCode) loadFromID(modified, scope);
        addObjectToDelete(modified);
        // Compare the original objects with the reloaded.
        ChargeCodeFixture.assertEquals(original, modified, scope);
    }

    // =====
    // XPATH
    // =====

    public void testXPathPropertyMappingForChargeCode() throws Exception
    {
        ChargeCode cc = ChargeCodeFixture.createAndSaveSimpleChargeCode(this, true);
        doTestXPathPropertyMappings(cc);
    }

    public void testXPathPropertyMappingForChargeCodeCategory()
            throws Exception
    {
        doTestXPathPropertyMappings(ChargeCodeFixture.createChargeCodeCategory(
                null, true));
    }

    public void testXPathChildrenFromParentX() throws Exception
    {
        ChargeCodeCategory original = (ChargeCodeCategory) ChargeCodeFixture
                .createGenericChargeCode(3, false);
        ChargeCodeScope scope = ChargeCodeFixture
                .createUniversalChargeCodeScope();
        original = (ChargeCodeCategory) doTestCreate(original);

        // Build query of the form:
        // "/ChargeCodeCategory[Name='value']/GenericChargeCode".
        final String query = "/ChargeCodeCategory[Name='" + original.getName()
                + "']/GenericChargeCode";
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original set.
        RPMObject reloadedObjects[] = loadFromXpath(query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        GenericChargeCode[] originalChildren = original.getChildren();
        assertEquals(originalChildren.length, reloadedObjects.length);
        for (int i = 0; i < originalChildren.length; i++)
        {
            GenericChargeCode reloaded = ChargeCodeFixture.findInArray(
                    originalChildren[i], reloadedObjects);
            assertTrue("Original object not found in reloaded objects.",
                    reloaded != null);
            ChargeCodeFixture
                    .assertEquals(originalChildren[i], reloaded, scope);
        }
    }

    public void testXPathParentWithChildX() throws Exception
    {
        ChargeCodeCategory original = (ChargeCodeCategory) ChargeCodeFixture
                .createGenericChargeCode(2, false);
        ChargeCodeScope scope = ChargeCodeFixture
                .createUniversalChargeCodeScope();
        original = (ChargeCodeCategory) doTestCreate(original);

        // Build query.
        final String query = "/ChargeCodeCategory[GenericChargeCode[starts-with(Name, '"
                + HelperTestUtilities.unitTestPrefix + "code name')]]";
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original set.
        RPMObject reloadedObjects[] = loadFromXpath(query, scope);
        assertNotNull("Reloaded objects should not be null", reloadedObjects);
        //assertEquals(1, reloadedObjects.length);
        GenericChargeCode reloaded = ChargeCodeFixture.findInArray(original,
                reloadedObjects);
        assertTrue("Original object not fount in reloaded objects.",
                reloaded != null);

        // Compare the original objects with the reloaded.
        ChargeCodeFixture.assertEquals(original, reloaded, scope);
    }

    private void doTestXPathPropertyMappings(GenericChargeCode original)
            throws Exception
    {
        ChargeCodeScope scope = ChargeCodeFixture
                .createUniversalChargeCodeScope();
        if (original.getID() == null)
        {
            original = doTestCreate(original, scope);
        }
        // Test property mapping.
        if (original instanceof ChargeCode)
        {
            // loading with null scope otherwise too many objects are loaded
            // for restriction on billable field only
            doTestXPathPropertyMapping("[billable='"
                    + ((ChargeCode) original).getBillable().toString() + "']",
                original, null);
        }
        doTestXPathPropertyMapping("[name='" + original.getName() + "']",
            original, scope);
        doTestXPathPropertyMapping("[description='" + original.getDescription()
                + "']", original, scope);
        doTestXPathPropertyMapping("[externalID='" + original.getExternalID()
                + "']", original, scope);
    }

    private void doTestXPathPropertyMapping(String xpath,
            GenericChargeCode original,
            ChargeCodeScope scope) throws Exception
    {
        // Build query of the form: /ChargeCode[Property='value'].
        String className = StringUtil.getShortClassName(original.getClass());
        final String query = '/' + className + xpath;
        logger.debug("XPath query: " + query);

        // Reload the set of objects that match the specified property value
        // and then ensures that the set contains the original.
        RPMObject reloadedObjects[] = loadFromXpath(query, scope);
        GenericChargeCode reloaded = ChargeCodeFixture.findInArray(original,
                reloadedObjects);
        assertTrue("Original object not fount in reloaded objects.",
                reloaded != null);

        // Compare the original objects with the reloaded.
        ChargeCodeFixture.assertEquals(original, reloaded, scope);
    }

    // ======
    // DELETE
    // ======

    public void testDeleteSingle() throws Exception
    {
        doTestDelete(ChargeCodeFixture.createChargeCodeCategory(null, false));
    }

    public void testDeleteTree() throws Exception
    {
        doTestDelete(ChargeCodeFixture.createGenericChargeCode(3, false));
    }

    public void testDeleteTreeWithExternalIDs() throws Exception
    {
        doTestDeleteWithExternalIDs(ChargeCodeFixture.createGenericChargeCode(
                3, true));
    }

    public void testDeleteAll() throws Exception
    {
        ChargeCodeFixture.deleteAllChargeCodes(this);
        ChargeCodeFixture.assertAllDeleted(this);
    }

    public void testAllDeleted() throws Exception
    {
        ChargeCodeFixture.assertAllDeleted(this);
    }

    private void doTestDelete(GenericChargeCode original) throws Exception
    {
        // Write the object and read it back.
        ChargeCodeScope scope = ChargeCodeFixture
                .createUniversalChargeCodeScope();
        GenericChargeCode saved = doTestCreate(original, scope);
        // Ensure that deletion works even when parent is null
        // and object is generic.
        GenericChargeCode toDelete = new GenericChargeCode();
        toDelete.setID(saved.getID());
        // Delete the object.
        delete(toDelete);
        // Assert that the object is deleted.
        try
        {
            doLoadFromId(saved, scope);
            // Abnormal: the object was found after being deleted.
            fail("The charge code should have been deleted.");
        }
        catch (Exception x)
        {
            // Normal: the preceding load should throw an exception.
            logger.debug("Expected exception.", x);
        }
    }

    private void doTestDeleteWithExternalIDs(GenericChargeCode gcc)
            throws Exception
    {
        // Create the object, write it and read it back.
        GenericChargeCode saved = doTestCreate(gcc);
        // Delete the object. Should fail.
        try
        {
            delete(saved);
            // Abnormal: the delete call should have fail.
            fail("The charge code deletion should have fail because of ExternalID.");
        }
        catch (Exception x)
        {
            // Normal: the preceding load should throw an exception.
            logger.debug("Expected exception.", x);
        }
    }

    // =======
    // FROM QA
    // =======

    public void testQA_34467_AssociateChargeCode() throws Exception
    {
        ChargeCodeScope mainScope = null;
        ChargeCodeCategory mainCategory = null;
        ChargeCodeCategory subCategory = null;
        ChargeCode expense = null;
        String CCCatName = ChargeCodeFixture.PREFIX + " QACategoryTC4";
        mainScope = new ChargeCodeScope();

        ChargeCodeCategory category = new ChargeCodeCategory();
        category.setName(CCCatName);
        mainScope.setChildren(new ChargeCodeScope());

        ChargeCodeCategory category1 = new ChargeCodeCategory();
        category1.setName(ChargeCodeFixture.PREFIX + " QACategory1");
        category.setChildren(new ChargeCodeCategory[]
        {
            category1
        });
        mainScope.getChildren().setChildren(new ChargeCodeScope());

        expense = new ChargeCode();
        expense.setName(ChargeCodeFixture.PREFIX + " QAExpense");
        expense.setDescription("QA Expense");
        expense.setFinancialCategory(FinancialCategory.Expense);

        ChargeCode capital = new ChargeCode();
        capital.setName(ChargeCodeFixture.PREFIX + " QACapital");
        capital.setDescription("QA Capital");
        capital.setFinancialCategory(FinancialCategory.Capital);

        ChargeCode benefit = new ChargeCode();
        benefit.setName(ChargeCodeFixture.PREFIX + " QABenefit");
        benefit.setDescription("QA Benefit");
        benefit.setBillable(new Boolean(true));
        benefit.setFinancialCategory(FinancialCategory.Benefit);

        ChargeCode elimination = new ChargeCode();
        elimination.setName(ChargeCodeFixture.PREFIX + " QAElimination");
        elimination.setDescription("QA Elimination");
        elimination.setFinancialCategory(FinancialCategory.Elimination);

        ChargeCode contingency = new ChargeCode();
        contingency.setName(ChargeCodeFixture.PREFIX + " QAContingency");
        contingency.setDescription("QA Contingency");
        contingency.setFinancialCategory(FinancialCategory.Contingency);
        category1.setChildren(new ChargeCode[]
        {
                expense, capital, benefit, elimination, contingency
        });
        mainScope.getChildren().setChildren(new ChargeCodeScope());

        SingleResult result = save(category, mainScope);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof ChargeCodeCategory);

        //public void testQueryChargeCodeCategory() throws Exception

        //load from XPath
        RPMObject[] loadResult = loadFromXpath(
                "/ChargeCodeCategory[name='" + CCCatName + "']", mainScope);
        assertTrue(loadResult[0] instanceof ChargeCodeCategory);

        mainCategory = (ChargeCodeCategory) loadResult[0];
        assertNotNull(mainCategory);
        subCategory = (ChargeCodeCategory) mainCategory.getChildren()[0];
        assertNotNull(subCategory);
        assertNotNull(subCategory.getChildren());
        
        for (int i = 0; i < subCategory.getChildren().length; i++)
        {
            ChargeCode cCode = (ChargeCode) subCategory.getChildren()[i];

            if (cCode != null && cCode.getDescription() != null
                    && cCode.getDescription().equals("QA Expense"))
            {
                expense = cCode;
            }
            else if (cCode != null && cCode.getDescription() != null
                    && cCode.getDescription().equals("QA Benefit"))
            {
                benefit = cCode;
            }
        }
        
        assertNotNull(expense);
        assertNotNull(benefit);
        
        expense.setAssociatedChargeCode(benefit);
        
        mainScope.getChildren().getChildren().setAssociatedChargeCode(
                new ChargeCodeScope());
        result = save(mainCategory, mainScope);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof ChargeCodeCategory);
    }

    public void testQA_34461_AssociatedChargeCodeFinancialCategory()
            throws Exception
    {
        ChargeCodeCategory mainCategory = null;
        ChargeCodeCategory subCategory = null;
        String CCCatName = ChargeCodeFixture.PREFIX + " QACategoryTC4";

        ChargeCodeScope mainScope = new ChargeCodeScope();

        ChargeCodeCategory category = new ChargeCodeCategory();
        category.setName(CCCatName);
        mainScope.setChildren(new ChargeCodeScope());

        ChargeCodeCategory category1 = new ChargeCodeCategory();
        category1.setName(ChargeCodeFixture.PREFIX + " QACategory1");
        category.setChildren(new ChargeCodeCategory[]
        {
            category1
        });
        mainScope.getChildren().setChildren(new ChargeCodeScope());

        ChargeCode expense = new ChargeCode();
        expense.setName(ChargeCodeFixture.PREFIX + " QAExpense");
        expense.setDescription("QA Expense");
        expense.setFinancialCategory(FinancialCategory.Expense);

        ChargeCode capital = new ChargeCode();
        capital.setName(ChargeCodeFixture.PREFIX + " QACapital");
        capital.setDescription("QA Capital");
        capital.setFinancialCategory(FinancialCategory.Capital);

        ChargeCode benefit = new ChargeCode();
        benefit.setName(ChargeCodeFixture.PREFIX + " QABenefit");
        benefit.setDescription("QA Benefit");
        benefit.setBillable(new Boolean(true));
        benefit.setFinancialCategory(FinancialCategory.Benefit);

        ChargeCode elimination = new ChargeCode();
        elimination.setName(ChargeCodeFixture.PREFIX + " QAElimination");
        elimination.setDescription("QA Elimination");
        elimination.setFinancialCategory(FinancialCategory.Elimination);

        ChargeCode contingency = new ChargeCode();
        contingency.setName(ChargeCodeFixture.PREFIX + " QAContingency");
        contingency.setDescription("QA Contingency");
        contingency.setFinancialCategory(FinancialCategory.Contingency);
        category1.setChildren(new ChargeCode[]
        {
                expense, capital, benefit, elimination, contingency
        });
        mainScope.getChildren().setChildren(new ChargeCodeScope());

        SingleResult result = save(category, mainScope);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof ChargeCodeCategory);

        //public void testQueryChargeCodeCategory() throws Exception

        //load from XPath
        RPMObject[] loadResult = loadFromXpath(
                "/ChargeCodeCategory[name='" + CCCatName + "']", mainScope);
        assertTrue(loadResult[0] instanceof ChargeCodeCategory);

        mainCategory = (ChargeCodeCategory) loadResult[0];
        assertNotNull(mainCategory);
        subCategory = (ChargeCodeCategory) mainCategory.getChildren()[0];
        assertNotNull(subCategory);
        assertNotNull(subCategory.getChildren());
        
        for (int i = 0; i < subCategory.getChildren().length; i++)
        {
            ChargeCode cCode = (ChargeCode) subCategory.getChildren()[i];

            if (cCode != null && cCode.getDescription() != null
                    && cCode.getDescription().equals("QA Expense"))
            {
                expense = cCode;
            }
            else if (cCode != null && cCode.getDescription() != null
                    && cCode.getDescription().equals("QA Benefit"))
            {
                benefit = cCode;
            }
        }
        
        assertNotNull(expense);
        assertNotNull(benefit);

        // public void testAssociateChargeCode() throws Exception

         expense.setAssociatedChargeCode(benefit);
        mainScope.getChildren().getChildren().setAssociatedChargeCode(
                new ChargeCodeScope());
        result = save(mainCategory, mainScope);
        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.isSuccessful());
        assertTrue(result.getRpmObject() instanceof ChargeCodeCategory);
    }
}