package com.ibm.rpm.client.junit.financial.testcases.validation;

import com.ibm.rpm.client.junit.financial.fixtures.ChargeCodeFixture;
import com.ibm.rpm.client.junit.financial.fixtures.FinancialFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimePhasedFinancialFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.containers.TimePhasedFinancial;
import com.ibm.rpm.financial.containers.WbsFinancial;
import com.ibm.rpm.financial.containers.WorksheetFinancial;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.scope.FinancialScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;

/**
 * @author ikusakov
 */
public class TestFinancialValidation extends GenericValidation
{
    private static final String FINANCIAL_CLASS_NAME = WorksheetFinancial.class
            .getName();

    private static final String WBS_FINANCIAL_CLASS_NAME = WbsFinancial.class
            .getName();

    private FinancialScope mainScope;

    private WorksheetFinancial mainFinancial;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainScope = FinancialFixture.createAllScopeNoTimephased();
        mainFinancial = FinancialFixture
                .createWorksheetFixture(mainScope, this);
    }

    protected void tearDown() throws Exception
    {
        logger.info("------------- tearDown");
        super.tearDown();
    }

    public void testFailureWithInvalidParent() throws Exception
    {
        ChargeCode wrongParent = new ChargeCode();
        mainFinancial.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainFinancial
                    .getClass().getName(), "parent", mainFinancial.getID())
        };

        verifySaveMustFail(mainFinancial, mainScope, expectedExceptions);
    }

    public void testSuccessValidationMax() throws Exception
    {
        mainFinancial.setActual(new Double(ValidationConstants.FINANCIAL_MAX));
        mainFinancial
                .setEstimate(new Double(ValidationConstants.FINANCIAL_MIN));
        mainFinancial.setETC(new Double(ValidationConstants.FINANCIAL_ETC_MIN));
        //mainFinancial.setProbabilityPercent(new Double(100));

        saveMustSucceed(mainFinancial, WorksheetFinancial.class, mainScope);
    }

    public void testSuccessValidationNull() throws Exception
    {
        // Set field with null values.
        mainFinancial.setActual(null);
        mainFinancial.setEstimate(null);
        mainFinancial.setETC(null);
        mainFinancial.setProbabilityPercent(null);

        saveMustSucceed(mainFinancial, WorksheetFinancial.class, mainScope);
    }

    public void testFailureValidation() throws Exception
    {
        // Set field with invalid values.
        mainFinancial.setActual(new Double(
                ValidationConstants.FINANCIAL_MAX + 1));
        mainFinancial.setEstimate(new Double(
                ValidationConstants.FINANCIAL_MIN - 1));
        mainFinancial.setETC(new Double(
                ValidationConstants.FINANCIAL_ETC_MIN - 1));

        String mainId = mainFinancial.getID();
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        FINANCIAL_CLASS_NAME, "actual", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        FINANCIAL_CLASS_NAME, "estimate", mainId),
                createRPMException(ErrorCodes.OUT_OF_RANGE,
                        FINANCIAL_CLASS_NAME, "ETC", mainId),
        };

        verifySaveMustFail(mainFinancial, mainScope, expectedExceptions);
    }

    public void testFailureWithoutParentID() throws Exception
    {
        mainFinancial.setParent(new Proposal());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID, FINANCIAL_CLASS_NAME,
                    "parent", mainFinancial.getID()),
        };

        verifySaveMustFail(mainFinancial, mainScope, expectedExceptions);
    }

    public void testFailureWBSFinancialIsReadonlyForSystemCC_59817()
            throws Exception
    {
        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainScope, this);

        //System Charge Codes
        String SystemChargeCodeName = "System Charge Codes";
        RPMObject[] resultArray = null;
        ChargeCodeCategory wCCCategory = null;
        ChargeCodeCategory wExpenseCategory = null;

        // ChargeCodeScope needed to load the ChargeCodeCategory
        ChargeCodeScope wCCMainScope = new ChargeCodeScope();
        wCCMainScope.setChildren(new ChargeCodeScope());
        wCCMainScope.getChildren().setChildren(new ChargeCodeScope());

        resultArray = loadFromXpath("/ChargeCodeCategory[name='"
                + SystemChargeCodeName + "']", wCCMainScope);

        if (resultArray != null && resultArray[0] != null
                && (resultArray[0] instanceof ChargeCodeCategory))
        {
            wCCCategory = (ChargeCodeCategory) resultArray[0];
            wExpenseCategory = (ChargeCodeCategory) wCCCategory.getChildren()[0];

            wbsFinancial.setChargeCode((ChargeCode) wExpenseCategory
                .getChildren()[0]);
        }

        // first save should pass
        save(wbsFinancial, mainScope);

        //should not be able to modify wbsFinancial Actual
        wbsFinancial.setActual(new Double(-1));

        //Expected Warning
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", wbsFinancial.getID()),
        };

        verifySaveMustFail(wbsFinancial, mainScope, null, expectedWarnings);
    }
    
    
    public void testWorkElementWithWbsFinancialForSystemCC_59817_Create()
            throws Exception
    {
        Proposal parent = null;

        // Proposal
        parent = new Proposal();
        parent.setName(HelperTestUtilities.createName());
        parent.setPublished(ProjectPublishedType.None);

        SaveResult sr = save(parent, new WorkElementScope());
        assertTrue(sr.getRpmObject() instanceof Proposal);
        parent = (Proposal) sr.getRpmObject();

        // Task
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setPublished(ProjectPublishedType.None);

        task.setParent(parent);
        parent.setChildren(new WorkElement[]
        {
            task
        });

        // WbsFinancials
        WbsFinancial[] wbsFin = FinancialFixture
            .createEmptyWbsFinancialSystemChargeCodesArray(task, this);

        // Hour = 0
        wbsFin[0].setEstimate(new Double(20));
        // Avg cost = 1
        wbsFin[1].setEstimate(new Double(20));

        task.setWbsFinancials(wbsFin);

        //Scopes
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());

        WorkElementScope childScope = new WorkElementScope();
        childScope.setWbsFinancials(finScope);
        childScope.setParent(new WorkElementScope());

        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(childScope);

        //Expected Warnings
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", task.getID()),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", task.getID()),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", task.getID()),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", task.getID())
        };

        verifySaveMustFail(task, childScope, null, expectedWarnings);

    }
    
    public void testWorkElementWithWbsFinancialForSystemCC_59817_Update()
            throws Exception
    {
        Proposal parent = null;

        // Proposal
        parent = new Proposal();
        parent.setName(HelperTestUtilities.createName());
        parent.setPublished(ProjectPublishedType.None);

        SaveResult sr = save(parent, new WorkElementScope());
        assertTrue(sr.getRpmObject() instanceof Proposal);
        parent = (Proposal) sr.getRpmObject();

        // Task
        Task task = new Task();
        task.setName(HelperTestUtilities.createName());
        task.setPublished(ProjectPublishedType.None);

        task.setParent(parent);
        parent.setChildren(new WorkElement[]
        {
            task
        });

        // WbsFinancials
        WbsFinancial[] wbsFin = FinancialFixture
            .createEmptyWbsFinancialSystemChargeCodesArray(task, this);

        //Scopes
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());

        WorkElementScope childScope = new WorkElementScope();
        childScope.setWbsFinancials(finScope);
        childScope.setParent(new WorkElementScope());

        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(childScope);

        // first save should pass
        SingleResult result = null;
        result = save(task, childScope);
        assertTrue(result.isWarningFree());

        task = (Task) result.getRpmObject();

        // shouldn't be able to do that
        task.setWbsFinancials(wbsFin);
        // Hour = 0
        wbsFin[0].setEstimate(new Double(20));
        // Avg cost = 1
        wbsFin[1].setEstimate(new Double(20));

        
        //Expected Warnings
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null)
        };

        verifySaveMustFail(task, childScope, null, expectedWarnings);

    }



    
    public void testFailureWBSFinancialIsReadonlyForCustomCC_59817()
            throws Exception
    {
        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainScope, this);

        save(wbsFinancial, mainScope);

        wbsFinancial.setActual(new Double(-1));

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", wbsFinancial.getID()),
        };

        verifySaveMustFail(wbsFinancial, mainScope, null, expectedWarnings);
    }

    public void testFailureWBSFinancialActionParentIsReadonlyForCustomCC_59817()
            throws Exception
    {
        //Creating Scopes
        FinancialScope mainFinancialScope = FinancialFixture
            .createAllScopeNoTimephased();

        ScopeElementScope mainElementScope = new ScopeElementScope();
        mainElementScope.setParent(new ScopeElementScope());
        mainElementScope.setWbsFinancials(new FinancialScope());
        mainElementScope.getWbsFinancials().setChargeCode(true);

        //Creating parent for ScopeElement
        ChangeRequest scopeParent = ScopeElementFixture
            .createSaveChangeRequest(this);

        //Creating Action ScopeElement
        Action action = ScopeElementFixture.createActionFixture(
            mainElementScope, 1, scopeParent, this);

        //Creating custom wbsFinancial
        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainFinancialScope, this);

        action.setWbsFinancials(new WbsFinancial[]
        {
            wbsFinancial
        });

        //Expected Warning
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", wbsFinancial.getID()),
        };

        verifySaveMustFail(action, mainElementScope, null, expectedWarnings);

    }

    public void testFailureWBSFinancialDefectParentIsReadonlyForsystemCC_59817()
            throws Exception
    {
        //Create Scopes
        ScopeElementScope mainElementScope = new ScopeElementScope();
        mainElementScope.setParent(new ScopeElementScope());
        mainElementScope.setWbsFinancials(new FinancialScope());
        mainElementScope.getWbsFinancials().setChargeCode(true);

        //Creating parent ScopeElement
        ChangeRequest scopeParent = ScopeElementFixture
            .createSaveChangeRequest(this);

        //Creating Defect ScopeElement
        Defect defect = ScopeElementFixture.createDefectFixture(
            mainElementScope, 1, scopeParent, this);

        //Creating wbsFinancial system charge code
        WbsFinancial[] wbsFinancial = FinancialFixture
            .createEmptyWbsFinancialSystemChargeCodesArrayForScopElement(
                defect, this);

        defect.setWbsFinancials(wbsFinancial);

        //Expected Warning
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null)
        };

        verifySaveMustFail(defect, mainElementScope, null, expectedWarnings);

    }

    public void testFailureWBSFinancialScopeTaskParentIsReadonlyForsystemCC_59817()
            throws Exception
    {
        //Scopes
        ScopeElementScope mainElementScope = new ScopeElementScope();
        mainElementScope.setParent(new ScopeElementScope());
        mainElementScope.setWbsFinancials(new FinancialScope());
        mainElementScope.getWbsFinancials().setChargeCode(true);

        //creating Parent ChangeRequest
        ChangeRequest scopeParent = ScopeElementFixture
            .createSaveChangeRequest(this);

        //Creating ScopeTask 
        ScopeTask scopeTask = ScopeElementFixture.createScopeTaskFixture(
            mainElementScope, 1, scopeParent, this);

        //Creating wbsfinancial System System ChargeCodes
        WbsFinancial[] wbsFinancial = FinancialFixture
            .createEmptyWbsFinancialSystemChargeCodesArrayForScopElement(
                scopeTask, this);

        scopeTask.setWbsFinancials(wbsFinancial);

        //Expected Warnings
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    "WbsFinancial", "WbsFinancial", null)
        };

        verifySaveMustFail(scopeTask, mainElementScope, null, expectedWarnings);

    }

    public void testFailureWBSFinancialDeliverableParentIsReadonlyForCustomCC_59817()
            throws Exception
    {
        //Creating Scopes
        FinancialScope mainFinancialScope = FinancialFixture
            .createAllScopeNoTimephased();

        WorkElementScope mainWEScope = new WorkElementScope();
        mainWEScope.setParent(new WorkElementScope());
        mainWEScope.setChildren(mainWEScope);
        mainWEScope.setWbsFinancials(new FinancialScope());
        mainWEScope.getWbsFinancials().setChargeCode(true);

        mainWEScope.getChildren().setWbsFinancials(new FinancialScope());
        mainWEScope.getChildren().getWbsFinancials().setChargeCode(true);

        //Creating parent Project
        Project parentProject = (Project) WorkElementFixture.createSaveFixture(
            new Project(), new WorkElementScope(), this);

        //Creating child deliverable
        Deliverable deliverable = (Deliverable) WorkElementFixture
            .createFixture(new Deliverable(), new WorkElementScope(), this);

        //Creating custom wbsFinancial
        WbsFinancial wbsFinancial = FinancialFixture.createWbsFixture(
            mainFinancialScope, this);

        deliverable.setParent(parentProject);
        deliverable.setWbsFinancials(new WbsFinancial[]
        {
            wbsFinancial
        });

        //Expected Warnings
        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", wbsFinancial.getID()),
        };

        verifySaveMustFail(deliverable, mainWEScope, null, expectedWarnings);
    }
    
    public void testFailureFTEIsReadonly() throws Exception
    {
        // first we need existing HOURS financial, to calculate FTE
        ChargeCode chargeCode = ChargeCodeFixture
            .createAndSaveSimpleChargeCode(this);
        mainFinancial.setChargeCode(chargeCode);
        save(mainFinancial, mainScope);

        chargeCode.setID("BENEFIT_FULL_TIME_EQUIVALENT");
        mainFinancial.setChargeCode(chargeCode);
        mainFinancial.setID(null);

        // first save should pass
        save(mainFinancial, mainScope);

        mainFinancial.setActual(new Double(-1));

        RPMExceptionMock[] expectedWarnings =
        {
            createRPMException(com.ibm.rpm.financial.constants.ErrorCodes.
                    FINANCIAL_FTE_READONLY, FINANCIAL_CLASS_NAME, 
                    null, mainFinancial.getID()),
        };

        verifySaveMustFail(mainFinancial, mainScope, null, expectedWarnings);
    }
    
    public void testFailureModifyTotalsWithTimePhased() throws Exception
    {
        TimePhasedFinancial timephased = TimePhasedFinancialFixture
            .createWorksheetFixture(null, this);
        
        // Cannot modify Worksheet totals at the same time 
        // as timePhased data (created in fixture)
        mainFinancial.setActual(new Double(55));
        mainFinancial.setActualTimePhased(new TimePhasedFinancial[]
        {
            timephased
        });

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.financial.constants.ErrorCodes.TIMEPHASED_INDIVIDUAL_MODIFICATION_REQUIRED,
                FINANCIAL_CLASS_NAME),
        };

        FinancialScope scope = new FinancialScope();
        scope.setActualTimePhased(true);
        scope.setEstimateTimePhased(true);

        verifySaveMustFail(mainFinancial, scope, expectedExceptions);
    }
    
    public void testFailureWbsFinancialDeliverableParentIsReadonlyForSystemCC_59817()
            throws Exception
    {
        Proposal parent = null;

        // Proposal
        parent = new Proposal();
        parent.setName(HelperTestUtilities.createName());
        parent.setPublished(ProjectPublishedType.None);

        SaveResult sr = save(parent, new WorkElementScope());
        assertTrue(sr.getRpmObject() instanceof Proposal);
        parent = (Proposal) sr.getRpmObject();

        // Task
        Deliverable deliverable = new Deliverable();
        deliverable.setName(HelperTestUtilities.createName());
        deliverable.setPublished(ProjectPublishedType.None);

        deliverable.setParent(parent);
        parent.setChildren(new WorkElement[]
        {
            deliverable
        });

        // WbsFinancials
        WbsFinancial[] wbsFin = FinancialFixture
            .createEmptyWbsFinancialSystemChargeCodesArray(deliverable, this);

        // Hour = 0
        wbsFin[0].setEstimate(new Double(20));
        // Avg cost = 1
        wbsFin[1].setEstimate(new Double(20));

        deliverable.setWbsFinancials(wbsFin);

        //Scopes
        FinancialScope finScope = new FinancialScope();
        finScope.setChargeCode(true);
        finScope.setParent(new WorkElementScope());

        WorkElementScope childScope = new WorkElementScope();
        childScope.setWbsFinancials(finScope);
        childScope.setParent(new WorkElementScope());

        WorkElementScope wes = new WorkElementScope();
        wes.setChildren(childScope);

        //Expected Warnings
        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", deliverable
                        .getID()),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", deliverable
                        .getID()),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", deliverable
                        .getID()),
                createRPMException(
                    com.ibm.rpm.financial.constants.ErrorCodes.FINANCIAL_WBS_READONLY,
                    WBS_FINANCIAL_CLASS_NAME, "WbsFinancial", deliverable
                        .getID())
        };

        verifySaveMustFail(deliverable, childScope, null, expectedWarnings);

    }
    
    
    public void testNoFailureWBSFinancialScopeTaskParentIsReadonlyForsystemCC_59817_NoScope()
            throws Exception
    {
        // Scopes
        ScopeElementScope mainElementScope = new ScopeElementScope();
        mainElementScope.setParent(new ScopeElementScope());

        // creating Parent ChangeRequest
        ChangeRequest scopeParent = ScopeElementFixture
            .createSaveChangeRequest(this);

        // Creating ScopeTask
        ScopeTask scopeTask = ScopeElementFixture.createScopeTaskFixture(
            mainElementScope, 1, scopeParent, this);

        // Creating wbsfinancial System System ChargeCodes
        WbsFinancial[] wbsFinancial = FinancialFixture
            .createEmptyWbsFinancialSystemChargeCodesArrayForScopElement(
                scopeTask, this);
        // Hour = 0
        wbsFinancial[0].setEstimate(new Double(20));
        // Avg cost = 1
        wbsFinancial[1].setEstimate(new Double(20));

        scopeTask.setWbsFinancials(wbsFinancial);

        SingleResult result = null;
        result = save(scopeTask,mainElementScope);
        
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
    }
       
    
        
}
