/*
 * Created on 29-Nov-2005
 */
package com.ibm.rpm.client.junit.framework.testcases;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.resource.containers.Pool;

/**
 * Tests that saving succeeds on with all possible extended characters on the
 * update of a Pool's description.
 * 
 * @author dominiqu
 */
public class TestExtendedCharacters extends ClientTestController
{
    // 8-bit encodings

    private final int SMALLEST_ASCII_EXTCHAR_DECIMAL = 160;

    private final int LARGEST_ASCII_EXTCHAR_DECIMAL = 255;

    // Unicode Latin-1 Supplement subset
    // http://www.unicode.org/charts/PDF/U0080.pdf

    private final int SMALLEST_UNICODE_LATIN1 = 0x00B0;

    private final int LARGEST_UNICODE_LATIN1 = 0x00FF;

    // Unicode Arabic subset
    // http://www.unicode.org/charts/PDF/U0600.pdf

    private final int SMALLEST_UNICODE_ARABIC = 0x0621;
    
    private final int LARGEST_UNICODE_ARABIC = 0x064A;
    
    private final int MAX_ARABIC_SEGMENT_LENGTH = 512;

    // Container to test with

    private Pool mainPool;

    protected void setUp() throws Exception
    {
        super.setUp();

        mainPool = PoolFixture.createFixture(false, 0, null, this);
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void test8BitExtendedCharacters() throws AssertionError, Exception
    {
        // Set description with extended characters
        mainPool.setDescription(create8bitExtendedCharacterString());

        // Attempt to save the Pool object
        Pool saved = (Pool) saveMustSucceed(mainPool, Pool.class, null);
        mainPool.setID(saved.getID());

        // Load the inserted pool from ID
        RPMObject rpmObject = loadFromID(mainPool, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);
    }

    public void testUnicodeCharacters() throws AssertionError, Exception
    {
        // Set productsExpertiseUsed with extended characters
        mainPool
            .setProductsExpertiseUsed("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????");

        // Attempt to save the Pool object
        Pool saved = (Pool) saveMustSucceed(mainPool, Pool.class, null);
        mainPool.setID(saved.getID());

        // Load the inserted pool from ID
        RPMObject rpmObject = loadFromID(mainPool, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);
    }

    public void testUnicodeCharactersLatin1Supplement() throws AssertionError, Exception
    {
        // Set productsExpertiseUsed with extended characters
        mainPool.setProductsExpertiseUsed(new String(createUnicodeCharacterString(
            SMALLEST_UNICODE_LATIN1, LARGEST_UNICODE_LATIN1)));

        // Attempt to save the Pool object
        Pool saved = (Pool) saveMustSucceed(mainPool, Pool.class, null);
        mainPool.setID(saved.getID());

        // Load the inserted pool from ID
        RPMObject rpmObject = loadFromID(mainPool, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Pool);
    }

    public void testUnicodeCharactersArabic() throws AssertionError, Exception
    {
        // create a String of extended characters
        StringBuffer arabicCharacters = createUnicodeCharacterString(
            SMALLEST_UNICODE_ARABIC, LARGEST_UNICODE_ARABIC);
        
        // Each character takes up 4 spaces in the DB, so we can't 
        // safely save anything that is more than 512 long - DB column length is 2048
        // so we'll divide the string of extended characters into segments of
        // length 512 and save them individually
        int start = 0;
        int end = MAX_ARABIC_SEGMENT_LENGTH;
        String segment = "";
        
        while (start < end)
        {
            // get the next segment of extended characters
            segment = arabicCharacters.substring(start, end);
           
            // Set productsExpertiseUsed with extended characters
            mainPool.setProductsExpertiseUsed(segment);
            
            Pool saved = (Pool) saveMustSucceed(mainPool, Pool.class, null);

            // Load the inserted / updated pool from ID
            RPMObject rpmObject = loadFromID(saved, null);
            assertNotNull(rpmObject);
            assertTrue(rpmObject instanceof Pool);
            
            start += MAX_ARABIC_SEGMENT_LENGTH;
            end = arabicCharacters.length() < start + MAX_ARABIC_SEGMENT_LENGTH ? 
                    arabicCharacters.length() : start + MAX_ARABIC_SEGMENT_LENGTH;
        }
    }

    private String create8bitExtendedCharacterString()
    {
        int length = LARGEST_ASCII_EXTCHAR_DECIMAL - SMALLEST_ASCII_EXTCHAR_DECIMAL + 1;

        StringBuffer extendedCharacters = new StringBuffer(length);

        for (int i = 0; i < length; ++i)
            extendedCharacters.append((char) (SMALLEST_ASCII_EXTCHAR_DECIMAL + i));

        return new String(extendedCharacters);
    }

    private StringBuffer createUnicodeCharacterString(int startCharacter, int endCharacter)
    {
        StringBuffer extendedCharacters = new StringBuffer(endCharacter - startCharacter
            + 1);

        for (int i = 0; i != endCharacter; ++i)
            extendedCharacters.append(new Character((char) (startCharacter + i)));

        return extendedCharacters;
    }

    /**
     * Generic save function for saves that must succeed
     */
    private RPMObject saveMustSucceed(RPMObject obj, Class objType, RPMObjectScope scope)
            throws AssertionError, Exception
    {
        assertNotNull(obj);
        assertTrue(obj.getClass().equals(objType));

        SingleResult result = save(obj, scope);

        assertNotNull(result);
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject().getClass().equals(objType));

        obj = result.getRpmObject();
        return obj;
    }
}
