package com.ibm.rpm.client.junit.framework.testcases;

import java.rmi.RemoteException;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
// import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * Tests LDAP Authentication.
 * 
 * Usage: please ensure that (1) an active (with a calendar and effective start
 * date) resource exists in the test database with its username equal to
 * USERNAME; (2) an entry exists in the LDAP directory with its email equal to
 * USERNAME and its password equal to PASSWORD; (3) all options in the following
 * option files are still correct: com.ibm.rpm.GeneralOptions.properties and
 * com.ibm.rpm.LdapOptions.properties.
 * 
 * @author legaults
 */
public class TestLdapAuthentication extends ClientTestController
{
    // Credentials for JUnit RPM-WS in IBM's test Bluepages.
    public static final String USERNAME = "-05054649@tst.ibm.com";

    public static final String PASSWORD = "r1p2m3w4s";

    public static final String BAD_USERNAME = "BAD-05054649@tst.ibm.com";

    public static final String BAD_PASSWORD = "r1p2m3w4s_BAD";

    public static final String LDAP_CONFIG;
    
    public boolean switchToLdapMode = false;

    static
    {
        String assignLdapConfig = "";

        assignLdapConfig += "java.naming.provider.url=\"ldaps://tstbluepages.mkm.can.ibm.com:636\"";
        assignLdapConfig += "java.naming.factory.initial=\"com.sun.jndi.ldap.LdapCtxFactory\"";
        assignLdapConfig += "java.naming.ldap.version=\"3\"";
        assignLdapConfig += "java.naming.security.protocol=\"simple\"";

        assignLdapConfig += "ldapsearchcontext=\"o=ibm.com\"";
        assignLdapConfig += "ldapuseridattr=\"email\"";
        assignLdapConfig += "ldapuserauthdn=\"distinguishedName\"";

        LDAP_CONFIG = assignLdapConfig;
    }

    public String getLdapConfig()
    {
        return LDAP_CONFIG;
    }

    protected void setUp() throws Exception
    {
        switchToLdapMode = false;

        // check that the LDAP user exists and is active in the DB
        super.setUp();

        RPMObject[] ldapUserArray = loadFromXpath("/resource[@fullname='"
                + USERNAME + "']", null);
        
        if (ldapUserArray == null || ldapUserArray.length == 0)
        {
            // create the user
            Pool newPool = new Pool();
            newPool.setName(USERNAME);
            Resource newResource = new Resource();
            newResource.setActive(Boolean.TRUE);
            newResource.setFullName(USERNAME);
            newResource.setNickName(USERNAME);
            newResource.setFamilyName(USERNAME);
            newResource.setUserName(USERNAME);
            newResource.setEmailAddress(USERNAME);
            newResource.setGivenName(USERNAME);
            newResource.setMiddleName(USERNAME);
            newResource.setPassword(PASSWORD);
            RPMCalendar cal = RPMCalendarFixture
                .retrieveStaticFixtureForInserts(this);
            newResource.setCalendar(cal);

            newPool.setResources(new Resource[]
            {
                newResource
            });

            PoolScope poolScope = new PoolScope();
            ResourceScope resourceScope = new ResourceScope();
            CalendarScope calendarScope = new CalendarScope();
            resourceScope.setCalendar(calendarScope);
            poolScope.setResources(resourceScope);

            saveWithoutDelete(newPool, poolScope);
        }
    }
    
    protected void tearDown() throws Exception
    {
        SpecificTestController.clearOverrideOptions();
        super.tearDown();
    }

    private void applyLdapConfigAndLogin() throws Exception
    {
        if (isRemote())
            return;
        
        // switch to LDAP mode
        SpecificTestController.overrideOption("useLdapAuthentication",
            Boolean.TRUE);

        // configure LDAP
        SpecificTestController.overrideOption("ldapConfiguration",
            getLdapConfig());

        // ignore all security constraints for this test
        SpecificTestController.overrideOption(
            "webServicesUseApiSecurityFlagOnLogin", Boolean.FALSE);

        // ignore all security constraints for this test
        SpecificTestController.overrideOption(
            "webServicesSecurityEnabled", Boolean.FALSE);
        
        switchToLdapMode = true;

        super.setUp();
    }

    public void testLdapAuthentication() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();
    }

    public void testLdapAuthenticationBadUser() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();
        try
        {
            getAuthenticateInterface().login(BAD_USERNAME, PASSWORD, DSN);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.FAILED_LOGIN, e.getErrorID());
        }
    }

    public void testLdapAuthenticationBadPassword() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();
        try
        {
            getAuthenticateInterface().login(USERNAME, BAD_PASSWORD, DSN);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.FAILED_LOGIN, e.getErrorID());
        }
    }

    public void testLdapAuthenticationNullUser() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();
        try
        {
            getAuthenticateInterface().login(null, PASSWORD, DSN);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.FAILED_LOGIN, e.getErrorID());
        }
    }

    public void testLdapAuthenticationNullPassword() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();
        try
        {
            getAuthenticateInterface().login(USERNAME, null, DSN);
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.FAILED_LOGIN, e.getErrorID());
        }
    }

    public void testBadLdapConfigurationURL() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();

        // configure LDAP with bad URL
        SpecificTestController.overrideOption("ldapConfiguration",
            getLdapConfig().replaceFirst("tstbluepages", "tstBADbluepages"));

        try
        {
            super.setUp();
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR, e.getErrorID());
        }

    }

    public void testBadLdapConfigurationFactory() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();

        // configure LDAP with bad factory
        SpecificTestController
            .overrideOption("ldapConfiguration", getLdapConfig()
                .replaceFirst("LdapCtxFactory", "LdapBADCtxFactory"));

        try
        {
            super.setUp();
        }
        catch (RPMException e)
        {
            assertEquals(ErrorCodes.EXTERNAL_AUTHENTICATION_CONFIGURATION_ERROR, e.getErrorID());
        }
    }

    public void testChangePasswordFails() throws Exception
    {
        if (isRemote())
            return;
        applyLdapConfigAndLogin();

        try
        {
            final String newPassword = "Stub";
            getAuthenticateInterface().changePassword(USERNAME, PASSWORD,
                newPassword, DSN);
        }
        catch (RPMException x)
        {
            if (x.getErrorID() != ErrorCodes.CHANGER_PASSWORD_NOT_SUPPORTED_ON_EXTERNAL_AUTHENTICATION)
            {
                throw x;
            }
            return;
        }
        fail("Change password was successful but should have thrown an error");
    }

    public void performLogin() throws RemoteException, RemoteException,
            Exception
    {
        if( switchToLdapMode )
        {
            performLogin(USERNAME, PASSWORD);
        }
        else
        {
            super.performLogin();
        }
           
    }
}