package com.ibm.rpm.client.junit.resource.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.EmploymentHistoryFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;

/**
 * @author dominiqu
 */
public class TestEmploymentHistoryValidation extends GenericValidation
{
    private Resource mainResource;

    private EmploymentHistory mainEmploymentHistory;

    private Calendar calendar1;

    private Calendar calendar2;

    public void setUp() throws Exception
    {
        super.setUp();
        mainResource = ResourceFixture.createSaveFixture(this);
        mainEmploymentHistory = EmploymentHistoryFixture.createFixture(mainResource);
        mainResource.setEmploymentHistories(new EmploymentHistory[]
        {
            mainEmploymentHistory
        });

        calendar1 = Calendar.getInstance();
        calendar1.set(2007, 12, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2007, 12, 30);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainEmploymentHistory.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainEmploymentHistory.getClass()
                .getName(), "parent", mainEmploymentHistory.getID())
        };

        verifySaveMustFail(mainEmploymentHistory, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainEmploymentHistory.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainEmploymentHistory
                .getClass().getName(), "parent", mainEmploymentHistory.getID())
        };

        verifySaveMustFail(mainEmploymentHistory, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainEmploymentHistory
            .setCompany(getTestString(ValidationConstants.EMPLOYMENTHISTORY_COMPANY_MAX));
        mainEmploymentHistory
            .setTitle(getTestString(ValidationConstants.EMPLOYMENTHISTORY_TITLE_MAX));
        mainEmploymentHistory
            .setDescription(getTestString(ValidationConstants.EMPLOYMENTHISTORY_DESCRIPTION_MAX));
        mainEmploymentHistory
            .setLocation(getTestString(ValidationConstants.EMPLOYMENTHISTORY_LOCATION_MAX));
        mainEmploymentHistory
            .setResponsibilities(getTestString(ValidationConstants.EMPLOYMENTHISTORY_RESPONSIBILITIES_MAX));
        mainEmploymentHistory.setSalary(new Double(
            ValidationConstants.EMPLOYMENTHISTORY_SALARY_MAX));
        mainEmploymentHistory.setStartDate(calendar1);
        mainEmploymentHistory.setEndDate(calendar1);

        // Attempt to save the EmploymentHistory object
        mainEmploymentHistory = (EmploymentHistory) saveMustSucceed(
            mainEmploymentHistory, EmploymentHistory.class, null);

        // Load the inserted employment history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/EmploymentHistory[id='"
            + mainEmploymentHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof EmploymentHistory);

        // Check that the loaded employment history is identical to the one we
        // requested to store
        EmploymentHistoryFixture.compare(mainEmploymentHistory,
            (EmploymentHistory) rpmObjects[0]);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        mainEmploymentHistory = EmploymentHistoryFixture.createFixture(mainResource);
        mainResource.setEmploymentHistories(new EmploymentHistory[]
        {
            mainEmploymentHistory
        });

        // Set field values with min and verify that they are accepted
        mainEmploymentHistory.setSalary(new Double(
            ValidationConstants.EMPLOYMENTHISTORY_SALARY_MIN));

        // Attempt to save the EmploymentHistory object
        mainEmploymentHistory = (EmploymentHistory) saveMustSucceed(
            mainEmploymentHistory, EmploymentHistory.class, null);

        // Load the inserted employment history from xpath
        RPMObject[] rpmObjects = loadFromXpath("/EmploymentHistory[id='"
            + mainEmploymentHistory.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof EmploymentHistory);

        // Check that the loaded employment history is identical to the one we
        // requested to store
        EmploymentHistoryFixture.compare(mainEmploymentHistory,
            (EmploymentHistory) rpmObjects[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainEmploymentHistory
            .setCompany(getTestString(ValidationConstants.EMPLOYMENTHISTORY_COMPANY_MAX + 1));
        mainEmploymentHistory
            .setTitle(getTestString(ValidationConstants.EMPLOYMENTHISTORY_TITLE_MAX + 1));
        mainEmploymentHistory
            .setDescription(getTestString(ValidationConstants.EMPLOYMENTHISTORY_DESCRIPTION_MAX + 1));
        mainEmploymentHistory
            .setLocation(getTestString(ValidationConstants.EMPLOYMENTHISTORY_LOCATION_MAX + 1));
        mainEmploymentHistory
            .setResponsibilities(getTestString(ValidationConstants.EMPLOYMENTHISTORY_RESPONSIBILITIES_MAX + 1));
        mainEmploymentHistory.setSalary(new Double(
            ValidationConstants.EMPLOYMENTHISTORY_SALARY_MAX + 0.01));

        // calendar2 < calendar1
        mainEmploymentHistory.setStartDate(calendar1);
        mainEmploymentHistory.setEndDate(calendar2);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, mainEmploymentHistory.getClass()
                .getName(), "company", mainEmploymentHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEmploymentHistory.getClass()
                .getName(), "title", mainEmploymentHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEmploymentHistory.getClass()
                .getName(), "description", mainEmploymentHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEmploymentHistory.getClass()
                .getName(), "location", mainEmploymentHistory.getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, mainEmploymentHistory.getClass()
                .getName(), "responsibilities", mainEmploymentHistory.getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainEmploymentHistory.getClass()
                .getName(), "salary", mainEmploymentHistory.getID()),
            createRPMException(ErrorCodes.INVALID_DATES, mainEmploymentHistory.getClass()
                .getName(), "startDate", mainEmploymentHistory.getID())
        };

        verifySaveMustFail(mainEmploymentHistory, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        mainEmploymentHistory = EmploymentHistoryFixture.createFixture(mainResource);
        mainResource.setEmploymentHistories(new EmploymentHistory[]
        {
            mainEmploymentHistory
        });

        // Set field values with min-1 and verify that they are not accepted
        mainEmploymentHistory.setSalary(new Double(
            ValidationConstants.EMPLOYMENTHISTORY_SALARY_MIN - 0.01));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, mainEmploymentHistory.getClass()
                .getName(), "salary", mainEmploymentHistory.getID())
        };

        verifySaveMustFail(mainEmploymentHistory, null, expectedExceptions);
    }
}
