package com.ibm.rpm.client.junit.resource.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceAttributeAssignmentFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author dominiqu
 */
public class TestResourceAttributeAssignmentValidation extends GenericValidation
{
    private ResourceScope mainScope;

    private Resource mainResource;

    private ResourceAttributeAssignment[] competencies;

    private ResourceAttributeAssignment[] skills;

    private Pool parentPool;

    public void setUp() throws Exception
    {
        super.setUp();

        mainScope = new ResourceScope();
        mainScope.setCompetencyAttributeAssignments(true);
        mainScope.setSkillAttributeAssignments(true);
        parentPool = PoolFixture.createSaveFixture(this);
        mainResource = ResourceFixture.createFixture(parentPool, 1, mainScope, this);

        assertNotNull(mainResource.getCompetencyAttributeAssignments());
        assertNotNull(mainResource.getSkillAttributeAssignments());
        assertNotNull(mainResource.getCompetencyAttributeAssignments()[0]);
        assertNotNull(mainResource.getSkillAttributeAssignments()[0]);
        assertNotNull(mainResource.getCompetencyAttributeAssignments()[0]
            .getProficiencyLevel());
        assertNotNull(mainResource.getSkillAttributeAssignments()[0]
            .getProficiencyLevel());

        competencies = mainResource.getCompetencyAttributeAssignments();
        skills = mainResource.getSkillAttributeAssignments();
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        competencies[0].setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT,
                competencies[0].getClass().getName(), "parent", competencies[0].getID())
        };

        verifySaveMustFail(competencies[0], null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        competencies[0].setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, competencies[0].getClass()
                .getName(), "parent", competencies[0].getID())
        };

        verifySaveMustFail(competencies[0], null, expectedExceptions);
    }
    
    public void testFailureWithoutMandatoryFields() throws AssertionError, Exception
    {
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        competencies = mainResource.getCompetencyAttributeAssignments();
        skills = mainResource.getSkillAttributeAssignments();

        competencies[0].setProficiencyLevel(null);
        competencies[0].setExperience(null);
        competencies[0].setPreferenceRank(null);

        skills[0].setProficiencyLevel(null);
        skills[0].setExperience(null);
        skills[0].setPreferenceRank(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID,
                competencies[0].getClass().getName(), "proficiencyLevel", competencies[0]
                    .getID()),
            createRPMException(ErrorCodes.NO_ID,
                competencies[0].getClass().getName(), "experience", competencies[0]
                    .getID()),
            createRPMException(ErrorCodes.NO_ID,
                competencies[0].getClass().getName(), "preferenceRank", competencies[0]
                    .getID()),
            createRPMException(ErrorCodes.NO_ID, skills[0].getClass().getName(),
                "proficiencyLevel", skills[0].getID()),
            createRPMException(ErrorCodes.NO_ID, skills[0].getClass().getName(),
                "experience", skills[0].getID()),
            createRPMException(ErrorCodes.NO_ID, skills[0].getClass().getName(),
                "preferenceRank", skills[0].getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }

    public void testSuccessValidationCompetencies() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        competencies[0].setDefaultFlag(new Integer(1));
        competencies[0]
            .setDescription(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX));
        competencies[0]
            .setLastUsedLocation(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX));
        competencies[0]
            .setMandatoryDetails(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX));
        competencies[0].setStrategicFlag("1");

        // Attempt to save the Resource object with the compentency and skill
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted Resource from id
        RPMObject obj = new Resource();
        obj.setID(mainResource.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Resource);

        // Check that the loaded competency assignments are identical to the
        // ones we requested to store
        ResourceAttributeAssignmentFixture.compare(mainResource
            .getCompetencyAttributeAssignments(), ((Resource) rpmObject)
            .getCompetencyAttributeAssignments());
    }

    public void testSuccessValidationSkills() throws AssertionError, Exception
    {
        // Set field values with max and verify that they are accepted
        skills[0].setDefaultFlag(new Integer(1));
        skills[0]
            .setDescription(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX));
        skills[0]
            .setLastUsedLocation(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX));
        skills[0]
            .setMandatoryDetails(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX));
        skills[0].setStrategicFlag("1");

        // Attempt to save the Resource object with the compentency and skill
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class, mainScope);

        // Load the inserted Resource from id
        RPMObject obj = new Resource();
        obj.setID(mainResource.getID());
        RPMObject rpmObject = loadFromID(obj, mainScope);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof Resource);

        // Check that the loaded skill assignments are identical to the ones we
        // requested to store
        ResourceAttributeAssignmentFixture.compare(mainResource
            .getSkillAttributeAssignments(), ((Resource) rpmObject)
            .getSkillAttributeAssignments());
    }

    public void testFailureValidationCompetencies() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        competencies[0].setDefaultFlag(new Integer(2));
        competencies[0]
            .setDescription(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX + 1));
        competencies[0]
            .setLastUsedLocation(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX + 1));
        competencies[0]
            .setMandatoryDetails(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX + 1));
        competencies[0].setStrategicFlag("2");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, competencies[0].getClass()
                .getName(), "description", competencies[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, competencies[0].getClass()
                .getName(), "lastUsedLocation", competencies[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, competencies[0].getClass()
                .getName(), "mandatoryDetails", competencies[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, competencies[0].getClass()
                .getName(), "defaultFlag", competencies[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, competencies[0].getClass()
                .getName(), "strategicFlag", competencies[0].getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }

    public void testFailureValidationSkills() throws AssertionError, Exception
    {
        // Set field values with max+1 and verify that they are not accepted
        skills[0].setDefaultFlag(new Integer(2));
        skills[0]
            .setDescription(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX + 1));
        skills[0]
            .setLastUsedLocation(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX + 1));
        skills[0]
            .setMandatoryDetails(getTestString(ValidationConstants.RESOURCEATTRIBUTEASSIGNMENT_MAX + 1));
        skills[0].setStrategicFlag("2");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MAX_LENGTH, skills[0].getClass().getName(),
                "description", skills[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, skills[0].getClass().getName(),
                "lastUsedLocation", skills[0].getID()),
            createRPMException(ErrorCodes.MAX_LENGTH, skills[0].getClass().getName(),
                "mandatoryDetails", skills[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, skills[0].getClass().getName(),
                "defaultFlag", skills[0].getID()),
            createRPMException(ErrorCodes.OUT_OF_RANGE, skills[0].getClass().getName(),
                "strategicFlag", skills[0].getID())
        };

        verifySaveMustFail(mainResource, mainScope, expectedExceptions);
    }
}
