/*
 * Created on Sep 29, 2005
 */
package com.ibm.rpm.client.junit.resource.testcases.validation;

import java.util.Calendar;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.constants.ValidationConstants;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;

/**
 * @author gosselin
 */
public class TestResourceValidation extends GenericValidation
{
    private Resource mainResource;

    private Pool parentPool;

    private Calendar calendar1;

    private Calendar calendar2;

    public void setUp() throws Exception
    {
        super.setUp();

        // Parent pool is needed by all of the tests
        parentPool = PoolFixture.createSaveFixture(this);
        mainResource = ResourceFixture.createFixture(parentPool, null, this);

        calendar1 = Calendar.getInstance();
        calendar1.set(2008, 10, 31);
        calendar2 = Calendar.getInstance();
        calendar2.set(2008, 10, 30);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureWithoutParent() throws AssertionError, Exception
    {
        mainResource.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, mainResource.getClass()
                .getName(), "parent", mainResource.getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureWithInvalidParent() throws AssertionError, Exception
    {
        ClientCostCenter wrongParent = new ClientCostCenter();
        mainResource.setParent(wrongParent);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, mainResource
                .getClass().getName(), "parent", mainResource.getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class,
            null);

        // username is only mandatory on update
        mainResource.setUserName(null);
        mainResource.setFullName(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MANDATORY, mainResource
                    .getClass().getName(), "userName", mainResource.getID()),
                createRPMException(ErrorCodes.MANDATORY, mainResource
                    .getClass().getName(), "fullName", mainResource.getID()),
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testInsertWithoutParentID() throws AssertionError, Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool, null, this);
        Pool invalidPool = new Pool();
        invalidPool.setID(null);
        mainResource.setParent(invalidPool);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT_ID, mainResource.getClass()
                .getName(), "parent", mainResource.getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureInvalidChars() throws AssertionError, Exception
    {
        mainResource
            .setUserName(com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS);
        mainResource
            .setFullName(com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS);
        mainResource
            .setNickName(com.ibm.rpm.framework.constants.ValidationConstants.GENERIC_INVALIDCHARS);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.INVALID_CHARS, mainResource
                    .getClass().getName(), "userName", mainResource.getID()),
                createRPMException(ErrorCodes.INVALID_CHARS, mainResource
                    .getClass().getName(), "fullName", mainResource.getID()),
                createRPMException(ErrorCodes.INVALID_CHARS, mainResource
                    .getClass().getName(), "nickName", mainResource.getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureWithLeadingSpaceInUsername() throws AssertionError,
            Exception
    {
        mainResource.setUserName(" Username with LeadingSpace");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.LEADING_SPACE, mainResource
                .getClass().getName(), "userName", mainResource.getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureFullNameSameAsPassword() throws AssertionError,
            Exception
    {
        mainResource.setFullName("same thing");
        mainResource.setPassword("same thing");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.FIELDS_CANNOT_BE_THE_SAME,
                mainResource.getClass().getName(), "fullName", "password",
                mainResource.getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testSuccessFullNameNotSameAsPassword() throws AssertionError,
            Exception
    {
        Resource resource = new Resource();
        resource.setParent(parentPool);

        resource.setFullName("thing");
        resource.setPassword("not same thing");

        // Attempt to save the Resource object
        resource = (Resource) saveMustSucceed(resource, Resource.class, null);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='"
                + resource.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the loaded resource is identical to the one we requested
        // to store
        ResourceFixture.compare(resource, (Resource) rpmObjects[0], null);
    }

    public void testFailureWithDuplicateUsername() throws AssertionError,
            Exception
    {
        mainResource.setUserName("DuplicateUsername5432");
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class,
            null);

        Resource mainResource2 = mainResource = ResourceFixture.createFixture(
            parentPool, null, this);
        mainResource.setUserName("DuplicateUsername5432");

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NOT_UNIQUE, mainResource.getClass()
                .getName(), "userName", mainResource.getID())
        };

        verifySaveMustFail(mainResource2, null, expectedExceptions);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength and verify that they are accepted
        mainResource
            .setAddress1(getTestString(ValidationConstants.RESOURCE_ADDRESS1_MAX));
        mainResource
            .setAddress2(getTestString(ValidationConstants.RESOURCE_ADDRESS2_MAX));
        mainResource
            .setAlternativePhone(getTestString(ValidationConstants.RESOURCE_ALTERNATIVEPHONE_MAX));
        mainResource
            .setCompanyCode(getTestString(ValidationConstants.RESOURCE_COMPANYCODE_MAX));
        mainResource
            .setCountryCode(getTestString(ValidationConstants.RESOURCE_COUNTRYCODE_MAX));
        mainResource
            .setCustom1(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX));
        mainResource
            .setCustom2(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX));
        mainResource
            .setCustom3(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX));
        mainResource
            .setCustom4(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX));
        mainResource
            .setCustom5(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX));
        // TODO Insert RESOURCE_DASHBOARD_MAX
        mainResource
            .setDepartmentCode(getTestString(ValidationConstants.RESOURCE_DEPARTMENTCODE_MAX));
        mainResource
            .setDivisionCode(getTestString(ValidationConstants.RESOURCE_DIVISIONCODE_MAX));
        mainResource
            .setEmailAddress(getTestString(ValidationConstants.RESOURCE_EMAILADDRESS_MAX));
        mainResource
            .setEmployeeCode(getTestString(ValidationConstants.RESOURCE_EMPLOYEECODE_MAX));
        mainResource
            .setExternalID(getTestString(ValidationConstants.RESOURCE_EXTERNALID_MAX));
        mainResource
            .setFamilyName(getTestString(ValidationConstants.RESOURCE_FAMILYNAME_MAX));
        mainResource
            .setFax(getTestString(ValidationConstants.RESOURCE_FAX_MAX));
        mainResource
            .setGivenName(getTestString(ValidationConstants.RESOURCE_GIVENNAME_MAX));
        mainResource
            .setHomePhone(getTestString(ValidationConstants.RESOURCE_HOMEPHONE_MAX));
        mainResource
            .setInterviewExperience(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX));
        // TODO Insert RESOURCE_LOGONREMINDER_MAX
        mainResource
            .setMiddleName(getTestString(ValidationConstants.RESOURCE_MIDDLENAME_MAX));
        mainResource
            .setMobile(getTestString(ValidationConstants.RESOURCE_MOBILE_MAX));
        mainResource
            .setMyAchievements(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX));
        mainResource
            .setMyPersonalPassions(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX));
        mainResource
            .setMySummaryOfSkills(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX));
        mainResource
            .setMyWebsite(getTestString(ValidationConstants.RESOURCE_MYWEBSITE_MAX));
        mainResource
            .setPager(getTestString(ValidationConstants.RESOURCE_PAGER_MAX));
        mainResource
            .setTitle(getTestString(ValidationConstants.RESOURCE_TITLE_MAX));
        mainResource
            .setWhatIDoReallyWell(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX));
        mainResource
            .setWorkPhone(getTestString(ValidationConstants.RESOURCE_WORKPHONE_MAX));
        mainResource
            .setZipPostalCode(getTestString(ValidationConstants.RESOURCE_ZIPPOSTALCODE_MAX));

        // Set verified fields
        Resource verResource = new Resource();
        verResource.setID("PMO_SUPERVISOR");
        verResource = (Resource) loadFromID(verResource, null);
        mainResource.setReVerifiedBy(verResource);
        mainResource.setVerifiedBy(verResource);

        // Dates
        mainResource.setEffectiveStartDate(calendar1);
        mainResource.setDiscontinueDate(calendar1);

        // Create a userName, fullName and password of maxLength without special
        // characters
        mainResource
            .setPassword(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_PASSWORD_MAX));
        mainResource
            .setUserName(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_USERNAME_MAX));
        mainResource
            .setFullName(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_FULLNAME_MAX));
        mainResource
            .setNickName(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_NICKNAME_MAX));

        // myPortalURL strings of maxSize
        mainResource.setMyPortalURL1(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX));
        mainResource.setMyPortalURL2(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX));
        mainResource.setMyPortalURL3(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX));
        mainResource.setMyPortalURL4(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX));
        mainResource.setMyPortalURL5(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX));
        
        // Range validation - max
        mainResource.setWeeklyMaxHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMAXHOURS_MAX));
        mainResource.setWeeklyMinHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMINHOURS_MAX));

        // Attempt to save the Resource object
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class,
            null);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='"
                + mainResource.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the loaded resource is identical to the one we requested
        // to store
        ResourceFixture.compare(mainResource, (Resource) rpmObjects[0], null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Range validation - min
        mainResource.setWeeklyMaxHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMAXHOURS_MIN));
        mainResource.setWeeklyMinHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMINHOURS_MIN));

        // Attempt to save the Resource object
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class,
            null);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='"
                + mainResource.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the loaded resource is identical to the one we requested
        // to store
        ResourceFixture.compare(mainResource, (Resource) rpmObjects[0], null);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set field values with maxLength+1 and verify that they are not
        // accepted
        mainResource
            .setAddress1(getTestString(ValidationConstants.RESOURCE_ADDRESS1_MAX + 1));
        mainResource
            .setAddress2(getTestString(ValidationConstants.RESOURCE_ADDRESS2_MAX + 1));
        mainResource
            .setAlternativePhone(getTestString(ValidationConstants.RESOURCE_ALTERNATIVEPHONE_MAX + 1));
        mainResource
            .setCompanyCode(getTestString(ValidationConstants.RESOURCE_COMPANYCODE_MAX + 1));
        mainResource
            .setCountryCode(getTestString(ValidationConstants.RESOURCE_COUNTRYCODE_MAX + 1));
        mainResource
            .setCustom1(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX + 1));
        mainResource
            .setCustom2(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX + 1));
        mainResource
            .setCustom3(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX + 1));
        mainResource
            .setCustom4(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX + 1));
        mainResource
            .setCustom5(getTestString(ValidationConstants.RESOURCE_CUSTOM_MAX + 1));
        // TODO Insert RESOURCE_DASHBOARD_MAX
        mainResource
            .setDepartmentCode(getTestString(ValidationConstants.RESOURCE_DEPARTMENTCODE_MAX + 1));
        mainResource
            .setDivisionCode(getTestString(ValidationConstants.RESOURCE_DIVISIONCODE_MAX + 1));
        mainResource
            .setEmailAddress(getTestString(ValidationConstants.RESOURCE_EMAILADDRESS_MAX + 1));
        mainResource
            .setEmployeeCode(getTestString(ValidationConstants.RESOURCE_EMPLOYEECODE_MAX + 1));
        mainResource
            .setExternalID(getTestString(ValidationConstants.RESOURCE_EXTERNALID_MAX + 1));
        mainResource
            .setFamilyName(getTestString(ValidationConstants.RESOURCE_FAMILYNAME_MAX + 1));
        mainResource
            .setFax(getTestString(ValidationConstants.RESOURCE_FAX_MAX + 1));
        mainResource
            .setGivenName(getTestString(ValidationConstants.RESOURCE_GIVENNAME_MAX + 1));
        mainResource
            .setHomePhone(getTestString(ValidationConstants.RESOURCE_HOMEPHONE_MAX + 1));
        mainResource
            .setInterviewExperience(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX + 1));
        // TODO Insert RESOURCE_LOGONREMINDER_MAX
        mainResource
            .setMiddleName(getTestString(ValidationConstants.RESOURCE_MIDDLENAME_MAX + 1));
        mainResource
            .setMobile(getTestString(ValidationConstants.RESOURCE_MOBILE_MAX + 1));
        mainResource
            .setMyAchievements(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX + 1));
        mainResource
            .setMyPersonalPassions(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX + 1));
        mainResource
            .setMySummaryOfSkills(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX + 1));
        mainResource
            .setMyWebsite(getTestString(ValidationConstants.RESOURCE_MYWEBSITE_MAX + 1));
        mainResource
            .setPager(getTestString(ValidationConstants.RESOURCE_PAGER_MAX + 1));
        mainResource
            .setTitle(getTestString(ValidationConstants.RESOURCE_TITLE_MAX + 1));
        mainResource
            .setWhatIDoReallyWell(getTestString(ValidationConstants.RESOURCE_INTERVIEWPORTLET_MAX + 1));
        mainResource
            .setWorkPhone(getTestString(ValidationConstants.RESOURCE_WORKPHONE_MAX + 1));
        mainResource
            .setZipPostalCode(getTestString(ValidationConstants.RESOURCE_ZIPPOSTALCODE_MAX + 1));

        // calendar2 < calendar1
        mainResource.setEffectiveStartDate(calendar1);
        mainResource.setDiscontinueDate(calendar2);

        // Create a username and password of maxLength+1 without special
        // characters
        mainResource
            .setPassword(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_PASSWORD_MAX + 1));
        mainResource
            .setUserName(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_USERNAME_MAX + 1));
        mainResource
            .setFullName(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_FULLNAME_MAX + 1));
        mainResource
            .setNickName(createFixedLengthStringAlpha(ValidationConstants.RESOURCE_NICKNAME_MAX + 1));

        // myPortalURL strings of maxSize + 1
        mainResource.setMyPortalURL1(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX + 1));
        mainResource.setMyPortalURL2(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX + 1));
        mainResource.setMyPortalURL3(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX + 1));
        mainResource.setMyPortalURL4(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX + 1));
        mainResource.setMyPortalURL5(getTestString(ValidationConstants.RESOURCE_MYPORTALURLSSTRING_MAX + 1));

        // Range validation - max+1
        mainResource.setWeeklyMaxHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMAXHOURS_MAX + 1));
        mainResource.setWeeklyMinHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMINHOURS_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "address1", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "address2", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "alternativePhone", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "companyCode", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "countryCode", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "custom1", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "custom2", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "custom3", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "custom4", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "custom5", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "departmentCode", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "divisionCode", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "emailAddress", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "employeeCode", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "externalID", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "familyName", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "fax", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "fullName", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "givenName", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "homePhone", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "interviewExperience", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "middleName", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "mobile", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "myAchievements", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "myPersonalPassions", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "mySummaryOfSkills", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "myWebsite", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "nickName", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "pager", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "title", mainResource.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainResource
                    .getClass().getName(), "weeklyMaxHours", mainResource
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainResource
                    .getClass().getName(), "weeklyMinHours", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "whatIDoReallyWell", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "workPhone", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "zipPostalCode", mainResource
                    .getID()),
                createRPMException(ErrorCodes.INVALID_DATES, mainResource
                    .getClass().getName(), "effectiveStartDate", mainResource
                    .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "password", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource
                    .getClass().getName(), "userName", mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource.getClass()
                    .getName(),
                    ValidationConstants.RESOURCE_MY_PORTAL_URL1_FIELD,
                    mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource.getClass()
                    .getName(),
                    ValidationConstants.RESOURCE_MY_PORTAL_URL2_FIELD,
                    mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource.getClass()
                    .getName(),
                    ValidationConstants.RESOURCE_MY_PORTAL_URL3_FIELD,
                    mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource.getClass()
                    .getName(),
                    ValidationConstants.RESOURCE_MY_PORTAL_URL4_FIELD,
                    mainResource.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, mainResource.getClass()
                    .getName(),
                    ValidationConstants.RESOURCE_MY_PORTAL_URL5_FIELD,
                    mainResource.getID()),
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool, null, this);

        // Range validation - min-1
        mainResource.setWeeklyMaxHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMAXHOURS_MIN - 1));
        mainResource.setWeeklyMinHours(new Integer(
            ValidationConstants.RESOURCE_WEEKLYMINHOURS_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainResource
                    .getClass().getName(), "weeklyMaxHours", mainResource
                    .getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, mainResource
                    .getClass().getName(), "weeklyMinHours", mainResource
                    .getID())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testFailureValidationReVerified() throws AssertionError,
            Exception
    {
        mainResource = ResourceFixture.createFixture(parentPool, null, this);

        // Set the reVerified resource but leave verified to null and ensure
        // that
        // saving fails
        Resource verResource = new Resource();
        verResource.setID("PMO_SUPERVISOR");
        verResource = (Resource) loadFromID(verResource, null);
        mainResource.setReVerifiedBy(verResource);
        mainResource.setVerifiedBy(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.resource.constants.ErrorCodes.REVERIFIEDBY_SPECIFIED_WITHOUT_A_VERIFIEDBY_RESOURCE,
                mainResource.getClass().getName())
        };

        verifySaveMustFail(mainResource, null, expectedExceptions);
    }

    public void testSuccessNullable() throws AssertionError, Exception
    {
        // Set nullable fields to null.
        // TODO: Set all other nullable fields to null.
        mainResource.setDiscontinueDate(null);

        // Attempt to create the Resource object with null fields.
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class,
            null);

        // Load the inserted resource from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='"
                + mainResource.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the loaded resource is identical to the one we requested
        // to store
        ResourceFixture.compare(mainResource, (Resource) rpmObjects[0], null);

        // Attempt to update the Resource object with null fields.
        mainResource = (Resource) saveMustSucceed(mainResource, Resource.class,
            null);

        // Load the inserted resource from xpath
        rpmObjects = loadFromXpath("/Resource[id='" + mainResource.getID()
                + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the loaded resource is identical to the one we requested
        // to store
        ResourceFixture.compare(mainResource, (Resource) rpmObjects[0], null);
    }
    
    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        Resource resource = new Resource();
        resource.setParent(parentPool);
        resource.setUserName("test resource userName");
        resource.setFullName("test resource fullName");
        
        ResourceScope scope = new ResourceScope();
        scope.setCurrency(true);
        scope.setExperience(true);
        scope.setWorkLocation(true);

        // Leave default values as null and save
        resource = (Resource) saveMustSucceed(resource, Resource.class, scope);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='"
                + resource.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the default values are now set
        Resource savedResource = (Resource) rpmObjects[0];
        assertNotNull(savedResource.getActive());
        assertNotNull(savedResource.getCurrency());
        assertNotNull(savedResource.getEffectiveStartDate());
        assertNotNull(savedResource.getExperience());
        assertNotNull(savedResource.getOverrideOriginalDocumentOnCheckout());
        assertNotNull(savedResource.getPassword());
        assertNotNull(savedResource.getWeeklyMaxHours());
        assertNotNull(savedResource.getWorkLocation());

        // Save again to validate the default values
        resource = (Resource) saveMustSucceed(resource, Resource.class, scope);

        // Load the inserted project from xpath
        rpmObjects = loadFromXpath("/Resource[id='" + resource.getID() + "']",
            scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the loaded Project is identical to the one we
        // requested to store
        ResourceFixture.compare(resource, (Resource) rpmObjects[0], scope);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        Resource resource = new Resource();
        resource.setParent(parentPool);
        resource.setUserName("test resource userName");
        resource.setFullName("test resource fullName");
        
        ResourceScope scope = new ResourceScope();
        scope.setCurrency(true);
        scope.setExperience(true);
        scope.setWorkLocation(true);

        // Leave default values as null and save
        resource = (Resource) saveMustSucceed(resource, Resource.class, scope);

        // Load the inserted project from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Resource[id='"
                + resource.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Resource);

        // Check that the default values are now set
        Resource savedResource = (Resource) rpmObjects[0];
        assertNotNull(savedResource.getActive());
        assertNotNull(savedResource.getCurrency());
        assertNotNull(savedResource.getEffectiveStartDate());
        assertNotNull(savedResource.getExperience());
        assertNotNull(savedResource.getOverrideOriginalDocumentOnCheckout());
        assertNotNull(savedResource.getPassword());
        assertNotNull(savedResource.getWeeklyMaxHours());
        assertNotNull(savedResource.getWorkLocation());

        // Update the default values to null
        savedResource.setActive(null);
        savedResource.setCurrency(null);
        savedResource.setEffectiveStartDate(null);
        savedResource.setExperience(null);
        savedResource.setOverrideOriginalDocumentOnCheckout(null);
        savedResource.setPassword(null);
        savedResource.setWeeklyMaxHours(null);
        savedResource.setWorkLocation(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(), ValidationConstants.RESOURCE_ACTIVE_FIELD,
                    savedResource.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(), ValidationConstants.RESOURCE_CURRENCY_FIELD,
                    savedResource.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(),
                    ValidationConstants.RESOURCE_EFFECTIVE_START_DATE_FIELD,
                    savedResource.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(),
                    ValidationConstants.RESOURCE_EXPERIENCE_FIELD, savedResource.getID()),
                createRPMException(
                    ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedResource.getClass().getName(),
                    ValidationConstants.RESOURCE_OVERRIDE_ORIGINAL_DOCUMENT_ON_CHECKOUT_FIELD,
                    savedResource.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(), ValidationConstants.RESOURCE_PASSWORD_FIELD,
                    savedResource.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(),
                    ValidationConstants.RESOURCE_WEEKLY_MAX_HOURS_FIELD, savedResource
                        .getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD, savedResource
                    .getClass().getName(),
                    ValidationConstants.RESOURCE_WORK_LOCATION_FIELD, savedResource
                        .getID())

        };

        verifySaveMustFail(savedResource, scope, expectedExceptions);

    }
}
