/*
 * Created on 6-Jun-2006
 * @author mmatwiy
 * Validation tests for conversion of ScopeElement containers.
 */
package com.ibm.rpm.client.junit.scopemanagement.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.constants.ValidationConstants;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestScopeElementConversionValidation extends GenericValidation
{
    ScopeFolder scopeFolder;

    public void setUp() throws Exception
    {
        super.setUp();

        // create a Proposal and save it; this will create a ScopeFolder
        Proposal proposal = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);

        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + proposal.getName() + "']", null);

        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ScopeFolder);
        scopeFolder = (ScopeFolder) rpmObjects[0];
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testFailureNullInputID() throws Exception
    {
        Defect inputDefect = new Defect();

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), inputDefect, new Action(), null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, inputDefect.getClass()
                .getName()),
        };

        verifyFailure(inputDefect, expectedExceptions, null, result);
    }

    public void testFailureInvalidScope() throws Exception
    {
        Risk risk = new Risk();
        risk.setParent(scopeFolder);
        risk.setName(HelperTestUtilities.createName());
        save(risk, null);

        SingleResult result = getApplicationInterface().convertContainer(
            getSessionID(), risk, new Requirement(), new WorkElementScope());

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_SCOPE, risk.getClass()
                .getName()),
        };

        verifyFailure(risk, expectedExceptions, null, result);

    }

    public void testFailureConvertInvalidInputType() throws Exception
    {
        ClientCostCenter clientCostCenter = new ClientCostCenter();

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, clientCostCenter, new Risk(), null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_RPMOBJECT_TYPE, null),
        };

        verifyFailure(clientCostCenter, expectedExceptions, null, result);
    }

    public void testFailureConvertInvalidOutputType() throws Exception
    {
        Issue inputIssue = new Issue();
        inputIssue.setParent(scopeFolder);
        inputIssue.setName(HelperTestUtilities.createName());
        save(inputIssue, null);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputIssue, new Task(), null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputIssue.getClass().getName())
        };

        verifyFailure(inputIssue, expectedExceptions, null, result);
    }

    public void testFailureConvertNonTaskToTask() throws Exception
    {
        Action inputAction = new Action();
        inputAction.setParent(scopeFolder);
        inputAction.setName(HelperTestUtilities.createName());
        inputAction.setTask(new Boolean(false));
        save(inputAction, null);

        Risk riskTask = new Risk();
        riskTask.setTask(new Boolean(true));

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputAction, riskTask, null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputAction.getClass().getName())
        };

        verifyFailure(inputAction, expectedExceptions, null, result);
    }

    public void testFailureConvertTaskToNonTask() throws Exception
    {
        Action inputAction = new Action();
        inputAction.setParent(scopeFolder);
        inputAction.setName(HelperTestUtilities.createName());
        inputAction.setTask(new Boolean(true));
        save(inputAction, null);

        ServiceRequest serviceRequest = new ServiceRequest();
        serviceRequest.setTask(new Boolean(false));

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputAction, serviceRequest, null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputAction.getClass().getName())
        };

        verifyFailure(inputAction, expectedExceptions, null, result);
    } 

    public void testFailureConvertNullTaskToTask() throws Exception
    {
        Action inputAction = new Action();
        inputAction.setParent(scopeFolder);
        inputAction.setName(HelperTestUtilities.createName());
        save(inputAction, null);

        Risk riskTask = new Risk();
        riskTask.setTask(new Boolean(true));

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputAction, riskTask, null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputAction.getClass().getName())
        };

        verifyFailure(inputAction, expectedExceptions, null, result);
    }

    public void testFailureConvertTaskToNullTask() throws Exception
    {
        Action inputAction = new Action();
        inputAction.setParent(scopeFolder);
        inputAction.setName(HelperTestUtilities.createName());
        inputAction.setTask(new Boolean(true));
        save(inputAction, null);

        SingleResult result = getApplicationInterface().convertContainer(
            sessionid, inputAction, new ServiceRequest(), null);

        assertFalse(result.isSuccessful());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
                inputAction.getClass().getName())
        };

        verifyFailure(inputAction, expectedExceptions, null, result);
    }

    public void testFailureConvertRequirementToTaskWithChildren()
            throws Exception
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setChildren(new ScopeElementScope());
        
        Action action = new Action();
        action.setName(HelperTestUtilities.createName());

        Requirement inputRequirement = new Requirement();
        inputRequirement.setParent(scopeFolder);
        inputRequirement.setName(HelperTestUtilities.createName());
        inputRequirement.setChildren(new ScopeElement[]
        {
            action
        });

        action.setParent(inputRequirement);
        inputRequirement = (Requirement) save(inputRequirement, scope)
            .getRpmObject();

        inputRequirement.setTask(new Boolean(true));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_BE_NULL, inputRequirement
                .getClass().getName(), ValidationConstants.CHILDREN_FIELD,
                inputRequirement.getID())
        };

        verifySaveMustFail(inputRequirement, scope, expectedExceptions);
    }

    public void testFailureCreateServiceRequestTaskWithChildren()
            throws Exception
    {
        ScopeElementScope scope = new ScopeElementScope();
        scope.setChildren(new ScopeElementScope());
        
        Action action = new Action();
        action.setName(HelperTestUtilities.createName());

        ServiceRequest inputServiceRequest = new ServiceRequest();
        inputServiceRequest.setParent(scopeFolder);
        inputServiceRequest.setName(HelperTestUtilities.createName());
        inputServiceRequest.setTask(new Boolean(true));
        inputServiceRequest.setChildren(new ScopeElement[]
        {
            action
        });

        action.setParent(inputServiceRequest);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_BE_NULL, inputServiceRequest
                .getClass().getName(), ValidationConstants.CHILDREN_FIELD,
                inputServiceRequest.getID())
        };

        verifySaveMustFail(inputServiceRequest, scope, expectedExceptions);
    }
}