/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.client.junit.security.testcases;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import junit.framework.Assert;

import org.apache.commons.lang.time.StopWatch;

import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.RpmTestProperties;
import com.ibm.rpm.client.junit.SpecificTestController;
import com.ibm.rpm.client.junit.UnitTestSessionUtility;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.resource.fixtures.ResourceFixture;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.interfaces.impl.ServerFactory;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceSecurityRights;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;

/**
 * @author Yasser Lulu, Marc-Andre Lamothe
 * 
 */
public class BaseSecurityTestLocal extends ClientTestController
{

    public static final String RPM_SEC = "com.ibm.rpm.framework.security.controller.impl.rpm.RPMSecurityControllerImpl";

    public static final String RES_SEC = "com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl";

    public static final String ROLE_SEC = "com.ibm.rpm.framework.security.controller.impl.role.RoleBasedSecurityControllerImpl";

    public static final String IGS_SEC = "com.ibm.rpm.framework.security.controller.impl.igs.IGSSecurityControllerImpl";
    
    public static final String NULL_SEC = "com.ibm.rpm.framework.security.controller.BaseSecurityController";

    private StopWatch watch = new StopWatch();

    protected boolean isEnableSecurity = true;
    
    protected boolean loginAsTestUser = true;

    protected String loggedAs = "";

    protected void setUp() throws Exception
    {
        watch.reset();
        watch.start();
        SpecificTestController.overrideOption("webServicesSecurityEnabled",
            new Boolean(isEnableSecurity));
        super.setUp();
        // Switch to the security tester resource
        String username = RpmTestProperties
            .getProperty("rpm.test.security.user.name");
        String password = RpmTestProperties
            .getProperty("rpm.test.security.user.password");
        if (loginAsTestUser)
            assertTrue(logoutAndLoginAs(username, password));
    }

    private Resource currentResource;

    protected Resource getCurrentUser()
    {
        return currentResource;
    }

    protected void saveCurrentUserSecurityRights(ResourceScope resourceScope)
            throws Exception
    {
        currentResource = saveAndReloadResourceWithSecurity(getCurrentUser(),
            resourceScope);
        assertNotNull(currentResource);
    }

    protected void tearDown() throws Exception
    {
//        // Switch back to the default resource (should be admin)
//        String username = RpmTestProperties.getProperty("rpm.test.user.name");
//        String password = RpmTestProperties
//            .getProperty("rpm.test.user.password");
//        if (loginAsTestUser)
//        	getAuthenticateInterface().logout(sessionid);//logoutAndLoginAs(username, password);
        super.tearDown();
        watch.stop();
        logger.info("===================================================");
        logger.info(" Security : " + ((isEnableSecurity)?"ON":"OFF"));
        logger.info(this.getName() + " finished in: " + watch);
        logger.info("===================================================");
    }

    protected Resource loadOrCreateUserResource(String username, String password)
            throws Exception
    {
        Resource securityUser = loadUserResource(username, password);
        if (securityUser == null)
        {
            securityUser = createUserResource(username, password);
        }
        enableAllSecurityRights(securityUser);
        return securityUser;
    }

    protected boolean logoutAndLoginAs(String username, String password)
            throws Exception
    {
        Resource res = loadOrCreateUserResource(username, password);
        getAuthenticateInterface().logout(sessionid);
        performLogin(username, password);
        loggedAs = username;
        ResourceScope resourceScope = new ResourceScope();
        resourceScope.setResourceSecurityRights(true);
        currentResource = (Resource) loadFromID(res, resourceScope);

        assertNotNull(currentResource);
        return true;
    }
    
    protected void checkOutAndValidate(RPMObject rpmObject, boolean canCheckOut) throws RPMException
    {
    	addObjectToDelete(rpmObject);
    	Result result = getApplicationInterface().checkOut(sessionid, rpmObject, false);
    	Assert.assertNotNull("Result is null", result);
    	if (canCheckOut)
    	{
    		actionMustSucceed(result, rpmObject.getClass());
    	}
    	else
    	{
    		actionMustFail(result);
    	}
    }

    protected RPMObject saveAndValidate(RPMObject rpmObject,
            RPMObjectScope scope, boolean canSave) throws Exception
    {
        addObjectToDelete(rpmObject);
        SingleResult result = getApplicationInterface().save(sessionid, rpmObject, scope,
            ReloadType.ReloadResult);
        Assert.assertNotNull("Result is null", result);
        if (canSave)
        {
            actionMustSucceed(result, rpmObject.getClass());
            return result.getRpmObject();
        }
        else
        {
            actionMustFail(result);
            return null;
        }
    }
    
    protected GenericDocument lockAndValidate(GenericDocument documentToLock, boolean canLock) throws Exception
    {
    	addObjectToDelete(documentToLock);
        SingleResult result = getApplicationInterface().lockDocument(sessionid, documentToLock);
        Assert.assertNotNull("Result is null", result);
        if (canLock)
        {
            actionMustSucceed(result, documentToLock.getClass());
            return (GenericDocument)result.getRpmObject();
        }
        else
        {
            actionMustFail(result);
            return null;
        }
    }
    
    protected GenericDocument unLockAndValidate(GenericDocument documentToUnlock, boolean canUnLock) throws Exception
    {
    	addObjectToDelete(documentToUnlock);
        SingleResult result = getApplicationInterface().unLockDocument(sessionid, documentToUnlock);
        Assert.assertNotNull("Result is null", result);
        if (canUnLock)
        {
            actionMustSucceed(result, documentToUnlock.getClass());
            return (GenericDocument)result.getRpmObject();
        }
        else
        {
            actionMustFail(result);
            return null;
        }
    }

    protected void deleteAndValidate(RPMObject rpmObject, boolean canDelete)
            throws Exception
    {
        Result result = getApplicationInterface().delete(sessionid, rpmObject);
        Assert.assertNotNull("Result is null", result);
        if (canDelete)
            actionMustSucceed(result, rpmObject.getClass());
        else
            actionMustFail(result);
    }

    protected void transferUnderWBS(AbstractScope abstractScope, boolean canTransfer) throws Exception
    {
    	Result result = getApplicationInterface().transferUnderWBS(sessionid, abstractScope.getProject(), abstractScope);
    	Assert.assertNotNull("Result is null", result);
        if (canTransfer)
            actionMustSucceed(result, abstractScope.getClass());
        else
            actionMustFail(result);
    }
    
    public SingleResult copyProposedToPlan(WorkElement rpmObject,
            WorkElementScope scope, boolean canTransfer) throws Exception
    {
        SingleResult result = getApplicationInterface().copyProposedToPlan(
            sessionid, rpmObject, scope);
        Assert.assertNotNull("Result is null", result);
        if (canTransfer)
            actionMustSucceed(result, rpmObject.getClass());
        else
            actionMustFail(result);
        return result;
    }
    
    public SingleResult convertContainer(
            RPMObject inputContainer, RPMObject outputContainer,
            RPMObjectScope rpmScope, boolean canConvert)
    {        
        SingleResult result = getApplicationInterface().convertContainer(sessionid, inputContainer,
            outputContainer, rpmScope);
        
        Assert.assertNotNull("Result is null", result);
        if (canConvert)
            actionMustSucceed(result, outputContainer.getClass());
        else
            actionMustFail(result);
        return result;
        
    }

    
    protected void actionMustSucceed(Result result, Class expectedClass)
    {
        String Error = "";
        if (result.getErrors() != null && result.getErrors()[0] != null)
            Error = result.getErrors()[0].toString();
        assertTrue(Error, result.isSuccessful());
        if (result instanceof SingleResult)
        {
            SingleResult singleResult = (SingleResult) result;
            assertNotNull(singleResult.getRpmObject());
            assertTrue(expectedClass.isAssignableFrom(singleResult
                .getRpmObject().getClass()));
            //Module objects don't have IDs so we shouldn't skip assertion
            if(!(singleResult.getRpmObject() instanceof AbstractModule))
            {
                assertNotNull(singleResult.getRpmObject().getID());                
            }
        }
    }

    protected void actionMustFail(Result result)
    {
        String Error = "";
        if (result.getErrors() != null && result.getErrors()[0] != null)
            Error = result.getErrors()[0].toString();
        assertFalse(Error, result.isSuccessful());
        assertTrue(
            Error,
            result.getErrors()[0].getErrorID() == ErrorCodes.INSUFFICIENT_SECURITY_RIGHTS);
    }

    protected void loadResourceWithSecurity(ResourceScope resourceScope)
            throws Exception
    {
        if (resourceScope == null)
        {
            resourceScope = new ResourceScope();
        }
        resourceScope.setResourceSecurityRights(true);
        currentResource = (Resource) loadFromID(getCurrentUser(), resourceScope);
//        currentResource = (Resource) loadOneFromXpath("/Resource[username='"
//                + loggedAs + "']", resourceScope);
        assertNotNull(currentResource);
    }

    private Resource saveAndReloadResourceWithSecurity(Resource resource,
            ResourceScope resourceScope) throws Exception
    {
        if (resourceScope == null)
        {
            resourceScope = new ResourceScope();
        }
        resourceScope.setResourceSecurityRights(true);
        SingleResult result = saveWithoutDelete(resource, resourceScope);
        assertTrue(result.isSuccessful());
        return (Resource) result.getRpmObject();
    }

    /* 
     * Private logout and login fonctions 
     */

    private Resource loadUserResource(String username, String password)
            throws Exception
    {
        RPMObject[] rpmObjects = loadFromXpath("/Resource[USERNAME='"
                + username + "']", null);
        if (rpmObjects == null || rpmObjects.length == 0)
        {
            return null;
        }
        else if (rpmObjects.length > 1)
        {
            throw new Exception(
                "BaseIGSFlagsTestLocal.loadSecurityTestUser : ERROR "
                        + rpmObjects.length + " security test users ["
                        + username + "] were found.");
        }
        else
        {
            Resource resource = (Resource) rpmObjects[0];
            return resource;
        }
    }

    private Resource createUserResource(String username, String password)
            throws Exception
    {
        // Create the Pool and Resource
        Pool pool = PoolFixture.retrieveStaticFixture(this);
        Resource securityUser = ResourceFixture.createFixture(pool, null, this);
        securityUser.setFullName(username + "_FULL_NAME");
        securityUser.setUserName(username);
        securityUser.setParent(pool);
        securityUser.setActive(new Boolean(true));
        securityUser.setCalendar(RPMCalendarFixture
            .retrieveStaticFixtureForInserts(this));
        securityUser.setPassword("ibm");
        ResourceScope scope = new ResourceScope();
        scope.setCalendar(new CalendarScope());
        securityUser = (Resource) saveWithoutDelete(securityUser, scope)
            .getRpmObject();

        // Changing resource passwords to avoid the password expired
        // error message on the first logon.
        // WARNING: don't change this password since the
        // rpm-webui-functional-test projects depends on it.
        changePassword(securityUser.getUserName(), "ibm", password);
        securityUser.setPassword(password);
        return securityUser;
    }

    private Resource enableAllSecurityRights(Resource securityUser)
            throws Exception
    {
        ResourceScope scope = new ResourceScope();
        scope.setResourceSecurityRights(true);

        // Enable all the security rights
        securityUser = (Resource) loadFromID(securityUser, scope);
        ResourceSecurityRights resourceSecurityRights = securityUser
            .getResourceSecurityRights();

        //        if (securityUser.getEffectiveStartDate() == null)
        //        {
        //            Calendar effectiveStartDate = Calendar.getInstance();
        //            effectiveStartDate.set(Calendar.YEAR, 2000);
        //            effectiveStartDate.set(Calendar.MONTH, 1);
        //            effectiveStartDate.set(Calendar.DAY_OF_MONTH, 1);
        //            securityUser.setEffectiveStartDate(effectiveStartDate);
        //        }

        setFlags(resourceSecurityRights, "setCan", true);
        setFlags(resourceSecurityRights, "setAll", true);
        return saveAndReloadResourceWithSecurity(securityUser, scope);
    }

    private void setFlags(Object object, String prefix, boolean flagValue)
            throws Exception
    {
        Method[] methods = object.getClass().getMethods();
        Method method = null;
        List setMethods = filterMethods(methods, prefix);
        for (int i = 0; i < setMethods.size(); i++)
        {
            method = (Method) setMethods.get(i);
            if (method.getParameterTypes().length == 1)
                setFlag(object, method, flagValue);
        }
    }

    private boolean setFlag(Object object, Method method, boolean flagValue)
            throws Exception
    {
        if (method != null && method.getParameterTypes().length == 1)
        {
            Class parameterType = method.getParameterTypes()[0];
            if (parameterType.getName().equals("java.lang.Boolean"))
            {
                method.invoke(object, new Boolean[]
                {
                    new Boolean(flagValue)
                });
                return true;
            }
            else
            {
                String setMethodName = method.getName();
                String getMethodName = "get" + setMethodName.substring(3);

                Method getMethod = object.getClass().getMethod(getMethodName,
                    null);
                Object output = getMethod.invoke(object, null);
                if (output instanceof Boolean[])
                {
                    Boolean[] castedOutput = (Boolean[]) output;
                    int length = castedOutput.length;
                    Boolean[] args = new Boolean[length];
                    for (int j = 0; j < length; j++)
                        args[j] = new Boolean(flagValue);
                    method.invoke(object, new Object[]
                    {
                        args
                    });
                    return true;
                }
            }
        }
        return false;
    }

    private List filterMethods(Method[] methods, String prefix)
    {
        Method[] rpmObjectMethods = RPMObject.class.getMethods();
        Set rpmObjectMethodNames = new TreeSet();
        for (int i = 0; i < rpmObjectMethods.length; i++)
            rpmObjectMethodNames.add(rpmObjectMethods[i].getName());
        String methodName = null;
        List filteredMethods = new ArrayList();
        if (methods != null)
            for (int i = 0; i < methods.length; i++)
            {
                methodName = methods[i].getName();
                if (methodName.startsWith(prefix)
                        && !rpmObjectMethodNames.contains(methodName))
                    filteredMethods.add(methods[i]);
            }
        return filteredMethods;
    }

    public MessageContext fabricateBasicMessageContext(String operationName)
            throws RPMException
    {
        MessageContext messageContext = new MessageContext(ServerFactory
            .getInstance());
        messageContext.setSession(UnitTestSessionUtility.INSTANCE, sessionid,
            operationName);
        return messageContext;
    }

}
