/*
 * Created on Feb 13, 2007
 * Defect 60645: WS - should not allow editing start forecast date 
 * if actual date is not null
 * 
 * Description - 
 * Steps to reproduce:
 * 1- report some time on a SummaryTimesheet and save.
 * 2- set SummaryTimesheet.forecastStartDate to some date and save.
 * 3- Notice how silently the forecastStartDate is set back to null. 

 * This is because forecastStartDate should be read-only when actual 
 * start date is not null. The validation layer should ensure of that.
 */
package com.ibm.rpm.client.junit.timesheet.testcases.fixes;

import java.util.Calendar;

import com.ibm.rpm.client.junit.ClientTestController;
import com.ibm.rpm.client.junit.timesheet.fixtures.SummaryTimesheetFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.TimesheetTestUtil;
import com.ibm.rpm.client.junit.wbs.fixtures.TaskAssignmentFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

/**
 * @author cobber
 * 
 */
public class Test60645 extends ClientTestController
{
    private SummaryTimesheet summaryTimesheet = null;

    private Calendar otherWeek = null;

    private SingleResult result = null;

    public void setUp() throws Exception
    {
        super.setUp();

        // Create a TaskAssignment
        TaskAssignmentScope taskAssignmentScope = TaskAssignmentFixture
            .createScope(true, false, false, false);
        TaskAssignment taskAssignment = TaskAssignmentFixture
            .createSaveFixture(taskAssignmentScope, this);
        assertNotNull(taskAssignment);

        // Create a SummaryTimesheet
        Calendar weekOf = TimesheetTestUtil.getValidWeekOf(taskAssignment
            .getAssignmentStartDate(), TimesheetTestUtil.CURRENT_WEEK);
        summaryTimesheet = SummaryTimesheetFixture.createSaveFixture(
            taskAssignment, weekOf, null, false, this);
        assertNotNull(summaryTimesheet);
        RPMObject rpmObject = this.loadFromID(summaryTimesheet, null);
        assertNotNull(rpmObject);
        assertTrue(rpmObject instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) rpmObject;
        assertNull(summaryTimesheet.getForecastStartDate());

        otherWeek = TimesheetTestUtil.getValidWeekOf(weekOf,
            TimesheetTestUtil.OTHER_WEEK);
    }

    private void updateForecastStartDate() throws Exception
    {
        // set forecast start date without any actuals
        summaryTimesheet.setForecastStartDate(otherWeek);
        result = this.save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
        assertNotNull(summaryTimesheet.getForecastStartDate());
        assertEquals(otherWeek, summaryTimesheet.getForecastStartDate());
    }

    public void testUpdateForecastStartDateWithoutActual() throws Exception
    {
        updateForecastStartDate();
    }

    public void testUpdateForecastStartDateWithActual() throws Exception
    {
        // set some values in forecast start date
        updateForecastStartDate();
        
        Calendar newForecastStartDate = TimesheetTestUtil.getValidWeekOf(
            otherWeek, TimesheetTestUtil.OTHER_WEEK);

        // Report hours
        summaryTimesheet.setRegularMonday(new Double(1.0));
        summaryTimesheet.setForecastStartDate(newForecastStartDate);
        result = this.save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        assertFalse(result.isWarningFree());
        assertNotNull(result.getWarnings());
        assertEquals(1, result.getWarnings().length);
        assertEquals(ErrorCodes.READ_ONLY_FIELD, result.getWarnings()[0]
            .getErrorID());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
        assertNull(summaryTimesheet.getForecastStartDate());
    }
    
    public void testUpdateActual() throws Exception
    {
        // set some values in forecast start date
        updateForecastStartDate();
        
        // Report hours
        summaryTimesheet.setRegularMonday(new Double(1.0));
        result = this.save(summaryTimesheet, null);
        assertTrue(result.isSuccessful());
        assertTrue(result.isWarningFree());
        assertNotNull(result.getRpmObject());
        assertTrue(result.getRpmObject() instanceof SummaryTimesheet);
        summaryTimesheet = (SummaryTimesheet) result.getRpmObject();
        assertNull(summaryTimesheet.getForecastStartDate());
    }
}
