package com.ibm.rpm.client.junit.timesheet.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.timesheet.fixtures.DefaultAdministrativeTaskFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.DefaultAdministrativeTask;
import com.ibm.rpm.wbs.containers.Task;


public class TestDefaultAdministrativeTaskValidation extends GenericValidation
{
    private DefaultAdministrativeTask adminTask;

    private Pool parentPool;

    public void setUp() throws Exception
    {
        super.setUp();

        adminTask = new DefaultAdministrativeTask();
        
        parentPool = PoolFixture.createSaveFixture(this);

    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(), this);

        adminTask
            .setName(getTestString(ValidationConstants.ADMIN_TASK_NAME_CHARACTER_LIMIT));

        adminTask.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, adminTask
                .getClass().getName(), ValidationConstants.PARENT_FIELD, adminTask
                .getID())
        };

        verifySaveMustFail(adminTask, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        // Set the mandatory fields
        adminTask.setParent(parentPool);
        adminTask
            .setName(getTestString(ValidationConstants.ADMIN_TASK_NAME_CHARACTER_LIMIT));

        // Attempt to save the adminTask object
        adminTask = (DefaultAdministrativeTask) saveMustSucceed(adminTask,
            DefaultAdministrativeTask.class, null);

        // Load the inserted adminTask from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DefaultAdministrativeTask[id='"
                + adminTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DefaultAdministrativeTask);

        // Check that the loaded adminTask is identical to the one we
        // requested to store
        DefaultAdministrativeTaskFixture.compare(adminTask, (DefaultAdministrativeTask) rpmObjects[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        adminTask.setParent(parentPool);

        // Set string values with max length + 1 and verify that they are not
        // accepted
        adminTask
            .setName(getTestString(ValidationConstants.ADMIN_TASK_NAME_CHARACTER_LIMIT + 1));
 
        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, adminTask.getClass()
                    .getName(), ValidationConstants.NAME_FIELD, adminTask
                    .getID()),
        };

        verifySaveMustFail(adminTask, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        adminTask.setParent(parentPool);
        adminTask
            .setName(getTestString(ValidationConstants.NAME_MIN -1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MIN_LENGTH, adminTask.getClass()
                .getName(), ValidationConstants.NAME_FIELD, adminTask.getID()),
        };

        verifySaveMustFail(adminTask, null, expectedExceptions);
    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        adminTask.setParent(parentPool);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, adminTask.getClass()
                .getName(), ValidationConstants.NAME_FIELD, adminTask.getID())
        };

        verifySaveMustFail(adminTask, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        // Set the mandatory fields
        adminTask
            .setName(getTestString(ValidationConstants.ADMIN_TASK_NAME_CHARACTER_LIMIT));

        adminTask.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, adminTask.getClass()
                .getName(), ValidationConstants.PARENT_FIELD, adminTask.getID())
        };

        verifySaveMustFail(adminTask, null, expectedExceptions);

    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        adminTask.setParent(parentPool);

        // Set field values with maxLength and verify that they are accepted
        adminTask
            .setName(getTestString(ValidationConstants.ADMIN_TASK_NAME_CHARACTER_LIMIT));

        // Attempt to save the AdministrativeTask object
        adminTask = (DefaultAdministrativeTask) saveMustSucceed(adminTask,
            DefaultAdministrativeTask.class, null);

        // Load the inserted adminTask from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DefaultAdministrativeTask[id='"
                + adminTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DefaultAdministrativeTask);

        // Check that the loaded AdministrativeTask is identical to the one we
        // requested to store
        DefaultAdministrativeTaskFixture.compare(adminTask, (DefaultAdministrativeTask) rpmObjects[0]);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        adminTask.setParent(parentPool);
        adminTask
            .setName(getTestString(ValidationConstants.ADMIN_TASK_NAME_CHARACTER_LIMIT));

         // Attempt to save the AdministrativeTask object
        adminTask = (DefaultAdministrativeTask) saveMustSucceed(adminTask,
            DefaultAdministrativeTask.class, null);

        // Load the inserted adminTask from xpath
        RPMObject[] rpmObjects = loadFromXpath("/DefaultAdministrativeTask[id='"
                + adminTask.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof DefaultAdministrativeTask);

        // Check that the loaded AdministrativeTask is identical to the one we
        // requested to store
        DefaultAdministrativeTaskFixture.compare(adminTask, (DefaultAdministrativeTask) rpmObjects[0]);
    }

}
