/*
 * Created on 18-Jan-07
 * @author mmatwiy
 * 
 * Defect 60352:
 * I try to assign a freshly created resource to a Project via a ResourceTaskAssignment. 
 * The following error message is being thrown:
 * ::ERROR:: Insufficient Work/Effort available within given period
 * 
 * Problem: Not allowed to set EETC on for a Project level assignment.
 * 
 * Resolution: Add a validation error message that disallows the user from setting the EETC
 * when the ResourceTaskAssignment's parent is of type GenericProject.
 * Test the case where the ResourceTaskAssignment's parent is null as well (forces
 * reload of parent in checkpoint).
 */
package com.ibm.rpm.client.junit.wbs.testcases.fixes;

import java.util.Calendar;
import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.RPMCalendar;
import com.ibm.rpm.applicationadministration.scope.CalendarScope;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.RPMCalendarFixture;
import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.helpers.HelperTestUtilities;
import com.ibm.rpm.client.junit.resource.fixtures.PoolFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.SaveResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.PoolScope;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class Test60352 extends GenericValidation
{

    WorkElementScope wes = null;

    Calendar resource1StartDate = null;

    Resource resource1 = null;

    public void setUp() throws Exception
    {
        super.setUp();

        resource1StartDate = new GregorianCalendar(2007, 1, 10);

        Pool pool = PoolFixture.createSaveFixture(this);
        RPMCalendar standardHoursRPMCalendar = RPMCalendarFixture
            .createSaveFixture(null, this);

        resource1 = createResource(pool, HelperTestUtilities.createName(),
            standardHoursRPMCalendar, new Boolean(true));

        wes = new WorkElementScope();
        wes.setResourceTaskAssignments(new ResourceTaskAssignmentScope());
        wes.getResourceTaskAssignments().setResource(new ResourceScope());
        wes.getResourceTaskAssignments().setTaskAssignment(
            new TaskAssignmentScope());
        wes.getResourceTaskAssignments().getResource().setCalendar(
            new CalendarScope());
        wes.setContainingProject(new WorkElementScope());
    }

    public void testAssignResourceToProjectInsert() throws Exception
    {
        ResourceTaskAssignment rta1 = createResourceTaskAssignment(resource1,
            getSecurityRole("Project Assignment"), resource1StartDate,
            new Double(2624.00));

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setContainingProject(new WorkElementScope());
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), workElementScope, this);

        project.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            rta1
        });

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_BE_NULL, rta1
                .getTaskAssignment().getClass().getName(),
                ValidationConstants.EETC_HOURS_FIELD, rta1.getTaskAssignment()
                    .getID())
        };

        verifySaveMustFail(project, wes, expectedExceptions);
    }

    public void testAssignResourceToProjectUpdate() throws Exception
    {
        // create with null EETCHours
        ResourceTaskAssignment rta1 = createResourceTaskAssignment(resource1,
            getSecurityRole("Project Assignment"), resource1StartDate, null);

        WorkElementScope workElementScope = new WorkElementScope();
        workElementScope.setContainingProject(new WorkElementScope());
        Project project = (Project) WorkElementFixture.createSaveFixture(
            new Project(), workElementScope, this);

        project.setResourceTaskAssignments(new ResourceTaskAssignment[]
        {
            rta1
        });

        // save in order to get an RTA with an ID so that we can reload it with a null parent
        save(project, wes);

        // save with an EETC value and a null RTA parent (forces reload in checkpoint)
        ResourceTaskAssignment reloaded = (ResourceTaskAssignment) loadFromID(
            rta1, wes.getResourceTaskAssignments());
        reloaded.getTaskAssignment().setEETCHours(new Double(40));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MUST_BE_NULL, rta1
                .getTaskAssignment().getClass().getName(),
                ValidationConstants.EETC_HOURS_FIELD, rta1.getTaskAssignment()
                    .getID())
        };

        verifySaveMustFail(reloaded, wes.getResourceTaskAssignments(),
            expectedExceptions);
    }

    private ResourceTaskAssignment createResourceTaskAssignment(
            Resource resource, SecurityRole secRole, Calendar startDate,
            Double EETC) throws Exception
    {
        ResourceTaskAssignment rta = new ResourceTaskAssignment();
        rta.setResource(resource);
        rta.setSecurityRole(secRole);
        rta.setTaskAssignment(new TaskAssignment());
        rta.getTaskAssignment().setAssignmentStartDate(startDate);

        // need a validation error - can't set EETC for a Project level assignment
        rta.getTaskAssignment().setEETCHours(EETC);

        return rta;
    }

    private SecurityRole getSecurityRole(String securityRoleName)
            throws Exception
    {
        String securityRoleXpath = "/SecurityRole[name='" + securityRoleName
                + "']";
        RPMObject[] lr = loadFromXpath(securityRoleXpath, null);
        SecurityRole secRole = (SecurityRole) lr[0];
        return secRole;
    }

    private Resource createResource(Pool parent, String name, RPMCalendar cal,
            Boolean active) throws Exception
    {
        Resource resource = new Resource();
        resource.setFullName(name);
        resource.setCalendar(cal);
        resource.setActive(active);
        resource.setParent(parent);
        ResourceScope resScope = new ResourceScope();
        resScope.setParent(new PoolScope());
        resScope.setCalendar(new CalendarScope());
        SaveResult sr = save(resource, resScope);
        assertTrue(sr.getRpmObject() instanceof Resource);
        resource = (Resource) sr.getRpmObject();
        return resource;
    }
}
