/*
 * Created on Nov 30, 2005 
 * @author mmatwiy 
 * Validation tests for container Deliverable.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import java.util.GregorianCalendar;

import com.ibm.rpm.applicationadministration.containers.State;
import com.ibm.rpm.client.junit.applicationadministration.fixtures.DatafieldFixture;
import com.ibm.rpm.client.junit.financial.fixtures.TimecodeFixture;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.scope.TimeCodeScope;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class TestDeliverableValidation extends AbstractTestWorkElementValidation
{
    private Deliverable deliverable;

    private Project parentProject;
    
    public void setUp() throws Exception
    {
        super.setUp();

        deliverable = new Deliverable();
        
        parentProject = (Project) WorkElementFixture.retrieveStaticFixture(
            new Project(), this);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Task task = (Task) WorkElementFixture.retrieveStaticFixture(new Task(),
            this);

        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(task);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, deliverable
                .getClass().getName(), ValidationConstants.PARENT, deliverable
                .getID())
        };

        verifySaveMustFail(deliverable, null, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        deliverable.setEstimatedStartDate(new GregorianCalendar(2005, 12, 11));
        deliverable.setEstimatedFinishDate(new GregorianCalendar(2006, 12, 12));

        // Attempt to save the object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (WorkElement) rpmObjects[0],
            null);
    }

    public void testFailureInvalidEstimatedFinishDate() throws AssertionError,
            Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set the estimatedFinishDate to be prior to the estimatedStartDate
        deliverable.setEstimatedStartDate(new GregorianCalendar(2005, 12, 12));
        deliverable.setEstimatedFinishDate(new GregorianCalendar(2005, 12, 11));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_DATES, deliverable.getClass()
                .getName(), ValidationConstants.ESTIMATED_START_DATE_FIELD,
                deliverable.getID())
        };

        verifySaveMustFail(deliverable, null, expectedExceptions);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);

        // Set string values with max length + 1 and verify that they are not
        // accepted
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT + 1));
        deliverable
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT + 1));
        deliverable
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT + 1));

        // Set integers to max value + 1 and verify that they are not accepted
        deliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.MAX_LENGTH, deliverable
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    deliverable.getID()),
                createRPMException(ErrorCodes.OUT_OF_RANGE, deliverable
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    deliverable.getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, deliverable
                    .getClass().getName(),
                    ValidationConstants.REFERENCE_NUMBER_FIELD, deliverable
                        .getID()),
                createRPMException(ErrorCodes.MAX_LENGTH, deliverable
                    .getClass().getName(),
                    ValidationConstants.EXTERNAL_ID_FIELD, deliverable.getID())

        };

        verifySaveMustFail(deliverable, null, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);

        // Set integers to min value -1 and verify that they are not accepted
        deliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MIN - 1));
        deliverable.setName("");

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.OUT_OF_RANGE, deliverable
                    .getClass().getName(), ValidationConstants.PRIORITY_FIELD,
                    deliverable.getID()),
                createRPMException(ErrorCodes.MIN_LENGTH, deliverable
                    .getClass().getName(), ValidationConstants.NAME_FIELD,
                    deliverable.getID())

        };

        verifySaveMustFail(deliverable, null, expectedExceptions);
    }

    public void testFailureReadOnlyScheduleDates() throws AssertionError,
            Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setScheduleDates(true);
        
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        WbsScheduleDate actualDate = new WbsScheduleDate();
        actualDate.setStartDate(new GregorianCalendar());
        actualDate.setDurationInMinutes(new Integer(4000));

        WbsScheduleDate forecastDate = new WbsScheduleDate();
        forecastDate.setDurationInMinutes(new Integer(5000));

        WbsScheduleDate planDate = new WbsScheduleDate();
        planDate.setStartDate(new GregorianCalendar());
        planDate.setDurationInMinutes(new Integer(3000));

        WbsScheduleDate proposedDate = new WbsScheduleDate();
        proposedDate.setStartDate(new GregorianCalendar());
        proposedDate.setDurationInMinutes(new Integer(3000));

        deliverable.setActualDate(actualDate);
        deliverable.setForecastDate(forecastDate);
        deliverable.setPlanDate(planDate);
        deliverable.setProposedDate(proposedDate);

        RPMExceptionMock[] expectedWarnings =
        {
                createRPMException(ErrorCodes.READ_ONLY_FIELD, deliverable
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_ACTUAL_FIELD, deliverable
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, deliverable
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_FORECAST_FIELD,
                    deliverable.getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, deliverable
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PLAN_FIELD, deliverable
                        .getID()),
                createRPMException(ErrorCodes.READ_ONLY_FIELD, deliverable
                    .getClass().getName(),
                    ValidationConstants.SCHEDULE_DATE_PROPOSED_FIELD,
                    deliverable.getID()),
        };

        verifySaveMustFail(deliverable, scope, null, expectedWarnings);
        
        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);
        
        //check that the values we set were not saved to the db
        Deliverable loadedDeliverable = (Deliverable) rpmObjects[0];

        // assert that the values we set were not saved in the db
        assertEquals(null, loadedDeliverable.getActualDate());
        assertEquals(null, loadedDeliverable.getForecastDate());
        assertEquals(null, loadedDeliverable.getPlanDate());
        assertNotSame(deliverable.getProposedDate(), loadedDeliverable
            .getProposedDate());

    }

    public void testInsertWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        deliverable.setParent(parentProject);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, deliverable.getClass()
                .getName(), ValidationConstants.NAME_FIELD, deliverable.getID())
        };

        verifySaveMustFail(deliverable, null, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, deliverable.getClass()
                .getName(), ValidationConstants.PARENT, deliverable.getID())
        };

        verifySaveMustFail(deliverable, null, expectedExceptions);

    }

    public void testInsertWithParentDeliverable() throws AssertionError,
            Exception
    {
        Deliverable parentDeliverable = (Deliverable) WorkElementFixture
            .retrieveStaticFixture(new Deliverable(), this);

        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(parentDeliverable);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {

        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);

        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(workOrganization);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testInsertWithParentWorkProduct() throws AssertionError,
            Exception
    {
        WorkProduct workProduct = (WorkProduct) WorkElementFixture
            .retrieveStaticFixture(new WorkProduct(), this);

        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(workProduct);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testInsertWithParentSummaryTask() throws AssertionError,
            Exception
    {

        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);

        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(summaryTask);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);

        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(parentProposal);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setParent(parentProject);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted deliverable from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testInsertWithParentProjectInPortfolio() throws AssertionError,
            Exception
    {
        Portfolio parentPortfolio = PortfolioFixture.createSaveFixture(this);

        // Set the mandatory fields
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        parentProject.setParent(parentPortfolio);
        deliverable.setParent(parentProject);

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);

        // Set field values with maxLength and verify that they are accepted
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        deliverable
            .setReferenceNumber(getTestString(ValidationConstants.PROJECT_ACTIVITY_REFERENCE_NUMBER_CHARACTER_LIMIT));
        deliverable
            .setExternalID(getTestString(ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT));

        // Set integers to max value and verify that they are accepted
        deliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MAX));

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        // Set integers to min value and verify that they are accepted
        deliverable.setPriority(new Integer(ValidationConstants.PRIORITY_MIN));

        // Attempt to save the Deliverable object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, null);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", null);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded Deliverable is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            null);
    }

    public void testFailureInvalidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);

        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setState(new State());

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.INVALID_TYPE, deliverable.getClass()
                .getName(), ValidationConstants.STATE_FIELD, deliverable
                .getID())
        };

        verifySaveMustFail(deliverable, scope, expectedExceptions);
    }

    public void testSuccessValidState() throws AssertionError, Exception
    {
        WorkElementScope scope = new WorkElementScope();
        scope.setState(true);
        
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));

        deliverable.setState(DatafieldFixture.createState(deliverable));

        // Attempt to save the object
        deliverable = (Deliverable) saveMustSucceed(deliverable,
            Deliverable.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/Deliverable[id='"
                + deliverable.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof Deliverable);

        // Check that the loaded object is identical to the one we
        // requested to store
        WorkElementFixture.compare(deliverable, (Deliverable) rpmObjects[0],
            scope);
    }

    public void testFailureInvalidTimeCodes() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        
        WorkElementScope scope = new WorkElementScope();
        scope.setTimeCodes(true);

        // assign the timecodes to the project
        TimeCodeScope timeCodeScope = TimecodeFixture.createScope(true, true);
        TimeCode timecode1 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, true, this);
        TimeCode timecode2 = TimecodeFixture.createSaveTimeCodeFixture(
            timeCodeScope, false, this);

        deliverable.setTimeCode1(timecode2);
        deliverable.setTimeCode2(timecode1);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, deliverable
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE1_FIELD, deliverable.getID()),
                createRPMException(
                    ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, deliverable
                        .getClass().getName(),
                    ValidationConstants.TIME_CODE2_FIELD, deliverable.getID()),
        };

        verifySaveMustFail(deliverable, scope, expectedExceptions);
    }
    
    public void testReadOnlyFields() throws AssertionError, Exception
    {
        // Set the mandatory fields
        deliverable.setParent(parentProject);
        deliverable
            .setName(getTestString(ValidationConstants.PROJECT_ACTIVITY_NAME_CHARACTER_LIMIT));
        
        super.testReadOnlyFields(deliverable);
    }
}
