/*
 * Created on Dec 2, 2005 
 * @author mmatwiy 
 * Validation tests for container ElementDependency.
 */
package com.ibm.rpm.client.junit.wbs.testcases.validation;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.DependencyFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.PortfolioFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Portfolio;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.scope.ElementDependencyScope;

public class TestElementDependencyValidation extends GenericValidation
{
    private ElementDependency elementDependency;

    private ElementDependencyScope scope;

    public void setUp() throws Exception
    {
        super.setUp();

        elementDependency = new ElementDependency();
        scope = DependencyFixture.createScope();
        
        // Set mandatory field predecessor
        // Note that we can't use the static fixture for both the parent and
        // the predecessor because we can't add the same dependency more than once
        Proposal predecessor = (Proposal) WorkElementFixture.createSaveFixture(
            new Proposal(), null, this);
        elementDependency.setPredecessor(predecessor);
    }

    public void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testInsertWithInvalidParent() throws AssertionError, Exception
    {
        // create and set invalid parent
        Portfolio portfolio = PortfolioFixture.createSaveFixture(this);

        elementDependency.setParent(portfolio);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.WRONG_PARENT_TYPE, elementDependency
                .getClass().getName(), ValidationConstants.PARENT,
                elementDependency.getID())
        };

        verifySaveMustFail(elementDependency, scope, expectedExceptions);
    }

    public void testInsertSuccess() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        elementDependency.setParent(parentProject);
        elementDependency.setAmount(new Integer((int) (Math.random() * 1000 - 1)));

        // Attempt to save the object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded object is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testFailureValidationMax() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        elementDependency.setParent(parentProject);

        // Set doubles to max value + 1 and verify that they are not accepted
        elementDependency.setAmount(new Integer(
            ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_MAX + 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, elementDependency
                .getClass().getName(),
                ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_FIELD,
                elementDependency.getID())
        };

        verifySaveMustFail(elementDependency, scope, expectedExceptions);
    }

    public void testFailureValidationMin() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the parent
        elementDependency.setParent(parentProject);

        // Set doubles to min value -1 and verify that they are not accepted
        elementDependency.setAmount(new Integer(
            ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_MIN - 1));

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.OUT_OF_RANGE, elementDependency
                .getClass().getName(),
                ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_FIELD,
                elementDependency.getID()),
        };

        verifySaveMustFail(elementDependency, scope, expectedExceptions);
    }

    public void testInsertWithoutParent() throws AssertionError, Exception
    {
        elementDependency.setParent(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_PARENT, elementDependency
                .getClass().getName(), ValidationConstants.PARENT,
                elementDependency.getID())
        };

        verifySaveMustFail(elementDependency, scope, expectedExceptions);

    }

    public void testFailureWithoutMandatoryFields() throws AssertionError,
            Exception
    {
        Deliverable parentDeliverable = (Deliverable) WorkElementFixture
            .retrieveStaticFixture(new Deliverable(), this);
        
        elementDependency.setParent(parentDeliverable);
        elementDependency.setPredecessor(null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.MANDATORY, elementDependency
                .getClass().getName(),
                ValidationConstants.ELEMENT_DEPENDENCY_PREDECESSOR_FIELD,
                elementDependency.getID())
        };

        verifySaveMustFail(elementDependency, scope, expectedExceptions);
    }

    public void testInsertWithParentDeliverable() throws AssertionError,
            Exception
    {
        Deliverable parentDeliverable = (Deliverable) WorkElementFixture
            .retrieveStaticFixture(new Deliverable(), this);
        elementDependency.setParent(parentDeliverable);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentWorkProduct() throws AssertionError,
            Exception
    {
        WorkProduct parentWorkProduct = (WorkProduct) WorkElementFixture
            .retrieveStaticFixture(new WorkProduct(), this);
        elementDependency.setParent(parentWorkProduct);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentWorkOrganization() throws AssertionError,
            Exception
    {
        WorkOrganization workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);
        elementDependency.setParent(workOrganization);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentSummaryTask() throws AssertionError,
            Exception
    {
        SummaryTask summaryTask = (SummaryTask) WorkElementFixture
            .retrieveStaticFixture(new SummaryTask(), this);
        elementDependency.setParent(summaryTask);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentProposal() throws AssertionError, Exception
    {
        Proposal parentProposal = (Proposal) WorkElementFixture
            .retrieveStaticFixture(new Proposal(), this);
        elementDependency.setParent(parentProposal);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentMilestone() throws AssertionError,
            Exception
    {
        Milestone milestone = (Milestone) WorkElementFixture
            .retrieveStaticFixture(new Milestone(), this);
        elementDependency.setParent(milestone);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentTask() throws AssertionError, Exception
    {
        Task parentTask = (Task) WorkElementFixture.retrieveStaticFixture(
            new Task(), this);
        elementDependency.setParent(parentTask);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testInsertWithParentProject() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);
        elementDependency.setParent(parentProject);

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testSuccessValidationMax() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        elementDependency.setParent(parentProject);

        // Set doubles to max value and verify that they are accepted
        elementDependency.setAmount(new Integer(
            ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_MAX));

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testSuccessValidationMin() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        elementDependency.setParent(parentProject);

        // Set doubles to min value and verify that they are accepted
        elementDependency.setAmount(new Integer(
            ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_MIN));

        // Attempt to save the ElementDependency object
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testDefaultValuesInsert() throws AssertionError, Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        elementDependency.setParent(parentProject);

        // Leave default values as null and save
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the default values are now set
        ElementDependency savedElementDependency = (ElementDependency) rpmObjects[0];
        assertNotNull(savedElementDependency.getAmount());
        assertNotNull(savedElementDependency.getDependencyType());
        assertNotNull(savedElementDependency.getLeadlagType());
        assertNotNull(savedElementDependency.getUnitType());

        // Save again to validate the default values
        savedElementDependency = (ElementDependency) saveMustSucceed(
            savedElementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        rpmObjects = loadFromXpath("/ElementDependency[id='"
                + savedElementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the loaded ElementDependency is identical to the one we
        // requested to store
        DependencyFixture.compare(elementDependency,
            (ElementDependency) rpmObjects[0]);
    }

    public void testDefaultValuesUpdateToNull() throws AssertionError,
            Exception
    {
        Project parentProject = (Project) WorkElementFixture
            .retrieveStaticFixture(new Project(), this);

        // Set the mandatory fields
        elementDependency.setParent(parentProject);

        // Leave default values as null and save
        elementDependency = (ElementDependency) saveMustSucceed(
            elementDependency, ElementDependency.class, scope);

        // Load the inserted object from xpath
        RPMObject[] rpmObjects = loadFromXpath("/ElementDependency[id='"
                + elementDependency.getID() + "']", scope);
        assertNotNull(rpmObjects);
        assertTrue(rpmObjects[0] instanceof ElementDependency);

        // Check that the default values are now set
        ElementDependency savedElementDependency = (ElementDependency) rpmObjects[0];
        assertNotNull(savedElementDependency.getAmount());
        assertNotNull(savedElementDependency.getDependencyType());
        assertNotNull(savedElementDependency.getLeadlagType());
        assertNotNull(savedElementDependency.getUnitType());

        // Update the default values to null
        savedElementDependency.setAmount(null);
        savedElementDependency.setDependencyType(null);
        savedElementDependency.setLeadlagType(null);
        savedElementDependency.setUnitType(null);

        RPMExceptionMock[] expectedExceptions =
        {
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedElementDependency.getClass().getName(),
                    ValidationConstants.ELEMENT_DEPENDENCY_AMOUNT_FIELD,
                    savedElementDependency.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedElementDependency.getClass().getName(),
                    ValidationConstants.ELEMENT_DEPENDENCY_TYPE_FIELD,
                    savedElementDependency.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedElementDependency.getClass().getName(),
                    ValidationConstants.ELEMENT_DEPENDENCY_LEAD_LAG_FIELD,
                    savedElementDependency.getID()),
                createRPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                    savedElementDependency.getClass().getName(),
                    ValidationConstants.ELEMENT_DEPENDENCY_UNIT_TYPE_FIELD,
                    savedElementDependency.getID()),
        };

        verifySaveMustFail(savedElementDependency, scope, expectedExceptions);

    }
}
