/*
 * Created on 4-Oct-2006
 * @author mmatwiy
 * 
 * WorkflowProcess control and xpath test cases for an AbstractScope.
 * This class tests these xpaths, with conditions:
 * - /Issue/AvailableWorkflowProcess
 * - /Issue[AvailableWorkflowProcess]
 * - /Issue/RunningWorkflowProcess
 * - /Issue[RunningWorkflowProcess]
 */
package com.ibm.rpm.client.junit.workflow.testcases;

import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.communications.types.WorkflowState;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.workflow.containers.AvailableWorkflowProcess;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.AvailableWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestWorkflowProcessScopeMgmt extends AbstractTestWorkflowProcess
{
    private WorkflowScope workflowScope = null;

    private RunningWorkflowProcessScope runningScope = null;
    
    private AvailableWorkflowProcessScope availableScope = null;
    
    private ScopeElementScope scopeElementScope = null;

    private WorkflowProcess process = null;

    private WorkOrganization workOrganization = null;

    private Issue issue = null;

    private static final String PROCESS_NAME = "IssueProcess1";

    public void setUp() throws Exception
    {
        logger.info("setUp");
        super.setUp();

        workflowScope = new WorkflowScope();
        workflowScope.setParent(new WorkflowScope());
        workflowScope.setChildren(new WorkflowScope());
        
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
                
        availableScope = new AvailableWorkflowProcessScope();
        availableScope.setChildren(workflowScope);

        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setWorkflowProcessesAvailable(availableScope);
        scopeElementScope.setWorkflowProcessesRunning(runningScope);

        workOrganization = (WorkOrganization) WorkElementFixture
            .createSaveFixture(new WorkOrganization(), null, this);
        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + workOrganization.getName() + "']", new ScopeElementScope());

        ScopeFolder scopeFolder = (ScopeFolder) rpmObjects[0];
        issue = ScopeElementFixture.createIssueFixture(null, 0, scopeFolder,
            this);
        save(issue, null);

        //TODO: replace this load once the API is able to create a WorkflowProcess
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        process = (WorkflowProcess) objects[0];
    }

    public void tearDown() throws Exception
    {
        // reload to get the running processes
        RPMObject[] objects = loadFromXpath("/Issue[id='" + issue.getID()
                + "']", scopeElementScope);
        assertNotNull(objects);
        issue = (Issue) objects[0];

        if (issue.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < issue.getWorkflowProcessesRunning().length; i++)
            {
                cancelProcess(issue.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    public void testWorkflowAssociation() throws Exception
    {
        associateProcess(process, workOrganization, false);

        // once the process has been associated to the workOrganization, 
        // it should be in the list of available processes for the Issue
        RPMObject[] objects = loadFromXpath("/Issue[id='" + issue.getID()
                + "']", scopeElementScope);
        assertNotNull(objects);
        issue = (Issue) objects[0];

        assertNotNull(issue.getWorkflowProcessesAvailable());
        assertNotNull(issue.getWorkflowProcessesAvailable()[0]);
        assertNotNull(issue.getWorkflowProcessesAvailable()[0]
            .getWorkflowProcess());
        assertFalse(issue.getWorkflowProcessesAvailable()[0].getDefaultFlag()
            .booleanValue());

        unAssociateProcess(process, workOrganization);

        // when we unassociate it, it should be gone from the list of
        // available processes
        objects = loadFromXpath("/Issue[id='" + issue.getID() + "']",
            scopeElementScope);
        assertNotNull(objects);
        issue = (Issue) objects[0];

        assertNull(workOrganization.getWorkflowProcessesAvailable());
    }

    public void testWorkflowControl() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, workOrganization, false);

        // start the process  
        RunningWorkflowProcess runningProcess = startProcess(process, issue,
            runningScope);
        assertNotNull(runningProcess);
        assertTrue(runningProcess.getState() == WorkflowState.Open);

        // suspend the process
        RunningWorkflowProcess suspendedProcess = suspendProcess(
            runningProcess, runningScope);
        assertTrue(suspendedProcess.getState() == WorkflowState.Suspended);

        // resume the process
        RunningWorkflowProcess resumedProcess = resumeProcess(suspendedProcess,
            runningScope);
        assertTrue(resumedProcess.getState() == WorkflowState.Open);

        // cancel the process
        cancelProcess(resumedProcess);

        // reload to check that the process has been cancelled
        RPMObject[] objects = loadFromXpath("/Issue[id='" + issue.getID()
                + "']", scopeElementScope);
        assertNotNull(objects);
        issue = (Issue) objects[0];

        assertNull(issue.getWorkflowProcessesRunning());

    }

    public void testLoadIssueRunningProcesses() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, workOrganization, false);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, issue,
            runningScope);

        RPMObject[] objects = loadFromXpath("/Issue[RunningWorkflowProcess]",
            scopeElementScope);
        assertNotNull(objects);
        
        for (int i = 0; i < objects.length; i++)
        {
            if (objects[i].getID().equals(issue.getID()))
            {
                Issue reloaded = (Issue) objects[i];
                assertNotNull(reloaded.getWorkflowProcessesRunning());
                
                findAndCompareRunningProcess(runningProcess, reloaded
                    .getWorkflowProcessesRunning(), runningScope);
                
                break;
            }
        }
    }

    public void testLoadIssueAvailableProcesses() throws Exception
    {
        // associate the process so that it shows up in the list of 
        // available processes
        associateProcess(process, workOrganization, false);

        RPMObject[] objects = loadFromXpath("/Issue[AvailableWorkflowProcess]",
            null);
        assertNotNull(objects);
        checkMatchingIssueId(objects);
    }

    public void testLoadAvailableProcessFromIssue() throws Exception
    {
        // associate the process so that it shows up in the list of 
        // available processes
        associateProcess(process, workOrganization, false);

        RPMObject[] objects = loadFromXpath("/Issue[id='" + issue.getID()
                + "']/AvailableWorkflowProcess", availableScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof AvailableWorkflowProcess);

        findAndCompareAvailableProcess(process, objects, workflowScope);
    }

    public void testLoadRunningProcessFromIssue() throws Exception
    {
        // associate the process so that we can start it
        associateProcess(process, workOrganization, false);

        // start the process
        RunningWorkflowProcess runningProcess = startProcess(process, issue,
            runningScope);

        RPMObject[] objects = loadFromXpath("/Issue[id='" + issue.getID()
                + "']/RunningWorkflowProcess", runningScope);
        assertNotNull(objects);

        findAndCompareRunningProcess(runningProcess, objects, runningScope);
    }

    private void checkMatchingIssueId(RPMObject[] objects)
    {
        boolean found = false;
        for (int i = 0; i < objects.length; i++)
        {
            assertTrue(objects[i] instanceof Issue);
            if (issue.getID().equals(objects[i].getID()))
            {
                found = true;
            }
        }
        assertTrue("Issue with correct ID not loaded", found);
    }

}