/*
 * Created on 2-Oct-2006
 * @author mmatwiy
 * Validation test methods for cancelling a workflow process.
 */
package com.ibm.rpm.client.junit.workflow.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.scopemanagement.fixtures.ScopeElementFixture;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestCancelWorkflowProcessValidation extends GenericValidation
{
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;

    private ScopeElementScope scopeElementScope = null;

    private Issue issue = null;

    private WorkOrganization workOrganization = null;

    public final static String ISSUE_PROCESS_NAME = "IssueProcess1";

    
    public void setUp() throws Exception
    {
        super.setUp();

        workflowScope = new WorkflowScope();
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
        
        scopeElementScope = new ScopeElementScope();
        scopeElementScope.setWorkflowProcessesRunning(runningScope);

        workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);
        RPMObject[] rpmObjects = loadFromXpath("/ScopeFolder[name='"
                + workOrganization.getName() + "']", new ScopeElementScope());

        ScopeFolder scopeFolder = (ScopeFolder) rpmObjects[0];
        issue = ScopeElementFixture.createIssueFixture(null, 0, scopeFolder,
            this);
        save(issue, null);
    }

    public void tearDown() throws Exception
    {
        // reload to get the running processes
        RPMObject[] objects = loadFromXpath("/Issue[id='" + issue.getID()
                + "']", scopeElementScope);
        assertNotNull(objects);
        issue = (Issue) objects[0];

        if (issue.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < issue.getWorkflowProcessesRunning().length; i++)
            {
                getApplicationInterface().cancelWorkflow(sessionid,
                    issue.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    private RunningWorkflowProcess loadRunningProcess() throws Exception
    {
        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // then start and suspend the process
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + ISSUE_PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkflowProcess);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        Result associateResult = getApplicationInterface().associateWorkflow(
            sessionid, process, workOrganization, true, false);
        assertTrue(associateResult.isSuccessful());

        SingleResult startResult = getApplicationInterface().startWorkflow(
            getSessionID(), process, issue, runningScope);
        assertTrue(startResult.isSuccessful());

        RunningWorkflowProcess runningProcess = (RunningWorkflowProcess) startResult
            .getRpmObject();
        return runningProcess;
    }

    public void testFailureCancelNullRunningProcess() throws Exception
    {
        // Cancel a null runningProcess object
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            null);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, "RunningWorkflowProcess")
        };

        verifyProcessFailure(null, expectedExceptions, null, result);
    }

    public void testFailureCancelUnsavedRunningProcess() throws Exception
    {
        RunningWorkflowProcess runningProcess = new RunningWorkflowProcess();

        // Cancel an unsaved runningProcess object
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            runningProcess);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, runningProcess.getClass()
                .getName())
        };

        verifyProcessFailure(runningProcess, expectedExceptions, null, result);
    }

    public void testFailureInvalidState() throws Exception
    {
        RunningWorkflowProcess runningProcess = loadRunningProcess();

        Result cancelResult = getApplicationInterface().cancelWorkflow(
            sessionid, runningProcess);
        assertTrue(cancelResult.isSuccessful());

        // Attempt to cancel a runningProcess that's already cancelled
        Result result = getApplicationInterface().cancelWorkflow(sessionid,
            runningProcess);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.INVALID_STATE_TRANSITION,
                runningProcess.getClass().getName())
        };

        verifyProcessFailure(runningProcess, expectedExceptions, null, result);
    }

    private void verifyProcessFailure(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result result)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, result);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}