/*
 * Created on 2-Oct-2006
 * @author mmatwiy
 * Validation test methods for resuming a workflow process.
 */
package com.ibm.rpm.client.junit.workflow.testcases.validation;

import junit.framework.Assert;

import com.ibm.rpm.client.junit.framework.testcases.GenericValidation;
import com.ibm.rpm.client.junit.framework.testcases.RPMExceptionMock;
import com.ibm.rpm.client.junit.wbs.fixtures.WorkElementFixture;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.Result;
import com.ibm.rpm.framework.SingleResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.workflow.scope.WorkflowScope;

public class TestResumeWorkflowProcessValidation extends GenericValidation
{
    private WorkflowScope workflowScope = null;
    
    private RunningWorkflowProcessScope runningScope = null;

    private WorkElementScope workElementScope = null;

    private Task task = null;

    private WorkOrganization workOrganization = null;

    public final static String TASK_PROCESS_NAME = "TaskProcess1";

    public void setUp() throws Exception
    {
        super.setUp();

        workflowScope = new WorkflowScope();
        runningScope = new RunningWorkflowProcessScope();
        runningScope.setChildren(workflowScope);
        
        workElementScope = new WorkElementScope();
        workElementScope.setWorkflowProcessesRunning(runningScope);

        workOrganization = (WorkOrganization) WorkElementFixture
            .retrieveStaticFixture(new WorkOrganization(), this);
        task = (Task) WorkElementFixture.createSaveFixture(workOrganization,
            new Task(), null, this);
    }

    public void tearDown() throws Exception
    {
        // reload the Task to get the running workflows
        RPMObject[] objects = loadFromXpath("/Task[id='" + task.getID() + "']",
            workElementScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof Task);
        task = (Task) objects[0];

        if (task.getWorkflowProcessesRunning() != null)
        {
            // cancel the running processes so that everything can be deleted
            for (int i = 0; i < task.getWorkflowProcessesRunning().length; i++)
            {
                getApplicationInterface().cancelWorkflow(sessionid,
                    task.getWorkflowProcessesRunning()[i]);
            }
        }

        super.tearDown();
    }

    private RunningWorkflowProcess loadRunningProcess() throws Exception
    {
        // TODO: replace this with a fixture that creates a WorkflowProcess
        // once it's possible to create a WorkflowProcess using the API,
        // then start and suspend the process
        RPMObject[] objects = loadFromXpath("/WorkflowProcess[name='"
                + TASK_PROCESS_NAME + "']", workflowScope);
        assertNotNull(objects);
        assertTrue(objects[0] instanceof WorkflowProcess);
        WorkflowProcess process = (WorkflowProcess) objects[0];

        Result associateResult = getApplicationInterface().associateWorkflow(
            sessionid, process, workOrganization, true, false);
        assertTrue(associateResult.isSuccessful());

        SingleResult startResult = getApplicationInterface().startWorkflow(
            getSessionID(), process, task, runningScope);
        assertTrue(startResult.isSuccessful());

        RunningWorkflowProcess runningProcess = (RunningWorkflowProcess) startResult
            .getRpmObject();

        SingleResult suspendResult = getApplicationInterface().suspendWorkflow(
            getSessionID(), runningProcess, runningScope);
        assertTrue(suspendResult.isSuccessful());

        return (RunningWorkflowProcess) suspendResult.getRpmObject();
    }

    public void testFailureResumeNullRunningProcess() throws Exception
    {
        // Resume a null runningProcess object
        Result result = getApplicationInterface().resumeWorkflow(sessionid,
            null, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, "RunningWorkflowProcess")
        };

        verifyProcessFailure(null, expectedExceptions, null, result);
    }

    public void testFailureResumeUnsavedRunningProcess() throws Exception
    {
        RunningWorkflowProcess runningProcess = new RunningWorkflowProcess();

        // Resume an unsaved runningProcess object
        Result result = getApplicationInterface().resumeWorkflow(sessionid,
            runningProcess, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(ErrorCodes.NO_ID, runningProcess.getClass()
                .getName())
        };

        verifyProcessFailure(runningProcess, expectedExceptions, null, result);
    }

    public void testFailureInvalidState() throws Exception
    {
        RunningWorkflowProcess runningProcess = loadRunningProcess();

        Result cancelResult = getApplicationInterface().cancelWorkflow(
            sessionid, runningProcess);
        assertTrue(cancelResult.isSuccessful());

        // Attempt to resume a cancelled runningProcess
        Result result = getApplicationInterface().resumeWorkflow(sessionid,
            runningProcess, runningScope);

        RPMExceptionMock[] expectedExceptions =
        {
            createRPMException(
                com.ibm.rpm.workflow.constants.ErrorCodes.INVALID_STATE_TRANSITION,
                runningProcess.getClass().getName())
        };

        verifyProcessFailure(runningProcess, expectedExceptions, null, result);
    }

    private void verifyProcessFailure(RPMObject obj,
            RPMExceptionMock[] expectedExceptions,
            RPMExceptionMock[] expectedWarnings, Result result)
            throws AssertionError, Exception
    {
        // Check that we received exceptions if we were expecting some
        // and that they are the correct ones
        String errorMessage = checkAndLogExceptions(obj, expectedExceptions,
            expectedWarnings, result);

        if (errorMessage.length() != 0)
            Assert.fail(errorMessage);
    }
}