package com.ibm.rpm.customfield.checkpoints;

import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

import com.ibm.rpm.applicationadministration.checkpoints.DatafieldCheckpoint;
import com.ibm.rpm.applicationadministration.containers.CustomDatafield;
import com.ibm.rpm.applicationadministration.containers.CustomDatafieldCategory;
import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.asset.containers.Asset;
import com.ibm.rpm.clientcostcenters.containers.ClientCostCenter;
import com.ibm.rpm.customfield.constants.ErrorCodes;
import com.ibm.rpm.customfield.constants.ValidationConstants;
import com.ibm.rpm.customfield.containers.CustomField;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.customfield.managers.CustomFieldAssignmentUtil;
import com.ibm.rpm.customfield.managers.CustomFieldManager;
import com.ibm.rpm.customfield.types.FieldInputType;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.wbs.containers.WorkElement;

public class CustomFieldAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final String VALUE_FIELD_NAME = "value";
    
    private static final String CUSTOMFIELD_FIELD_NAME = "customField";

    private static final List CUSTOM_FIELD_ASSIGNMENT_PARENTS = new LinkedList();

    static
    {
        CUSTOM_FIELD_ASSIGNMENT_PARENTS.add(WorkElement.class);
        CUSTOM_FIELD_ASSIGNMENT_PARENTS.add(ScopeElement.class);
        CUSTOM_FIELD_ASSIGNMENT_PARENTS.add(ClientCostCenter.class);
        CUSTOM_FIELD_ASSIGNMENT_PARENTS.add(Asset.class);
        CUSTOM_FIELD_ASSIGNMENT_PARENTS.add(Resource.class);
        CUSTOM_FIELD_ASSIGNMENT_PARENTS.add(GenericDocument.class);
    }

    protected static CustomFieldAssignmentCheckpoint instance = new CustomFieldAssignmentCheckpoint();

    public static CustomFieldAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        super.isValidParent(rpmObject, msgCtx, true, true,
            CUSTOM_FIELD_ASSIGNMENT_PARENTS);

        GenericValidator.validateInstance(rpmObject,
            CustomFieldAssignment.class, msgCtx);

        CustomFieldAssignment assignment = (CustomFieldAssignment) rpmObject;

        GenericValidator.validateMandatory(assignment, "customField",
            assignment.getCustomField(), msgCtx);
        
        if (assignment.getCustomField() != null)
        {
            GenericValidator.validateMandatory(assignment,
                "assignment.getCustomField().getID()", assignment.getCustomField().getID(), msgCtx);
            
            if(assignment.getCustomField().getID() != null && assignment.getParent() != null)
            {
                validateCustomFieldGroup(assignment, msgCtx);
            }
        }

        areValidTypeAndValue(assignment, msgCtx);
    }

    private void areValidTypeAndValue(CustomFieldAssignment assignment,
            MessageContext msgCtx)
    {
        if (assignment.testValueModified())
        {
            FieldInputType inputType = assignment.getCustomField()
                .getInputType();
            Object defaultValue = assignment.getValue();
            if (FieldInputType.Checkbox.equals(inputType))
            {
                isValidValueType(assignment, Boolean.class, msgCtx);
            }
            else if (FieldInputType.Currency.equals(inputType))
            {
                if (isValidValueType(assignment, Double.class, msgCtx))
                {
                    GenericValidator.validateRange(assignment, 
                        ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_FIELD,
                        (Double) assignment.getValue(), 
                        ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_CURRENCY_MIN,
                        ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_CURRENCY_MAX, 
                        msgCtx);
                }
            }
            else if (FieldInputType.Date.equals(inputType))
            {
                isValidValueType(assignment, Calendar.class, msgCtx);
            }
            else if (FieldInputType.Numeric.equals(inputType))
            {
                if (isValidValueType(assignment, Integer.class, msgCtx))
                {
                    GenericValidator.validateRange(assignment, 
                        ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_FIELD,
                        (Integer) assignment.getValue(), 
                        ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_INTEGER_MIN,
                        ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_INTEGER_MAX, 
                        msgCtx);
                }
            }
            else if (FieldInputType.Text.equals(inputType))
            {
                if (isValidValueType(assignment, String.class, msgCtx))
                {
	                GenericValidator.validateMaxLength(assignment,
	                    VALUE_FIELD_NAME, (String) defaultValue,
	                    ValidationConstants.CUSTOMFIELDASSIGNMENT_VALUE_TEXT_MAX, msgCtx);
	            }
            }
            else if (FieldInputType.Datafield.equals(inputType))
            {
                isValidDatafield(assignment, msgCtx);
            }
        }
    }

    private boolean isValidValueType(CustomFieldAssignment assignment,
            Class expectedType, MessageContext msgCtx)
    {
        Object defaultValue = assignment.getValue();
        if (defaultValue != null)
        {
           return GenericValidator.validateType(assignment, expectedType,
                VALUE_FIELD_NAME, defaultValue, msgCtx);
        }
        return true;
    }

    private void isValidDatafield(CustomFieldAssignment assignment,
            MessageContext msgCtx)
    {
        DatafieldCategory category = assignment.getCustomField()
            .getDatafieldCategory();
        Datafield datafield = (Datafield) assignment.getValue();

        if (category == null)
        {
            if (datafield instanceof CustomDatafield)
            {
                RPMException e = new RPMException(
                    ErrorCodes.NULL_CATEGORY_WITH_NON_NULL_VALUE, null,
                    assignment.getClass().getName(), VALUE_FIELD_NAME,
                    assignment.getID());
                addException(e, msgCtx);
            }
        }
        else if (datafield != null)
        {
            // Validate that the default value class matches the datafield
            // category class.
            if (category instanceof CustomDatafieldCategory)
            {
                GenericValidator.validateExactType(assignment,
                    CustomDatafield.class, VALUE_FIELD_NAME, datafield, msgCtx);
            }
            else
            {
                GenericValidator.validateType(assignment, Datafield.class,
                    VALUE_FIELD_NAME, datafield, msgCtx);
            }
            // Validate that the default value is in the specified datafield
            // category (valid domain).
            boolean isInCategory = false;
            try
            {
                isInCategory = DatafieldCheckpoint.isDatafieldInCategory(datafield, category,
                    msgCtx);
            }
            catch (Exception x)
            {
                RPMException e = new RPMException(x, CustomField.class
                    .getName());
                addException(e, msgCtx);
            }

            if (!isInCategory)
            {
                String[] arguments =
                {
                        datafield.getName(), category.getName()
                };
                RPMException e = new RPMException(
                    ErrorCodes.DATAFIELD_NOT_IN_CATEGORY, arguments, assignment
                        .getClass().getName(), VALUE_FIELD_NAME, assignment
                        .getID());
                addException(e, msgCtx);
            }
        }
    }
  
    private void validateCustomFieldGroup(CustomFieldAssignment assignment,
            MessageContext msgCtx)
    {

        String parentContext = CustomFieldAssignmentUtil
            .getContextByParent(assignment.getParent());
        
        if(parentContext != null){
	        
	        Integer parentType = CustomFieldAssignmentUtil
	            .getTableType(parentContext);
	        
	        SqlBuffer sql = new SqlBuffer();
	        sql.appendEqualQuestionMark(CustomFieldManager.NAME_ELEMENT_ID);
	
            Object[] arguments = new Object[]
            {
                assignment.getCustomField().getID()
            };
            
	        Integer tableType = null;
	        try
	        {
	            tableType = (Integer) ManagerUtil.selectColumnValue(Integer.class, null,
	                null,
	                CustomFieldManager.NAME_TABLE_TYPE, CustomFieldManager.TABLE_NAME, sql, arguments, msgCtx);
	        }
	        catch (RPMException e1)
	        {
	            e1.printStackTrace();
	        }
	        if (!parentType.equals(tableType))
	        {
	
	            String[] errorParams =
	            {
	                parentContext
	            };
	
	            RPMException e = new RPMException(ErrorCodes.INVALID_CUSTOM_FIELD,
	                errorParams, assignment.getClass().getName(),
	                CUSTOMFIELD_FIELD_NAME, assignment.getID());
	            addException(e, msgCtx);
	        }
        }
    }
}
