package com.ibm.rpm.customfield.types;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.Datafield;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.DateUtil;

public class FieldInputTypeToDBMap
{
    private static Map fieldInputTypeToDBMap;

    private static Map dbToFieldInputTypeMap;

    static
    {
        fieldInputTypeToDBMap = new HashMap();
        dbToFieldInputTypeMap = new HashMap();

        addToMap(FieldInputType.Checkbox, 5);
        addToMap(FieldInputType.Currency, 1);
        addToMap(FieldInputType.Datafield, 4);
        addToMap(FieldInputType.Date, 2);
        addToMap(FieldInputType.Numeric, 3);
        addToMap(FieldInputType.Text, 0);
    }

    private static void addToMap(FieldInputType fit, int dbCode)
    {
        Integer codeObject = new Integer(dbCode);
        fieldInputTypeToDBMap.put(fit, codeObject);
        dbToFieldInputTypeMap.put(codeObject, fit);
    }

    /**
     * Returns the TMT_CHARGECODE.CAPITAL_EXPENSE value that matches the
     * specified fc FinancialCategory.
     * 
     * @param fit
     *            Financial category to convert.
     * @return Matching DB code.
     * @throws RPMException
     */
    public static Integer toDBCode(FieldInputType fit) throws RPMException
    {
        Integer value = (Integer) fieldInputTypeToDBMap.get(fit);
        if (value == null)
        {
            String[] errorParams =
            {
                "Unknown FieldInputType " + fit.toString()
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, errorParams);
        }
        return value;
    }

    /**
     * Returns the FinancialCategory value that matches the specified database
     * code, as found in TMT_CHARGECODE.CAPITAL_EXPENSE.
     * 
     * @param dbCode
     *            Database code to convert.
     * @return Matching FinancialCategory.
     * @throws RPMException
     */
    public static FieldInputType toFieldInputType(int dbCode)
            throws RPMException
    {
        Integer key = new Integer(dbCode);
        FieldInputType value = (FieldInputType) dbToFieldInputTypeMap.get(key);
        if (value == null)
        {
            String[] errorParams =
            {
                key.toString()
            };
            
            RPMException rpmEx = new RPMException(com.ibm.rpm.customfield.constants.ErrorCodes.INVALID_CUSTOM_FIELD_TYPE_IN_DATABASE,
                errorParams);
            rpmEx.setSeverity(SeverityLevel.Error);
            
            throw rpmEx;
        }
        return value;
    }

    /**
     * Converts DB string value into corresponding field type object
     * 
     * @param objectBeingRead the object currently being read. Used for error message generation
     * @param fieldName name of the field being read from the DB
     * @param inputType
     *            CustomField.getInputType()
     * @param dbValue
     * @param messageContext
     *            required because of Datafield type. You can pass null and get
     *            null for all Datafield types.
     * @return Object corresponding to each InputType or null if database value is null
     * @throws ParseException
     * @throws RPMException
     * @throws SQLException
     */
    public static Object dbStringToObject(RPMObject objectBeingRead,
        String fieldName, FieldInputType inputType, String dbValue,
        MessageContext messageContext) throws ParseException, RPMException,
        SQLException
    {
        Object value = null;
        try
        {
            if (dbValue != null)
            {
                if (inputType.equals(FieldInputType.Checkbox))
                {
                    value = new Boolean("Y".equalsIgnoreCase(dbValue));
                }
                else if (inputType.equals(FieldInputType.Currency))
                {
                    value = new Double(dbValue);
                }
                else if (inputType.equals(FieldInputType.Datafield))
                {
                    value = loadDatafield(dbValue, messageContext);
                }
                else if (inputType.equals(FieldInputType.Date))
                {
                    value = DateUtil.convertToCalendarNoTime(dbValue);
                }
                else if (inputType.equals(FieldInputType.Numeric))
                {
                    value = new Integer(dbValue);
                }
                else if (inputType.equals(FieldInputType.Text))
                {
                    value = dbValue;
                }
                else
                {
                    String[] errorParams =
                    {
                        "Unsupported FieldInputType: " + inputType
                    };
                    RPMException rpmEx = new RPMException(
                        ErrorCodes.INTERNAL_ERROR,
                        errorParams, objectBeingRead.getClass().getName(),
                        fieldName, objectBeingRead.getID());
                    rpmEx.setSeverity(SeverityLevel.Warning);
                    messageContext.addExceptionNoThrow(rpmEx);
                }
            }
        }
        catch (Exception e)
        {
            String[] errorParams =
            {
                fieldName,
                inputType.getValue(),
                dbValue,
                e.toString()
            };
            
            RPMException rpmEx = new RPMException(
                com.ibm.rpm.customfield.constants.ErrorCodes.INVALID_CUSTOM_FIELD_VALUE_IN_DATABASE,
                errorParams, objectBeingRead.getClass().getName(), fieldName,
                objectBeingRead.getID());
            rpmEx.setSeverity(SeverityLevel.Warning);
            messageContext.addExceptionNoThrow(rpmEx);
        }
        return value;
    }

    /**
     * Converts object value of custom field into its database string
     * representation
     * 
     * @param inputType
     *            CustomField.getInputType()
     * @param value
     * @return
     * @throws RPMException
     */
    public static String objectToDBString(FieldInputType inputType, Object value)
            throws RPMException
    {
        String dbString = null;
        if (inputType == null) 
        {
            String[] errorParams =
            {
                "InputType cannot be null"
            };
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, errorParams);
        }
        if (value != null)
        {
            if (inputType.equals(FieldInputType.Checkbox))
            {
                dbString = (value != null) && ((Boolean) value).booleanValue()
                        ? "Y" : "N";
            }
            else if (inputType.equals(FieldInputType.Datafield))
            {
                dbString = ((Datafield) value).getID();
            }
            else if (inputType.equals(FieldInputType.Date))
            {
                dbString = DateUtil.convertToString((Calendar) value);
            }
            else
            {
                dbString = value.toString();
            }
        }
        return dbString;
    }

    private static Datafield loadDatafield(String ID,
            MessageContext messageContext) throws RPMException, ParseException,
            SQLException
    {
        Datafield datafield = null;

        if (ID != null && messageContext != null)
        {
            datafield = new Datafield();
            datafield.setID(ID);
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(Datafield.class);
            datafield = (Datafield) manager.loadByPrimaryKey(datafield, null,
                    messageContext, false);
        }

        return datafield;
    }
}
