package com.ibm.rpm.financial.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.applicationadministration.checkpoints.DatafieldCheckpoint;
import com.ibm.rpm.applicationadministration.containers.ChargeCodeState;
import com.ibm.rpm.applicationadministration.containers.DatafieldCategory;
import com.ibm.rpm.financial.constants.ErrorCodes;
import com.ibm.rpm.financial.constants.ValidationConstants;
import com.ibm.rpm.financial.containers.ChargeCode;
import com.ibm.rpm.financial.containers.ChargeCodeCategory;
import com.ibm.rpm.financial.scope.ChargeCodeScope;
import com.ibm.rpm.financial.types.FinancialCategory;
import com.ibm.rpm.financial.util.FinancialCategoryUtil;
import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;

/**
 * @author legaults
 */
public class ChargeCodeCheckpoint extends GenericChargeCodeCheckpoint
{
    private static final List CHARGE_CODE_PARENT_LIST = new ArrayList();

    static
    {
        CHARGE_CODE_PARENT_LIST.add(ChargeCodeCategory.class);
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // validate the instance and the parent
        if (!GenericValidator.validateInstance(rpmObject, ChargeCode.class,
            msgCtx)
                || !isValidParent(rpmObject, msgCtx, true,
                    CHARGE_CODE_PARENT_LIST))
        {
            return;
        }

        super.validateSave(rpmObject, rpmScope, msgCtx);

        ChargeCode chargeCode = (ChargeCode) rpmObject;
        ChargeCodeScope scope = (ChargeCodeScope) rpmScope;

        // Validate financialCategory.
        if (chargeCode.testFinancialCategoryModified()
                || (chargeCode.getID() == null))
        {
            if (chargeCode.getID() != null
                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
            {
                GenericValidator.validateReadOnly(chargeCode,
                    "financialCategory", msgCtx);
            }
            else
            {
                GenericValidator.validateMandatory(chargeCode,
                    "financialCategory", chargeCode.getFinancialCategory(),
                    msgCtx);
                validateFinancialCategory(chargeCode, msgCtx);
            }
        }
        if (scope != null)
        {
            // Validate associated charge code.
            if (scope.getAssociatedChargeCode() != null
                    && chargeCode.testAssociatedChargeCodeModified()
                    || chargeCode.testFinancialCategoryModified())
        {
            if (chargeCode.getID() != null
                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
            {
                GenericValidator.validateReadOnly(chargeCode,
                    "associatedChargeCode", msgCtx);
            }
            else
            {
                validateAssociatedChargeCode(chargeCode, msgCtx);
            }
        }

            // Validate state.
            if (scope.isState() && chargeCode.testStateModified()
                    && chargeCode.getState() != null)
            {
	            if (chargeCode.getID() != null
	                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
	            {
	                GenericValidator.validateReadOnly(chargeCode, "state", msgCtx);
	            }
	            else
	            {
	                DatafieldCheckpoint.validateDatafieldInCategory(chargeCode,
	                    "state", chargeCode.getState(), chargeCodeStageCategory,
	                    msgCtx);
	            }
            }
        }

        // validate billable
        if (chargeCode.testBillableModified())
        {
            if (chargeCode.getID() != null
                    && isChargeCodePartOfDefaultChargeCodeList(chargeCode.getID()))
            {
                GenericValidator.validateReadOnly(chargeCode, "billable",
                    msgCtx);
            }
            else if (FinancialCategory.Benefit.equals(chargeCode
                .getFinancialCategory()))
            {
                GenericValidator.validateMandatory(chargeCode,
                    ValidationConstants.BILLABLE_FIELD, chargeCode
                        .getBillable(), msgCtx);
            }
        }
    }

    private static void validateAssociatedChargeCode(ChargeCode chargeCode,
            MessageContext msgCtx)
    {
        boolean isValid = true;

        if (chargeCode.getAssociatedChargeCode() != null)
        {
            FinancialCategory thisFinCat = chargeCode.getFinancialCategory();
            FinancialCategory assocFinCat = chargeCode
                .getAssociatedChargeCode().getFinancialCategory();
            isValid = FinancialCategoryUtil.isValidAssociatedFinancialCategory(
                thisFinCat, assocFinCat);

            if (!isValid)
            {
                String[] arguments =
                {
                        thisFinCat.toString(), assocFinCat.toString()
                };
                RPMException e = new RPMException(
                    ErrorCodes.FINANCIAL_CATEGORY_MISMATCH, arguments,
                    chargeCode.getClass().getName(), "associatedChargeCode",
                    chargeCode.getID());
                addException(e, msgCtx);
            }
        }
    }

    private static boolean validateFinancialCategory(ChargeCode chargeCode,
            MessageContext msgCtx)
    {
        // Validate that the financial category is not a system category.
        FinancialCategory finCat = chargeCode.getFinancialCategory();
        boolean isValid = true;

        if (finCat != null)
        {
            try
            {
                isValid = !FinancialCategoryUtil
                    .isSystemFinancialCategory(finCat);
            }
            catch (RPMException x)
            {
                addException(x, msgCtx);
            }

            if (!isValid)
            {
                String[] arguments =
                {
                    finCat.toString()
                };
                RPMException e = new RPMException(
                    ErrorCodes.SYSTEM_FINANCIAL_CATEGORY, arguments, chargeCode
                        .getClass().getName(), "financialCategory", chargeCode
                        .getID());
                addException(e, msgCtx);
            }
        }
        return isValid;
    }

    private static final String CHARGE_CODE_STAGE_CATEGORY_ID = "6800";

    private static final DatafieldCategory chargeCodeStageCategory;
    static
    {
        chargeCodeStageCategory = new DatafieldCategory();
        chargeCodeStageCategory.setID(CHARGE_CODE_STAGE_CATEGORY_ID);
        chargeCodeStageCategory.assignTypeID(new Integer(
            ChargeCodeState.TYPE_ID));
    }
    
    public void validateDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {
        super.validateDelete(rpmObject, messageContext);

        ChargeCode chargeCode = (ChargeCode) rpmObject;

        // should not be able to delete a chargeCode that is assigned
        //should not be able to delete a chargecode that has an exernal_id
        //set or is a system_chargecode.
        if (chargeCode.getID() != null
                && (chargeCode.getExternalID() != null
                        || isChargeCodePartOfDefaultChargeCodeList(chargeCode
                            .getID()) || FinancialUtil.isChargeCodeAssigned(
                    chargeCode, messageContext)))
        {
            GenericValidator.validateCannotDelete(chargeCode, messageContext);
        }

    }

}
