package com.ibm.rpm.framework;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.rpm.framework.checkpoints.ScopeCheckpoint;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.containers.AbstractModule;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.metadata.model.Container;
import com.ibm.rpm.metadata.model.MetadataStore;

/**
 * Base class for all checkpoints.
 * Note that matching checkpoint can be inherited if the same validation
 * can be applied to all containers in an inheritance branch 
 * (see DatafieldCheckpoint and its initialization in RPMInitializer).
 */
public abstract class AbstractCheckpoint
{
    // =====================================
    // VALIDATION TO BE CALLED FROM MANAGERS
    // =====================================

    public final boolean isValidSave(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
    {
        //Default behaviour based on the <saveable> property of the metadata.
        MetadataStore store = MetadataStore.getInstance();
        Container container = store.getContainerByJavaClass(rpmObject
            .getClass());

        if (!container.isSaveable())
        {
            if (!GenericValidator.validateCannotSave(rpmObject, messageContext))
                return false;
        }

        //note: This is necessary to validate the scope of the children
        if (!ScopeCheckpoint.getInstance().isValid(rpmObject, rpmScope,
            messageContext))
            return false;

        // Validate the object.
        validateSave(rpmObject, rpmScope, messageContext);

        return messageContext.isSuccessful();
    }

    public final boolean isValidLoad(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException
    {
        // TODO: Validate the scope.
        validateLoad(rpmObject, rpmScope, messageContext);
        return messageContext.isSuccessful();
    }

    public final boolean isValidXPath(Class containerClass,
            RPMObjectScope rpmScope, MessageContext messageContext)
            throws RPMException
    {
        // Validate the scope.
        ScopeCheckpoint.getInstance().isValid(
            containerClass.getClass().getName(), rpmScope);
        // Validate the object.
        validateXPath(containerClass, rpmScope, messageContext);
        return messageContext.isSuccessful();
    }

    public final boolean isValidDelete(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        //Default behaviour based on the <deletable> property of the metadata.
        MetadataStore store = MetadataStore.getInstance();
        Container container = store.getContainerByJavaClass(rpmObject
            .getClass());

        if (!container.isDeletable())
        {
            GenericValidator.validateCannotDelete(rpmObject, messageContext);
        }

        validateDelete(rpmObject, messageContext);
        return messageContext.isSuccessful();
    }

    // ========================================================
    // VALIDATION TO BE IMPLEMENTED BY SUB-CLASSES, IF REQUIRED
    // ========================================================

    /**
     * Override this method to validate the <code>save</code> operation for a
     * specific <code>rpmObject</code>. In case of invalidity, call
     * <code>messageContext.addException(Throwable)</code>. To be called ONLY
     * from within a checkpoint.
     * 
     * @see com.ibm.rpm.framework.RpmResult#addException(Throwable)
     */
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        // By default, save is invalid.
        RPMException x = new RPMException(
            "A checkpoint overriding AbstractCheckpoint.validateSave must be associated to class "
                    + rpmObject.getClass().getName());
        messageContext.addExceptionNoThrow(x);
    }

    /**
     * Override this method to validate the <code>loadFromID</code> operation
     * for a specific <code>rpmObject</code>. In case of invalidity, call
     * <code>messageContext.addException(Throwable)</code>. To be called ONLY
     * from within a checkpoint.
     * 
     * @see com.ibm.rpm.framework.RpmResult#addException(Throwable)
     */
    public void validateLoad(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        // By default, load is valid.
    }

    /**
     * Override this method to validate the <code>loadFromXPath</code>
     * operation for a specific <code>rpmObject</code>. In case of
     * invalidity, call <code>messageContext.addException(Throwable)</code>.
     * To be called ONLY from within a checkpoint.
     * 
     * @see com.ibm.rpm.framework.RpmResult#addException(Throwable)
     */
    public void validateXPath(Class containerClass, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        //Default behaviour based on the <supportxpath> property of the metadata.
        MetadataStore store = MetadataStore.getInstance();
        Container container = store.getContainerByJavaClass(containerClass);

        if (!container.isSupportsXPath())
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(containerClass)
            };
            RPMException ex = new RPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.XPATH_UNSUPPORTED_CONTAINER,
                errorParams);
            addException(ex, messageContext);
        }
    }

    /**
     * Override this method to validate the <code>delete</code> operation for
     * a specific <code>rpmObject</code>. In case of invalidity, call
     * <code>messageContext.addException(Throwable)</code>. To be called ONLY
     * from within a checkpoint.
     * 
     * @see com.ibm.rpm.framework.RpmResult#addException(Throwable)
     */
    public void validateDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {
        RPMException error = null;
        try
        {
            // attempt to load the object using its primary key before
            // deleting.
            RPMManagerFactory.getInstance().getRPMObjectManager(rpmObject)
                .loadByPrimaryKey(rpmObject, null, messageContext, false);
        }
        catch (RPMException e)
        {
            // failed to load object by primary key.
            error = e;
        }
        catch (Exception e)
        {
            // failed to load object by primary key for an unexpected reason.
            error = new RPMException(ErrorCodes.INTERNAL_ERROR, new String[]
            {}, e);
        }

        if (error != null)
        {
            addException(error, messageContext);
        }
    }

    // ===============
    // UTILITY METHODS
    // ===============

    /**
     * Verifies the RPMObject's parent. Ensures that it exists if it is
     * required, and that it is of the correct type. Exceptions are stored in
     * the MessageContext. If a parent is specified, then it must have an id,
     * indicating that is has previously been saved.
     * 
     * @param rpmObject
     * @param msgCtx
     * @param parentRequired
     * @param parentIDRequired
     *            defaults to true
     * @param parentType
     * @return
     */
    // TODO: Move the generic validation to isValidSave method.
    protected final boolean isValidParent(RPMObject rpmObject,
            MessageContext msgCtx, boolean parentRequired, Class parentType)
    {
        List validParentTypes = new ArrayList();
        if (parentType != null)
        {
            validParentTypes.add(parentType);
        }
        return isValidParent(rpmObject, msgCtx, parentRequired, true,
            validParentTypes);
    }

    protected final boolean isValidParent(RPMObject rpmObject,
            MessageContext msgCtx, boolean parentRequired,
            boolean parentIDRequired, Class parentType)
    {
        List validParentTypes = new ArrayList();
        if (parentType != null)
        {
            validParentTypes.add(parentType);
        }
        return isValidParent(rpmObject, msgCtx, parentRequired,
            parentIDRequired, validParentTypes);
    }

    /**
     * Verifies the RPMObject's parent. Ensures that it exists if it is
     * required, and that it is of a correct type.
     */
    protected final boolean isValidParent(RPMObject rpmObject,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {
        return isValidParent(rpmObject, msgCtx, parentRequired, true,
            validParentTypes);
    }

    protected final boolean isValidParent(RPMObject rpmObject,
            MessageContext msgCtx, boolean parentRequired,
            boolean parentIDRequired, List validParentTypes)
    {
        boolean valid = true;

        String[] errorParams =
        {
                StringUtil.getShortClassName(rpmObject.getClass()), "", ""
        };

        RPMObject parent = rpmObject.getParent();

        // Verify the parent if the object is being created for the first time,
        // or if we detect that the parent has been modified
        // Note that we do not enforce a mandatory parent to be there for updates
        if (rpmObject.getID() == null || rpmObject.testParentModified())
        {
            boolean validMandatoryParent = true;

            if (parentRequired)
            {
                if (parent == null)
                {
                    validMandatoryParent = false;
                }

                if (validParentTypes == null || validParentTypes.isEmpty())
                {
                    RPMException e = new RPMException(
                        ErrorCodes.NO_PARENT_TYPE, errorParams, rpmObject
                            .getClass().getName(), "parent", rpmObject.getID());
                    addException(e, msgCtx);
                    valid = false;
                }
            }

            String validParents = "";
            boolean validParentType = false;

            if (parent == null && !parentRequired)
            {
                validParentType = true;
            }
            else
            {
                // loop through the list of valid parents to see if the
                // parent type is valid
                Iterator iterator = validParentTypes.iterator();

                while (iterator.hasNext())
                {
                    Class parentType = (Class) iterator.next();

                    // add the valid parent class names to a string for use
                    // in the potential error message
                    validParents += StringUtil.getShortClassName(parentType);

                    if (parent != null && (parentType.isInstance(parent)))
                    {
                        validParentType = true;
                    }

                    if (iterator.hasNext())
                    {
                        validParents += ", ";
                    }
                }
            }

            if (!validMandatoryParent)
            {
                errorParams[1] = validParents;
                RPMException e = new RPMException(ErrorCodes.NO_PARENT,
                    errorParams, rpmObject.getClass().getName(), "parent",
                    rpmObject.getID());
                addException(e, msgCtx);
                valid = false;
            }
            else if (!validParentType)
            {
                errorParams[1] = StringUtil.getShortClassName(rpmObject
                    .getParent().getClass());
                errorParams[2] = validParents;
                RPMException e = new RPMException(ErrorCodes.WRONG_PARENT_TYPE,
                    errorParams, rpmObject.getClass().getName(), "parent",
                    rpmObject.getID());
                addException(e, msgCtx);
                valid = false;
            }

            else if (parentIDRequired && parent != null
                    && !(parent instanceof AbstractModule)
                    && parent.getID() == null)
            {
                ValidateSerializer vs = msgCtx.getObjectsToValidate();
                if (!vs.before(parent, rpmObject))
                {

                    errorParams[1] = StringUtil.getShortClassName(rpmObject
                        .getParent().getClass());
                    RPMException e = new RPMException(ErrorCodes.NO_PARENT_ID,
                        errorParams, rpmObject.getClass().getName(), "parent",
                        rpmObject.getID());
                    addException(e, msgCtx);
                    valid = false;
                }
            }
        }

        return valid;
    }

    protected static void addException(RPMException e,
            MessageContext messageContext)
    {
        messageContext.addExceptionNoThrow(e);
    }

    public static final boolean canDelete(RPMObject rpmObject,
            MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canDelete(messageContext,
                rpmObject).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

    public static final boolean canSave(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
    {
        try
        {
            if (rpmObject.getID() == null)
            {
                return (ISecurityController.INSTANCE.canCreate(messageContext,
                    rpmObject, rpmScope).type != SecurityValidationResult.FALSE);
            }
            else
            {
                return (ISecurityController.INSTANCE.canUpdate(messageContext,
                    rpmObject, rpmScope).type != SecurityValidationResult.FALSE);
            }
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    
    public static final boolean canUpdateOnCreate(RPMObject rpmObject,
            RPMObjectScope rpmScope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canUpdateOnCreate(
                messageContext, rpmObject, rpmScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    
    public static final boolean restoreRelevantFilteredFields(RestoreRelevantFilteredFieldsParameter param)
    {
        try
        {
            return (ISecurityController.INSTANCE.restoreRelevantFilteredFields(param).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                param.getMessageContext());
            return false;
        }
    }
}