package com.ibm.rpm.framework;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ContextUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.PreparedStatementHelper;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;

/**
 * Collection of static methods used to implement a manager. This class cannot
 * be instanciated.
 */
public abstract class ManagerUtil
{
    private static Log logger = LogFactory.getLog(ManagerUtil.class);

    private static Integer ZERO = new Integer(0);

    private static Integer ONE = new Integer(1);

    /**
     * this is a config parameters indicating that for certain types of
     * SQLException we allow the request to be re-issued a number of time.
     */
    private static final int nbOfRetry = 2;

    /**
     * List of DB errors for which we can retry the operation.
     */
    private static final ArrayList temporaryDBError = new ArrayList();
    static
    {
        temporaryDBError.add(new Integer(-911));
        temporaryDBError.add(new Integer(-1040));
        temporaryDBError.add(new Integer(-964));
    }

    public static boolean canRetryDBCall(int errorType,
            MessageContext messageContext)
    {
        Integer retryCounter = messageContext.getNbCallRetry();

        if (retryCounter == null)
        {
            retryCounter = new Integer(0);
            messageContext.addNbCallRetry(retryCounter);
        }
        else
        {
            Integer counter = new Integer(retryCounter.intValue() + 1);
            messageContext.addNbCallRetry(counter);
        }

        if (messageContext.getNbCallRetry().intValue() < nbOfRetry
                && temporaryDBError.contains(new Integer(errorType)))
        {
            try
            {
                Thread.sleep(2000);
            }
            catch (Exception e)
            {
                // Swallow exception.
            }

            return true;
        }

        messageContext.addNbCallRetry(new Integer(0));
        return false;
    }

    /**
     * Gets the parent's (STRING) id from the object or else, from the database.
     * Uses "ELEMENT_ID" and "PARENT_ID" table column name. Parent's id may be
     * null.
     * 
     * @param object
     *            Child object.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getParentId(RPMObjectManager manager,
            RPMObject object, MessageContext messageContext, String tableName)
            throws RPMException
    {
        return getParentId(manager, object, messageContext, tableName, null,
            null);
    }

    /**
     * Gets the parent's (STRING) id from the object or else, from the database.
     * Parent's id may be null.
     * 
     * @param object
     *            Child object.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @param idColumnName
     *            Database column name for the <code>object</code>'s id.
     * @param parentIdColumnName
     *            Database column name for the <code>object</code>'s parent
     *            id.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getParentId(RPMObjectManager manager,
            RPMObject object, MessageContext messageContext, String tableName,
            String idColumnName, String parentIdColumnName) throws RPMException
    {
        String parentId;

        if (object.getParent() == null || object.getParent().getID() == null)
        {
            parentId = getParentId(manager, object.getClass().getName(), object
                .getID(), messageContext, tableName, idColumnName,
                parentIdColumnName);
        }
        else
        {
            parentId = object.getParent().getID();
        }

        return parentId;
    }

    /**
     * Gets the parent's (STRING) id from the database. Uses "ELEMENT_ID" and
     * "PARENT_ID" table column name. Parent's id may be null.
     * 
     * @param id
     *            Child object's id.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @param idColumnName
     *            Database column name for the <code>object</code>'s id.
     * @param parentIdColumnName
     *            Database column name for the <code>object</code>'s parent
     *            id.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getParentId(RPMObjectManager manager,
            String className, String id, MessageContext messageContext,
            String tableName) throws RPMException
    {
        return getParentId(manager, className, id, messageContext, tableName,
            null, null);
    }

    /**
     * Gets the parent's (STRING) id from the database. Parent's id may be null.
     * 
     * @param id
     *            Child object's id.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @param idColumnName
     *            Database column name for the <code>object</code>'s id.
     * @param parentIdColumnName
     *            Database column name for the <code>object</code>'s parent
     *            id.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getParentId(RPMObjectManager manager,
            String className, String id, MessageContext messageContext,
            String tableName, String idColumnName, String parentIdColumnName)
            throws RPMException
    {
        return doGetParentId(manager, className, id, messageContext, tableName,
            idColumnName, parentIdColumnName);
    }

    /**
     * Gets the parent's (INTEGER) id from the object or else, from the
     * database. Uses "ELEMENT_ID" and "PARENT_ID" table column name. Parent's
     * id may be null.
     * 
     * @param object
     *            Child object.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getIntParentId(RPMObjectManager manager,
            RPMObject object, MessageContext messageContext, String tableName)
            throws RPMException
    {
        return getIntParentId(manager, object, messageContext, tableName, null,
            null);
    }

    /**
     * Gets the parent's (INTEGER) id from the object or else, from the
     * database. Parent's id may be null.
     * 
     * @param object
     *            Child object.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @param idColumnName
     *            Database column name for the <code>object</code>'s id.
     * @param parentIdColumnName
     *            Database column name for the <code>object</code>'s parent
     *            id.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getIntParentId(RPMObjectManager manager,
            RPMObject object, MessageContext messageContext, String tableName,
            String idColumnName, String parentIdColumnName) throws RPMException
    {
        String parentId;

        if (object.getParent() == null || object.getParent().getID() == null)
        {
            parentId = getIntParentId(manager, object.getClass().getName(),
                object.getID(), messageContext, tableName, idColumnName,
                parentIdColumnName);
        }
        else
        {
            parentId = object.getParent().getID();
        }

        return parentId;
    }

    /**
     * Gets the parent's (INTEGER) id from the database. Uses "ELEMENT_ID" and
     * "PARENT_ID" table column name. Parent's id may be null.
     * 
     * @param id
     *            Child object's id.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @param idColumnName
     *            Database column name for the <code>object</code>'s id.
     * @param parentIdColumnName
     *            Database column name for the <code>object</code>'s parent
     *            id.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getIntParentId(RPMObjectManager manager,
            String className, String id, MessageContext messageContext,
            String tableName) throws RPMException
    {
        return getIntParentId(manager, className, id, messageContext,
            tableName, null, null);
    }

    /**
     * Gets the parent's (INTEGER) id from the database. Parent's id may be
     * null.
     * 
     * @param id
     *            Child object's id.
     * @param messageContext
     *            Message context.
     * @param tableName
     *            Database table name.
     * @param idColumnName
     *            Database column name for the <code>object</code>'s id.
     * @param parentIdColumnName
     *            Database column name for the <code>object</code>'s parent
     *            id.
     * @return Id of <code>object</code>'s parent.
     * @throws RPMException
     */
    public static String getIntParentId(RPMObjectManager manager,
            String className, String id, MessageContext messageContext,
            String tableName, String idColumnName, String parentIdColumnName)
            throws RPMException
    {
        Integer intId = Integer.valueOf(id);
        return doGetParentId(manager, className, intId, messageContext,
            tableName, idColumnName, parentIdColumnName);
    }

    private static String doGetParentId(RPMObjectManager manager,
            String className, Object id, MessageContext messageContext,
            String tableName, String idColumnName, String parentIdColumnName)
            throws RPMException
    {
        // Sets default values.
        if (idColumnName == null)
        {
            idColumnName = "ELEMENT_ID";
        }
        if (parentIdColumnName == null)
        {
            parentIdColumnName = "PARENT_ID";
        }

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqual(idColumnName, id);

        //This filter is maybe too much for our needs since we are getting parent_id, however,
        //we need to filter out soft-deleted objects
        condition.appendCondition(manager.getFilter(ContextUtil.NONE));

        SqlBuffer statementBuffer = new SqlBuffer();
        statementBuffer.appendSelect();
        statementBuffer.append(parentIdColumnName);
        statementBuffer.appendFrom();
        statementBuffer.append(tableName);
        statementBuffer.appendWhere(condition.toString());

        // Design note: returning a null parent is valid.
        // => Cannot use the selectColumnValue method.
        List result = select(new Class[]
        {
            String.class
        }, null, "parentID", statementBuffer, null, messageContext);

        if (result == null || result.size() == 0)
        {
            ExceptionUtil.handleInvalidId(manager, className, id.toString());
        }

        Object parentIdObj = ((List) result.get(0)).get(0);
        String parentId = null;
        if (parentIdObj != null)
            parentId = parentIdObj.toString();

        return parentId;
    }

    /**
     * Executes
     * <code>"SELECT " + columnName + " FROM " + tableName [+ " WHERE " + condition]</code>,
     * and then returns the specified column's value for the first record that
     * matches the given condition. Returns <code>null</code> if no record
     * matches the condition. Use <code>SqlBuffer</code> to build the
     * <code>condition</code>.
     * 
     * @param returnType
     *            return type of the object. If null the method will not try to
     *            cast or convert the object into another type.
     * @param container
     *            Container that requires this select. Used for exception
     *            reporting only. Nullable.
     * @param containerFieldName
     *            Name of the container field that requires this select. Used
     *            for exception reporting only. Nullable.
     * @param columnName
     *            Name of the column to be selected.
     * @param tableName
     *            Name of the database table.
     * @param condition
     *            Condition of the "WHERE" clause. Nullable.
     * @param messageContext
     *            Context of the current message.
     * 
     * @return One value or <code>null</code>.
     * @throws RPMException
     * @see com.ibm.rpm.framework.util.SqlBuffer
     */
    public static Object selectColumnValue(Class returnType,
            RPMObject container, String containerFieldName, String columnName,
            String tableName, SqlBuffer condition, Object[] arguments,
            MessageContext messageContext) throws RPMException
    {
        SqlBuffer statementBuffer = new SqlBuffer();
        statementBuffer.appendSelect();
        statementBuffer.append(columnName);
        statementBuffer.appendFrom();
        statementBuffer.append(tableName);
        statementBuffer.appendWhere(condition.toString());

        List result = select(new Class[]
        {
            returnType
        }, container, containerFieldName, statementBuffer, arguments, messageContext);

        Object object = null;
        if (result != null && result.size() > 0)
        {
            object = ((List) result.get(0)).get(0);
        }

        return object;
    }

    public static Object selectColumnValue(Class returnType,
            RPMObject container, String containerFieldName, String columnName,
            String tableName, SqlBuffer condition, MessageContext messageContext)
            throws RPMException
    {
        Object[] arguments = null;
        return selectColumnValue(returnType, container, containerFieldName,
            columnName, tableName, condition, arguments, messageContext);
    }
    
    public static boolean areSame(RPMObject object1, RPMObject object2)
    {
        // Compare the objects.
        if (object1 == null || object2 == null)
        {
            return false;
        }
        if (object1 == object2)
        {
            return true;
        }

        // Compare the ids.
        String id1 = object1.getID();
        String id2 = object2.getID();
        if (id1 == null || id2 == null)
        {
            return false;
        }
        return id1.equals(id2);
    }

    public static boolean areSame(Object object1, Object object2)
    {
        if (object1 == null || object2 == null)
        {
            return false;
        }
        return (object1 == object2);
    }

    /**
     * Gets the <code>object</code>'s id, if possible. Returns null if the
     * <code>object</code> is <code>null</code> or its id is
     * <code>null</code>.
     * 
     * @param object
     * @return <code>object</code>'s id.
     */
    public static String getId(RPMObject object)
    {
        String id = null;
        if (object != null)
        {
            id = object.getID();
        }
        return id;
    }

    /**
     * Gets the <code>object</code>'s id, if possible, converted to an
     * integer. Returns null if the <code>object</code> is <code>null</code>
     * or its id is <code>null</code>.
     * 
     * @param object
     * @return <code>object</code>'s id, converted to an integer.
     */
    public static Integer getIntId(RPMObject object)
    {
        Integer id = null;
        if (object != null)
        {
            String strId = object.getID();
            if (strId != null)
            {
                id = Integer.valueOf(strId);
            }
        }
        if (id == null)
        {
            id = new Integer(0);
        }
        return id;
    }

    /**
     * Converts an <code>int</code> identifier to its <code>String</code>
     * equivalent. <code>0</code> is converted to <code>null</code>.
     * 
     * @param intId
     *            identifier to be converted.
     * @return <code>String</code> identifier.
     */
    public static String intIdToStringId(int intId)
    {
        String id = null;
        if (intId > 0)
        {
            id = String.valueOf(intId);
        }
        return id;
    }

    public static Double nullToZero(Double value)
    {
        if (value == null)
        {
            value = new Double(0.0D);
        }
        return value;
    }

    public static Double nullToZeroDouble(String value)
    {
        Double result;
        if (value == null)
        {
            result = new Double(0.0D);
        }
        else
        {
            result = new Double(value);
        }
        return result;
    }

    public static Double toDouble(String value)
    {
        Double result = null;
        if (value != null)
        {
            result = new Double(value);
        }
        return result;
    }

    public static Integer toInteger(Boolean value)
    {
        boolean booleanValue = value == null ? false : value.booleanValue();
        Integer result = booleanValue ? ONE : ZERO;
        return result;
    }

    public static Boolean toBoolean(int value, int truthMask)
    {
        return (value & truthMask) != 0 ? Boolean.TRUE : Boolean.FALSE;
    }

    public static Boolean toBoolean(int value)
    {
        return (value == 1) ? Boolean.TRUE : Boolean.FALSE;
    }

    public static Integer nullToZero(Integer value)
    {
        if (value == null)
        {
            value = new Integer(0);
        }
        return value;
    }

    /**
     * Executes a select statement and returns the result as a <code>List</code>
     * of <code>List</code> of values. The root <code>List</code> contains
     * one <code>List</code> for each result row. The sub- <code>List</code>
     * contains one value for each column specified in the
     * <code>selectStatement</code>.
     * 
     * @param columnTypes
     *            Array of expected return type of each column. ColumnTypes
     *            lenght must match the number of columns of the select
     *            statement. If null the method will not try to cast or convert
     *            the values.
     * @param container
     *            Container that requires this select. Used for exception
     *            reporting only. Nullable.
     * @param containerFieldName
     *            Name of the container field that requires this select. Used
     *            for exception reporting only. Nullable.
     * @param selectStatement
     *            Text of the select statement.
     * @param arguments
     *            Arguments of the select statment. Nullable.
     * @param messageContext
     *            Context of the current message.
     * 
     * @return <code>List</code> of <code>List</code> of values. Never null.
     */
    public static List select(Class[] columnTypes, RPMObject container,
            String containerFieldName, SqlBuffer selectStatement,
            Object[] arguments, MessageContext messageContext)
            throws RPMException
    {
        Connection c = null;
        ResultSet rs = null;
        List objects = new ArrayList();

        try
        {
            c = Manager.getConnection(messageContext);
            selectStatement.closeAllParenthesis();
            String selectStmtStr = Manager.makeStatementUncommitedRead(
                selectStatement.toString(), c);

            rs = executeSqlQuery(messageContext, c, selectStmtStr, arguments);

            int columnsCount = rs.getMetaData().getColumnCount();
            if (columnTypes != null && columnTypes.length != columnsCount)
            {
                String[] params = new String[]
                {
                    "ColumnTypes lenght must match the number of columns of your select statement."
                };
                throw new RPMException(ErrorCodes.INTERNAL_ERROR, params);
            }
            int rowsCount = 0;

            while (rs.next())
            {
                List columns = new ArrayList();
                for (int i = 1; i <= columnsCount; i++)
                {
                    Object value = convertToSpecificType(rs.getObject(i),
                        columnTypes, i - 1);
                    columns.add(value);
                }
                objects.add(columns);
                rowsCount++;
            }

            // print out a warning if we are loading too many object
            if (logger.isDebugEnabled())
            {
                if (rowsCount > 100)
                {
                    logger.debug("WARNING: Optimization required. Loaded over "
                            + "100 results in ManagerUtil.select().");
                }
            }
        }
        catch (Exception ex)
        {
            throw translateException(ex, container, containerFieldName);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.releaseConnection(messageContext, c);
        }

        return objects;
    }

    public static ResultSet executeSqlQuery(MessageContext msgContext,
            Connection c, String selectStmtStr, Object[] arguments)
            throws RPMException, SQLException
    {
        PreparedStatement ps = null;

        //Always use PreparedStatementHelper to use the caching mechanism (if enabled)
        PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
            msgContext, c);
        ps = preparedStatementHelper.createPreparedStatement(selectStmtStr,
            arguments);

        return executeSqlQuery(msgContext, ps, selectStmtStr, arguments);
    }

    public static ResultSet executeSqlQuery(MessageContext msgContext,
            PreparedStatement ps, String selectStmtStr, Object[] arguments)
            throws SQLException
    {
        if (msgContext != null)
        {
            msgContext.getStatsOperation().incrementSelectStatementCount();
        }
        logSqlStatement(selectStmtStr, arguments);
        ResultSet rs= ps.executeQuery();
        msgContext.reservePreparedStatement(rs, ps);
        return rs;
    }

    public static int executeSqlUpdate(MessageContext msgContext, Connection c,
            String selectStmtStr, Object[] arguments) throws RPMException,
            SQLException
    {
        PreparedStatement ps = null;
        //Always use PreparedStatementHelper to use the caching mechanism (if enabled)
        PreparedStatementHelper preparedStatementHelper = new PreparedStatementHelper(
            msgContext, c);
        ps = preparedStatementHelper.createPreparedStatement(selectStmtStr,
            arguments);

        return executeSqlUpdate(msgContext, ps, selectStmtStr, arguments);
    }

    public static int executeSqlUpdate(MessageContext msgContext,
            PreparedStatement ps, String selectStmtStr, Object[] arguments)
            throws SQLException
    {
        if (msgContext != null)
        {
            msgContext.getStatsOperation().incrementSelectStatementCount();
        }

        logSqlStatement(selectStmtStr, arguments);
        return ps.executeUpdate();
    }

    /**
     * Executes a delete statement.
     * 
     * @param container
     *            Container that requires this select. Used for exception
     *            reporting only. Nullable.
     * @param containerFieldName
     *            Name of the container field that requires this select. Used
     *            for exception reporting only. Nullable.
     * @param deleteStatement
     *            Text of the statement.
     * @param arguments
     *            Arguments of the statment. Nullable.
     * @param messageContext
     *            Context of the current message.
     * @return Number of rows deleted.
     */
    public static int delete(RPMObject container, String containerFieldName,
            SqlBuffer deleteStatement, Object[] arguments,
            MessageContext messageContext) throws RPMException
    {
        Connection c = null;
        int rowsCount = 0;
        try
        {
            c = Manager.getConnection(messageContext);
            deleteStatement.closeAllParenthesis();
            String deleteStmtStr = deleteStatement.toString();

            rowsCount = executeSqlUpdate(messageContext, c, deleteStmtStr,
                arguments);

            // Log a warning if we are loading too many rows.
            if (logger.isDebugEnabled())
            {
                if (rowsCount > 100)
                {
                    logger
                        .debug("WARNING: Optimization required. Deleted over "
                                + "100 results in ManagerUtil.delete().");
                }
            }
        }
        catch (Exception ex)
        {
            throw translateException(ex, container, containerFieldName);
        }
        finally
        {
            Manager.releaseConnection(messageContext, c);
        }

        return rowsCount;
    }

    /**
     * Logs the statement and its arguments, if any.
     * 
     * @param statement
     *            Text of the statement.
     * @param arguments
     *            Arguments of the statement.
     */
    public static void logSqlStatement(String statement, Object[] arguments)
    {
        if (logger.isDebugEnabled())
        {
            StringBuffer logString = new StringBuffer();
            logString.append(statement);

            if (arguments != null)
            {
                for (int i = 0; i < arguments.length; i++)
                {
                    logString.append("\n\targuments[");
                    logString.append(i);
                    logString.append("]: ");
                    if (arguments[i] == null)
                    {
                        logString.append("null");
                    }
                    else
                    {
                        logString.append(arguments[i].toString());
                    }
                }
            }
            logger.debug(logString.toString());
        }
    }

    public static String executeProcedure(Class callerClass,
            MessageContext context, String procedureName, Object[] args)
            throws SQLException, RPMException
    {
        return executeProcedure(callerClass, context, procedureName, args,
            false);
    }
    
    /**
     * @param procedureName
     * @param args
     * @param useDBStatementsForLOBS
     * @return the element_id of the object that was insert/update/delete
     * @throws SQLException
     * @throws RPMException
     */
    public static String executeProcedure(Class callerClass,
            MessageContext context, String procedureName, Object[] args, boolean useDBStatementsForLOBS)
            throws SQLException, RPMException
    {
        Connection c = null;
        ResultSet rs = null;
        int error = 0;
        String callStr = null;

        try
        {
            context.getStatsOperation().incrementStoredProcedureCount();
            c = Manager.getConnection(context);

            rs = Manager.executeStoredProcedure(c, procedureName, args, true, context, useDBStatementsForLOBS);

            rs.next();
            try
            {
                error = Manager.checkForError(rs);
            }
            catch (SQLException sqle)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(callerClass),
                        procedureName
                };
                throw new RPMException(ErrorCodes.AM_NO_RESULT_SET, errorParams);
            }
            if (error != 0)
            {
                context.addCurrentSPCall(callStr);
                MessageContext.dumpStoredProcedureStack();
                context.removeCurrentSPCall();
                if (ManagerUtil.canRetryDBCall(error, context))
                {
                    logger.debug("Retrying DB call: " + callStr
                            + "on DB error: " + error);
                    Manager.rollbackConnection(c);
                    Manager.closeConnection(context, c, true);
                    return executeProcedure(callerClass, context,
                        procedureName, args);
                }
                throw new RPMException(error);
            }
            return rs.getString(2);

        }
        catch (RPMException e)
        {
            Manager.rollbackConnection(c);
            Manager.closeConnection(context, c, true);
            throw e;
        }
        catch (SQLException e)
        {
            Manager.rollbackConnection(c);
            Manager.closeConnection(context, c, true);
            throw e;
        }
        finally
        {
            Manager.close(context, rs);
            if (error == 0)
            {
                Manager.releaseConnection(context, c);
            }
        }
    }

    /**
     * Dummy implementation
     * 
     * @param ResultSet
     * @return ArrayList
     */
    //    protected ArrayList convertResultSetToArrayList(ResultSet rs)
    //            throws RPMException, SQLException, ParseException
    //    {
    //        return null;
    //    }
    /**
     * @param procedureName
     * @param args
     * @return the element_id of the object that was insert/update/delete
     * @throws SQLException
     * @throws RPMException
     */
    public static ArrayList loadFromProcedure(MessageContext context,
            String procedureName, Object[] args) throws SQLException,
            RPMException
    {
        Connection c = null;
        ResultSet rs = null;
        try
        {
            c = Manager.getConnection(context);
            rs = Manager.executeStoredProcedure(c, procedureName, args, true,
                context);
            //ArrayList arrayList = convertResultSetToArrayList(rs);
            return null;
        }
        finally
        {
            if (rs != null)
            {
                Manager.close(context, rs);
            }
            if (c != null)
            {
                Manager.releaseConnection(context, c);
            }
        }
    }

    public static String retrieveRecStatus(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException, SQLException
    {
        if (rpmObject == null)
        {
            return null;
        }

        String status = null;

        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(rpmObject);

        String primaryKeyString = manager.getPrimaryKeyString(rpmObject);
        if (primaryKeyString != null)
        {
            Connection c = null;
            ResultSet rs = null;

            // Builds the SQL statement.
            SqlBuffer statement = new SqlBuffer();
            statement.appendSelect();
            statement.append(" REC_USER,REC_STATUS,REC_DATETIME");
            if (rpmObject.getContextName() == null)
            {
                statement.appendFrom(manager.getTableName());
            }
            else
            {
                statement.appendFrom(manager.getTableName(rpmObject
                    .getContextName()));
            }
            statement.appendWhere();
            String[] keyColumns = manager.getPrimaryKeyArray(rpmObject);
            Object[] keyValues = manager.getPrimaryKeyValues(rpmObject);

            for (int i = 0; i < keyColumns.length; i++)
            {
                statement.appendEqualQuestionMark(keyColumns[i]);
            }

            String selectStmt = statement.toString();

            try
            {
                c = Manager.getConnection(messageContext);

                selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);

                try
                {
                    rs = executeSqlQuery(messageContext, c, selectStmt,
                        keyValues);
                }
                catch (Exception s)
                {
                    logger.error(s);
                }
                if (rs != null && rs.next())
                {
                    Calendar cal = Manager.getTimestamp(rs, 3);
                    messageContext.setCheckOutRecDateTime(cal);

                    Resource resource = new Resource();
                    resource.setID(rs.getString(1));

                    if (resource.getID() != null)
                    {
                        RPMObjectManager resourceManager = RPMManagerFactory
                            .getInstance().getRPMObjectManager(resource);
                        try
                        {
                            resource = (Resource) resourceManager
                                .loadByPrimaryKey(resource, null,
                                    messageContext, false);
                            messageContext.setCheckOutRecUser(resource);
                        }
                        catch (ParseException e)
                        {
                            throw new RPMException(e);
                        }

                        status = rs.getString(2);
                    }
                }
            }
            finally
            {
                Manager.close(messageContext, rs);
                Manager.releaseConnection(messageContext,c);
            }
        }
        return status;
    }

    private static Object convertToSpecificType(Object object,
            Class[] columnTypes, int index) throws Exception
    {

        Class returnType = null;
        if (columnTypes != null && columnTypes[index] != null)
        {
            returnType = columnTypes[index];
        }

        //Convert the field into the desired return type when needed.
        if (object != null && returnType != null && returnType != Object.class)
        {
            if (returnType == Integer.class
                    && object.getClass() != Integer.class)
            {
                int value = ((Number) object).intValue();
                object = new Integer(value);
            }
            else if (returnType == Double.class
                    && object.getClass() != Double.class)
            {
                double value = ((Number) object).doubleValue();
                object = new Double(value);
            }
            else if (returnType == Long.class
                    && object.getClass() != Long.class)
            {
                long value = ((Number) object).longValue();
                object = new Long(value);
            }
            else if (returnType == Float.class
                    && object.getClass() != Float.class)
            {
                float value = ((Number) object).floatValue();
                object = new Float(value);
            }
            else if (returnType == Calendar.class
                    && object.getClass() != Calendar.class)
            {
                object = DateUtil.convertObjectToCalendar(object);
            }
        }

        return object;
    }

    private static RPMException translateException(Exception ex,
            RPMObject container, String fieldName)
    {
        RPMException exception;
        if (ex instanceof RPMException)
        {
            exception = (RPMException) ex;
        }
        else if (ex instanceof SQLException)
        {
            SQLException sqlException = (SQLException) ex;
            exception = new RPMException(sqlException.getErrorCode());
        }
        else
        {
            exception = new RPMException(ex);
        }

        if (container != null)
        {
            exception.assignContainer(container);
            exception.setFieldName(fieldName);
        }
        return exception;
    }

    //selectStatement processing :
    //Expected format SELECT OUTPUT[0] FROM TABLE_NAME WHERE OUTPUT[1] WITH UR
    //OUTPUT[0] = column list
    //OUTPUT[1] = where condition
    //Processing also includes
    //1- replacing the tableName by the tableAlias (expected by the SP)
    //2- replacing all the ? by argument values
    public static String[] processSelectStatementBeforeSecurityCall(
            String selectStmt, Object[] args)
    {
        String[] output = new String[3];
        if (selectStmt != null)
        {
            int selectIndex = selectStmt.indexOf("SELECT");
            int fromIndex = selectStmt.indexOf("FROM");
            int whereIndex = selectStmt.indexOf("WHERE");

            //here we assume that the provided selectStmt always contains 
            //'SELECT', 'FROM', 'WHERE'
            if (selectIndex != -1 && whereIndex != -1)
            {
                //output[0] = column list
                output[0] = selectStmt.substring(selectIndex + 6, fromIndex);

                //output[1] = table name
                output[1] = selectStmt.substring(fromIndex + 5, whereIndex)
                    .trim();

                //output[2] = where clause
                int beginIndex = whereIndex + 5;
                int endIndex = selectStmt.length();
                if (selectStmt.endsWith("WITH UR"))
                {
                    endIndex = selectStmt.lastIndexOf("WITH UR");
                }
                output[2] = selectStmt.substring(beginIndex, endIndex);
            }

            //replace all the ? by argument values
            if (output[2] != null && !isEmptyArray(args))
            {
                for (int i = 0; i < args.length; i++)
                {
                    output[2] = replaceArgument(output[2], args[i]);
                }
            }
        }

        return output;
    }

    private static boolean isEmptyArray(Object[] args)
    {
        if (args != null && args.length != 0)
        {
            for (int i = 0; i < args.length; i++)
            {
                if (args[i] != null)
                {
                    return false;
                }
            }
        }
        return true;
    }

    //replace ? with the argument value before calling the SP_S_*
    //security filtering stored procedures  
    private static String replaceArgument(String selectStmt, Object argument)
    {

        if (argument != null)
        {
            int index = selectStmt.indexOf("?");
            if (index != -1)
            {
                StringBuffer sb = new StringBuffer();
                sb.append(selectStmt.substring(0, index));

                if (argument instanceof Number)
                {
                    sb.append(argument);
                }
                else
                {
                    sb.append("'" + argument.toString() + "'");
                }

                sb.append(selectStmt.substring(index + 1));
                return sb.toString();
            }
        }
        return selectStmt;
    }

    public static void setSecurityRoleIds(MessageContext messageContext,
            Class objectClass, String objectId, String securityRoleIds)
            throws RPMException
    {
        Set securityRoleIdsByElement = new HashSet();
        if (securityRoleIds != null
                && (securityRoleIds = securityRoleIds.trim()).length() > 0)
        {
            String securityRoleId = null;
            int i = 0;
            while (i < securityRoleIds.length())
            {
                securityRoleId = securityRoleIds.substring(i, Math.min(i + 32,
                    securityRoleIds.length()));
                securityRoleId = securityRoleId.trim();
                securityRoleIdsByElement.add(securityRoleId);
                i = i + 32;
            }

            messageContext.getRPMSecurityCache().addSecurityRoleIds(
                objectClass, objectId, securityRoleIdsByElement);
        }
    }
    
    /**
     * Utility class to hold the project ID List and the SP idsType parameter that will be used 
     * to call secured stored procedure. 
     * @author mcduffs
     */
    public static class IDStorage
    {
        // the project ID list in a comma separated string
        public String idsList;
        
        // the SP idsType parameter that specifies to the SP wether it should use the project ID list or not
        public Integer idsType;
    }
    

    /**
     * Method used to find if there is a single Project ID in the result requested by the user.
     * 
     * If there is a single project in the result, we will then use that project ID in the call to the
     * secured stored procedure in order to prevent it from building a full project access list for the current
     * user (which takes around 2 seconds on a large DB).
     * 
     * @param messageContext the current message context
     * @param spName the name of the secured SP (used for log messages only)
     * @param whereClause the where clause that will be used to filter the results
     * @param idsType the current value used to call the secure SP idsType parameter.
     * @param idsList the current list of project IDs in a comma separated string
     * @param tableName the name of the table to query the project ID count
     * 
     * @return the IDStorage structure which contains the new list of project IDs to use and the 
     * stored procedure idsType paramter to use
     * 
     * @throws RPMException if the DB is unreachable
     */
    public static IDStorage getRestrictedProjectIdList(
            MessageContext messageContext, String spName, String whereClause,
            Integer idsType, String idsList, String tableName)
            throws RPMException
    {
        IDStorage storage = new IDStorage();
        storage.idsList = idsList;
        storage.idsType = idsType;

        // find out the count of unique project IDs that we will get as a result of the current request
        SqlBuffer buff = new SqlBuffer(
            "select count(distinct PROJECT_ID) from " + tableName + " where "
                    + whereClause);

        Class[] projectIdCountParam = new Class[]
        {
            Integer.class
        };

        List resultList = ManagerUtil.select(projectIdCountParam, null, null,
            buff, null, messageContext);

        if (resultList == null)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected null result list size for count before SP_S_WBS_PAGES");
            }
            return storage;
        }

        if (resultList.size() != 1)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected result list size for count before SP_S_WBS_PAGES : "
                            + resultList.size());
            }
            return storage;
        }

        List subList = (List) resultList.get(0);
        if (subList == null || subList.size() != 1)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected null or empty sub-list result list size for count before SP_S_WBS_PAGES");
            }
            return storage;
        }

        Integer result = (Integer) subList.get(0);
        int intResult = result.intValue();

        if (intResult != 1)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("Found "
                            + intResult
                            + " matching projects for a call to SP_S_WBS_PAGES, will not specify project IDs to use.");
            }
            return storage;
        }

        // we have a single project fetch the unique project ID we need
        SqlBuffer projectIDBuff = new SqlBuffer(
            "select distinct PROJECT_ID from " + tableName + " where "
                    + whereClause);

        Class[] projectIDClasses = new Class[]
        {
            String.class
        };

        List projectIDResultList = ManagerUtil.select(projectIDClasses, null,
            null, projectIDBuff, null, messageContext);

        if (projectIDResultList == null)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected result list size for distinct PROJECT_ID before "
                            + spName + " : null list");
            }
            return storage;
        }

        if (projectIDResultList.size() != 1)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected result list size for distinct PROJECT_ID before "
                            + spName + " : " + projectIDResultList.size());
            }
            return storage;
        }

        List subProjectIDList = (List) projectIDResultList.get(0);
        if (subProjectIDList == null)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected result sub-list size for distinct PROJECT_ID before "
                            + spName + " : null");
            }
            return storage;
        }

        if (subProjectIDList.size() != 1)
        {
            if (logger.isDebugEnabled())
            {
                logger
                    .debug("unexpected result sub-list size for distinct PROJECT_ID before "
                            + spName + " : " + subProjectIDList.size());
            }
            return storage;
        }

        // override the project ID to use to call the SP to the SELECT result
        storage.idsList = (String) subProjectIDList.get(0);

        // override the idsType parameter to use to call the SP with "2" This specifies
        // to the SP that a project ID list is specified and that it should use it for filtering.
        storage.idsType = new Integer(2);
        
        if (logger.isDebugEnabled())
        {
            logger.debug("Found unique project ID for " + spName + " "
                    + storage.idsList + ". Will use this specific ID to optimize Secure SP call");
        }

        return storage;
    }
    
}
