package com.ibm.rpm.framework;

import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.ErrorCodeProperties;
import com.ibm.rpm.framework.util.StringUtil;

public class RPMException extends Exception
{
    public static final ErrorCodeProperties ecp = ErrorCodeProperties.getInstance();

    private String localMessage;
    
    private int errorID = 0;
    
    private String className;

    private String fieldName;

    private String secondaryFieldName;

    private String containerID;

    private SeverityLevel severity;
    
    private String[] errorParams;
    
    // ============
    // CONSTRUCTORS
    // ============

    public RPMException()
    {
        this("", null);
    }

    /**
     * Deprecated.
     */ 
    public RPMException(String message)
    {
        // TODO: Eventually delete.
        this(message, null);
    }

    /**
     * Deprecated.
     */ 
    public RPMException(String message, Throwable cause)
    {
        // Design note: Root constructor. Add mandatory initialization here.
        // => DO NOT call super(...) anywhere else.
        // TODO: Eventually make private.
        super(message, cause);
        localMessage = null;
        setSeverity(SeverityLevel.Error);
    }

    public RPMException(Throwable cause)
    {
        this(formatMessage(cause), cause);
    }

    public RPMException(Throwable cause, String className)
    {
        this(cause);
        this.setClassName(className);
    }

    public RPMException(int errorID)
    {
        this(getMessageFromErrorID(errorID, null));
        this.setErrorID(errorID);
    }

    public RPMException(int errorID, String[] errorParams)
    {
        this(getMessageFromErrorID(errorID, errorParams));
        this.setErrorID(errorID);
        this.setErrorParameters(errorParams);
    }

    public RPMException(int errorID, String[] errorParams, Throwable cause)
    {
        this(getMessageFromErrorID(errorID, errorParams), cause);
        this.setErrorID(errorID);
        this.setErrorParameters(errorParams);
    }

    public RPMException(int errorID, String className)
    {
        this(errorID, className, null, null);
    }

    public RPMException(int errorID, String[] errorParams, String className)
    {
        this(errorID, errorParams, className, null, null);
    }

    public RPMException(int errorID, String className, String fieldName)
    {
        this(errorID, className, fieldName, null);
    }

    public RPMException(int errorID, String[] errorParams, String className,
            String fieldName)
    {
        this(errorID, errorParams, className, fieldName, null);
    }

    public RPMException(int errorID, String className, String fieldName,
            String containerID)
    {
        this(errorID);
        setClassName(className);
        setFieldName(fieldName);
        setContainerID(containerID);
    }

    public RPMException(int errorID, String[] errorParams, String className,
            String fieldName, String containerID)
    {
        this(errorID, errorParams, className, fieldName, null, containerID);
    }

    public RPMException(int errorID, String[] errorParams, String className,
            String fieldName, String secondaryFieldName, String containerID)
    {
        this(errorID, errorParams, className, fieldName, secondaryFieldName, containerID, null);
    }

    public RPMException(int errorID, String[] errorParams, String className,
            String fieldName, String secondaryFieldName, String containerID,
            Throwable cause)
    {
        this(errorID, errorParams, cause);
        setClassName(className);
        setFieldName(fieldName);
        setContainerID(containerID);
        setSecondaryFieldName(secondaryFieldName);
        this.setErrorParameters(errorParams);
    }

    public RPMException(String className, String fieldName, String containerID,
            int errorID)
    {
        this(errorID, className);
        setFieldName(fieldName);
        setContainerID(containerID);
    }

    // ===================
    // GETTERS AND SETTERS
    // ===================
    
    public String getRpmMessage()
    {
        // Design note: this method is redefined only to make it available
        // in the Web service.
        return getMessage();
    }
    
    public void setRpmMessage(String message)
    {
        // Design note: this method is redefined only to make it available
        // in the Web service.
        setMessage(message);
    }
    
    public String getMessage()
    {
        if (localMessage != null)
            return localMessage;
        return super.getMessage();
    }
    
    public void setMessage(String message)
    {
        localMessage = message;
    }
    
    public String getClassName()
    {
        return className;
    }

    public void setClassName(String className)
    {
        this.className = StringUtil.getShortClassName(className);
    }

    public String getFieldName()
    {
        return fieldName;
    }

    public void setFieldName(String fieldName)
    {
        this.fieldName = fieldName;
    }

    public String getSecondaryFieldName()
    {
        return secondaryFieldName;
    }

    public void setSecondaryFieldName(String secondaryFieldName)
    {
        this.secondaryFieldName = secondaryFieldName;
    }

    public String getContainerID()
    {
        return containerID;
    }

    public void setContainerID(String containerID)
    {
        this.containerID = containerID;
    }

    public void assignContainer(RPMObject container)
    {
        if (container != null)
        {
            this.setClassName(container.getClass().getName());
            this.setContainerID(container.getID());
        }
    }

    public int getErrorID()
    {
        return errorID;
    }

    public void setErrorID(int errorID)
    {
        this.errorID = errorID;
    }

    public SeverityLevel getSeverity()
    {
        return severity;
    }

    public void setSeverity(SeverityLevel severity)
    {
        this.severity = severity;
    }

    public static String getMessageFromErrorID(int errorID,
            String[] errorParams)
    {
        return ecp.getString(Integer.toString(errorID), errorParams);
    }

    private static String formatMessage(Throwable cause)
    {
        if (cause != null)
        {
	        String message = cause.getMessage();
	        String name = cause.getClass().getName();
	        String retVal = name;
	        if (message != null)
	        {
	            retVal += ":" + message;
	        }
	        return retVal;
        }
        return "";
    }

    public String toString()
    {
        StringBuffer msg = new StringBuffer(getMessage());
        msg.append(" - Severity: ");
        msg.append(severity.getValue());
        if (className != null)
        {
            msg.append(" - Class: ");
            msg.append(className);
        }
        if (containerID != null)
        {
            msg.append(" - Object ID: ");
            msg.append(containerID);
        }
        if (fieldName != null)
        {
            msg.append(" - Field name: ");
            msg.append(fieldName);
        }
        if (secondaryFieldName != null)
        {
            msg.append(" - Other field name: ");
            msg.append(secondaryFieldName);
        }
        if (errorID != 0)
        {
            msg.append(" - Error ID: ");
            msg.append(errorID);
        }

        return msg.toString();
    }
    // =======
    // LOGGING
    // =======
    
    private static RpmLog logger = RpmLog.getLog();

    public RPMException log()
    {
        logger.log(severity, "", this);
        return this;
    }

    public String[] getErrorParameters()
    {
        return errorParams;
    }

    public void setErrorParameters(String[] errorParams)
    {
        this.errorParams = errorParams;
    }
}
