package com.ibm.rpm.framework;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;

/**
 * @author gmeloche
 * @author mcduffs
 *  */
public class RPMObjectCache
{
    
    public static class RemovedContainerFromCache
    {
        public final RPMObject container;

        private final List scopeList;

        private final Object key;

        private RemovedContainerFromCache(final RPMObject container,
                final List scopeList, final Object key)
        {
            this.container = container;
            this.scopeList = scopeList;
            this.key = key;
        }
    }
    
    private Map containerCache = new HashMap();
    
    private Map scopeCache = new IdentityHashMap();
    
    private int cacheHitCount = 0;
    
    private int cacheMissCount = 0;
    
    private static ArrayList skipContainers = new ArrayList();
    
    static 
    {
        // list of containers that should never be cached
        skipContainers.add(WbsScheduleDate.class);
    }

    public RPMObjectCache()
    {
        super();
    }

    private Map getContainerCache()
    {
        return containerCache;
    }

    private Map getScopeCache()
    {
        return scopeCache;
    }
    
    public int getCacheHitCount()
    {
        return cacheHitCount;
    }

    public int getCacheMissCount()
    {
        return cacheMissCount;
    }

    public void addCacheHit()
    {
        ++cacheHitCount;
    }

    public void addCacheMiss()
    {
        ++cacheMissCount;
    }

    public boolean isContainerInCacheWithScope(RPMObject object, RPMObjectScope scope)
    {
        Map scopeCacheToUse = getScopeCache();
        List scopeList = (List) scopeCacheToUse.get(object);
        if( scopeList != null )
        {
            for (Iterator iter = scopeList.iterator(); iter.hasNext();)
            {
                RPMObjectScope cachedScope = (RPMObjectScope) iter.next();
                if( scope == cachedScope)
                {
                    return true;
                }
            }
        }
        return false;
    }

    public void addContainerInCache(RPMObjectManager manager, Object[] primaryKey,
            RPMObject object, RPMObjectScope scope)
    {
        if (!skipContainers.contains(object.getClass()))
        {
            String key = createUniqueKeyFromPrimaryKeyValues(manager,
                primaryKey);
            getContainerCache().put(key, object);
            
            addScopeInCache(object, scope);
        }
    }
    
    public void addScopeInCache(RPMObject object, RPMObjectScope scope)
    {
    	if (scope != null)
        {
            Map scopeCache = getScopeCache();
            List scopeList = (List) scopeCache.get(object);
            if (scopeList == null)
            {
                scopeList = new ArrayList();
                scopeCache.put(object, scopeList);
            }
            scopeList.add(scope);
        }
    }

    /**
     * @param manager The manager that is used.
     * @param primaryKey
     *            in the case of a multiple field primary key, primarykey will
     *            be a concatenation of fields
     * @param messageContext
     * @return
     */
    public boolean containerExistInCache(RPMObjectManager manager,
            Object[] primaryKey)
    {
        String primaryKeyString = createUniqueKeyFromPrimaryKeyValues(manager, primaryKey);

        boolean exist = false;
        RPMObject object = getContainerInCache(manager, primaryKeyString);

        if (object != null)
        {
            exist = true;
        }
        return exist;
    }

    public boolean containerExistInCacheWithScope(RPMObjectManager manager,
            Object[] primaryKey, RPMObjectScope rpmObjectScope)
    {
        String primaryKeyString = createUniqueKeyFromPrimaryKeyValues(manager, primaryKey);

        boolean existWithScope = false;
        RPMObject object = getContainerInCache(manager, primaryKeyString);

        if (object != null)
        {
        	existWithScope = isContainerInCacheWithScope(object, rpmObjectScope); 
        }
        return existWithScope;
    }
    
    public static String createUniqueKeyFromPrimaryKeyValues(RPMObject object)
            throws RPMException
    {
        RPMObjectManager manager = RPMManagerFactory.getInstance()
            .getRPMObjectManager(object.getClass());

        return createUniqueKeyFromPrimaryKeyValues(manager, object);
    }

    public static String createUniqueKeyFromPrimaryKeyValues(
            RPMObjectManager manager, RPMObject object)
    {
        Object[] primaryKeyValues = manager.getPrimaryKeyValues(object);

        return createUniqueKeyFromPrimaryKeyValues(manager, primaryKeyValues);
    }
    
    public static String createUniqueKeyFromPrimaryKeyValues(
            RPMObjectManager manager, Object[] primaryKey)
    {
        StringBuffer sb = new StringBuffer();
        sb.append(StringUtil.getShortClassName(manager.getClass()));
        for (int i = 0; i < primaryKey.length; i++)
        {
            sb.append(primaryKey[i]);
        }
        String primaryKeyString = sb.toString();
        return primaryKeyString;
    }
    
    public RPMObject getContainerInCache(RPMObject object) throws RPMException
    {
        if (object != null)
        {
            //Manager is concatenated to the primaryKey because primaryKeys
            // are not unique between tables.
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(object.getClass());
            if (manager == null)
            {
                return null;
            }
            Object[] primaryKeyValues = manager.getPrimaryKeyValues(object);
            if (primaryKeyValues == null)
            {
                return null;
            }
            return getContainerInCache(manager, primaryKeyValues);
        }
        return null;
    }   


    public RPMObject getContainerInCache(RPMObjectManager manager,
            Object[] primaryKey)
    {
        String key = createUniqueKeyFromPrimaryKeyValues(manager, primaryKey);
        return getContainerInCache(manager, key);
    }
    
    private RPMObject getContainerInCache(RPMObjectManager manager,
            String key)
    {
        return (RPMObject) getContainerCache().get(
            key);
    }
    
    public void restoreContainerToCache(RemovedContainerFromCache param)
    {

        getContainerCache().put(param.key, param.container);

        if (param.scopeList != null)
        {
            getScopeCache().put(param.container, param.scopeList);
        }
    }

    public RemovedContainerFromCache removeContainerFromCache(RPMObject object)
            throws RPMException
    {
        if (object != null)
        {
            RPMObject removedContainer = null;

            List scopeList = (List) getScopeCache().remove(object);

            //Manager is concatenated to the primaryKey because primaryKeys
            // are not unique between tables.
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(object.getClass());

            Object[] primaryKeyValues = manager.getPrimaryKeyValues(object);

            String removeKey = createUniqueKeyFromPrimaryKeyValues(manager,
                primaryKeyValues);

            if (removeKey != null)
            {
                removedContainer = (RPMObject) getContainerCache().remove(
                    removeKey);
            }
            else
            {
                String[] params = new String[]
                {
                    StringUtil.getShortClassName(manager.getClass())
                };
                ExceptionUtil.createAndHandleRPMException(this.getClass(),
                    ErrorCodes.REMOVE_FROM_CACHE_INVALID_PRIMARY_KEY, params);
            }

            return (removedContainer != null) ? new RemovedContainerFromCache(
                removedContainer, scopeList, removeKey) : null;
        }
        return null;
    }
    
    
    

    public void resetLoadedContainers()
    {
        getContainerCache().clear();
        getScopeCache().clear();
    }

    
}
