/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.RPMObjectKey;
import com.ibm.rpm.security.containers.SecurityRole;

/**
 * @author Yasser Lulu
 *
 */
public final class RPMSecurityCache
{
    private final MessageContext messageContext;

    private RPMObjectManager securityRoleManager;

    private Map securityRoleIdsToSecurityRoleMap = new HashMap();

    private Map securityRoleByType = new HashMap();

    private Map genericCacheMap = new HashMap();

    private Map rpmObjectToCacheMaps = new HashMap();

    private Map containerSecurityDescriptorsMap = new HashMap();

    public RPMSecurityCache(MessageContext messageContext)
    {
        this.messageContext = messageContext;
    }

    public MessageContext getMessageContext()
    {
        return messageContext;
    }

    private Set getSecurityRoles(Set securityRoleIds) throws RPMException
    {
        Set roles = new HashSet();
        Iterator it = securityRoleIds.iterator();
        while (it.hasNext())
        {
            String id = (String) it.next();
            SecurityRole role = (SecurityRole) securityRoleIdsToSecurityRoleMap
                .get(id);
            if (role == null)
            {
                role = loadSecurityRole(id);
                if (role != null)
                {
                    securityRoleIdsToSecurityRoleMap.put(id, role);
                }
            }

            if (role != null)
            {
                roles.add(role);
            }
        }

        return roles;
    }

    private RPMObjectManager getSecurityRoleManager() throws RPMException
    {
        if (securityRoleManager == null)
        {
            securityRoleManager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(SecurityRole.class);
        }
        return securityRoleManager;
    }

    /**
     * @param id
     * @return
     */
    private SecurityRole loadSecurityRole(String securityRoleId)
            throws RPMException
    {
        try
        {
            SecurityRole securityRole = new SecurityRole();
            securityRole.setID(securityRoleId);
            return (SecurityRole) getSecurityRoleManager().loadByPrimaryKey(
                securityRole, null, messageContext, false);
        }
        catch (Exception ex)
        {
            throw SecurityValidationResult.makeException(ex);
        }
    }

    public void addSecurityRoleIds(Class objectClass, Object objectId,
            Set securityRoleIds) throws RPMException
    {
        //although callers should not call us with empty set, but just to be sure        
        if (securityRoleIds.isEmpty())
        {
            return;
        }
        Map objectIdsToSecurityRoleMap = (Map) securityRoleByType
            .get(objectClass);
        if (objectIdsToSecurityRoleMap == null)
        {
            objectIdsToSecurityRoleMap = new HashMap();
            securityRoleByType.put(objectClass, objectIdsToSecurityRoleMap);
        }

        Set elementRoles = (Set) objectIdsToSecurityRoleMap.get(objectId);

        if (elementRoles == null)
        {
            elementRoles = new HashSet();
            objectIdsToSecurityRoleMap.put(objectId, elementRoles);
        }
        //since we cache loaded roles, then we'll end up getting the same 
        //instance fo the same id,so using a HashSet to add them all will work
        //fine (no duplicates)
        elementRoles.addAll(getSecurityRoles(securityRoleIds));

    }

    /**
     * @param rpmObject
     * @return
     */
    public Set getLoadedSecurityRolesSet(RPMObject rpmObject)
    {
        if (!securityRoleByType.isEmpty())
        {
            Map objectIdsToSecurityRoleMap = (Map) SecurityControllerUtil
                .getValueFromMap(rpmObject.getClass(), securityRoleByType);
            if (objectIdsToSecurityRoleMap != null)
            {
                return (Set) objectIdsToSecurityRoleMap.get(rpmObject.getID());
            }
        }
        return null;
    }

    public void associateWithEmptyRoleSet(RPMObject rpmObject)
    {
        Map objectIdsToSecurityRoleMap = (Map) securityRoleByType.get(rpmObject
            .getClass());
        if (objectIdsToSecurityRoleMap == null)
        {
            objectIdsToSecurityRoleMap = new HashMap();
            securityRoleByType.put(rpmObject.getClass(),
                objectIdsToSecurityRoleMap);
        }
        objectIdsToSecurityRoleMap
            .put(rpmObject.getID(), Collections.EMPTY_SET);
    }

    public Map getGenericCacheMap()
    {
        return genericCacheMap;
    }

    public Map getRPMObjectCacheMap(RPMObject rpmObject)
    {
        RPMObjectKey key = new RPMObjectKey(rpmObject);
        Map map = (Map) rpmObjectToCacheMaps.get(key);
        if (map == null)
        {
            map = new HashMap();
            rpmObjectToCacheMaps.put(key, map);
        }
        return map;
    }

    public Map getContainerSecurityDescriptorsMap()
    {
        return containerSecurityDescriptorsMap;
    }

}
