package com.ibm.rpm.framework;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.types.SeverityLevel;

/**
 * Extension to org.apache.commons.logging.Log. Adds the "severe" and "critical"
 * error levels. Logs trace, debug, info, warning and error and severe error
 * messages in the "regularLog". Logs only critical errors and fatal errors in
 * the "criticalLog".
 * 
 * <h3>Severity Levels</h3>
 * <p>
 * <table border=1 cellspacing=0>
 * <tr>
 * <th>Severity</th>
 * <th>Meaning</th>
 * </tr>
 * <tr>
 * <td>Information</td>
 * <td>Indicates a situation that is normal but worth a notification.</td>
 * </tr>
 * <tr>
 * <td>Warning</td>
 * <td>Indicates a situation that is abnormal but not sufficient to make the
 * operation fail.</td>
 * </tr>
 * <tr>
 * <td>Error</td>
 * <td>Indicates an error situation that will cause the operation to fail. For
 * instance, many such errors can accumulate during the validation phase.</td>
 * </tr>
 * <tr>
 * <td>SevereError</td>
 * <td>Indicates a severe error situation that will cause the operation to fail
 * and terminate immediately.</td>
 * </tr>
 * <tr>
 * <td>CriticalError</td>
 * <td>Indicates a critical error situation that might cause RPM Web service to
 * become unstable or unusable.</td>
 * </tr>
 * <tr>
 * <td>FatalError</td>
 * <td>Indicates a fatal (unrecoverable) error situation that will cause RPM
 * Web service to stop.</td>
 * </tr>
 * </table>
 * 
 * @author legaults
 */
public class RpmLog implements Log
{
    // =======
    // FACTORY
    // =======

    private static final RpmLog instance = new RpmLog();

    public static RpmLog getLog()
    {
        return instance;
    }

    // =======
    // LOGGING
    // =======

    private Log regularLog = LogFactory.getLog("com.ibm.rpm.regularLog");

    private Log criticalLog = LogFactory.getLog("com.ibm.rpm.criticalLog");

    public void log(SeverityLevel severity, Object message, Throwable t)
    {
        if (SeverityLevel.Information.equals(severity))
        {
            info(message, t);
        }
        else if (SeverityLevel.Warning.equals(severity))
        {
            warn(message, t);
        }
        else if (SeverityLevel.Error.equals(severity))
        {
            error(message, t);
        }
        else if (SeverityLevel.SevereError.equals(severity))
        {
            severe(message, t);
        }
        else if (SeverityLevel.CriticalError.equals(severity))
        {
            critical(message, t);
        }
        else if (SeverityLevel.FatalError.equals(severity))
        {
            fatal(message, t);
        }
        else
        {
            error(message, t);
        }
    }

    public boolean isSevereEnabled()
    {
        return regularLog.isErrorEnabled();
    }

    public void severe(Object arg0)
    {
        regularLog.error(arg0);
    }

    public void severe(Object arg0, Throwable arg1)
    {
        regularLog.error(arg0, arg1);
    }

    public boolean isCriticalEnabled()
    {
        return criticalLog.isErrorEnabled();
    }

    public void critical(Object arg0)
    {
        criticalLog.error(arg0);
    }

    public void critical(Object arg0, Throwable arg1)
    {
        criticalLog.error(arg0, arg1);
    }

    public boolean isFatalEnabled()
    {
        return criticalLog.isFatalEnabled();
    }

    public void fatal(Object arg0)
    {
        criticalLog.fatal(arg0);
    }

    public void fatal(Object arg0, Throwable arg1)
    {
        criticalLog.fatal(arg0, arg1);
    }

    public void debug(Object arg0)
    {
        regularLog.debug(arg0);
    }

    public void debug(Object arg0, Throwable arg1)
    {
        regularLog.debug(arg0, arg1);
    }

    public boolean equals(Object obj)
    {
        return regularLog.equals(obj);
    }

    public void error(Object arg0)
    {
        regularLog.error(arg0);
    }

    public void error(Object arg0, Throwable arg1)
    {
        regularLog.error(arg0, arg1);
    }

    public int hashCode()
    {
        return regularLog.hashCode();
    }

    public void info(Object arg0)
    {
        regularLog.info(arg0);
    }

    public void info(Object arg0, Throwable arg1)
    {
        regularLog.info(arg0, arg1);
    }

    public boolean isDebugEnabled()
    {
        return regularLog.isDebugEnabled();
    }

    public boolean isErrorEnabled()
    {
        return regularLog.isErrorEnabled();
    }

    public boolean isInfoEnabled()
    {
        return regularLog.isInfoEnabled();
    }

    public boolean isTraceEnabled()
    {
        return regularLog.isTraceEnabled();
    }

    public boolean isWarnEnabled()
    {
        return regularLog.isWarnEnabled();
    }

    public String toString()
    {
        return regularLog.toString();
    }

    public void trace(Object arg0)
    {
        regularLog.trace(arg0);
    }

    public void trace(Object arg0, Throwable arg1)
    {
        regularLog.trace(arg0, arg1);
    }

    public void warn(Object arg0)
    {
        regularLog.warn(arg0);
    }

    public void warn(Object arg0, Throwable arg1)
    {
        regularLog.warn(arg0, arg1);
    }
}
