/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller;

import java.util.ArrayList;
import java.util.Map;

import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;

/**
 * @author Yasser Lulu
 *
 */
public interface ISecurityController
{
    ISecurityController INSTANCE = new SecurityControllerDispatcher();

    void fillContainerSecurityDescriptors(Map containerSecurityDescriptorsMap,
            MessageContext messageContext) throws RPMException;

    SecurityValidationResult canRead(RPMObject rpmObject, String fieldName,
            MessageContext messageContext) throws RPMException;

    /**
     * Returns if user has the rights to edit the value in an object's field.
     * @param object, fieldName, messageContext 
     */
    SecurityValidationResult canWrite(RPMObject rpmObject, String fieldName,
            MessageContext messageContext) throws RPMException;

    /**
     * @param messageContext
     * @param rpmObject
     * @return SecurityValidationResult
     */
    SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException;

    /**
     * @param messageContext
     * @param rpmObject
     * @param rpmObjectScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canUpdate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException;

    /**
     * @param messageContext
     * @param rpmObject
     * @param rpmObjectScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException;

    //DOES NOT ADD EXCEPTION to message context
    SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException;

    SecurityValidationResult canUpdateOnCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmScope) throws RPMException;

    /**
     * @param messageContext
     * @param loadedObjects
     * @return
     */
    ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects) throws RPMException;

    //IMPORTANT NOTE: returns if the rpmObject itself can be viewed, and if -and only if- the rpmObject 
    //is Viewable, it will nullify non-viewable fields of the rpmObject itself, since otherwise,
    //the whole object is non-viewwable and will be filtered out, so, 
    //there is no point in filtering its fields.  
    SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException;

    SecurityValidationResult restoreRelevantFilteredFields(
            RestoreRelevantFilteredFieldsParameter param) throws RPMException;

    /**
     * @param messageContext
     * @param taskAssignment
     * @param scope
     * @param acceptEETC
     * @param acceptForecastDates
     * @return SecurityValidationResult
     */
    SecurityValidationResult canAcceptTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean acceptEETC,
            boolean acceptForecastDates) throws RPMException;

    /**
     * @param messageContext
     * @param objectToAssign
     * @param objectToAssignScope
     * @param scoreCardToAssign
     * @param reloadType
     * @return SecurityValidationResult
     */
    SecurityValidationResult canAssignScoreCard(MessageContext messageContext,
            RPMObject objectToAssign, RPMObjectScope objectToAssignScope,
            Scorecard scoreCardToAssign, ReloadType reloadType)
            throws RPMException;

    /**
     * @param messageContext
     * @param process
     * @param genericProject
     * @param associate
     * @param isDefault
     * @return SecurityValidationResult
     */
    SecurityValidationResult canAssociateWorkflow(
            MessageContext messageContext, WorkflowProcess process,
            GenericProject genericProject, boolean associate, boolean isDefault)
            throws RPMException;

    /**
     * @param messageContext
     * @param runningProcess
     * @return SecurityValidationResult
     */
    SecurityValidationResult canCancelWorkflow(MessageContext messageContext,
            RunningWorkflowProcess runningProcess) throws RPMException;//XXX required!

    /**
     * @param messageContext
     * @param rpmObject
     * @param isTimeless
     * @return SecurityValidationResult
     */
    SecurityValidationResult canCheckOut(MessageContext messageContext,
            RPMObject rpmObject, boolean isTimeless) throws RPMException;

    /**
     * @param messageContext
     * @param inputContainer
     * @param outputContainer
     * @param rpmScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canConvertContainer(MessageContext messageContext,
            RPMObject inputContainer, RPMObject outputContainer,
            RPMObjectScope rpmScope) throws RPMException;

    /**
     * @param messageContext
     * @param resourceTaskAssignment
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canCopyAssignmentProposedToPlan(
            MessageContext messageContext,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope) throws RPMException;

    /**
     * @param messageContext
     * @param workElement
     * @param workElementScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canCopyProposedToPlan(
            MessageContext messageContext, WorkElement workElement,
            WorkElementScope workElementScope) throws RPMException;

    /**
     * @param messageContext
     * @param inputObject
     * @param template
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canCreateObjectFromTemplate(
            MessageContext messageContext, RPMObject inputObject,
            RPMObject template, RPMObjectScope scope) throws RPMException;

    /**
     * @param messageContext
     * @param workElement
     * @param abstractScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canMoveAndTransferUnderWBS(
            MessageContext messageContext, WorkElement workElement,
            AbstractScope abstractScope) throws RPMException;

    /**
     * @param messageContext
     * @param workElement
     * @param abstractScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canMoveUnderWBS(MessageContext messageContext,
            WorkElement workElement, AbstractScope abstractScope)
            throws RPMException;

    /**
     * @param messageContext
     * @param assignedScorecard
     * @param scorecardScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canPublishScorecard(MessageContext messageContext,
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope) throws RPMException;

    /**
     * @param messageContext
     * @param taskAssignment
     * @param scope
     * @param rejectEETC
     * @param rejectForecastDates
     * @return SecurityValidationResult
     */
    SecurityValidationResult canRejectTaskAssignmentChanges(
            MessageContext messageContext, TaskAssignment taskAssignment,
            TaskAssignmentScope scope, boolean rejectEETC,
            boolean rejectForecastDates) throws RPMException;

    /**
     * @param messageContext
     * @param rta
     * @param rtaScope
     * @param resource
     * @return SecurityValidationResult
     */
    SecurityValidationResult canReplaceProfileWithResource(
            MessageContext messageContext, ResourceTaskAssignment rta,
            ResourceTaskAssignmentScope rtaScope, Resource resource)
            throws RPMException;

    /**
     * @param messageContext
     * @param assignment
     * @param rtaScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canReplaceResourceWithProfile(
            MessageContext messageContext, ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope) throws RPMException;

    /**
     * @param messageContext
     * @param runningProcess
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canResumeWorkflow(MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException;//XXX required!

    /**
     * @param messageContext
     * @param rpmObject
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canRollupProjectEV(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope scope) throws RPMException;

    /**
     * @param messageContext
     * @param rpmObject
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canSaveProjectBaseline(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope scope) throws RPMException;

    /**
     * @param messageContext
     * @param process
     * @param object
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canStartWorkflow(MessageContext messageContext,
            WorkflowProcess process, RPMObject object,
            RunningWorkflowProcessScope scope) throws RPMException;//XXX required!

    /**
     * @param messageContext
     * @param runningProcess
     * @param scope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canSuspendWorkflow(MessageContext messageContext,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope) throws RPMException;//XXX required!

    /**
     * @param messageContext
     * @param scopeElement
     * @param workElement
     * @return SecurityValidationResult
     */
    SecurityValidationResult canTransferUnderScope(
            MessageContext messageContext, AbstractAggregateScope scopeElement,
            WorkElement workElement) throws RPMException;

    /**
     * @param messageContext
     * @param workElement
     * @param abstractScope
     * @return SecurityValidationResult
     */
    SecurityValidationResult canTransferUnderWBS(MessageContext messageContext,
            WorkElement workElement, AbstractScope abstractScope)
            throws RPMException;
    
    /**
     * @param messageContext
     * @param documentToLock
     * @return SecurityValidationResult
     * @throws RPMException
     */
    SecurityValidationResult canLockDocument(MessageContext messageContext,
           GenericDocument documentToLock) throws RPMException;
    
    /**
     * @param messageContext
     * @param documentToUnlock
     * @return SecurityValidationResult
     * @throws RPMException
     */
    SecurityValidationResult canUnlockDocument(MessageContext messageContext,
            GenericDocument documentToUnlock) throws RPMException;
}