package com.ibm.rpm.framework.security.controller.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.framework.security.ContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.OperationSecurityDescriptor;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult.ValueType;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

public abstract class ContainerSecurityController
{

    private static final String NAME_FIELD = "name";

    private static final CheckOutOperationMappingEntry DEFAULT_CHECKOUT_OPERATION = new CheckOutOperationMappingEntry(
        RPMObject.class);

    private static final DeletionOperationMappingEntry DEFAULT_DELETION_OPERATION = new DeletionOperationMappingEntry(
        RPMObject.class);

    private static final ViewOperationMappingEntry DEFAULT_VIEW_OPERATION = new ViewOperationMappingEntry(
        RPMObject.class);

    private Map fieldMappingEntries;

    private Map operationMappingEntries;

    private Map creationOperationMappingEntries;

    private DeletionOperationMappingEntry deletionOperationMappingEntry;

    private ViewOperationMappingEntry viewOperationMappingEntry;

    private CheckOutOperationMappingEntry checkOutOperationMappingEntry;

    /**
     * Returns the container class controlled by the controller.
     */
    public abstract Class getContainerClass();

    public abstract Object getCategoryIdentifier();

    protected abstract ContainerSecurityController getContainerSecurityController(
            Class rpmObjectClass);

    /**
     * Returns if user has the rights to create an instance of the child object under this object.
     * @param object
     * @param securityFlags
     * @param messageContext 
     */
    public final SecurityValidationResult canCreateChild(
            RPMObject childRpmObject, CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        CreationOperationMappingEntry entry = getCreationOperationMappingEntry(childRpmObject
            .getClass());
        if (entry != null)
        {
            return entry.canPerform(childRpmObject, securityFlags,
                messageContext);
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public final SecurityValidationResult canUpdateOnCreate(
            RPMObject rpmObject, CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        FieldMappingEntry fieldEntry = null;
        try
        {
            fieldEntry = (FieldMappingEntry) getFieldMappingEntries().remove(
                NAME_FIELD);
            return canUpdate(rpmObject, securityFlags, messageContext);
        }
        finally
        {
            if (fieldEntry != null)
            {
                getFieldMappingEntries().put(fieldEntry.name, fieldEntry);
            }
        }
    }

    /**
     * Returns if user has the rights to delete an instance of the object.
     * @param object
     * @param securityFlags
     * @param messageContext 
     */
    public final SecurityValidationResult canDelete(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        return getDeletionOperationMappingEntry().canPerform(rpmObject,
            securityFlags, messageContext);
    }

    public final SecurityValidationResult canView(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        return getViewOperationMappingEntry().canPerform(rpmObject,
            securityFlags, messageContext);
    }

    public final SecurityValidationResult canCheckOut(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        return getCheckOutOperationMappingEntry().canPerform(rpmObject,
            securityFlags, messageContext);
    }

    protected final OperationMappingEntry getCheckOutOperationMappingEntry()
    {
        if (checkOutOperationMappingEntry == null)
        {
            checkOutOperationMappingEntry = generateCheckOutOperationMappingEntry();
        }
        return checkOutOperationMappingEntry;
    }

    protected CheckOutOperationMappingEntry generateCheckOutOperationMappingEntry()
    {
        return DEFAULT_CHECKOUT_OPERATION;
    }

    public final void applyFieldFiltering(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        Map cachMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(rpmObject);
        Map canReadFieldMap = (Map) cachMap.get(getCategoryIdentifier());
        if (canReadFieldMap == null)
        {
            canReadFieldMap = new HashMap();
            cachMap.put(getCategoryIdentifier(), canReadFieldMap);
        }

        Iterator it = getFieldMappingEntries().values().iterator();
        while (it.hasNext())
        {
            FieldMappingEntry entry = (FieldMappingEntry) it.next();
            if (entry.canRead(rpmObject, securityFlags, messageContext).type == SecurityValidationResult.FALSE)
            {
                canReadFieldMap.put(entry, Boolean.FALSE);
                entry.applyFiltering(rpmObject);
            }
            else
            {
                canReadFieldMap.put(entry, Boolean.TRUE);
            }
        }
    }

    public final void restoreRelevantFilteredFields(
            RestoreRelevantFilteredFieldsParameter param,
            CombinedSecurityFlags securityFlags)
    {
        Iterator it = getFieldMappingEntries().values().iterator();
        while (it.hasNext())
        {
            FieldMappingEntry entry = (FieldMappingEntry) it.next();
            entry.restoreValue(param, securityFlags);
        }
    }

    /**
     * Returns if user has the rights to save the object to the database.
     * @param object
     * @param securityFlags
     * @param messageContext
     */
    public final SecurityValidationResult canUpdate(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        Iterator it = getFieldMappingEntries().values().iterator();
        while (it.hasNext())
        {
            FieldMappingEntry entry = (FieldMappingEntry) it.next();
            if (entry.isModified(rpmObject) == SecurityValidationResult.TRUE)
            {
                SecurityValidationResult result = entry.canWrite(rpmObject,
                    securityFlags, messageContext);
                if (result.type == SecurityValidationResult.FALSE)
                {
                    return result;
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    /**
     * Returns if user has the rights to view the value in an object's field.
     * @param object
     * @param securityFlags
     * @param messageContext
     */
    public final SecurityValidationResult canRead(RPMObject rpmObject,
            String fieldName, CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        FieldMappingEntry entry = (FieldMappingEntry) getFieldMappingEntries()
            .get(fieldName);
        if (entry != null)
            return entry.canRead(rpmObject, securityFlags, messageContext);
        else
            return SecurityValidationResult.UNDEFINED_RESULT;
    }

    /**
     * Returns if user has the rights to edit the value in an object's field.
     * @param object
     * @param securityFlags
     * @param messageContext
     */
    public final SecurityValidationResult canWrite(RPMObject rpmObject,
            String fieldName, CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        FieldMappingEntry entry = (FieldMappingEntry) getFieldMappingEntries()
            .get(fieldName);
        if (entry != null)
            return entry.canWrite(rpmObject, securityFlags, messageContext);
        else
            return SecurityValidationResult.UNDEFINED_RESULT;
    }

    /**
     * Returns if user has the rights to edit the value in an object's field.
     * @param object
     * @param securityFlags
     * @param messageContext
     */
    public final SecurityValidationResult canPerform(RPMObject rpmObject,
            String operationName, CombinedSecurityFlags securityFlags,
            MessageContext messageContext)
    {
        OperationMappingEntry entry = (OperationMappingEntry) getOperationMappingEntries()
            .get(operationName);
        if (entry != null)
            return entry.canPerform(rpmObject, securityFlags, messageContext);
        else
            return SecurityValidationResult.UNDEFINED_RESULT;
    }

    /**
     * Returns if any field in this container has been modified.
     * @param object
     * @param securityFlags
     * @param messageContext
     */
    public final ValueType isModified(RPMObject rpmObject)
    {
        Iterator it = getFieldMappingEntries().values().iterator();
        while (it.hasNext())
        {
            FieldMappingEntry entry = (FieldMappingEntry) it.next();
            if (entry.isModified(rpmObject) == SecurityValidationResult.TRUE)
            {
                return SecurityValidationResult.TRUE;
            }
        }
        return SecurityValidationResult.FALSE;
    }

    /**
     * Fills a map with creation operation mapping entries for creating all the children
     * types of this container's field.
     */
    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
    }

    /**
     * Returns an operation entry for the deletion of the container.
     */
    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        return DEFAULT_DELETION_OPERATION;
    }

    /**
     * Returns an operation entry for the viewing of the container.
     */
    protected ViewOperationMappingEntry generateViewOperationMappingEntry()
    {
        return DEFAULT_VIEW_OPERATION;
    }

    /**
     * Fills a map with field mapping entries for all the field in the container.
     */
    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
    }

    /**
     * Fills a map with field mapping entries for all the operations that affect
     * the container (other than create or delete).
     */
    protected void generateOperationMappingEntries(Map mappingEntryMap)
    {
    }

    /**
     * Returns a mapping entry for the deletion of the container.
     */
    protected final Map getCreationOperationMappingEntries()
    {
        if (creationOperationMappingEntries == null)
        {
            creationOperationMappingEntries = new HashMap();
            generateCreationOperationMappingEntries(creationOperationMappingEntries);
        }
        return creationOperationMappingEntries;
    }

    private final CreationOperationMappingEntry getCreationOperationMappingEntry(
            Class rpmObjectClass)
    {
        return (CreationOperationMappingEntry) SecurityControllerUtil
            .getValueFromMap(rpmObjectClass,
                getCreationOperationMappingEntries());
    }

    /**
     * Returns a mapping entry for the deletion of the container.
     */
    protected final DeletionOperationMappingEntry getDeletionOperationMappingEntry()
    {
        if (deletionOperationMappingEntry == null)
        {
            deletionOperationMappingEntry = generateDeletionOperationMappingEntry();
        }
        return deletionOperationMappingEntry;
    }

    protected final OperationMappingEntry getViewOperationMappingEntry()
    {
        if (viewOperationMappingEntry == null)
        {
            viewOperationMappingEntry = generateViewOperationMappingEntry();
        }
        return viewOperationMappingEntry;
    }

    /**
     * Returns a map of field mapping entries for all the field in the container.
     * Was made public because of tests.
     */
    public final Map getFieldMappingEntries()
    {
        if (fieldMappingEntries == null)
        {
            fieldMappingEntries = new HashMap();
            generateFieldMappingEntries(fieldMappingEntries);
        }
        return fieldMappingEntries;
    }

    /**
     * Returns a map of operation mapping entries for all the operations
     * that affect the container (other than create or delete).
     * Was made public because of tests.
     */
    public final Map getOperationMappingEntries()
    {
        if (operationMappingEntries == null)
        {
            operationMappingEntries = new HashMap();
            generateOperationMappingEntries(operationMappingEntries);
        }
        return operationMappingEntries;
    }

    protected Map buildFieldSecurityDescriptor(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        if (getFieldMappingEntries().isEmpty())
        {
            return Collections.EMPTY_MAP;
        }

        Map cachMap = messageContext.getRPMSecurityCache()
            .getRPMObjectCacheMap(objectInfo.rpmObject);
        Map canReadFieldMap = (Map) cachMap.get(getCategoryIdentifier());
        if (canReadFieldMap == null)
        {
            canReadFieldMap = Collections.EMPTY_MAP;
        }

        Map fieldSecurityDescrs = new HashMap();
        Iterator it = getFieldMappingEntries().values().iterator();
        while (it.hasNext())
        {
            FieldMappingEntry fieldMappingEntry = (FieldMappingEntry) it.next();

            Boolean cachedCanViewResult = (Boolean) canReadFieldMap
                .get(fieldMappingEntry);

            FieldSecurityDescriptor fieldSecurityDescr = new FieldSecurityDescriptor();
            fieldSecurityDescr.setName(fieldMappingEntry.name);
            if (cachedCanViewResult != null)
            {
                fieldSecurityDescr.setCanView(cachedCanViewResult
                    .booleanValue());
            }
            else
            {
                fieldSecurityDescr.setCanView(fieldMappingEntry
                    .canRead(objectInfo.rpmObject, combinedSecurityFlags,
                        messageContext).type != SecurityValidationResult.FALSE);
            }

            //As far as WebUI is concerned, if the user can't view, the he shouldn't be able to edit
            //But, could this break WebUI???
            if (fieldSecurityDescr.isCanView())
            {
                fieldSecurityDescr.setCanEdit(fieldMappingEntry
                    .canWrite(objectInfo.rpmObject, combinedSecurityFlags,
                        messageContext).type != SecurityValidationResult.FALSE);
            }
            else
            {
                fieldSecurityDescr.setCanEdit(false);
            }

            fieldSecurityDescrs.put(fieldMappingEntry.name, fieldSecurityDescr);
        }
        return fieldSecurityDescrs;

    }

    protected Map buildCreationOperationSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        if (getCreationOperationMappingEntries().isEmpty())
        {
            return Collections.EMPTY_MAP;
        }

        Map creationOperationSecurityDescrs = new HashMap();
        Iterator it = getCreationOperationMappingEntries().values().iterator();
        while (it.hasNext())
        {
            CreationOperationMappingEntry creationOperationMappingEntry = (CreationOperationMappingEntry) it
                .next();
            creationOperationMappingEntry
                .buildCreationOperationSecurityDescriptors(
                    creationOperationSecurityDescrs, objectInfo,
                    combinedSecurityFlags, messageContext);
        }
        return creationOperationSecurityDescrs;
    }

    protected Map buildOperationSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        if (getOperationMappingEntries().isEmpty())
        {
            return Collections.EMPTY_MAP;
        }

        Map operationSecurityDescrs = new HashMap();
        Iterator it = getOperationMappingEntries().values().iterator();
        while (it.hasNext())
        {
            OperationMappingEntry operationMappingEntry = (OperationMappingEntry) it
                .next();
            OperationSecurityDescriptor operationSecurityDescr = new OperationSecurityDescriptor();
            operationSecurityDescr.setName(operationMappingEntry.name);
            operationSecurityDescr.setCanPerform(operationMappingEntry
                .canPerform(objectInfo.rpmObject, combinedSecurityFlags,
                    messageContext).type != SecurityValidationResult.FALSE);
            operationSecurityDescrs.put(operationMappingEntry.name,
                operationSecurityDescr);
        }
        return operationSecurityDescrs;
    }

    protected Map buildRTFSecurityDescriptors(ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        return Collections.EMPTY_MAP;
    }

    protected Map buildCustomFieldSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        return Collections.EMPTY_MAP;
    }

    public final void buildContainerSecurityDescriptor(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext, Map containerSecurityDescriptorsMap,
            final Set flatRPMObjectInfos)
    {
        ContainerSecurityDescriptor containerSecurityDescriptor = new ContainerSecurityDescriptor(
            objectInfo.rpmObject.getID());
        //----------------------------------------------------------
        // Delete flag
        containerSecurityDescriptor
            .setCanDelete(getDeletionOperationMappingEntry().canPerform(
                objectInfo.rpmObject, combinedSecurityFlags, messageContext).type != SecurityValidationResult.FALSE);

        // -----------------------------------------------------------
        // Fields
        Map fieldDescrs = buildFieldSecurityDescriptor(objectInfo,
            combinedSecurityFlags, messageContext);
        if (!fieldDescrs.isEmpty())
        {
            containerSecurityDescriptor
                .setFieldSecurityDescriptors(fieldDescrs);
        }

        // -----------------------------------------------------------
        // Operations
        Map operationDescrs = buildOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
        if (!operationDescrs.isEmpty())
        {
            containerSecurityDescriptor
                .setOperationSecurityDescriptors(operationDescrs);
        }

        // -----------------------------------------------------------
        // Creation Operations
        Map creationOperationDescrs = buildCreationOperationSecurityDescriptors(
            objectInfo, combinedSecurityFlags, messageContext);
        if (!creationOperationDescrs.isEmpty())
        {
            containerSecurityDescriptor
                .setCreationOperationSecurityDescriptors(creationOperationDescrs);
        }

        // -----------------------------------------------------------
        // Rich Text Field
        Map rtfDescrs = buildRTFSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
        if (!rtfDescrs.isEmpty())
        {
            containerSecurityDescriptor.setRTFSecurityDescriptors(rtfDescrs);
        }

        // -----------------------------------------------------------
        // Custom Field
        Map customFieldDescrs = buildCustomFieldSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
        if (!customFieldDescrs.isEmpty())
        {
            containerSecurityDescriptor
                .setCustomFieldSecurityDescriptors(customFieldDescrs);
        }
        // Add descriptor to map
        containerSecurityDescriptorsMap.put(objectInfo.rpmObject,
            containerSecurityDescriptor);
    }

}
