package com.ibm.rpm.framework.security.controller.impl;

import java.lang.reflect.Method;
import java.text.MessageFormat;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult.ValueType;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.metadata.model.Field;
import com.ibm.rpm.metadata.model.MetadataStore;

public class FieldMappingEntry extends AbstractSecurityMappingEntry
{
    protected static final Class[] NULL_PARAM = new Class[]
    {
        null
    };

    protected String viewErrorMessage;

    protected String editErrorMessage;

    public Integer[] canReadSecurityFlags;

    public Integer[] canWriteSecurityFlags;

    protected Field field;

    public FieldMappingEntry(String fieldName, Class container)
    {
        super(fieldName, container);
        field = MetadataStore.getInstance().getContainerByJavaClass(container)
            .getField(fieldName);
        if (field == null)
        {
            throw new IllegalArgumentException("Field \"" + fieldName
                    + "\" does not exist in container \"" + container.getName()
                    + "\"");
        }
    }

    /**
     * Returns true if at least one of the fields in rpmObject has been
     * filtered.
     */
    public ValueType applyFiltering(RPMObject rpmObject)
    {
        try
        {
            field.getSetterMethod().invoke(rpmObject, NULL_PARAM);
            return SecurityValidationResult.TRUE;
        }
        catch (Exception e)
        {
            return SecurityValidationResult.FALSE;
        }
    }

    /**
     * Returns true if the field was modified inside the rpmObject.
     */
    public ValueType isModified(RPMObject rpmObject)
    {
        try
        {
            if (!field.isArrayType())
            {
                Method method = field.getTestModifiedMethod();
                return (method != null) ? (((Boolean) method.invoke(rpmObject,
                    null)).booleanValue()) ? SecurityValidationResult.TRUE
                        : SecurityValidationResult.FALSE
                        : SecurityValidationResult.UNDEFINED;
            }
            return SecurityValidationResult.UNDEFINED;
        }
        catch (Exception e)
        {
            return SecurityValidationResult.FALSE;
        }
    }

    public ValueType restoreValue(RestoreRelevantFilteredFieldsParameter param,
            CombinedSecurityFlags securityFlags)
    {
        try
        {
            Object value = field.getGetterMethod().invoke(param.getRpmObject(),
                null);
            if (value == null)
            {
                if (canRead(param.getRpmObject(), securityFlags, param
                    .getMessageContext()).type == SecurityValidationResult.FALSE)
                {
                    Object otherValue = field.getGetterMethod().invoke(
                        param.getOtherRpmObject(), null);
                    if (otherValue != null)
                    {
                        field.getSetterMethod().invoke(param.getRpmObject(),
                            new Object[]
                            {
                                otherValue
                            });
                    }
                }
            }
            return SecurityValidationResult.TRUE;
        }
        catch (Exception e)
        {
            return SecurityValidationResult.FALSE;
        }
    }

    /**
     * Returns false if all the flags in readSecurityFlags array are false, if
     * at least one is true, return true.
     */
    public SecurityValidationResult canRead(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (securityFlags.isALL_FALSE())
        {
            return SecurityValidationResult.makeFalseResult(MessageFormat
                .format(getViewErrorMessage(), new Object[]
                {
                    StringUtil.getShortClassName(rpmObject.getClass())
                }));
        }
        if (securityFlags.isALL_TRUE())
        {
            return SecurityValidationResult.UNDEFINED_RESULT;
        }
        return SecurityControllerUtil.calculateFlagResultUsingOr(
            canReadSecurityFlags, securityFlags, getViewErrorMessage(),
            rpmObject.getClass());
    }

    /**
     * Returns false if all the flags in writeSecurityFlags array are false, if
     * at least one is true, return true.
     */
    public SecurityValidationResult canWrite(RPMObject rpmObject,
            CombinedSecurityFlags securityFlags, MessageContext messageContext)
    {
        if (securityFlags.isALL_FALSE())
        {
            return SecurityValidationResult.makeFalseResult(MessageFormat
                .format(getEditErrorMessage(), new Object[]
                {
                    StringUtil.getShortClassName(rpmObject.getClass())
                }));
        }
        if (securityFlags.isALL_TRUE())
        {
            return SecurityValidationResult.UNDEFINED_RESULT;
        }
        return SecurityControllerUtil.calculateFlagResultUsingOr(
            canWriteSecurityFlags, securityFlags, getEditErrorMessage(),
            rpmObject.getClass());
    }

    protected String getViewErrorMessage()
    {
        if (viewErrorMessage == null)
        {
            viewErrorMessage = "view field " + name + " of {0}"; // {0} will be replaced with container type
        }
        return viewErrorMessage;
    }

    protected String getEditErrorMessage()
    {
        if (editErrorMessage == null)
        {
            editErrorMessage = "edit field " + name + " of {0}"; // {0} will be replaced with container type
        }
        return editErrorMessage;
    }

}
