/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.RestoreRelevantFilteredFieldsParameter;
import com.ibm.rpm.framework.security.NullContainerSecurityDescriptor;
import com.ibm.rpm.framework.security.controller.BaseSecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;

/**
 * @author Yasser Lulu
 * 
 */
public abstract class SecurityControllerImpl extends BaseSecurityController
{

    protected static final void addContainerController(
            ContainerSecurityController controller, Map controllerMap)
    {
        controllerMap.put(controller.getContainerClass(), controller);
    }

    public void doFillContainerSecurityDescriptors(
            Map containerSecurityDescriptorsMap, Set flatRPMObjectInfos,
            MessageContext messageContext) throws RPMException
    {
        Iterator rpmObjectsIt = flatRPMObjectInfos.iterator();
        while (rpmObjectsIt.hasNext())
        {
            ReferencedObjectInfo referencedObjectInfo = (ReferencedObjectInfo) rpmObjectsIt
                .next();
            RPMObject rpmObject = referencedObjectInfo.rpmObject;
            ContainerSecurityController controller = getContainerSecurityController(rpmObject
                .getClass());
            if (controller != null)
            {
                CombinedSecurityFlags combinedSecurityFlags = getCombinedSecurityFlags(
                    rpmObject, messageContext);
                if (combinedSecurityFlags != null)
                {
                    controller.buildContainerSecurityDescriptor(
                        referencedObjectInfo, combinedSecurityFlags,
                        messageContext, containerSecurityDescriptorsMap,
                        flatRPMObjectInfos);
                }
                else
                {
                    containerSecurityDescriptorsMap.put(rpmObject,
                        new NullContainerSecurityDescriptor(rpmObject.getID()));
                }
            }
            else
            {
                containerSecurityDescriptorsMap.put(rpmObject,
                    new NullContainerSecurityDescriptor(rpmObject.getID()));
            }
        }
    }

    public void fillContainerSecurityDescriptors(
            Map containerSecurityDescriptorsMap, MessageContext messageContext)
            throws RPMException
    {
        Set flatRPMObjectInfos = SecurityControllerUtil
            .getFilteringResultFlatRPMObjectInfos(messageContext, this);

        if (flatRPMObjectInfos == null)
        {
            flatRPMObjectInfos = SecurityControllerUtil
                .getFlatRPMObjectInfos(messageContext);
        }

        doFillContainerSecurityDescriptors(containerSecurityDescriptorsMap,
            flatRPMObjectInfos, messageContext);

    }

    public ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects) throws RPMException
    {

        ArrayList filteringResult = SecurityControllerUtil.applyFiltering(
            messageContext, loadedObjects, this);
        if (filteringResult != null)
        {
            return filteringResult;
        }
        return loadedObjects;
    }

    protected abstract CombinedSecurityFlags getCombinedSecurityFlags(
            RPMObject rpmObject, MessageContext messageContext)
            throws RPMException;

    protected abstract ContainerSecurityController getContainerSecurityController(
            Class rpmObjectClass);

    public SecurityValidationResult canRead(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canRead(rpmObject, fieldName, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canView(rpmObject, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canCheckOut(MessageContext messageContext,
            RPMObject rpmObject, boolean isTimeless) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canCheckOut(rpmObject, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;

    }

    public SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        return applyFieldFiltering(rpmObject, messageContext, false);
    }

    public SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext, boolean always) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                if (always)
                {
                    controller.applyFieldFiltering(rpmObject, securityFlags,
                        messageContext);
                }
                else
                {
                    SecurityValidationResult result = controller.canView(
                        rpmObject, securityFlags, messageContext);
                    if (result.type != SecurityValidationResult.FALSE)
                    {
                        controller.applyFieldFiltering(rpmObject,
                            securityFlags, messageContext);
                    }
                    return result;
                }
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canWrite(RPMObject rpmObject,
            String fieldName, MessageContext messageContext)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canWrite(rpmObject, fieldName, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canDelete(rpmObject, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canUpdate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canUpdate(rpmObject, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult canUpdateOnCreate(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObjectScope rpmScope) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(rpmObject
            .getClass());
        if (controller != null)
        {
            if (rpmObject.getParent() != null)
            {
                SecurityControllerUtil.cacheParentOfCreatedRPMObject(
                    messageContext, rpmObject, rpmObject.getParent());
            }

            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                rpmObject, messageContext);
            if (securityFlags != null)
            {
                return controller.canUpdateOnCreate(rpmObject, securityFlags,
                    messageContext);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

    public SecurityValidationResult restoreRelevantFilteredFields(
            RestoreRelevantFilteredFieldsParameter param) throws RPMException
    {
        ContainerSecurityController controller = getContainerSecurityController(param
            .getRpmObject().getClass());
        if (controller != null)
        {
            CombinedSecurityFlags securityFlags = getCombinedSecurityFlags(
                param.getRpmObject(), param.getMessageContext());
            if (securityFlags != null)
            {
                controller.restoreRelevantFilteredFields(param, securityFlags);
            }
        }
        return SecurityValidationResult.UNDEFINED_RESULT;
    }

}
