/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.igs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.RtfAssignment;
import com.ibm.rpm.customfield.containers.CustomFieldAssignment;
import com.ibm.rpm.document.containers.DocumentBlobDetails;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.scope.DocumentScope;
import com.ibm.rpm.financial.containers.AbstractFinancial;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.RpmResult;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.LoadingResult;
import com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl;
import com.ibm.rpm.resource.containers.ResourceRoleAssignment;
import com.ibm.rpm.scopemanagement.containers.Duplicate;
import com.ibm.rpm.scopemanagement.containers.MitigationFactor;
import com.ibm.rpm.scopemanagement.containers.ReqProConnection;
import com.ibm.rpm.scopemanagement.containers.ReqProImportStatus;
import com.ibm.rpm.scopemanagement.containers.RiskMatrix;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.scope.ScopeElementScope;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.types.ProjectSecurityRoleType;
import com.ibm.rpm.timesheet.containers.DefaultStep;
import com.ibm.rpm.timesheet.containers.GenericStep;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.ElementDependency;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Opportunity;
import com.ibm.rpm.wbs.containers.ProjectOrganizationalAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;

/**
 * @author Yasser Lulu
 *
 */
public class IGSSecurityControllerImpl extends
        ResourceBasedSecurityControllerImpl
{
    private static final int UNDEFINED = 0;

    private static final int NONE = 1;

    private static final int ASSIGNED = 2;

    private static final int MANAGER = 3;

    private int extractResourceRoleType(Set securityRoles)
    {
        if (securityRoles != null && !securityRoles.isEmpty())
        {
            Iterator roleIt = securityRoles.iterator();
            while (roleIt.hasNext())
            {
                Object role = roleIt.next();
                if (role instanceof ProjectSecurityRole
                        && (((ProjectSecurityRole) role).getType() == ProjectSecurityRoleType.ProjectManager))
                {
                    return MANAGER;
                }
            }
            return ASSIGNED;
        }
        return NONE;
    }

    private int getResourceRoleTypeFromGenericProject(
            MessageContext messageContext, GenericProject rpmObject)
    {
        //optimistic check, may be it is there already
        if (messageContext.getLoadParams() != null)
        {
            Set securityRoles = messageContext.getRPMSecurityCache().getLoadedSecurityRolesSet(
                rpmObject);
            int type = extractResourceRoleType(securityRoles);
            if (type != NONE)
            {
                return type;
            }
        }

        LoadingResult result = SecurityControllerUtil.loadByPrimaryKey(
            rpmObject, null, messageContext, true);

        if ((result.loadedRPMObject == null) || !messageContext.isSuccessful())
        {
            return NONE;
        }

        Set securityRoles = messageContext.getRPMSecurityCache().getLoadedSecurityRolesSet(
            rpmObject);
        return extractResourceRoleType(securityRoles);
    }

    private int getResourceRoleTypeFromGenericStep(
            MessageContext messageContext, GenericStep genericStep)
            throws RPMException
    {
        if (genericStep.getID() == null)
        {
            if (genericStep.getProject() != null)
            {
                return getResourceRoleTypeFromGenericProject(messageContext,
                    genericStep.getProject());
            }
            else if (genericStep.getWorkElement() != null)
            {
                return getResourceRoleType(messageContext, genericStep
                    .getWorkElement());
            }
            else
            {
                return getResourceRoleType(messageContext, genericStep
                    .getTaskAssignment());
            }
        }

        //ignore the project value passed and load the project a fresh
        //to be sure that no one tried to pass in a different project
        StepScope scope = new StepScope();
        scope.setProject(true);
        GenericStep loadedStep = (GenericStep) SecurityControllerUtil
            .loadByPrimaryKey(genericStep, scope, messageContext, false).loadedRPMObject;
        if (loadedStep == null || loadedStep.getProject() == null)
        {
            return NONE;
        }
        return getResourceRoleTypeFromGenericProject(messageContext, loadedStep
            .getProject());
    }

    //XXX IMPORTANT: check if we can use the corresponding functions in the SecurityControllerUtil
    private int getResourceRoleType(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        if (rpmObject == null)
        {
            return NONE;
        }
        if (rpmObject instanceof GenericProject)
        {
            if (rpmObject.getID() == null)
            {
                return UNDEFINED;//Project creation is special case, there is a flag for it
            }
            return getResourceRoleTypeFromGenericProject(messageContext,
                (GenericProject) rpmObject);
        }
        else if (rpmObject instanceof WorkElement)
        {
            return getResourceRoleTypeFromNonProjectWorkElement(messageContext,
                (WorkElement) rpmObject);
        }
        else if (rpmObject instanceof ScopeElement)
        {
            return getResourceRoleTypeFromScopeElement(messageContext,
                (ScopeElement) rpmObject);
        }
        else if (rpmObject instanceof DocumentElement)
        {
            return getResourceRoleTypeFromDocumentElement(messageContext,
                (DocumentElement) rpmObject);
        }
        else if (rpmObject instanceof GenericStep)
        {
            return getResourceRoleTypeFromGenericStep(messageContext,
                (GenericStep) rpmObject);
        }
        else if (rpmObject instanceof SummaryTimesheet)
        {
            return getResourceRoleTypeFromSummaryTimesheet(messageContext,
                (SummaryTimesheet) rpmObject);
        }
        else if (rpmObject instanceof ProjectOrganizationalAssignment)
        {
            //The manager for this ProjectOrganizationalAssignment does not support loading the parent
            //which is the Project, so we have to do it manually in this convoluted way here
            return getResourceRoleTypeFromProjectOrganizationalAssignment(
                messageContext, (ProjectOrganizationalAssignment) rpmObject);
        }
        else if (rpmObject instanceof Opportunity
                || rpmObject instanceof RiskMatrix
                || rpmObject instanceof MitigationFactor
                || rpmObject instanceof ReqProConnection
                || rpmObject instanceof ReqProImportStatus
                || rpmObject instanceof DefaultStep
                || rpmObject instanceof AbstractFinancial
                || rpmObject instanceof TimeCodeAssignment
                || rpmObject instanceof DocumentBlobDetails
                || rpmObject instanceof WorkflowRoleMapping
                || rpmObject instanceof ResourceRoleAssignment
                || rpmObject instanceof ElementDependency
                || rpmObject instanceof Duplicate
                || rpmObject instanceof AssignedScorecard
                || rpmObject instanceof CustomFieldAssignment
                || rpmObject instanceof AttributeAssignment
                || rpmObject instanceof RtfAssignment
                || rpmObject instanceof GenericTaskAssignment)
        {
            return getResourceRoleTypeFromParent(messageContext, rpmObject);//GenericTaskAssignment
        }

        //all other rpmobjects as not related to projects
        return UNDEFINED;
    }

    /**
     * @param messageContext
     * @param assignment
     * @return
     */
    private int getResourceRoleTypeFromProjectOrganizationalAssignment(
            MessageContext messageContext,
            ProjectOrganizationalAssignment assignment) throws RPMException
    {

        //The manager for this ProjectOrganizationalAssignment does not support loading the parent
        //which is the Project, so we have to do it manually in this convoluted way here
        if (assignment.getID() == null)
        {
            return getResourceRoleType(messageContext, assignment.getParent());
        }

        RpmResult res = messageContext.getFactory().getManagerCaller().load(
            messageContext.getSession(),
            "/GenericProject[ProjectOrganizationalAssignment[id='"
                    + assignment.getID() + "']]", null);

        if (res.isSuccessful())
        {
            GenericProject proj = (GenericProject) res.getFirstObject();
            if (proj != null)
            {
                return getResourceRoleTypeFromGenericProject(messageContext,
                    proj);
            }
        }

        return NONE;
    }

    /**
     * @param messageContext
     * @param rpmObject
     * @return
     */
    private int getResourceRoleTypeFromNonProjectWorkElement(
            MessageContext messageContext, WorkElement workElement)
            throws RPMException
    {
        if (workElement.getID() == null)
        {
            return getResourceRoleType(messageContext, workElement.getParent());
        }

        WorkElementScope wbsScope = new WorkElementScope();
        wbsScope.setContainingProject(new WorkElementScope());

        WorkElement loadedWorkElement = (WorkElement) SecurityControllerUtil
            .loadByPrimaryKey(workElement, wbsScope, messageContext, false).loadedRPMObject;
        if (loadedWorkElement == null
                || loadedWorkElement.getContainingProject() == null)
        {
            return NONE;
        }
        return getResourceRoleTypeFromGenericProject(messageContext,
            (GenericProject) loadedWorkElement.getContainingProject());
    }

    /**
     * @param messageContext
     * @param rpmObject
     * @return
     */
    private int getResourceRoleTypeFromScopeElement(
            MessageContext messageContext, ScopeElement scopeElement)
            throws RPMException
    {
        if (scopeElement.getID() == null)
        {
            return getResourceRoleType(messageContext, scopeElement.getParent());
        }

        ScopeElementScope scopeScope = new ScopeElementScope();
        scopeScope.setProject(new WorkElementScope());

        ScopeElement loadedScopeElement = (ScopeElement) SecurityControllerUtil
            .loadByPrimaryKey(scopeElement, scopeScope, messageContext, false).loadedRPMObject;
        if (loadedScopeElement == null
                || loadedScopeElement.getProject() == null)
        {
            return NONE;
        }
        return getResourceRoleTypeFromGenericProject(messageContext,
            loadedScopeElement.getProject());
    }

    /**
     * @param messageContext
     * @param rpmObject
     * @return
     */
    private int getResourceRoleTypeFromDocumentElement(
            MessageContext messageContext, DocumentElement documentElement)
            throws RPMException
    {
        if (documentElement.getID() == null)
        {
            return getResourceRoleType(messageContext, documentElement
                .getParent());
        }
        DocumentScope documentScope = new DocumentScope();
        documentScope.setContainingProject(new WorkElementScope());

        DocumentElement loadedDocumentElement = (DocumentElement) SecurityControllerUtil
            .loadByPrimaryKey(documentElement, documentScope, messageContext,
                false).loadedRPMObject;
        if (loadedDocumentElement == null)
        {
            return NONE;
        }

        //XXX document elements are special case, because they need not to belong to a project
        if (loadedDocumentElement.getContainingProject() == null)
        {
            return UNDEFINED;
        }

        return getResourceRoleTypeFromGenericProject(messageContext,
            loadedDocumentElement.getContainingProject());
    }

    /**
     * @param messageContext
     * @param summaryTimesheet
     * @param rpmObjectScope
     * @param object
     * @return
     */
    private int getResourceRoleTypeFromSummaryTimesheet(
            MessageContext messageContext, SummaryTimesheet summaryTimesheet)
            throws RPMException
    {

        if (summaryTimesheet.getID() == null)
        {
            if (summaryTimesheet.getProject() != null)
            {
                return getResourceRoleTypeFromGenericProject(messageContext,
                    summaryTimesheet.getProject());
            }
            else if (summaryTimesheet.getWorkElement() != null)
            {
                return getResourceRoleType(messageContext, summaryTimesheet
                    .getWorkElement());
            }
            else
            {
                GenericTaskAssignment taskAssignment = summaryTimesheet
                    .getTaskAssignment();
                //we'll ignore Personal/AdministrativeTaskAssignment
                if (!(taskAssignment instanceof TaskAssignment))
                {
                    return UNDEFINED;//NONE;//
                }
                return getResourceRoleType(messageContext, taskAssignment);
            }
        }

        //ignore the project value passed and load the project a fresh
        //to be sure that no one tried to pass in a different project
        SummaryTimesheetScope scope = new SummaryTimesheetScope();
        scope.setProject(true);
        SummaryTimesheet loadedSummaryTimesheet = (SummaryTimesheet) SecurityControllerUtil
            .loadByPrimaryKey(summaryTimesheet, scope, messageContext, false).loadedRPMObject;
        if (loadedSummaryTimesheet == null
                || loadedSummaryTimesheet.getProject() == null)
        {
            return NONE;
        }
        return getResourceRoleTypeFromGenericProject(messageContext,
            loadedSummaryTimesheet.getProject());

    }

    private int getResourceRoleTypeFromParent(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        if (rpmObject.getID() == null)
        {
            return getResourceRoleType(messageContext, rpmObject.getParent());
        }

        RPMObjectScope scope = SecurityControllerUtil.getRPMObjectScope(
            rpmObject, new RPMObjectScope());

        if (scope == null)
        {
            return NONE;
        }

        RPMObject loadedObject = SecurityControllerUtil.loadByPrimaryKey(
            rpmObject, scope, messageContext, false).loadedRPMObject;
        if (loadedObject == null)
        {
            return NONE;
        }

        return getResourceRoleType(messageContext, loadedObject.getParent());
    }

    //################################################################
    //################################################################

    /**
     * @param messageContext
     * @param inputContainer
     * @return
     */
    private SecurityValidationResult doesResourceRoleAllowEditing(
            String operationName, MessageContext messageContext,
            RPMObject inputContainer) throws RPMException
    {
        if (!messageContext.isSuccessful() || (inputContainer == null))
        {
            return SecurityValidationResult.makeFalseResult(operationName
                    + ". Invalid input to check if user is a Project Manager");
        }
        int type = getResourceRoleType(messageContext, inputContainer);
        switch (type)
        {
        case UNDEFINED:
            return SecurityValidationResult.UNDEFINED_RESULT;
        case NONE:
        case ASSIGNED:
            return SecurityValidationResult.makeFalseResult(operationName
                    + ". The user is not a Project Manager");
        case MANAGER:
            return SecurityValidationResult.TRUE_RESULT;
        default:
            throw new RPMException("Unknown ResourceRole enum type");//should never happen        
        }
    }

    //NOTE caller must check if the call is an internal load for security, in which
    //case we allow viewing regardless of the resource-role-type
    private SecurityValidationResult doesResourceRoleAllowViewing(
            String operationName, MessageContext messageContext,
            RPMObject inputContainer) throws RPMException
    {
        if (!messageContext.isSuccessful() || (inputContainer == null))
        {
            return SecurityValidationResult.makeFalseResult(operationName
                    + ". Invalid input to check if user can view the data");
        }

        int type = getResourceRoleType(messageContext, inputContainer);
        switch (type)
        {
        case UNDEFINED:
            return SecurityValidationResult.UNDEFINED_RESULT;
        case NONE:
            return SecurityValidationResult
                .makeFalseResult("The user is not assigned to the Project");
        case ASSIGNED:
        case MANAGER:
            return SecurityValidationResult.TRUE_RESULT;
        default:
            throw new RPMException("Unknown ResourceRole enum type");//should never happen        
        }
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl#canConvertContainer(com.ibm.rpm.framework.MessageContext, com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope)
     */
    public SecurityValidationResult canConvertContainer(
            MessageContext messageContext, RPMObject inputContainer,
            RPMObject outputContainer, RPMObjectScope rpmScope)
            throws RPMException
    {
        SecurityValidationResult result = doesResourceRoleAllowEditing(
            "Perform a Convert Container operation", messageContext,
            inputContainer);
        if (result.type != SecurityValidationResult.TRUE)
        {
            SecurityValidationResult result_2 = super.canConvertContainer(
                messageContext, inputContainer, outputContainer, rpmScope);
            if (result_2.type != SecurityValidationResult.UNDEFINED)
            {
                return result_2;
            }
        }
        return result;
    }

    public SecurityValidationResult canCreateObjectFromTemplate(
            MessageContext messageContext, RPMObject rpmObject,
            RPMObject template, RPMObjectScope scope) throws RPMException
    {
        SecurityValidationResult result = doesResourceRoleAllowEditing(
            "Perform a Convert Container operation", messageContext, rpmObject);
        if (result.type != SecurityValidationResult.TRUE)
        {
            SecurityValidationResult result_2 = super
                .canCreateObjectFromTemplate(messageContext, rpmObject,
                    template, scope);
            if (result_2.type != SecurityValidationResult.UNDEFINED)
            {
                return result_2;
            }
        }
        return result;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl#canCreate(com.ibm.rpm.framework.MessageContext, com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope)
     */
    public SecurityValidationResult canCreate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        SecurityValidationResult result = doesResourceRoleAllowEditing(
            "Perform a Create operation", messageContext, rpmObject);
        if (result.type != SecurityValidationResult.TRUE)
        {
            SecurityValidationResult result_2 = super.canCreate(messageContext,
                rpmObject, rpmObjectScope);
            if (result_2.type != SecurityValidationResult.UNDEFINED)
            {
                return result_2;
            }
        }
        return result;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl#canDelete(com.ibm.rpm.framework.MessageContext, com.ibm.rpm.framework.RPMObject)
     */
    public SecurityValidationResult canDelete(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        SecurityValidationResult result = doesResourceRoleAllowEditing(
            "Perform a Delete operation", messageContext, rpmObject);
        if (result.type != SecurityValidationResult.TRUE)
        {
            SecurityValidationResult result_2 = super.canDelete(messageContext,
                rpmObject);
            if (result_2.type != SecurityValidationResult.UNDEFINED)
            {
                return result_2;
            }
        }
        return result;
    }

    /* (non-Javadoc)
     * @see com.ibm.rpm.framework.security.controller.impl.resource.ResourceBasedSecurityControllerImpl#canUpdate(com.ibm.rpm.framework.MessageContext, com.ibm.rpm.framework.RPMObject, com.ibm.rpm.framework.RPMObjectScope)
     */
    public SecurityValidationResult canUpdate(MessageContext messageContext,
            RPMObject rpmObject, RPMObjectScope rpmObjectScope)
            throws RPMException
    {
        SecurityValidationResult result = doesResourceRoleAllowEditing(
            "Perform an Update operation", messageContext, rpmObject);
        if (result.type != SecurityValidationResult.TRUE)
        {
            SecurityValidationResult result_2 = super.canUpdate(messageContext,
                rpmObject, rpmObjectScope);
            if (result_2.type != SecurityValidationResult.UNDEFINED)
            {
                return result_2;
            }
        }
        return result;
    }

    public ArrayList applyFiltering(MessageContext messageContext,
            ArrayList loadedObjects) throws RPMException
    {
        ArrayList filteredObjects = new ArrayList();
        SecurityValidationResult securityValidationResult = null;
        RPMObject rpmObject = null;
        Iterator iterObjects = loadedObjects.iterator();
        while (iterObjects.hasNext())
        {
            rpmObject = (RPMObject) iterObjects.next();

            securityValidationResult = doesResourceRoleAllowViewing("LOAD",
                messageContext, rpmObject);

            if (securityValidationResult.type != SecurityValidationResult.FALSE)
            {
                filteredObjects.add(rpmObject);
            }
        }
        return filteredObjects;
    }

    public SecurityValidationResult applyFieldFiltering(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        SecurityValidationResult securityValidationResult = doesResourceRoleAllowViewing(
            "LOAD", messageContext, rpmObject);

        if (securityValidationResult.type != SecurityValidationResult.FALSE)
        {
            return super.applyFieldFiltering(rpmObject, messageContext);
        }
        return securityValidationResult;
    }

    public SecurityValidationResult canView(RPMObject rpmObject,
            MessageContext messageContext) throws RPMException
    {
        SecurityValidationResult securityValidationResult = doesResourceRoleAllowViewing(
            "LOAD", messageContext, rpmObject);

        if (securityValidationResult.type != SecurityValidationResult.FALSE)
        {
            return super.canView(rpmObject, messageContext);
        }
        return securityValidationResult;
    }

}
