/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.rpm.framework.security.controller.impl.resource;

import java.util.HashMap;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AttributeAssignment;
import com.ibm.rpm.applicationadministration.containers.ContactGroupAssignment;
import com.ibm.rpm.applicationadministration.containers.GeographicalAssignment;
import com.ibm.rpm.applicationadministration.containers.OrganizationalAssignment;
import com.ibm.rpm.document.containers.DocumentElement;
import com.ibm.rpm.document.containers.DocumentFolder;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.security.FieldSecurityDescriptor;
import com.ibm.rpm.framework.security.SecuredFields;
import com.ibm.rpm.framework.security.SecurityFlags.RESOURCE;
import com.ibm.rpm.framework.security.SecurityFlags.VIRTUAL;
import com.ibm.rpm.framework.security.controller.impl.CombinedSecurityFlags;
import com.ibm.rpm.framework.security.controller.impl.ControllerSecuredOperations;
import com.ibm.rpm.framework.security.controller.impl.CreationOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.DeletionOperationMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.FieldMappingEntry;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil;
import com.ibm.rpm.framework.security.controller.impl.SecurityControllerUtil.ReferencedObjectInfo;
import com.ibm.rpm.resource.containers.EmploymentHistory;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.containers.ResourceAttributeAssignment;

/**
 * @author Marc-Andre Lamothe
 * 
 */

class ResourceBasedResourceSecurityController extends
        ResourceBasedContainerSecurityController
{
    public Class getContainerClass()
    {
        return Resource.class;
    }

    protected void generateCreationOperationMappingEntries(Map mappingEntryMap)
    {
        super.generateCreationOperationMappingEntries(mappingEntryMap);
        CreationOperationMappingEntry creationInfo = null;

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateAttributeAssignmentMappingEntry(
            ControllerSecuredOperations.CreateAttributeAssignment,
            getContainerClass(), AttributeAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateContactGroupAssignmentMappingEntry(
            ControllerSecuredOperations.CreateContactGroupAssignment,
            getContainerClass(), ContactGroupAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateDocumentElementMappingEntry(
            ControllerSecuredOperations.CreateDocumentElement,
            getContainerClass(), DocumentElement.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateEmploymentHistoryMappingEntry(
            ControllerSecuredOperations.CreateEmploymentHistory,
            getContainerClass(), EmploymentHistory.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateGeographicalAssignmentMappingEntry(
            ControllerSecuredOperations.CreateGeographicalAssignment,
            getContainerClass(), GeographicalAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateOrganizationalAssignmentMappingEntry(
            ControllerSecuredOperations.CreateOrganizationalAssignment,
            getContainerClass(), OrganizationalAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);

        // --------------------------------------------------------
        creationInfo = new ResourceBasedCreateResourceAttributeAssignmentMappingEntry(
            ControllerSecuredOperations.CreateResourceAttributeAssignment,
            getContainerClass(), ResourceAttributeAssignment.class);
        mappingEntryMap.put(creationInfo.childrenClass, creationInfo);
    }

    protected DeletionOperationMappingEntry generateDeletionOperationMappingEntry()
    {
        DeletionOperationMappingEntry deletionEntry = new DeletionOperationMappingEntry(
            getContainerClass());
        deletionEntry.securityFlag = RESOURCE.CanDeleteResources;
        return deletionEntry;
    }

    protected void generateFieldMappingEntries(Map mappingEntryMap)
    {
        FieldMappingEntry fieldInfo = null;

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("address1",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceAddress,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalAddress,
                RESOURCE.CanEditResourceAddress,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("address2",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceAddress,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalAddress,
                RESOURCE.CanEditResourceAddress,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("calendar",
            getContainerClass());
        //        fieldInfo.canReadSecurityFlags = new Integer[]
        //        {
        //                RESOURCE.CanViewPersonalCalendar, VIRTUAL.FALSE,
        //        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalCalendar,
                RESOURCE.CanEditResourceCalendar,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "geographicalAssignment", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalOrganizationGroupGeographical,
                RESOURCE.CanViewResourceOrganizationGroupGeographical,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalOrganizationGroupGeographical,
                RESOURCE.CanEditResourceOrganizationGroupGeographical,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("effectiveStartDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalEffectiveDiscontinueActiveDates,
                RESOURCE.CanViewResourceEffectiveDiscontinueActiveDates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalEffectiveDiscontinueActiveDates,
                RESOURCE.CanEditResourceEffectiveDiscontinueActiveDates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("familyName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceName,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalName, RESOURCE.CanEditResourceName,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("currency",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalRates, RESOURCE.CanViewResourceRates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalRates, RESOURCE.CanEditResourceRates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("givenName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceName,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalName, RESOURCE.CanEditResourceName,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("rates",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalRates, RESOURCE.CanViewResourceRates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalRates, RESOURCE.CanEditResourceRates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("fullName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, VIRTUAL.TRUE,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanEditResourceFullName,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new FieldMappingEntry("externalID", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            RESOURCE.CanEditExternalIdentifier,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("mySummaryOfSkills",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("userName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceLogon,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalUserNameEmail,
                RESOURCE.CanEditResourceUserNameEmail,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("emailAddress",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceLogon,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalUserNameEmail,
                RESOURCE.CanEditResourceUserNameEmail,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceSecurityRightsMappingEntry(
            "resourceSecurityRights", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSecurity,
                RESOURCE.CanViewResourceSecurity,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
            RESOURCE.CanEditResourceSecurity,
        };

        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("password",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceLogon,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalPassword,
                RESOURCE.CanEditResourcePassword,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("nickName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalNickName,
                RESOURCE.CanViewResourceNickName,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalNickName,
                RESOURCE.CanEditResourceNickName,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("active",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalEffectiveDiscontinueActiveDates,
                RESOURCE.CanViewResourceEffectiveDiscontinueActiveDates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalEffectiveDiscontinueActiveDates,
                RESOURCE.CanEditResourceEffectiveDiscontinueActiveDates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("countryCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("departmentCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("workLocation",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("divisionCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("divisionCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myPersonalPassions",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("workPhone",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("documentFolder",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalDocuments,
                RESOURCE.CanViewResourceDocuments,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalDocuments,
                RESOURCE.CanEditResourceDocuments,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("employeeCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("zipPostalCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "executiveSecurityRole", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.FALSE, RESOURCE.CanViewResourceExecutiveAccessLevel,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                VIRTUAL.FALSE, RESOURCE.CanEditResourceExecutiveAccessLevel,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("whatIDoReallyWell",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("middleName",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                VIRTUAL.TRUE, RESOURCE.CanViewResourceName,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalName, RESOURCE.CanEditResourceName,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "locationPreferences", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("companyCode",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("discontinueDate",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalEffectiveDiscontinueActiveDates,
                RESOURCE.CanViewResourceEffectiveDiscontinueActiveDates,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalEffectiveDiscontinueActiveDates,
                RESOURCE.CanEditResourceEffectiveDiscontinueActiveDates,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "overrideOriginalDocumentOnCheckout", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSettings,
                RESOURCE.CanViewResourceSettings,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalSettings,
                RESOURCE.CanEditResourceSettings,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("fax",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("custom5",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myWebsite",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("alternativePhone",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("homePhone",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("pager",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("Custom1",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("weeklyMaxHours",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("experience",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "organizationalAssignment", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalOrganizationGroupGeographical,
                RESOURCE.CanViewResourceOrganizationGroupGeographical,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalOrganizationGroupGeographical,
                RESOURCE.CanEditResourceOrganizationGroupGeographical,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("workingStatus",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("mobile",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("title",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("weeklyMinHours",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("Custom4",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myPortalURL1",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSettings,
                RESOURCE.CanViewResourceSettings,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalSettings,
                RESOURCE.CanEditResourceSettings,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("Custom3",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("Custom2",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myPortalURL5",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSettings,
                RESOURCE.CanViewResourceSettings,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalSettings,
                RESOURCE.CanEditResourceSettings,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("passport",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myAchievements",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myPortalURL2",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSettings,
                RESOURCE.CanViewResourceSettings,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalSettings,
                RESOURCE.CanEditResourceSettings,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "interviewExperience", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("employeeStatus",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalInformation,
                RESOURCE.CanViewResourceInformation,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalInformation,
                RESOURCE.CanEditResourceInformation,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myPortalURL3",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSettings,
                RESOURCE.CanViewResourceSettings,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalSettings,
                RESOURCE.CanEditResourceSettings,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("myPortalURL4",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalSettings,
                RESOURCE.CanViewResourceSettings,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalSettings,
                RESOURCE.CanEditResourceSettings,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "ContactGroupAssignments", getContainerClass());
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalOrganizationGroupGeographical,
                RESOURCE.CanEditResourceOrganizationGroupGeographical,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "skillAttributeAssignments", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalCompetencySkills,
                RESOURCE.CanViewResourceCompetencySkills,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalCompetencySkills,
                RESOURCE.CanEditResourceCompetencySkills,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "executiveAttributes", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalExecutiveAttributes,
                RESOURCE.CanViewResourceExecutiveAttributes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalExecutiveAttributes,
                RESOURCE.CanEditResourceExecutiveAttributes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "CompetencyAttributeAssignments", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalCompetencySkills,
                RESOURCE.CanViewResourceCompetencySkills,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalCompetencySkills,
                RESOURCE.CanEditResourceCompetencySkills,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "executiveAttributeClassifications", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalExecutiveAttributes,
                RESOURCE.CanViewResourceExecutiveAttributes,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalExecutiveAttributes,
                RESOURCE.CanEditResourceExecutiveAttributes,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("projectHistories",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "industryExperiences", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "employmentHistories", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("educations",
            getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "languageProficiencies", getContainerClass());
        fieldInfo.canReadSecurityFlags = new Integer[]
        {
                RESOURCE.CanViewPersonalBackground,
                RESOURCE.CanViewResourceBackground,
        };
        fieldInfo.canWriteSecurityFlags = new Integer[]
        {
                RESOURCE.CanEditPersonalBackground,
                RESOURCE.CanEditResourceBackground,
        };
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("securityGroup",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "competencyAttributeAssignments", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("dashboard",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("reVerifiedBy",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "administrativeTaskAssignments", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "resourceCostCenterAssignments", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry(
            "personalTaskAssignments", getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("verifiedBy",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);

        // --------------------------------------------------------
        fieldInfo = new ResourceBasedResourceMappingEntry("logonReminder",
            getContainerClass());
        mappingEntryMap.put(fieldInfo.name, fieldInfo);
    }

    protected Map buildCustomFieldSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map customFieldSecurityDescrs = new HashMap();
        FieldSecurityDescriptor customFieldSecurityDescr = null;

        if (objectInfo.rpmObject instanceof Resource
                && messageContext.getUser().getID().equals(
                    objectInfo.rpmObject.getID()))
        {
            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup1);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups1,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups1,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[1],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup2);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups2,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups2,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[2],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup3);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups3,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups3,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[3],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup4);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups4,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups4,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[4],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup5);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups5,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups5,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[5],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup6);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups6,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups6,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[6],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup7);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups7,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups7,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[7],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup8);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups8,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups8,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[8],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup9);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups9,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups9,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[9],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup10);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewPersonalRecordCustomFieldsGroups10,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditPersonalRecordCustomFieldsGroups10,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[10],
                customFieldSecurityDescr);
        }
        else
        {
            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup1);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups1,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups1,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[1],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup2);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups2,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups2,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[2],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup3);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups3,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups3,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[3],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup4);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups4,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups4,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[4],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup5);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups5,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups5,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[5],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup6);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups6,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups6,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[6],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup7);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups7,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups7,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[7],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup8);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups8,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups8,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[8],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup9);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups9,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups9,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[9],
                customFieldSecurityDescr);

            // --------------------------------------------------------
            customFieldSecurityDescr = new FieldSecurityDescriptor();
            customFieldSecurityDescr.setName(SecuredFields.CustomFieldGroup10);
            customFieldSecurityDescr.setCanView(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanViewOtherResourcesRecordCustomFieldsGroups10,
                    combinedSecurityFlags));
            customFieldSecurityDescr.setCanEdit(SecurityControllerUtil
                .calculateFlagValue(
                    RESOURCE.CanEditOtherResourcesRecordCustomFieldsGroups10,
                    combinedSecurityFlags));
            customFieldSecurityDescrs.put(SecuredFields.INTEGERS[10],
                customFieldSecurityDescr);
        }
        return customFieldSecurityDescrs;
    }

    // Adds the CreationOperation descriptors of the resource's DocumentFolder 
    // to the resource's CreationOperation descriptors because the WebUI does not
    // receive the resource's DocumentFolder when it is empty but still requires security 
    protected Map buildCreationOperationSecurityDescriptors(
            ReferencedObjectInfo objectInfo,
            CombinedSecurityFlags combinedSecurityFlags,
            MessageContext messageContext)
    {
        Map map = super.buildCreationOperationSecurityDescriptors(objectInfo,
            combinedSecurityFlags, messageContext);
        ResourceBasedDocumentFolderSecurityController controller = (ResourceBasedDocumentFolderSecurityController) getContainerSecurityController(DocumentFolder.class);
        if (controller != null)
        {
            map.putAll(controller.buildCreationOperationSecurityDescriptors(
                objectInfo, combinedSecurityFlags, messageContext));
        }
        return map;
    }

}
