package com.ibm.rpm.framework.util;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.constants.ErrorCodes;

/**
 * @author gmeloche
 */
public class ExceptionUtil
{
    public ExceptionUtil()
    {
        super();
    }

    /**
     * Use this exception handler for static methods
     * 
     * @param objectClass
     *            class where the exception occurs
     * @param exception
     * @throws RPMException
     */
    public static void handleException(Class objectClass, Exception exception)
            throws RPMException
    {        
        RPMException e = null;
        if(exception instanceof RPMException){
            e = (RPMException)exception;
        }else{
            e = new RPMException(exception);
        }

        throw e.log();
    }
    
    /**
     * Generic function to avoid creation of a handle method for each error
     * @param object where the exception occurs
     * @param exception
     * @throws RPMException
     */
    public static void handleException(Object object, Exception exception)
            throws RPMException
    {        
        handleException(object.getClass(), exception);
    }

    /**
     * Use this method to create and handle a new RPMException with a specific
     * errorCode and parameters
     * 
     * @param objectClass
     * @param errorCode
     * @param params
     * @throws RPMException
     */
    public static void createAndHandleRPMException(Class objectClass, int errorCode,
            String[] params) throws RPMException
    {
        RPMException e = new RPMException(errorCode, params);
        handleException(objectClass, e);
    }
    
    public static void handleException(Object manager, Exception exception,
            int errorCode, String[] message) throws RPMException
    {
        RPMException e = new RPMException(errorCode, message, null, null, null,
            null, exception);
        throw e.log();
    }

    /**
     * Rises RPMException with a given errorCode Passes manager class name as
     * first parameter for RPMException error params
     * 
     * @param manager
     * @param errorCode
     * @throws RPMException
     */
    public static void handleRPMException(RPMObjectManager manager,
            int errorCode) throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(manager.getClass())
        };
        RPMException e = new RPMException(errorCode, errorParams);
        throw e.log();
    }

    /**
     * Rises RPMException with given errorCode and errorParams
     * 
     * @param manager
     * @param errorCode
     * @param errorParams
     * @throws RPMException
     */
    public static void handleRPMException(Object manager, int errorCode,
            String[] errorParams) throws RPMException
    {
        RPMException e = new RPMException(errorCode, errorParams);
        throw e.log();
    }

    /**
     * Rises RPMException for given errorCode, using object class as first
     * errorParam and fieldValue as a second errorParam
     * 
     * @param manager
     * @param errorCode
     * @param obj
     * @param fieldName
     * @param fieldValue
     * @throws RPMException
     */
    public static void handleRPMException(RPMObjectManager manager,
            int errorCode, RPMObject obj, String fieldName, String fieldValue)
            throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(obj.getClass()), fieldValue
        };
        RPMException e = new RPMException(errorCode, errorParams, obj
            .getClass().getName(), fieldName, obj.getID());
        throw e.log();
    }

    /**
     * Method names for the generic method error handler
     */

    public static String CREATE_OBJECT = "createObject";

    public static String UPDATE_OBJECT = "updateObject";

    public static String SP_U_POOL_COST_CENTER = "SP_U_POOL_COST_CENTER";

    public static String SP_U_POOL_REVISION = "SP_U_POOL_REVISION";

    public static String JOIN = "getJoinCondition";

    public static String SUB_TABLE_JOIN = "getSubTableJoinCondition";

    public static String FOREIGN_KEY = "getForeignKeyCondition";

    public static String GET_CONTAINER_TYPE = "getContainerType";

    public static String DECODE_ROW = "decodeRow";

    public static String GET_PROPERTIES = "getProperties";

    public static void handleMethodException(Object manager, int errorCode,
            String method) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(manager.getClass()), method
        };
        RPMException e = new RPMException(errorCode, errorParams);
        throw e.log();
    }

    /**
     * Specific handle functions common to multiple managers
     */
   
    /**
     * OBSOLETE use handleNotImplementedJoinCondition(RPMObjectManager manager,
            RPMObjectManager managerParam, String method)
     */
    public static void handleUnsupportedJoinCondition(RPMObjectManager manager,
            RPMObjectManager managerParam, String method) throws RPMException
    {
        	handleNotImplementedJoinCondition(manager, managerParam, method);
    }
    
    
    public static void handleNotImplementedJoinCondition(RPMObjectManager manager,
            RPMObjectManager managerParam, String method) throws RPMException
    {
        RPMException e = new RPMException(ErrorCodes.NOT_IMPLEMENTED_JOIN);
        throw e.log();
    }

    public static String GET_TABLE_NAME = "getTableName(contextName)";

    public static void handleGetTableName(RPMObjectManager manager)
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(manager.getClass()),
                GET_TABLE_NAME
        };
        RPMException e = new RPMException(ErrorCodes.MANDATORY_METHOD,
            errorParams, manager.getClass().getName());
        e.log();
    }

    public static void handleNoParentManager(RPMObjectManager manager,
            String containerName) throws RPMException
    {
        String[] errorParams =
        {
            containerName
        };
        RPMException e = new RPMException(ErrorCodes.NO_PARENT_MANAGER,
            errorParams);
        throw e.log();
    }

    public static void handleNoTypeId(RPMObjectManager manager,
            String containerName) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(manager.getClass()), containerName
        };
        RPMException e = new RPMException(ErrorCodes.NO_TYPEID, errorParams);
        throw e.log();
    }

    public static void handleUnsupportedTypeId(Class managerClass,
            int typeId) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(managerClass),
                String.valueOf(typeId)
        };
        RPMException e = new RPMException(ErrorCodes.UNSUPPORTED_TYPEID,
            errorParams);
        throw e.log();
    }

    public static void handleUnsupportedTypeId(RPMObjectManager manager,
            int typeId) throws RPMException
    {
        handleUnsupportedTypeId(manager.getClass(), typeId);
    }

    public static void handleUnsupportedSubTypeId(Class managerClass,
            int subTypeId) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(managerClass),
                String.valueOf(subTypeId)
        };
        RPMException e = new RPMException(ErrorCodes.UNSUPPORTED_SUBTYPEID,
            errorParams);
        throw e.log();
    }

    public static void handleUnsupportedSubTypeId(RPMObjectManager manager,
            int subTypeId) throws RPMException
    {
        handleUnsupportedSubTypeId(manager.getClass(), subTypeId);
    }
    
    public static void handleUnsupportedAPICall(Object manager,
            String containerName) throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(containerName)
        };
        RPMException e = new RPMException(ErrorCodes.UNSUPPORTED_API_CALL,
            errorParams, containerName);
        throw e.log();
    }

    public static void handleNotRPMContainer(Object manager,
            String containerName) throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(containerName)
        };
        RPMException e = new RPMException(ErrorCodes.NOT_RPM_CONTAINER,
            errorParams, containerName);
        throw e.log();
    }

    public static void handleNoDelete(RPMObjectManager manager, RPMObject obj)
            throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(obj.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.NO_DELETE, errorParams,
            obj.getClass().getName());
        throw e.log();
    }
    
    public static void handleNoUpdate(RPMObjectManager manager, RPMObject obj)
            throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(obj.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.NO_UPDATE, errorParams,
            obj.getClass().getName());
        throw e.log();
    }

    public static void handleNoCreate(RPMObjectManager manager, RPMObject obj)
            throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(obj.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.NO_CREATE, errorParams,
            obj.getClass().getName());
        throw e.log();
    }

    public static void handleParentReference(RPMObjectManager manager,
            RPMObject obj) throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(obj.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.PARENT_REFERENCE,
            errorParams, obj.getClass().getName(), "parent", obj.getID());
        throw e.log();
    }

    /**
     * Unique Id field names for the handleNoId function
     */

    public static String ASSET_ID = "assetId";

    public static String RESOURCE_ID = "resourceId";

    public static String EXPERIENCE_ID = "experienceId";

    public static String PREFERENCE_RANK_ID = "preferenceRankId";

    public static String COST_CENTER_ID = "costcenterId";

    public static String ELEMENT_ID = "elementId";

    public static String ORGANIZATION_ID = "organizationId";

    public static String PROJECT_OR_CLIENT_ID = "projectOrClientId";
    
    public static String PROFILE_ID = "profileId";

    public static void handleNoId(RPMObjectManager manager, RPMObject obj,
            String id) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(manager.getClass()),
                StringUtil.getShortClassName(obj.getClass()) + " with ID '"
                        + obj.getID() + "'", id
        };
        RPMException e = new RPMException(ErrorCodes.NO_ID_ON_LOAD,
            errorParams, obj.getClass().getName(), "parent", obj.getID());
        throw e.log();
    }

    public static void handleInvalidId(RPMObjectManager manager,
            String className, String id) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(className), id
        };
        RPMException e = new RPMException(ErrorCodes.AM_INVALID_RPMOBJECTID,
            errorParams);
        throw e.log();
    }

    /**
     * Method name for context error message
     */

    public static String INIT_RTF = ".initRtf";

    /**
     * Use this method when the context is associated to a table name e.g. Pool
     * is a possible contextName for an Attribute
     */
    public static void handleUnsupportedContext(RPMObjectManager manager,
            RPMObject obj, String method) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(manager.getClass()) + "." + method,
                obj.getContextName()
        };
        RPMException e = new RPMException(ErrorCodes.UNSUPPORTED_CONTEXT,
            errorParams, obj.getClass().getName(), "contextName", obj.getID());
        throw e.log();
    }

    /**
     * Use this method when the context is associated to a field name e.g.
     * actualDate is a possible contextName for a WbsScheduleDate
     */
    public static void handleUnsupportedContext(RPMObjectManager manager,
            String contextName, String method) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(manager.getClass()) + method,
                contextName
        };
        RPMException e = new RPMException(ErrorCodes.UNSUPPORTED_CONTEXT,
            errorParams);
        throw e.log();
    }
    
    public static void handleInvalidEnumeratedType(
            Class enumeratedType, Object value) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(enumeratedType),
                String.valueOf(value)
        };
        RPMException e = new RPMException(
            ErrorCodes.INVALID_VALUE_OF_ENUMERATEDTYPE, errorParams);
        throw e.log();
    }

    public static void handleMissingPrimaryKey(RPMObjectManager manager,
            RPMObject rpmObject, String fields) throws RPMException
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(rpmObject.getClass()), fields
        };
        RPMException e = new RPMException(ErrorCodes.MISSING_PRIMARY_KEY,
            errorParams, rpmObject.getClass().getName());
        throw e.log();
    }


    public static void handleUnsupportedFieldInXpath(String containerName, String fieldName)
            throws RPMException
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(containerName), fieldName
        };
        RPMException e = new RPMException(ErrorCodes.XPATH_UNSUPPORTED_FIELD,
            errorParams);
        throw e.log();
    }   
    
}
