/*
 * Created on 5-Oct-2005
 */
package com.ibm.rpm.framework.util;

import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.financial.containers.AbstractTimeCode;
import com.ibm.rpm.financial.containers.TimeCodeGroup;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMContainer;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.types.SeverityLevel;


/**
 * @author dominiqu
 */
public class GenericValidator
{
    private static Log logger = LogFactory.getLog(RPMObjectManager.class);

    public static boolean validateInstance(RPMObject object,
            Class expectedType, MessageContext msgCtx)
    {
        if (!expectedType.isInstance(object))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(expectedType),
                    StringUtil.getShortClassName(object.getClass())
            };
            RPMException e = new RPMException(
                ErrorCodes.INVALID_RPMOBJECT_TYPE, errorParams, object
                    .getClass().getName());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateDefaultCount(RPMObject object,
            String fieldName, MessageContext msgCtx)
    {

        String[] errorParams =
        {
                StringUtil.getShortClassName(object.getClass()), fieldName
        };

        RPMException e = new RPMException(ErrorCodes.WRONG_NUMBER_OF_DEFAULTS,
            errorParams, object.getClass().getName(), fieldName, object.getID());
        addException(e, msgCtx);

        return false;
    }

    public static boolean validateActiveFlag(RPMObject object,
            String fieldName, MessageContext msgCtx)
    {

        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass())
        };

        RPMException e = new RPMException(ErrorCodes.NOT_ACTIVE, errorParams,
            object.getClass().getName(), fieldName, object.getID());
        addException(e, msgCtx);

        return false;

    }

    /**
     * Returns <code>fieldValue</code> is assignment compatible with the
     * <code>expectedTypes</code>.
     */
    public static boolean validateType(RPMObject object, Class[] expectedTypes,
            String fieldName, Object fieldValue, MessageContext msgCtx)
    {
        boolean isValid = false;
        if (expectedTypes != null)
        {
            for (int i = 0; i < expectedTypes.length && !isValid; i++)
            {
                if (expectedTypes[i].isInstance(fieldValue))
                {
                    isValid = true;
                }
            }
        }
        if (!isValid)
        {
            addInvalidType(object, expectedTypes, fieldName, fieldValue, msgCtx);
            return false;
        }
        return true;
    }

    /**
     * Returns <code>fieldValue</code> is assignment compatible with the
     * <code>expectedType</code>.
     */
    public static boolean validateType(RPMObject object, Class expectedType,
            String fieldName, Object fieldValue, MessageContext msgCtx)
    {
        if (!expectedType.isInstance(fieldValue))
        {
            addInvalidType(object, expectedType, fieldName, fieldValue, msgCtx);
            return false;
        }
        return true;
    }

    /**
     * Returns <code>fieldValue</code>'s type is equal to the
     * <code>expectedType</code>.
     */
    public static boolean validateExactType(RPMObject object,
            Class expectedType, String fieldName, Object fieldValue,
            MessageContext msgCtx)
    {
        if (!expectedType.equals(fieldValue.getClass()))
        {
            addInvalidType(object, expectedType, fieldName, fieldValue, msgCtx);
            return false;
        }
        return true;
    }

    private static void addInvalidType(RPMObject object, Class[] expectedTypes,
            String fieldName, Object fieldValue, MessageContext msgCtx)
    {
        StringBuffer sbExpectedTypes = new StringBuffer();
        for (int i = 0; i < expectedTypes.length; i++)
        {
            sbExpectedTypes.append(StringUtil
                .getShortClassName(expectedTypes[i]));
            if (i + 1 < expectedTypes.length)
            {
                sbExpectedTypes.append(",");
            }
        }
        String[] errorParams =
        {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName,
                StringUtil.getShortClassName(fieldValue.getClass()),
                sbExpectedTypes.toString()
        };
        RPMException e = new RPMException(ErrorCodes.INVALID_TYPE, errorParams,
            object.getClass().getName(), fieldName, object.getID());
        addException(e, msgCtx);
    }

    private static void addInvalidType(RPMObject object, Class expectedType,
            String fieldName, Object fieldValue, MessageContext msgCtx)
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName,
                StringUtil.getShortClassName(fieldValue.getClass()),
                StringUtil.getShortClassName(expectedType)
        };
        RPMException e = new RPMException(ErrorCodes.INVALID_TYPE, errorParams,
            object.getClass().getName(), fieldName, object.getID());
        addException(e, msgCtx);
    }

    public static boolean validateReadOnly(RPMObject object, String fieldName,
            MessageContext msgCtx)
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass()) + " " + fieldName
        };
        RPMException e = new RPMException(ErrorCodes.READ_ONLY_FIELD,
            errorParams, object.getClass().getName(), fieldName, object.getID());
        e.setSeverity(SeverityLevel.Warning);
        addException(e, msgCtx);
        return false;

    }
    
    public static boolean validateReadOnlyError(RPMObject object, String fieldName,
            MessageContext msgCtx)
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass()) + " " + fieldName
        };
        RPMException e = new RPMException(ErrorCodes.READ_ONLY_FIELD,
            errorParams, object.getClass().getName(), fieldName, object.getID());
        addException(e, msgCtx);
        return false;
    }

    public static boolean validateReadOnlyConditionExists(RPMObject object,
            String fieldName, String conditionalFieldName, MessageContext msgCtx)
    {
        String[] errorParams =
        {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName, conditionalFieldName
        };
        RPMException e = new RPMException(ErrorCodes.READ_ONLY_VALUE_EXISTS,
            errorParams, object.getClass().getName(), fieldName,
            conditionalFieldName, object.getID());
        e.setSeverity(SeverityLevel.Warning);
        addException(e, msgCtx);
        return false;

    }

    public static boolean validateReadOnlyConditionExists(RPMObject object,
            String fieldName, String conditionalFieldName,
            Calendar conditionalFieldValue, MessageContext msgCtx)
    {
        if (conditionalFieldValue != null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, conditionalFieldName
            };
            RPMException e = new RPMException(
                ErrorCodes.READ_ONLY_VALUE_EXISTS, errorParams, object
                    .getClass().getName(), fieldName, conditionalFieldName,
                object.getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateReadOnlyConditionMissing(RPMObject object,
            String fieldName, String conditionalFieldName,
            Calendar conditionalFieldValue, MessageContext msgCtx)
    {
        if (conditionalFieldValue == null)
        {

            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, conditionalFieldName
            };
            RPMException e = new RPMException(
                ErrorCodes.READ_ONLY_VALUE_MISSING, errorParams, object
                    .getClass().getName(), fieldName, conditionalFieldName,
                object.getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
            return false;

        }
        return true;
    }

    public static boolean validateConditionMissing(RPMObject object,
            String fieldName, String fieldValue, String conditionalFieldName,
            Calendar conditionalFieldValue, MessageContext msgCtx)
    {
        if (fieldValue != null && fieldValue.length() != 0)
        {
            if (conditionalFieldValue == null)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()) + " "
                                + fieldName, fieldValue, conditionalFieldName
                };
                RPMException e = new RPMException(
                    ErrorCodes.NOT_VALID_VALUE_MISSING, errorParams, object
                        .getClass().getName(), fieldName, conditionalFieldName,
                    object.getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validateReadOnlyConditionIsZero(RPMObject object,
            String fieldName, String conditionalFieldName,
            int conditionalFieldValue, MessageContext msgCtx)
    {
        if (conditionalFieldValue == 0)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, conditionalFieldName
            };
            RPMException e = new RPMException(
                ErrorCodes.READ_ONLY_CONDITIONAL_VALUE_ZERO, errorParams,
                object.getClass().getName(), fieldName, conditionalFieldName,
                object.getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
            return false;

        }
        return true;
    }

    public static boolean validateReadOnlyConditionalFieldsSame(
            RPMObject object, String fieldName, String conditionalFieldName1,
            String conditionalFieldName2, int conditionalFieldValue1,
            int conditionalFieldValue2, MessageContext msgCtx)
    {
        if (conditionalFieldValue1 == conditionalFieldValue2)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, conditionalFieldName1,
                    conditionalFieldName2,
                    String.valueOf(conditionalFieldValue1)
            };
            RPMException e = new RPMException(
                ErrorCodes.READ_ONLY_CONDITIONAL_VALUES_SAME, errorParams,
                object.getClass().getName(), fieldName, conditionalFieldName1,
                object.getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
            return false;

        }
        return true;
    }

    public static boolean validateConversionType(RPMObject inputObject,
            RPMObject outputObject, MessageContext msgCtx)
    {
        // invalid output type
        String[] errorParams =
        {
                StringUtil.getShortClassName(inputObject.getClass()
                    .getName()),
                StringUtil.getShortClassName(outputObject.getClass()
                    .getName()),
        };
        RPMException e = new RPMException(
            ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID,
            errorParams, inputObject.getClass().getName());
        addException(e, msgCtx);

        return false;
    }

    public static boolean validateCannotDelete(RPMObject object,
            MessageContext msgCtx)
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.NO_DELETE, errorParams,
            object.getClass().getName());
        addException(e, msgCtx);
        return false;
    }
    
    public static boolean validateCannotSave(RPMObject object,
            MessageContext msgCtx)
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.NO_SAVE, errorParams,
            object.getClass().getName());
        addException(e, msgCtx);
        return false;
    }

    public static boolean validateCannotCreate(RPMObject object, List expectedTypes,
            MessageContext msgCtx)
    {
        StringBuffer expectedTypeString = new StringBuffer();

        Iterator it = expectedTypes.iterator();
        boolean isFirst = true;
        while (it.hasNext())
        {
            if ( isFirst )
            {
                isFirst = false;
            }
            else
            {
                expectedTypeString.append(", ");
            }
            Class element = (Class) it.next();
            expectedTypeString.append( StringUtil.getShortClassName(element) );
        }

        String[] errorParams =
        {
                expectedTypeString.toString(),
                StringUtil.getShortClassName(object.getClass())
        };

        RPMException e = new RPMException(ErrorCodes.INVALID_RPMOBJECT_TYPE, errorParams,
            object.getClass().getName());

        addException(e, msgCtx);
        return false;
    }

    public static boolean validateCannotCreate(RPMObject object,
            MessageContext msgCtx)
    {
        if (object.getID() == null)
        {

            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass())
            };

            RPMException e = new RPMException(
                ErrorCodes.CANNOT_CREATE, errorParams, object
                    .getClass().getName());

            addException(e, msgCtx);
            return false;
        }
        return true;
    }
    
    public static boolean validateCannotCreateOrUpdate(RPMObject object,
            MessageContext msgCtx)
    {
        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass())
        };
        RPMException e = new RPMException(ErrorCodes.CANNOT_CREATE,
            errorParams, object.getClass().getName());
        addException(e, msgCtx);
        return false;
    }

    public static boolean validateMustBeNull(RPMObject object,
            String fieldName, String fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (fieldValue != null && fieldValue.trim().length() != 0)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.MUST_BE_NULL,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMustBeNull(RPMObject object,
            String fieldName, Integer fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (!(fieldValue == null || fieldValue.intValue() == 0))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.MUST_BE_NULL,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMustBeNull(RPMObject object,
            String fieldName, Object fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (!(fieldValue == null))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.MUST_BE_NULL,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMustBeNull(RPMObject object,
            String fieldName, Object[] fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (!(fieldValue == null || fieldValue.length == 0))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.MUST_BE_NULL,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMustAlwaysBeNull(RPMObject object,
            String fieldName, Object fieldValue,
            MessageContext msgCtx)
    {
        if (!(fieldValue == null))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.MUST_ALWAYS_BE_NULL,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateNotNull(RPMObject object, String fieldName,
            Object fieldValue, MessageContext msgCtx)
    {
        if (fieldValue == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.CANNOT_CLEAR_FIELD,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMandatory(RPMObject object, String fieldName,
            Object fieldValue, MessageContext msgCtx)
    {
        if (fieldValue == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }
    
    public static boolean validateMandatory(RPMObject object, String fieldName,
            Object [] fieldValue, MessageContext msgCtx)
    {
        if (fieldValue == null || fieldValue.length == 0)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMandatory(RPMObject object, String fieldName,
            String fieldValue, MessageContext msgCtx)
    {
        if (fieldValue == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMandatory(RPMObject object, String fieldName,
            RPMObject field, MessageContext msgCtx)
    {
        if (field == null)
        {
            String objectClassName = null;
            String shortObjectClassName = null;
            String objectId = null;
            if (object != null)
            {
                objectClassName = object.getClass().getName();
                shortObjectClassName = StringUtil.getShortClassName(object
                    .getClass());
                objectId = object.getID();
            }

            String[] errorParams =
            {
                shortObjectClassName + " " + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, objectClassName, fieldName, objectId);
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateDefaultValueUpdate(RPMObject object,
            String fieldName, Object fieldValue, MessageContext msgCtx)
    {
        if (object.getID() != null)
        {
            if (fieldValue == null)
            {
                String[] errorParams =
                {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName
                };
                RPMException e = new RPMException(
                    ErrorCodes.CANNOT_CLEAR_FIELD, errorParams, object
                        .getClass().getName(), fieldName, object.getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validateDefaultValueUpdate(RPMObject object,
            String fieldName, String fieldValue, MessageContext msgCtx)
    {
        if (object.getID() != null)
        {
            if (fieldValue == null || fieldValue.trim().length() <= 0)
            {
                String[] errorParams =
                {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName
                };
                RPMException e = new RPMException(
                    ErrorCodes.CANNOT_CLEAR_FIELD, errorParams, object
                        .getClass().getName(), fieldName, object.getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validateDefaultValueUpdate(RPMObject object,
            String fieldName, RPMObject field, MessageContext msgCtx)
    {
        if (object.getID() != null)
        {
            if (field == null)
            {
                String[] errorParams =
                {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName
                };
                RPMException e = new RPMException(
                    ErrorCodes.CANNOT_CLEAR_FIELD, errorParams, object
                        .getClass().getName(), fieldName, object.getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validateFieldUpdateInvalidOnContainerUpdate(
            RPMObject object, String fieldName, RPMObject fieldValue, MessageContext msgCtx)
    {
        if (object.getID() != null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, fieldName
            };
            RPMException e = new RPMException(
                ErrorCodes.CANNOT_UPDATE_FIELD_ON_CONTAINER_UPDATE,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }

        return GenericValidator.validateMandatoryID(object, fieldName,
            fieldValue, msgCtx);

    }

    public static boolean validateDeleteFieldExists(RPMObject object,
            String fieldName, Object field, MessageContext msgCtx)
    {
        if (field != null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()), fieldName
            };
            RPMException e = new RPMException(
                ErrorCodes.CANNOT_DELETE_VALUE_EXISTS, errorParams, object
                    .getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateConditionalMandatory(RPMObject object,
            String fieldName, String fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (fieldValue == null || fieldValue.trim().length() == 0)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.CONDITIONAL_MANDATORY,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateConditionalMandatory(RPMObject object,
            String fieldName, RPMObject fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (fieldValue == null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.CONDITIONAL_MANDATORY,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateConditionalMandatory(RPMObject object,
            String fieldName, Integer fieldValue, String condition,
            MessageContext msgCtx)
    {
        if (fieldValue == null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, condition
            };
            RPMException e = new RPMException(ErrorCodes.CONDITIONAL_MANDATORY,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMandatoryID(String objectName,
            RPMObject object, MessageContext msgCtx)
    {
        if (object == null || object.getID() == null)
        {
            String[] errorParams =
            {
                object == null ? objectName : StringUtil
                    .getShortClassName(object.getClass())
            };
            RPMException e = new RPMException(ErrorCodes.NO_ID, errorParams,
                object == null ? objectName : object.getClass().getName());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMandatoryID(RPMObject object,
            String fieldName, RPMObject obj, MessageContext msgCtx)
    {
        if (obj == null || obj.getID() == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass()) + " "
                        + fieldName
            };
            RPMException e = new RPMException(ErrorCodes.NO_ID, errorParams,
                object.getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateMaxLength(RPMObject object, String fieldName,
            String fieldValue, int maxLength, MessageContext msgCtx)
    {
        String utf8FieldValue = null;
        try
        {
            utf8FieldValue = StringUtil.convertUnicodeToUtf8(fieldValue);
        }
        catch (UnsupportedEncodingException e)
        {
            addException(new RPMException(e), msgCtx);
        }
        
        if (exceedsMaxLength(utf8FieldValue, maxLength))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(maxLength),
                    getShortString(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.MAX_LENGTH,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

	public static void validateMaxLength(String fieldName,
			String fieldValue, int maxLength) throws RPMException
	{
        String utf8FieldValue = null;
        try
        {
            utf8FieldValue = StringUtil.convertUnicodeToUtf8(fieldValue);
        }
        catch (UnsupportedEncodingException e)
        {
            throw new RPMException(e);
        }
        
		if (exceedsMaxLength(utf8FieldValue, maxLength))
		{
			String[] errorParams = { fieldName, String.valueOf(maxLength),
					getShortString(fieldValue) };
			RPMException e = new RPMException(ErrorCodes.MAX_LENGTH,
					errorParams);
			throw e;
		}
	}

    public static boolean validateMinLength(RPMObject object, String fieldName,
            String fieldValue, int minLength, MessageContext msgCtx)
    {
        // null value no longer fails min length test
        if (fieldValue != null && fieldValue.trim().length() < minLength)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(minLength),
                    getShortString(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.MIN_LENGTH,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

	public static void validateMinLength(String fieldName,
			String fieldValue, int minLength) throws RPMException
	{
		if (fieldValue != null && fieldValue.trim().length() < minLength)
		{
			String[] errorParams = { fieldName, String.valueOf(minLength),
					getShortString(fieldValue) };
			RPMException e = new RPMException(ErrorCodes.MIN_LENGTH,
					errorParams);
			throw e;
		}
	}

    /**
     * Ensure that the given decimal value has only two digits in the fractional
     * part.
     *
     * @param object
     * @param fieldName
     * @param fieldValue
     * @param maxLength
     * @param msgCtx
     * @return
     */
    public static boolean validateDecimalDigits(RPMObject object,
            String fieldName, Double fieldValue, MessageContext msgCtx)
    {
        double percentageDouble = fieldValue.doubleValue();
        if (decimalCount(percentageDouble) > 2)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(fieldValue)
            };
            RPMException e = new RPMException(
                ErrorCodes.MORE_THAN_TWO_DECIMAL_DIGITS, errorParams, object
                    .getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static int decimalCount(double value)
    {
        String converted = String.valueOf(value);
        int index = converted.indexOf('.');
        int count = 0;
        if (index >= 0)
        {
            count = converted.length() - index - 1;
        }
        return count;
    }

    public static boolean validateMaxSize(RPMObject object, String fieldName,
            int fieldSize, int maxSize, MessageContext msgCtx)
    {
        if (exceedsMaxSize(fieldSize, maxSize))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(maxSize)
            };
            RPMException e = new RPMException(ErrorCodes.MAX_SIZE, errorParams,
                object.getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateRange(RPMObject object, String fieldName,
            String fieldValue, int min, int max, MessageContext msgCtx)
    {
        if (fieldValue != null)
        {
            try
            {
                if (GenericValidator.outOfRange(new Integer(fieldValue), min,
                    max))
                {
                    throw new Exception();
                }
            }
            catch (Exception exc)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()) + " "
                                + fieldName, String.valueOf(min),
                        String.valueOf(max), fieldValue
                };
                RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                    errorParams, object.getClass().getName(), fieldName, object
                        .getID());
                addException(e, msgCtx);
            }
        }
        return true;
    }

    public static boolean validateRange(RPMObject object, String fieldName,
            Integer fieldValue, int min, int max, MessageContext msgCtx)
    {
        if (GenericValidator.outOfRange(fieldValue, min, max))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(min),
                    String.valueOf(max), String.valueOf(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateRange(RPMContainer object, String fieldName,
            Integer fieldValue, int min, int max, MessageContext msgCtx)
    {
        if (GenericValidator.outOfRange(fieldValue, min, max))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(min),
                    String.valueOf(max), String.valueOf(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                errorParams, object.getClass().getName(), fieldName, null);
            addException(e, msgCtx);
            return false;
        }
        return true;
    }
    
    public static boolean validateRange(RPMObject object, String fieldName,
            Calendar fieldValue, Calendar min, Calendar max,
            MessageContext msgCtx)
    {
        if (GenericValidator.outOfRange(fieldValue, min, max))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, min.getTime().toString(),
                    max.getTime().toString(), fieldValue.getTime().toString()
            };

            RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            e.setFieldName(fieldName);
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    /**
     * The time unit has to be in a 15 minutes increment. (i.e. 1.25, or 1.50,
     * are supported but not 1.1 or 1.2, etc.)
     *
     * @param object
     * @param fieldName
     * @param fieldValue
     * @param msgCtx
     * @return
     */
    public static boolean validateWorkingHour(RPMObject object,
            String fieldName, Double fieldValue, MessageContext msgCtx)
    {        
        if(fieldValue != null){
        
        double hours = fieldValue.doubleValue();
            // round to the nearest 0.25 value (0.25 hours correspond to 15
            // minutes)
            double validHours = (double) (Math.round(hours * 4.0)) / 4;

            if (hours % 0.25 != 0)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()) + " "
                                + fieldName, String.valueOf(fieldValue),
                        Double.toString(validHours)
                };
                RPMException e = new RPMException(ErrorCodes.INVALID_HOUR,
                    errorParams, object.getClass().getName(), fieldName, object
                        .getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validateRange(RPMObject object, String fieldName,
            Double fieldValue, double min, double max, MessageContext msgCtx)
    {
        if (GenericValidator.outOfRange(fieldValue, min, max))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(min),
                    String.valueOf(max), String.valueOf(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateRange(RPMContainer object, String fieldName,
            Double fieldValue, double min, double max, MessageContext msgCtx)
    {
        if (GenericValidator.outOfRange(fieldValue, min, max))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(min),
                    String.valueOf(max), String.valueOf(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                errorParams, object.getClass().getName(), fieldName, null);
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateBooleanDependancy(RPMObject object,
            String fieldName1, Boolean fieldValue1, boolean validFieldValue1,
            String fieldName2, Boolean fieldValue2, boolean validFieldValue2,
            MessageContext msgCtx)
    {
        if (getBoolean(fieldValue1) == validFieldValue1
                && getBoolean(fieldValue2) != validFieldValue2)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName2, String.valueOf(validFieldValue2),
                    fieldName1 + " is " + String.valueOf(validFieldValue1)
            };
            RPMException e = new RPMException(
                ErrorCodes.INVALID_CONDITIONAL_VALUE, errorParams, object
                    .getClass().getName(), fieldName2, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateGreaterThanOrEqual(RPMObject object,
            String fieldName1, Double fieldValue1, String fieldName2,
            Double fieldValue2, MessageContext msgCtx)
    {
        if (fieldValue1.doubleValue() < fieldValue2.doubleValue())
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName1, fieldName2
            };

            RPMException e = new RPMException(
                ErrorCodes.GREATER_THAN_OR_EQUAL_FIELD, errorParams, object
                    .getClass().getName(), fieldName1, fieldName2, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateGreaterThanOrEqual(RPMObject object,
            String fieldName1, Integer fieldValue1, String fieldName2,
            Integer fieldValue2, MessageContext msgCtx)
    {
        boolean result = true;
        
        if (fieldValue1.intValue() < fieldValue2.intValue())
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName1, fieldName2
            };

            RPMException e = new RPMException(
                ErrorCodes.GREATER_THAN_OR_EQUAL_FIELD, errorParams, object
                    .getClass().getName(), fieldName1, fieldName2, object
                    .getID());
            addException(e, msgCtx);
           result = false;
        }
        
        return result;
    }

    public static boolean validateGreaterThanOrEqual(RPMObject object,
            String fieldName, Double fieldValue, double min,
            MessageContext msgCtx)
    {
        if (GenericValidator.lessThanMinimum(fieldValue, min))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, String.valueOf(min),
                    String.valueOf(fieldValue)
            };
            RPMException e = new RPMException(
                ErrorCodes.GREATER_THAN_OR_EQUAL_VALUE, errorParams, object
                    .getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateLessThanOrEqual(RPMObject object,
            String fieldName1, Double fieldValue1, String fieldName2,
            Double fieldValue2, MessageContext msgCtx)
    {
        if (fieldValue1 == null)
            return true;
        if (fieldValue1.doubleValue() > fieldValue2.doubleValue())
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName1, fieldName2
            };

            RPMException e = new RPMException(ErrorCodes.LESS_THAN_OR_EQUAL,
                errorParams, object.getClass().getName(), fieldName1,
                fieldName2, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateLessThanOrEqual(RPMObject object,
            String fieldName1, int fieldValue1, String fieldName2,
            int fieldValue2, MessageContext msgCtx)
    {
        if (fieldValue1 > fieldValue2)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName1, fieldName2
            };

            RPMException e = new RPMException(ErrorCodes.LESS_THAN_OR_EQUAL,
                errorParams, object.getClass().getName(), fieldName1,
                fieldName2, object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateDates(RPMObject object, String fieldName1,
            Calendar fieldValue1, String fieldName2, Calendar fieldValue2,
            MessageContext msgCtx)
    {
        if (GenericValidator.invalidDates(fieldValue1, fieldValue2))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName1, fieldName2
            };

            RPMException e = new RPMException(ErrorCodes.INVALID_DATES,
                errorParams, object.getClass().getName(), fieldName1, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateCharacters(RPMObject object,
            String fieldName, String fieldValue, String invalidCharacters,
            MessageContext msgCtx)
    {
        if (invalidCharacters(fieldValue, invalidCharacters))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, invalidCharacters,
                    getShortString(fieldValue, invalidCharacters)
            };
            RPMException e = new RPMException(ErrorCodes.INVALID_CHARS,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateLeadingSpace(RPMObject object,
            String fieldName, String fieldValue, MessageContext msgCtx)
    {
        if (fieldValue != null && fieldValue.length() != 0
                && fieldValue.charAt(0) == ' ')
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, getShortString(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.LEADING_SPACE,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validatePossibleIntegerValues(RPMObject object,
            String fieldName, Integer fieldValue, ArrayList possibleValues,
            MessageContext msgCtx)
    {
        if (fieldValue != null)
        {
            boolean validValue = false;
            StringBuffer possibleValueString = new StringBuffer();
            for (int i = 0; i < possibleValues.size(); ++i)
            {
                if (fieldValue.equals(possibleValues.get(i)))
                {
                    validValue = true;
                    break;
                }
                possibleValueString.append(possibleValues.get(i) + "");
            }
            if (!validValue)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()) + " "
                                + fieldName, fieldValue.toString(),
                        possibleValueString.toString()
                };
                RPMException e = new RPMException(ErrorCodes.INVALID_VALUE,
                    errorParams, object.getClass().getName(), fieldName, object
                        .getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validatePossibleObjectValue(RPMObject object,
            String fieldName, Object fieldValue, ArrayList possibleValues,
            MessageContext msgCtx)
    {
        if (fieldValue != null)
        {
            if (!possibleValues.contains(fieldValue))
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()) + " "
                                + fieldName, fieldValue.toString(),
                        possibleValues.toString()
                };
                RPMException e = new RPMException(ErrorCodes.INVALID_VALUE,
                    errorParams, object.getClass().getName(), fieldName, object
                        .getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }

    public static boolean validateUnique(RPMObject object, String fieldName,
            String fieldValue, String table, String fieldColumn,
            String idColumn, String existingID, MessageContext msgCtx)
    {
        if (fieldValue == null || fieldValue.length() <= 0)
            return true;

        if (!isUnique(msgCtx, table, fieldColumn, fieldValue, idColumn,
            existingID))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, getShortString(fieldValue)
            };
            RPMException e = new RPMException(ErrorCodes.NOT_UNIQUE,
                errorParams, object.getClass().getName(), fieldName, object
                    .getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateNestingLevel(RPMObject object, String table,
            int maxLevel, MessageContext msgCtx)
    {
        // The parent is not being modified, or the object has no parent,
        // thus we don't need to perform validation
        if (object.getParent() == null)
            return true;

        String[] errorParams =
        {
                StringUtil.getShortClassName(object.getClass()), String.valueOf(maxLevel)
        };
        RPMException e = new RPMException(ErrorCodes.EXCEEDS_MAX_NESTING_LEVEL,
            errorParams, object.getClass().getName(), object.getID());

        while (object.getParent().getID() == null)
        {
            object = object.getParent();

            if (--maxLevel <= 0)
            {
                addException(e, msgCtx);
                return false;
            }
            else if (object.getParent() == null)
                return true;
        }

        if (exceedsNestingLevel(msgCtx, table, object.getParent().getID(), maxLevel))
        {
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    public static boolean validateFieldsNotTheSame(RPMObject object,
            String fieldName1, String fieldValue1, String fieldName2,
            String fieldValue2, MessageContext msgCtx)
    {
        if (fieldValue1 != null && fieldValue2 != null)
        {
            if (fieldValue1.equals(fieldValue2))
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()),
                        fieldName1, fieldName2
                };

                RPMException e = new RPMException(
                    ErrorCodes.FIELDS_CANNOT_BE_THE_SAME, errorParams, object
                        .getClass().getName(), fieldName1, fieldName2, object
                        .getID());
                addException(e, msgCtx);
                return false;
            }
        }
        return true;
    }
    
    
    public static void validateDurationQuarterHour(RPMObject rpmObject,
            int durationInMinutes, String fieldName, MessageContext msgCtx)
    {
        // Intended for use by SimpleNode and Scope containers.
        // If the duration is not set in quarter-hour increments, 
        // it will be rounded up to the nearest quarter-hour.
        int roundedDuration = ScheduleDateUtil
            .roundDurationUpToQuarterHour(new Integer(durationInMinutes));

        if (durationInMinutes != roundedDuration)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()) + " "
                            + fieldName,
                    new Integer(durationInMinutes).toString(),
                    new Integer(roundedDuration).toString()
            };
            RPMException e = new RPMException(
                ErrorCodes.DURATION_IN_MINUTES_ROUND_WARNING, errorParams,
                rpmObject.getClass().getName(), fieldName, rpmObject
                    .getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
        }
    }

    private static boolean exceedsMaxLength(String s, int max)
    {
        if (s == null)
            return false;
        return (s.length() > max);
    }

    private static boolean exceedsMaxSize(int arraySize, int max)
    {
        return (arraySize > max);
    }

    private static boolean lessThanMinimum(Double i, double min)
    {
        if (i == null)
            return false;
        return (i.doubleValue() < min);
    }

    private static boolean outOfRange(Integer i, int min, int max)
    {
        if (i == null)
            return false;
        return (i.intValue() < min || i.intValue() > max);
    }

    public static boolean outOfRange(Double d, double min, double max)
    {
        if (d == null)
            return false;
        return (d.doubleValue() < min || d.doubleValue() > max);
    }

    public static boolean outOfRange(Calendar c, Calendar min, Calendar max)
    {
        if (c == null)
        {
            return false;
        }
        c.set(Calendar.MILLISECOND, 0);
        return (c.before(min) || c.after(max));
    }

    private static boolean invalidDates(Calendar before, Calendar after)
    {
        boolean invalid = true;

        if (before != null)
        {
            if (after != null)
            {
                invalid = before.after(after);
            }
            else
            {
                invalid = false;
            }
        }
        else
        {
            if (after != null)
            {
                invalid = true;
            }
            else
            {
                invalid = false;
            }
        }

        return invalid;
    }

    private static boolean invalidCharacters(String checkStr,
            String invalidCharacters)
    {
        if (checkStr == null)
            return false;

        for (int i = 0; i < invalidCharacters.length(); ++i)
        {
            if (checkStr.indexOf(invalidCharacters.charAt(i)) >= 0)
                return true;
        }

        return false;
    }

    public static boolean getBoolean(Boolean b)
    {
        if (b == null)
            return false;
        return (b.booleanValue());
    }

    /**
     * @return Returns true if the objectToFind can be loaded using its manager.
     * Otherwise, returns false.
     */
    public static boolean validateExistingRPMObject(RPMObject object,
            String fieldName, RPMObject objectToFind, MessageContext msgCtx)
    {
        if (!isExistingRPMObject(objectToFind, msgCtx))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, "RPMObject", objectToFind.getID(),
                    "ID"
            };
            RPMException e = new RPMException(ErrorCodes.NOT_EXISTING_ID,
                errorParams, object.getClass().getName(), fieldName,
                object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }
    
    /**
     * @return Returns true if the objectToFind can be loaded using its manager.
     * Otherwise, returns false.
     */
    private static boolean isExistingRPMObject(RPMObject objectToFind,
            MessageContext msgCtx)
    {
        try
        {
            RPMObjectManager manager = RPMManagerFactory.getInstance().getRPMObjectManager(
                objectToFind.getClass().getName());
            objectToFind = manager.loadByPrimaryKey(objectToFind, null, msgCtx,
                false);
            if (objectToFind != null)
                return true;
        }
        catch (Exception e)
        {
            return false;
        }
        return false;
    }
    
    /**
     * @return Returns true if uniqueName does not exist in the table, false
     *         otherwise.
     */
    private static boolean isUnique(MessageContext msgCtx, String table,
            String fieldColumn, String uniqueName, String idColumn,
            String existingID)
    {
        String selectStmt;
        Object[] params;
        if (existingID == null)
        {
            selectStmt = "SELECT count(*) from " + table + " WHERE "
                    + fieldColumn + "=?";
            params = new Object[]
            {
                uniqueName.trim()
            };
        }
        else
        {
            selectStmt = "SELECT count(*) from " + table + " WHERE "
                    + fieldColumn + "=? AND " + idColumn + "<>?";
            params = new Object[]
            {
                    uniqueName.trim(), existingID
            };
        }

        return !executeStatement(msgCtx, selectStmt, params);
    }

    /**
     * @return Returns true if the parent is already at the maximum nesting level,
     *         false otherwise.
     */
    private static boolean exceedsNestingLevel(MessageContext msgCtx,
            String table, String parentID, int maxLevel)
    {
        String selectStmt = "SELECT count(*) from " + table + " WHERE "
                    + "ELEMENT_ID=? and LEVEL_ID>=?";
        Object[] params = new Object[]
        {
                parentID,
                new Integer(maxLevel-1)
        };

        return executeStatement(msgCtx, selectStmt, params);
    }

    /**
     * Executes the selectStmt with the provided parameters.
     *
     * @return Returns true if a result is obtained, false otherwise.
     */
    protected static boolean executeStatement(MessageContext msgCtx,
            String selectStmt, Object[] params)
    {
        Connection c = null;
        ResultSet rs = null;

        try
        {
            c = Manager.getConnection(msgCtx);
            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            try
            {
                rs = ManagerUtil.executeSqlQuery(msgCtx, c, selectStmt, params);
            }
            catch (Exception s)
            {
                logger.error(s);
            }

            if (rs.next())
            {
                if (rs.getInt(1) > 0)
                    return true;
            }
        }
        catch (Exception s)
        {
            logger.error(s);
        }
        finally
        {
            Manager.close(msgCtx, rs);
            Manager.releaseConnection(msgCtx, c);
        }

        return false;
    }

    /**
     * Adds an RPMException to the MessageContext.
     */
    public static void addException(RPMException e,
            MessageContext messageContext)
    {
        messageContext.addExceptionNoThrow(e);
    }

    /**
     * The maximum length of a string to be displayed in error messages
     */
    private static final int MAX_STRING_SIZE = 32;

    /**
     * Returns a substring of longString to be display in an error message If a
     * field is up to 3000 characters in length, we only want to display the
     * beginning of the string in the error message
     */
    protected static String getShortString(String longString)
    {
        String retVal = null;
        if (longString != null)
        {
            if (longString.length() > MAX_STRING_SIZE)
            {
                retVal = longString.substring(0, MAX_STRING_SIZE) + "...";
            }
            else
            {
                retVal = longString;
            }
            retVal += " of length: " + longString.length();
        }
        return retVal;
    }

    /**
     * Returns a substring containing any of the characters in invalidCharacters
     * that were found in longString
     */
    private static String getShortString(String longString,
            String invalidCharacters)
    {
        if (longString.length() > MAX_STRING_SIZE)
        {
            for (int i = 0; i < invalidCharacters.length(); ++i)
            {
                int index = longString.indexOf(invalidCharacters.charAt(i));
                if (index < 2)
                    return longString.substring(0, MAX_STRING_SIZE);
                else if (index + MAX_STRING_SIZE - 2 < longString.length())
                    return longString.substring(index - 2, index
                            + MAX_STRING_SIZE - 2);
                else
                    return longString.substring(index - 2, longString.length());
            }
        }
        return longString;
    }
    
    public static boolean validateTimeCode2(RPMObject object,
            AbstractTimeCode timecode, String fieldName, MessageContext msgCtx)
    {
        boolean found = true;

        if (!isInTimeCodeGroup(timecode, 2, msgCtx))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, timecode.getName().trim(),
                    getTimeCodeGroupName(false, msgCtx).trim()
            };
            RPMException e = new RPMException(
                ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, errorParams,
                object.getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            found = false;
        }

        return found;
    }

    public static boolean validateTimeCode1(RPMObject object,
            AbstractTimeCode timecode, String fieldName, MessageContext msgCtx)
    {
        boolean found = true;

        if (!isInTimeCodeGroup(timecode, 1, msgCtx))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, timecode.getName().trim(),
                    getTimeCodeGroupName(true, msgCtx).trim()
            };
            RPMException e = new RPMException(
                ErrorCodes.TIMECODE_NOT_IN_TIME_CODE_CATEGORY, errorParams,
                object.getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
            found = false;
        }

        return found;
    }

    // implemented by Simon L.
    private static boolean isInTimeCodeGroup(AbstractTimeCode timeCode,
            int timeCodeGroup, MessageContext msgCtx)
    {
        boolean result = false;
        TimeCodeGroup group = getTimeCodeGroup(timeCode);
        if (group != null)
        {
            String groupId = group.getID();
            if (groupId != null)
            {
                String expectedId = "TIMECODE_" + timeCodeGroup;
                result = groupId.startsWith(expectedId);
            }
        }
        else
        {
            try
            {
                SqlBuffer condition = new SqlBuffer();
                condition.appendEqualQuestionMark(TimecodeManager.NAME_ELEMENT_ID);
                
                Object[] arguments = new Object[]
                {
                    timeCode.getID()
                };
                
                String rank = (String) ManagerUtil.selectColumnValue(
                    String.class, timeCode, "parent",
                    TimecodeManager.NAME_RANK, TimecodeManager.TABLE_NAME,
                    condition, arguments, msgCtx);
                if (rank == null)
                {
                    logger.error("TimeCode not found: " + timeCode.getID());
                }
                else
                {
                    String expectedRank = "0000000" + timeCodeGroup;
                    result = rank.startsWith(expectedRank);
                }
            }
            catch (Exception x)
            {
                logger.error("TimeCodeGroup not found for TimeCode: "
                        + timeCode.getID(), x);
            }
        }

        return result;
    }

    // implemented by Simon L.
    private static TimeCodeGroup getTimeCodeGroup(AbstractTimeCode timeCode)
    {
        TimeCodeGroup result = null;
        AbstractTimeCode parent = (AbstractTimeCode) timeCode.getParent();
        if (parent != null)
        {
            while (parent.getParent() != null)
                parent = (AbstractTimeCode) parent.getParent();
        }
        if (parent instanceof TimeCodeGroup)
        {
            result = (TimeCodeGroup) parent;
        }
        return result;
    }

    private static String getTimeCodeGroupName(boolean timeCode1,
            MessageContext messageContext)
    {
        String id = null;
        if (timeCode1)
            id = "TIMECODE_1";
        else
            id = "TIMECODE_2";

        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(TimecodeManager.NAME_ELEMENT_ID);

        Object[] arguments = new Object[]
        {
            id
        };
        
        try
        {
            return (String) ManagerUtil.selectColumnValue(String.class, null,
                "name", TimecodeManager.NAME_ELEMENT_NAME,
                TimecodeManager.TABLE_NAME, condition, arguments, messageContext);
        }
        catch (RPMException e)
        {
            addException(e, messageContext);
            return null;
        }
    }

}
