package com.ibm.rpm.framework.util;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import org.apache.commons.lang.StringUtils;

/**
 * StringBuffer extension specialised in building the textual part of an SQL
 * statement. It can also be used to build only part of the statement, like, for
 * instance, the condition after the "WHERE" keyword.
 *
 * @author Simon Legault
 */
public class SqlBuffer
{

    /**
     * Length of <code>buffer</code> when the last condition started.
     */
    private int lastConditionStart = 0;

    /**
     * Buffer that contains the textual part of the SQL statement.
     */
    protected StringBuffer buffer = new StringBuffer();

    /**
     * Creates an empty <code>SqlBuffer</code>.
     *
     * @param initialContent
     *            Initial textual content.
     */
    public SqlBuffer()
    {
        // Nothing special to initialize.
    }

    /**
     * Creates a <code>SqlBuffer</code> with the given
     * <code>initialContent</code>.
     *
     * @param initialContent
     *            Initial textual content.
     */
    public SqlBuffer(String initialContent)
    {
        append(initialContent);
    }

    /**
     * Creates a <code>SqlBuffer</code> with the given
     * <code>initialContent</code>.
     *
     * @param initialContent
     *            Initial textual content.
     */
    public SqlBuffer(SqlBuffer initialContent)
    {
        append(initialContent);
    }

    /**
     * Returns true is this SQL buffer is empty. Otherwise, returns false.
     */
    public boolean isEmpty()
    {
        return buffer.length() == 0;
    }

    /**
     * Empties this SQL buffer. Its state is like after calling the default
     * constructor.
     *
     * @see #SqlBuffer()
     */
    public void empty()
    {
        lastConditionStart = 0;
        buffer.setLength(0);
    }

    /**
     * Indicates to this <code>SqlBuffer</code> that a new condition starts.
     * This is essential for the correct behavior of <code>appendAnd()</code>.
     * This situation happens, for instance, when appending "WHERE" or "(".
     *
     * @see #appendAnd()
     */
    public void startCondition()
    {
        lastConditionStart = buffer.length();
    }

    /**
     * Appends <code>" AND "</code> to this condition, if the latter is not
     * empty.
     *
     * @see #startCondition()
     */
    public void appendAnd()
    {
        if (buffer.length() > lastConditionStart)
        {
            buffer.append(" AND ");
        }
    }

    /**
     * Appends <code>" OR "</code> to this condition, if the latter is not
     * empty.
     *
     * @see #startCondition()
     */
    public void appendOr()
    {
        if (buffer.length() > lastConditionStart)
        {
            buffer.append(" OR ");
        }
    }

    /**
     * Appends <code>','</code>.
     */
    public void appendComma()
    {
        buffer.append(',');
    }

    /**
     * Appends a single quote <code>'\''</code>.
     */
    public void appendSingleQuote()
    {
        buffer.append('\'');
    }

    /**
     * Appends <code>'='</code>.
     */
    public void appendEqual()
    {
        buffer.append('=');
    }
    
    /**
     * Appends <code>'!='</code>.
     */
    public void appendNotEqual()
    {
        buffer.append("!=");
    }

    

    /**
     * Appends <code>"=?"</code>.
     */
    public void appendEqualQuestionMark()
    {
        buffer.append("=?");
    }

    /**
     * Appends <code>columnName + "=?"</code>.
     */
    public void appendEqualQuestionMark(String columnName)
    {
        appendAnd();
        buffer.append(columnName);
        appendEqualQuestionMark();
    }

    /**
     * Appends <code>" IS NULL"</code>.
     */
    public void appendIsNull()
    {
        buffer.append(" IS NULL");
    }

    /**
     * Appends <code>[" AND " +] columnName + " IS NULL"</code>.
     *
     * @param columnName
     *            Name of the database column.
     * @see #appendAnd()
     */
    public void appendIsNull(String columnName)
    {
        appendAnd();
        buffer.append(columnName);
        appendIsNull();
    }

    /**
     * Appends <code>" IS NOT NULL"</code>.
     */
    public void appendIsNotNull()
    {
        buffer.append(" IS NOT NULL");
    }

    /**
     * Appends <code>[" AND " +] columnName + " IS NOT NULL"</code>.
     *
     * @param columnName
     *            Name of the database column.
     * @see #appendAnd()
     */
    public void appendIsNotNull(String columnName)
    {
        appendAnd();
        buffer.append(columnName);
        appendIsNotNull();
    }

    public void appendOrderBy(String columnName)
    {
        if (columnName != null)
        {
            append(" ORDER BY ");
            append(columnName);
        }
    }

    /**
     * Appends <code>'('</code> and starts a new condition.
     *
     * @see #startCondition()
     */
    public void appendOpenParenthesis()
    {
        buffer.append('(');
        startCondition();
    }

    /**
     * Appends <code>')'</code>.
     */
    public void appendCloseParenthesis()
    {
        buffer.append(')');
    }

    /**
     * Closes all parenthesis that were left open.
     */
    public void closeAllParenthesis()
    {
        // TODO: Check this implementation.
        String currentContent = toString();
        int left = StringUtil.countOccurence(currentContent, "\\(");
        int right = StringUtil.countOccurence(currentContent, "\\)");

        int missing = left - right;
        for (int i = 0; i < missing; i++)
        {
            appendCloseParenthesis();
        }
    }

    /**
     * Appends <code>"SELECT "</code>.
     */
    public void appendSelect()
    {
        buffer.append("SELECT ");
    }

    /**
     * Appends <code>"SELECT columnName"</code>, if columnName is not null.
     */
    public void appendSelect(String columnName)
    {
        appendSelect();
        append(columnName);
    }

    /**
     * Appends <code>"DELETE FROM tableName WHERE "</code>.
     */
    public void appendDelete(String tableName)
    {
        appendDelete();
        appendFrom(tableName);
        appendWhere();
    }

    /**
     * Appends <code>"DELETE "</code>.
     */
    public void appendDelete()
    {
        append("DELETE ");
    }    

    /**
     * Appends <code>"DISTINCT "</code>.
     */
    public void appendDistinct()
    {
        append("DISTINCT ");
    }

    /**
     * Appends <code>" FROM "</code>.
     */
    public void appendFrom()
    {
        buffer.append(" FROM ");
    }

    /**
     * Appends <code>" FROM " + tableName</code> when <code>tableName</code>
     * is not empty.
     */
    public void appendFrom(String tableName)
    {
        if (!StringUtil.isEmpty(tableName))
        {
            appendFrom();
            append(tableName);
        }
    }

    /**
     * Appends <code>" WHERE "</code>.
     *
     * @see #startCondition()
     */
    public void appendWhere()
    {
        buffer.append(" WHERE ");
        startCondition();
    }

    /**
     * Appends <code>[" WHERE " + condition]</code> when
     * <code>condition</code> is not empty.
     *
     * @see #startCondition()
     */
    public void appendWhere(String condition)
    {
        if (!StringUtil.isEmpty(condition))
        {
            appendWhere();
            appendCondition(condition);
        }
    }

    /**
     * Appends <code>[" WHERE " + condition]</code> when
     * <code>condition</code> is not empty.
     *
     * @see #startCondition()
     */
    public void appendWhere(SqlBuffer condition)
    {
        if (!StringUtil.isEmpty(condition.buffer))
        {
            appendWhere();
            appendCondition(condition);
        }
    }

    /**
     * Appends <code>" IN("</code>.
     */
    public void appendIn()
    {
        buffer.append(" IN(");
        startCondition();
    }

    /**
     * Appends <code>" IN(" + paramList + ")"</code>.
     *
     * @param paramList
     *            List of parameters to be appended.
     * @see #appendParamList(Collection)
     */
    public void appendIn(Collection paramList)
    {
        appendIn(paramList, 0);
    }
    
    public void appendIn(Collection paramList, int fieldSize)
    {
        appendIn();
        appendParamList(paramList, fieldSize);
        appendCloseParenthesis();
    }

    /**
     * Appends nothing if <code>paramList</code> is null or empty. Appends
     * <code>[" AND " +] columnName + " IN(" + paramList + ")"</code> if
     * <code>paramList.size() > 1</code>.
     * <code>appendEqual(columnName, param)</code> if
     * <code>paramList.size() == 1</code>.
     *
     * @param columnName
     *            Name of the database column.
     * @param paramList
     *            Set of valid values.
     * @see #appendList(Collection)
     * @see #appendEqual(String, Object)
     * @see #appendIn(String, int[])
     */
    public void appendIn(String columnName, Collection paramList)
    {
        if (paramList != null)
        {
            int count = paramList.size();
            if (count > 0)
            {
                if (count == 1)
                {
                    appendEqual(columnName, paramList.iterator().next());
                }
                else
                {
                    appendAnd();
                    buffer.append(columnName);
                    appendIn();
                    appendParamList(paramList);
                    appendCloseParenthesis();
                }
            }
        }
    }

    /**
     * Appends nothing if <code>values</code> is null or empty. Appends
     * <code>[" AND " +] columnName + " IN(" + values + ")"</code> if
     * <code>values.length > 1</code>.
     * <code>appendEqual(columnName, value)</code> if
     * <code>values.length == 1</code>.
     *
     * @param columnName
     *            Name of the database column.
     * @param values
     *            Set of valid values.
     * @see #appendList(Collection)
     * @see #appendEqual(String, Object)
     * @see #appendIn(String, Collection)
     */
    public void appendIn(String columnName, int[] values)
    {
        appendIn(columnName, intArrayToCollection(values));
    }

    /**
     * Appends the given <code>list</code> of <code>Object</code> as
     * <code>String</code>, comma separated.
     *
     * @param list
     *            List of <code>String</code> to be appended.
     * @see #append(Object)
     */
    public void appendList(Collection list)
    {
        boolean isFirst = true;
        Iterator i = list.iterator();
        while (i.hasNext())
        {
            Object obj = i.next();
            if (obj != null)
            {
                if (isFirst)
                {
                    isFirst = false;
                }
                else
                {
                    appendComma();
                }
                append(obj);
            }
        }
    }

    /**
     * Appends the given <code>list</code> of <code>Object</code> as
     * a parameter in a condition, comma separated. Adds quotes where
     * needed.
     *
     * @param list
     *            List of <code>String</code> to be appended.
     * @see #appendParam(Object)
     */
    public void appendParamList(Collection list)
    {
    	appendParamList(list, 0);
    }
    
    public void appendParamList(Collection list, int fieldSize)
    {
        boolean isFirst = true;
        Iterator i = list.iterator();
        while (i.hasNext())
        {
            Object obj = i.next();
            if (obj != null)
            {
                if (isFirst)
                {
                    isFirst = false;
                }
                else
                {
                    appendComma();
                }
                appendParam(obj, fieldSize);
            }
        }
    }

    /**
     * Appends the given <code>list</code> of <code>int</code> as
     * <code>int</code>, comma separated.
     *
     * @param list
     *            List of <code>int</code> to be appended.
     * @see #append(Object)
     */
    public void appendList(int[] list)
    {
        boolean isFirst = true;
        for (int i = 0; i < list.length; i++)
        {
            int value = list[i];
            if (isFirst)
            {
                isFirst = false;
            }
            else
            {
                appendComma();
            }
            append(value);
        }
    }

    /**
     * Appends <code>[[" AND "] + otherCondition]</code>.
     * <code>" AND "</code> is appended only if required.
     * <code>[" AND "] + otherCondition</code> is appended only if
     * <code>otherCondition</code> is not empty.
     *
     * @param otherCondition
     *            Condition to be appended.
     * @see #appendAnd()
     */
    public void appendCondition(String otherCondition)
    {
        if (!StringUtil.isEmpty(otherCondition))
        {
            appendAnd();
            buffer.append(otherCondition);
        }
    }

    /**
     * Appends <code>[[" AND "] + otherCondition]</code>.
     * <code>" AND "</code> is appended only if required.
     * <code>[" AND "] + otherCondition</code> is appended only if
     * <code>otherCondition</code> is not empty.
     *
     * @param otherCondition
     *            Condition to be appended.
     * @see #appendAnd()
     */
    public void appendCondition(SqlBuffer otherCondition)
    {
        if (!StringUtil.isEmpty(otherCondition.buffer))
        {
            appendAnd();
            buffer.append(otherCondition);
        }
    }

    /**
     * Appends <code>[[" AND "] + otherCondition]</code>.
     * <code>" AND "</code> is appended only if required.
     * <code>[" AND "] + otherCondition</code> is appended only if
     * <code>otherCondition</code> is not empty.
     *
     * @param otherCondition
     *            Condition to be appended.
     * @see #appendAnd()
     */
    public void appendCondition(StringBuffer otherCondition)
    {
        if (!StringUtil.isEmpty(otherCondition))
        {
            appendAnd();
            buffer.append(otherCondition);
        }
    }

    /**
     * Appends <code>[" AND " +] columnName + '=' + value</code> to this SQL
     * buffer. Appends <code>[" AND " +] columnName + " IS NULL"</code> if
     * value is null. String or Character value is put between single quotes.
     *
     * @see #appendAnd()
     * @see #appendParam(Object)
     */
    public void appendEqual(String columnName, Object value)
    {
        appendAnd();
        buffer.append(columnName);
        if (value == null)
        {
            appendIsNull();
        }
        else
        {
            appendEqual();
            appendParam(value);
        }
    }
    
    
    public void appendNotEqual(String columnName, Object value)
    {
        appendAnd();
        buffer.append(columnName);
        if (value == null)
        {
            appendIsNull();
        }
        else
        {
            appendNotEqual();
            appendParam(value);
        }
    }

    /**
     * Appends <code>[" AND " +] columnName1 + '=' + columnName2</code> to
     * this SQL buffer.
     * 
     * @see #appendAnd()
     */
    public void appendEqualFields(String columnName1, String columnName2)
    {
        appendAnd();
        buffer.append(columnName1);
        appendEqual();
        buffer.append(columnName2);
    }

    /**
     * Appends <code>[" AND " +] columnName + '=' + value</code> to this SQL
     * buffer.
     *
     * @see #appendAnd()
     */
    public void appendEqual(String columnName, int value)
    {
        appendAnd();
        buffer.append(columnName);
        appendEqual();
        buffer.append(value);
    }

    /**
     * Appends <code>[" AND " +] columnName + "='" + value + "'"</code> to
     * this SQL buffer.
     *
     * @see #appendAnd()
     * @see #append(char)
     */
    public void appendEqual(String columnName, char value)
    {
        appendAnd();
        buffer.append(columnName);
        appendEqual();
        append(value);
    }

    /**
     * Appends the given <code>value</code> as a parameter in a condition.
     * String and Character instances are put between single quotes. Use
     * <code>appendParam(Boolean value)</code> for <code>Boolean</code>
     * value. It will handle <code>null Boolean</code> value correctly.
     *
     * @param value
     * @see #appendParam(Boolean)
     */
    public void appendParam(Object value)
    {
    	appendParam(value, 0);
    }
    
    public void appendParam(Object value, int fieldSize)
    {
        if (value instanceof String)
        {
            appendSingleQuote();
            buffer.append(StringUtils.rightPad((String)value, fieldSize));
            appendSingleQuote();
        }
        else if (value instanceof Character)
        {
        	appendSingleQuote();
            buffer.append(value);
            appendSingleQuote();
        }
        else if (value instanceof Date)
        {
            appendSingleQuote();
            buffer.append(DateUtil.convertToStringNoTime((Date) value));
            appendSingleQuote();
        }
        else if (value instanceof Calendar)
        {
            appendSingleQuote();
            buffer.append(DateUtil.convertToString((Calendar) value));
            appendSingleQuote();
        }
        else
        {
            append(value);
        }
    }

    /**
     * Appends the given boolean <code>value</code> as an integer in a
     * condition. <code>value</code> is translated to <code>0</code> or
     * <code>1</code> (specialization for RPM's database). <code>null</code>
     * is translated to <code>0</code>.
     *
     * @param value
     *            Boolean value to be appended.
     * @see #append(Boolean)
     */
    public void appendParam(Boolean value)
    {
        append(value);
    }

    /**
     * Appends "'" + value + "'".
     *
     * @param value
     */
    public void appendParam(char value)
    {
        appendSingleQuote();
        buffer.append(value);
        appendSingleQuote();
    }

    /**
     * Appends
     * <code>[" AND " +] foreignColumnName + "IN (SELECT " + localColumnName + " FROM " + tableName [+ " WHERE " + condition]</code>.
     * <code>" AND "</code> is appended only if required.
     * <code>" WHERE " + condition</code> is appended only if condition is not
     * empty.
     *
     * @param foreignColumnName
     *            Name of the first field of the sub-select
     * @param localColumnName
     *            Name of the second field of the sub-select
     * @param tableName
     *            Name of the database table
     * @param condition
     *            Condition to be added after "WHERE", if any.
     * @see #appendAnd()
     * @see com.ibm.rpm.framework.util.StringUtil#isEmpty(String)
     */
    public void appendSubSelect(String foreignColumnName,
            String localColumnName, String tableName, String condition)
    {
        appendAnd();
        buffer.append(foreignColumnName + " IN (SELECT " + localColumnName
                + " FROM " + tableName);
        appendWhere(condition);
    }
    
    /**
     * Appends
     * <code>[" OR " +] foreignColumnName + "IN (SELECT " + localColumnName + " FROM " + tableName [+ " WHERE " + condition]</code>.
     * <code>" WHERE " + condition</code> is appended only if condition is not
     * empty.
     *
     * @param foreignColumnName
     *            Name of the first field of the sub-select
     * @param localColumnName
     *            Name of the second field of the sub-select
     * @param tableName
     *            Name of the database table
     * @param condition
     *            Condition to be added after "WHERE", if any.
     * @see #appendOr()
     * @see com.ibm.rpm.framework.util.StringUtil#isEmpty(String)
     */
    public void appendOrSubSelect(String foreignColumnName,
            String localColumnName, String tableName, String condition)
    {
        appendOr();
        buffer.append(foreignColumnName + " IN (SELECT " + localColumnName
                + " FROM " + tableName);
        appendWhere(condition);
    }

    // =======================
    // StringBuffer Delegation
    // =======================

    /**
     * Appends <code>o</code> to this condition, if <code>o</code> is not
     * null.
     *
     * TODO: Special case for Calendar, Boolean, SqlBuffer, etc.?
     *
     * @param o
     *            Object to be appended.
     */
    public void append(Object o)
    {
        if (o != null)
        {
            buffer.append(o);
        }
    }

    /**
     * Appends <code>str</code> to this condition, if <code>str</code> is
     * not null.
     *
     * @param str
     *            String to be appended.
     */
    public void append(String str)
    {
        if (str != null)
        {
            buffer.append(str);
        }
    }

    /**
     * Appends <code>buffer</code> to this condition, if <code>buffer</code>
     * is not null.
     *
     * @param buffer
     *            String buffer to be appended.
     */
    public void append(StringBuffer otherBuffer)
    {
        if (otherBuffer != null)
        {
            buffer.append(otherBuffer);
        }
    }

    /**
     * Appends <code>buffer</code> to this condition, if <code>buffer</code>
     * is not null.
     *
     * @param buffer
     *            String buffer to be appended.
     */
    public void append(SqlBuffer otherBuffer)
    {
        if (otherBuffer != null)
        {
            buffer.append(otherBuffer.toString());
        }
    }

    /**
     * Appends the given boolean <code>value</code> as an integer in a
     * condition. <code>value</code> is translated to <code>0</code> or
     * <code>1</code> (specialization for RPM's database). <code>null</code>
     * is translated to <code>0</code>.
     *
     * @param value
     *            Boolean value to be appended.
     */
    public void append(Boolean value)
    {
        boolean val = false;
        if (value != null)
        {
            val = value.booleanValue();
        }
        append(val);
    }

    /**
     * Appends <code>c</code> to this condition.
     *
     * @param c
     *            Character to be appended.
     * @see java.lang.StringBuffer#append(char)
     */
    public void append(char c)
    {
        buffer.append(c);
    }

    /**
     * Appends <code>value</code> to this condition. Specialized for RPM's
     * database: translates boolean to integer.
     *
     * @param value
     *            <code>boolean</code> value to be appended.
     * @see java.lang.StringBuffer#append(boolean)
     */
    public void append(boolean value)
    {
        int intB = value ? 1 : 0;
        buffer.append(intB);
    }

    /**
     * Appends <code>value</code> to this condition.
     *
     * @param value
     *            <code>int</code> value to be appended.
     * @see java.lang.StringBuffer#append(int)
     */
    public void append(int value)
    {
        buffer.append(value);
    }

    /**
     * Appends <code>l</code> to this condition.
     *
     * @param l
     *            Long integer to be appended.
     * @see java.lang.StringBuffer#append(long)
     */
    public void append(long l)
    {
        buffer.append(l);
    }

    /**
     * Appends <code>f</code> to this condition.
     *
     * @param f
     *            Float number to be appended.
     * @see java.lang.StringBuffer#append(float)
     */
    public void append(float f)
    {
        buffer.append(f);
    }

    /**
     * Appends <code>d</code> to this condition.
     *
     * @param d
     *            Double float number to be appended.
     * @see java.lang.StringBuffer#append(double)
     */
    public void append(double d)
    {
        buffer.append(d);
    }

    /**
     * Returns the String representation of this SQL buffer.
     */
    public String toString()
    {
        return buffer.toString();
    }

    private static Collection intArrayToCollection(int[] array)
    {
        Collection collection = null;
        if (array != null)
        {
            collection = new ArrayList();
            for (int i = 0; i < array.length; i++)
            {
                collection.add(new Integer(array[i]));
            }
        }
        return collection;
    }

}
