package com.ibm.rpm.interfaces.impl;

import java.sql.Connection;
import java.sql.ResultSet;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.SessionCheckoutsDAO;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.interfaces.Authentication;
import com.ibm.rpm.interfaces.SessionUtility;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.security.managers.SecurityMask;

/**
 * Authentication implementation that authenticates using RPM's database stored
 * procedures.
 */
public class DatabaseAuthentication implements Authentication
{
    private static Log logger = LogFactory.getLog(DatabaseAuthentication.class);
    
    private ServerFactory factory;

    public DatabaseAuthentication(ServerFactory setfactory)
    {
        factory = setfactory;
    }

    public String login(String username, String password, String dsn)
            throws RPMException
    {
        String retVal = login("SP_LOGON", username, password, dsn);
        logger.info("User logged in Web Services : \"" + username
            + "\" using dsn : \"" + dsn + "\". Session ID: " + retVal);
        return retVal;
    }

    protected String login(String procedureName, String username, String password, String dsn)
            throws RPMException
    {                       
        Connection c = null;
        ResultSet rs = null;
        MessageContext messageContext = null;
        try
        {
            DataSource dataSource = getSessionUtility().getDataSource(dsn);

            messageContext = createMessageContext();
            messageContext.registerMessageContext();
            messageContext.addDatasource(dataSource);
            messageContext.setAutomaticCheckout(true);
            
            // Checks database compatibility.
            DatabaseCompatibility compatibility = new DatabaseCompatibility(
                    messageContext);
            int compatibilityLevel = compatibility.getLevel();

            // Fail if incompatible or unknown.
            if (compatibilityLevel < DatabaseCompatibility.PARTIALLY_COMPATIBLE)
            {
                String message = compatibility.getMessage();
                throw new RPMException( ErrorCodes.FAILED_VERSION_COMPATIBILITY_TEST,  new String[]{message});
            }
            // Log warning if partially compatible.
            else if (compatibilityLevel < DatabaseCompatibility.COMPATIBLE)
            {
                compatibility.log();
            }
            
            c = Manager.getConnection(messageContext);

            rs = Manager.executeStoredProcedure(c, procedureName, new Object[]
            {
                    username, password, new Integer(481)
            }, false, messageContext);
            int error = -1;
            if ((rs != null) && (rs.next()))
            {
                error = rs.getInt(1);
                if (error == 0)
                {
                    String id = rs.getString(3);
                    
                    //close resultSet
                    Manager.close(messageContext, rs);
                    Manager.commitConnection(c);
                    
                    boolean useApiFlag = getOptions().isSecurityFlagUsed();
                    if (useApiFlag == true)
                    {
                        checkWebServicesSecurityFlag(id,messageContext);
                    } 
                    
                    String retVal = getSessionUtility().addResourceToContext(
                            messageContext, id, dsn);
                    
                    return retVal;
                }
                RPMException e = new RPMException(error);
                throw e;
            }
            RPMException e = new RPMException(ErrorCodes.INTERNAL_ERROR,
                    new String[]{"Result set returned from SP_LOGON is null."});
            throw ManagerCaller.handleException(e);
        }
        catch (Throwable e1)
        {
            throw ManagerCaller.handleException(e1);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.commitAndCloseConnection(messageContext, c);
        }
    }

    public void assignWorkflowToSession(String sessionId, String workflowId)
            throws RPMException
    {
        MessageContext messageContext = null;
        try{            
            messageContext = createMessageContext();

            DataSource dataSource = getSessionUtility().getDataSource();
            messageContext.addDatasource(dataSource);
            
            factory.getManagerCaller().fillSessionInformation(messageContext,
                sessionId, "assignWorkflowToSession");
            
            if (sessionId == null)
            {
                RPMException e = new RPMException(
                    ErrorCodes.INVALID_SESSION_ID, new String[]
                    {
                            "assignWorkflowToSession", sessionId
                    });
                throw ManagerCaller.handleException(e);
            }

            if (workflowId == null)
            {
                RPMException e = new RPMException(
                    ErrorCodes.INVALID_WORKFLOW_ID, new String[]
                    {
                        workflowId
                    });
                throw ManagerCaller.handleException(e);
            }

            getSessionUtility().assignWorkflowToSession(messageContext,
                sessionId, workflowId);

            logger.info("Workflow assigned to session. Session ID : "
                    + sessionId + ", Workflow ID : " + workflowId);
        }
        finally
        {
            Manager.commitAndCloseConnection(messageContext);
        }
    }

    public void logout(String sessionID) throws RPMException
    {
        MessageContext messageContext = null;
        try
        {
            messageContext = createMessageContext();

            DataSource dataSource = getSessionUtility().getDataSource();
            messageContext.addDatasource(dataSource);
            
            if (sessionID == null
                    || !getSessionUtility()
                        .isSessionRegistered(messageContext, Long.parseLong(sessionID)))
            {
                RPMException ex = new RPMException(ErrorCodes.INVALID_SESSION_ID,
                    new String[]
                    {
                        "logout", sessionID 
                    });
                throw ManagerCaller.handleException(ex);
            }
            
            factory.getManagerCaller().fillSessionInformation(messageContext, sessionID, "logout");

            try
            {
                getSessionUtility()
                    .deleteResourceFromContext(messageContext, sessionID);
            }
            catch (RPMException e4)
            {
                throw ManagerCaller.handleException(e4);
            }
            
            String userLoginName = "unknown user";
            RPMObject userObject = messageContext.getUser();
            if( userObject instanceof Resource )
            {
                Resource userResource = (Resource) userObject;
                userLoginName = userResource.getUserName();
            }
            
            logger.info("Session logged out : " + sessionID + ", for user: " + userLoginName);
        }
        finally
        {
            Manager.commitAndCloseConnection(messageContext);
        }
    }

    public void changePassword(String username, String oldPw, String newPw,
            String dsn) throws RPMException
    {
        Connection c = null;
        ResultSet rs = null;
        MessageContext messageContext = null;
        try
        {
            DataSource dataSource = getSessionUtility().getDataSource(dsn);

            messageContext = createMessageContext();
            messageContext.registerMessageContext();
            messageContext.addDatasource(dataSource);
            c = Manager.getConnection(messageContext);

            rs = Manager.executeStoredProcedure(c, "SP_U_RES_PASS_EXP",
                    new Object[]
                    {
                            username, oldPw, newPw
                    }, false, messageContext);
            int error = -1;
            if ((rs != null) && (rs.next()))
            {
                error = rs.getInt(1);
                if (error != 0)
                {
                    throw new RPMException(error);
                }
            }
        }
        catch (Throwable e1)
        {
            throw ManagerCaller.handleException(e1);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.commitAndCloseConnection(messageContext, c);
        }
        
        logger.info("User password change completed for user : " + username);
    }

    
    private void checkWebServicesSecurityFlag(String id, MessageContext messageContext)
            throws RPMException
    {
        SqlBuffer condition = new SqlBuffer();
        condition.appendEqualQuestionMark(ResourceManager.NAME_RESOURCE_ID);

        Integer iClearanceLevel4 = (Integer) ManagerUtil.selectColumnValue(
            Integer.class, null, null, ResourceManager.NAME_CLEARANCE_LEVEL_4,
            ResourceManager.TABLE_NAME, condition, new Object[]
            {
                id
            }, messageContext);

        if (iClearanceLevel4 != null)
        {
            int clearanceLevel4 = iClearanceLevel4.intValue();

            Boolean canUseWebServicesAPI = new Boolean(
                (clearanceLevel4 & SecurityMask.MASK_CAN_USE_WEB_SERVICES_API) == SecurityMask.MASK_CAN_USE_WEB_SERVICES_API);
            if (!canUseWebServicesAPI.booleanValue())
            {
                RPMException e = new RPMException(ErrorCodes.CANNOT_USE_WS_API);
                throw e;
            }
        }
    }
    
    public RpmOptions getOptions()
    {
        return factory.getCurrentOptions();
    }
    
    public SessionUtility getSessionUtility()
    {
        return factory.getSession();
    }
    
    public SessionCheckoutsDAO getSessionCheckoutsDAO()
    {
        return factory.getSessionCheckoutDAO();
    }

    private MessageContext createMessageContext()
    {
        return new MessageContext(factory);
    }
}
