package com.ibm.rpm.interfaces.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Calendar;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RpmOptions;
import com.ibm.rpm.framework.RpmResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.init.RPMInitializer;
import com.ibm.rpm.interfaces.SessionUtility;
import com.ibm.rpm.resource.containers.Resource;


/**
 * @author gosselin
 * 
 * Manage access to the API at the session level. Each active session is
 * identified by an entry in the WEBAPI_SESSION_MONITOR table
 */
public class DefaultSessionUtility implements SessionUtility
{
    public static String WEBAPI_SESSION_MONITOR_TABLE = "WEBAPI_SESSION_MONITOR";
    
    public static String SESSION_ID = "SESSION_ID";

    public static String RESOURCE_ID = "RESOURCE_ID";

    public static String DSN = "DSN";

    public static String ACTIVITY = "ACTIVITY";

    public static String AUTOMATIC_CHECKOUT = "AUTOMATIC_CHECKOUT";

    public static String WORKFLOW_ID = "WORKFLOW_ID";
    
    private static TimeoutCheckThread timeoutCheckThread;

    private static Thread sessionTimeoutThread;

    /**
     * Remove expired sessions every minute.
     */
    private int idleLoopDelay = 60000;

    protected static Log logger = LogFactory.getLog(DefaultSessionUtility.class);
    
    private ServerFactory factory;
    
    public DefaultSessionUtility(ServerFactory usefactory) {
        super();
        factory = usefactory;
        RPMInitializer.init();
        sessionTimeoutThread = new Thread(timeoutCheckThread);
    }

	/**
	 * @return Returns the idleLoopDelay.
	 */
	public int getIdleLoopDelay() {
		return idleLoopDelay;
	}

	/**
	 * @param idleLoopDelay
	 *            The idleLoopDelay to set.
	 */
	public void setIdleLoopDelay(int idleLoopDelay) {
		this.idleLoopDelay = idleLoopDelay;
	}
    
	public boolean isSessionTimeoutUsed() {
		return getOptions().isSessionTimeoutUsed();
	}

	public int getSessionTimeoutDelayInSeconds() {
		return getOptions().getSessionTimeoutInSeconds();
	}

    public DataSource getDataSource() throws RPMException 
    {
        return getDataSource(RPM_DATASOURCE);
    }
    
	public DataSource getDataSource(String dsn) throws RPMException {
		DataSource result = null;
		Context initCtx = null;
    	try
        {
            initCtx = new InitialContext();

            try
            {
                // first, try the standard java:comp/env
                Context envCtx = (Context) initCtx.lookup("java:comp/env");
                result = (DataSource) envCtx.lookup(dsn);
            }
            catch (NamingException e1)
            {
                // If it doesn't work, try to use the initial context directly.
                result = (DataSource) initCtx.lookup(dsn);
            }
        }
        catch (NamingException e)
        {
            throw new RPMException(
                "Can not find datasource in initial context or java:comp/env",
                e);
        }			
        
        return result;
    }

    public DataSource getRPMDataSource() throws RPMException {
        return getDataSource(RPM_DATASOURCE);

    }
    
    public boolean isSessionTableFunctionnal()
    {
        return getCurrentSessionTimeoutError() == null;
    }
    
    private Exception currentSessionTimeoutError = null;
    
    public synchronized Exception getCurrentSessionTimeoutError()
    {
        return currentSessionTimeoutError;
    }
    
    public synchronized void setCurrentSessionTimeoutError(Exception ex )
    {
        currentSessionTimeoutError = ex;
    }
    
    /**
     * We use the database's time in order to avoid developers in different
     * timezones inadvertantly remove valid sessions from other timezones.
     */
    public long getDatabaseTimestamp(MessageContext messageContext) throws RPMException
    {
        Calendar ts = (Calendar) factory.getManagerCaller()
            .getDatabaseTimestamp(messageContext);
        return ts.getTimeInMillis();
    }

    public void checkToTimeoutSessions() throws RPMException
    {
        if (!isSessionTimeoutUsed())
        {
            return;
        }

        MessageContext context = null;
        Connection c = null;
        ResultSet rs = null;
        try
        {
            context = new MessageContext(factory, null, false, "checkToTimeoutSessions");
            setDatasourceIfNecessary(context);
            
            String selectStmt = "SELECT " + SESSION_ID + ", " + RESOURCE_ID
                    + ", " + DSN + ", " + ACTIVITY + " FROM "
                    + WEBAPI_SESSION_MONITOR_TABLE + " WHERE " + ACTIVITY
                    + " < ?";

            int timeoutDelay = getSessionTimeoutDelayInSeconds();
            long databaseTimestamp = getDatabaseTimestamp(context);
            Timestamp currentDatabaseStamp = new Timestamp(databaseTimestamp);
            Timestamp timeoutStamp = new Timestamp(databaseTimestamp
                    - (timeoutDelay * 1000));
            Object[] arguments = new Object[]
            {
                timeoutStamp
            };


            c = Manager.getConnection(context);
            selectStmt = Manager.makeStatementUncommitedRead(selectStmt, c);
            rs = ManagerUtil.executeSqlQuery(context, c, selectStmt, arguments);
            
            while (rs.next())
            {
                String sessionID = rs.getString(SESSION_ID);
                String resourceID = rs.getString(RESOURCE_ID);
                String datasourceName = rs.getString(DSN);
                Timestamp currentActivity = rs.getTimestamp(ACTIVITY);
                deleteTimedOutSession(sessionID, resourceID, datasourceName, currentActivity, currentDatabaseStamp);
            }
            
            // the whole process worked, flush any current error
            setCurrentSessionTimeoutError(null);
        }
        catch (Exception e)
        {
            Exception currentError = getCurrentSessionTimeoutError();
            if( currentError != null 
                    && currentError.getClass() == e.getClass() 
                    && StringUtils.equals(e.getMessage(), currentError.getMessage()))
            {
                // skip the error handling when we detect the same exception twice
            }
            else
            {
                // new error detected, log the error and store it for future reference
                setCurrentSessionTimeoutError(e);
                ExceptionUtil.handleException(this, e);
            }
        }
        finally
        {
            Manager.commitAndCloseConnection(context, c);
        }
    }

    private void deleteTimedOutSession(String sessionID, String resourceID,
            String datasourceName, Timestamp sessionTimestamp,
            Timestamp databaseTimestamp) throws RPMException, SQLException,
            ParseException
    {
        MessageContext messageContext = new MessageContext(factory, sessionID, false, "SessionTimeout");
        setDatasourceIfNecessary(messageContext);
        
        Resource loadResource = new Resource();
        loadResource.setID(resourceID);

        RPMObjectManager resourceManager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager("RESOURCE");
        
        Resource retVal = (Resource) resourceManager.load(loadResource, null,
            messageContext);
        
        String userName = "Deleted User";
        if( retVal == null )
        {
            // the user has been deleted from the database, use the system
            // administrator to delete the session.
            loadResource.setID("PMO_SUPERVISOR");
            retVal = (Resource) resourceManager.load(loadResource, null,
                messageContext);
        }
        else
        {
            userName = retVal.getUserName();
        }

        messageContext.addUser(retVal);

        if( logger.isInfoEnabled())
        {
            StringBuffer timeoutLog = new StringBuffer();
            
            long inactivityTimeInSeconds = (databaseTimestamp.getTime() - sessionTimestamp.getTime())/1000;
            
            timeoutLog.append("Session Timeout detected on session: ");
            timeoutLog.append(sessionID);
            timeoutLog.append(", User Name: ");
            timeoutLog.append(userName);
            timeoutLog.append(", User ID: ");
            timeoutLog.append(resourceID);
            timeoutLog.append(", Last Activity Timestamp: ");
            timeoutLog.append(sessionTimestamp);
            timeoutLog.append(". Current Database Timestamp: ");
            timeoutLog.append(databaseTimestamp);
            timeoutLog.append(", Session was inactive for: ");
            timeoutLog.append(inactivityTimeInSeconds);
            timeoutLog.append(" seconds, Inactivity limit: ");
            timeoutLog.append(getSessionTimeoutDelayInSeconds());
            timeoutLog.append(" seconds.");
            
            logger.info(timeoutLog.toString());
        }
        
        deleteResourceFromContext(messageContext, sessionID);
    }
    
    public void deleteResourceFromContext(MessageContext messageContext,String sessionid){
        Connection c = null;

        try 
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);
            String deleteStatement = "DELETE FROM WEBAPI_SESSION_MONITOR WHERE SESSION_ID = ?";
            Object[] parameters = new Object[]{sessionid.toString()};
            ManagerUtil.executeSqlUpdate(messageContext, c, deleteStatement, parameters);
        } 
        catch (Exception e) 
        {
            try
            {
                // handle and log the error
                handleSessionDBError(e, c);
            }
            catch (Exception e1)
            {
                // ignore any errors
            }
        } 
        finally 
        {
            try
            {
                Manager.releaseConnection(messageContext, c);
            }
            catch (Exception e)
            {
                // ignore errors
            }
        }
        
        try
        {
            factory.getSessionCheckoutDAO().undoSessionCheckouts(messageContext, sessionid);
        }
        catch (RPMException e)
        {
            // Ignore errors
        }

    }

    private static Object SESSION_ID_MUTEX = new Object();

    public long getNewSessionID(MessageContext ctx) throws RPMException {
        long retVal = 0;

        double randomID = Math.random() * Long.MAX_VALUE;
        retVal = (long) randomID;

        while (isSessionRegistered(ctx, retVal)) {
            ++retVal;
        }

        return retVal;
    }

    /**
     * @param retVal
     * @return
     */
    public boolean isSessionRegistered(MessageContext messageContext, long sessionID) throws RPMException {
        boolean retVal = false;
        Connection c = null;
        ResultSet resultSet = null;

        try {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStatement = "SELECT RESOURCE_ID FROM WEBAPI_SESSION_MONITOR WHERE SESSION_ID = ?";

            String longSessionID = Long.toString(sessionID);
            Object[] args = new Object[]{longSessionID};
            resultSet = ManagerUtil.executeSqlQuery(messageContext, c, selectStatement, args);
            if (resultSet.next()) {
                retVal = true;
            }
        } catch (Exception e) {
            handleSessionDBError(e, c);
        } finally {
            Manager.close(messageContext, resultSet);
            Manager.releaseConnection(messageContext, c);
        }

        return retVal;
    }

    public String addResourceToContext(MessageContext messageContext, String resourceID, String dsn)
            throws RPMException {
        String retVal = null;
        Connection c = null;

        synchronized (SESSION_ID_MUTEX) {
            try {
                Long sessionid = new Long(getNewSessionID(messageContext));
                Timestamp timeStamp = new Timestamp(getDatabaseTimestamp(messageContext));

                setDatasourceIfNecessary(messageContext);                
                c = Manager.getConnection(messageContext);
                String insertStatement = "INSERT INTO WEBAPI_SESSION_MONITOR (SESSION_ID,RESOURCE_ID,DSN,ACTIVITY, AUTOMATIC_CHECKOUT) values (?,?,?,?,?)";
                
                Object[] parameters = new Object[]{sessionid.toString(), resourceID, dsn, timeStamp, Boolean.TRUE};
                
                ManagerUtil.executeSqlUpdate(messageContext, c, insertStatement, parameters);
                retVal = sessionid.toString();
            } catch (Exception e) {
                handleSessionDBError(e, c);
            } finally {
                Manager.releaseConnection(messageContext, c);
            }
        }

        return retVal;
    }
    
    public void assignWorkflowToSession(MessageContext messageContext,String sessionId, String workflowId)
            throws RPMException
    {
        Connection c = null;
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);
            String updateStatement = "UPDATE WEBAPI_SESSION_MONITOR SET WORKFLOW_ID = ? WHERE SESSION_ID = ?";
            Object[] parameters = new Object[]{workflowId, sessionId};
            ManagerUtil.executeSqlUpdate(messageContext, c, updateStatement, parameters);
        }
        catch (Exception e)
        {
            handleSessionDBError(e, c);
        }
        finally
        {
            Manager.releaseConnection(messageContext, c);
        }
    }

    public boolean isAutomaticCheckout(MessageContext messageContext, String sessionID) throws RPMException
    {
        Connection c = null;
        ResultSet resultSet = null;
        boolean isAutomaticCheckout = true; //default value

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStatement = "SELECT AUTOMATIC_CHECKOUT FROM WEBAPI_SESSION_MONITOR WHERE SESSION_ID = ?";

            Object[] arguments = new Object[]{sessionID};
            resultSet = ManagerUtil.executeSqlQuery(messageContext, c, selectStatement, arguments);
            if (resultSet.next())
            {
                isAutomaticCheckout = resultSet.getBoolean(1);
            }
        }
        catch (Exception e)
        {
            handleSessionDBError(e, c);
        }
        finally
        {
            Manager.close(messageContext, resultSet);
            Manager.releaseConnection(messageContext, c);
        }

        return isAutomaticCheckout;
    }
    
    public void setAutomaticCheckout(MessageContext messageContext, String sessionID,
            boolean isAutomaticCheckout) throws RPMException
    {
        Connection c = null;

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String updateStatement = "UPDATE WEBAPI_SESSION_MONITOR SET AUTOMATIC_CHECKOUT = ? WHERE SESSION_ID = ?";

            Object[] arguments = new Object[]{Boolean.valueOf(isAutomaticCheckout), sessionID};
            ManagerUtil.executeSqlUpdate(messageContext, c, updateStatement, arguments);
        }
        catch (Exception e)
        {
            handleSessionDBError(e, c);
        }
        finally
        {
            Manager.releaseConnection(messageContext, c);
        }
    }

    private void handleSessionDBError(Exception e, Connection c)
            throws RPMException 
    {
        String fullMessage = e.toString();
        if (e instanceof SQLException) 
        {
            SQLException x = (SQLException)e;
            fullMessage = RpmResult.getMessageFromSqlException(x);
            try 
            {
                rollbackConnection(c);
            } catch (RPMException e1) 
            {
                // Ignore rollback error.
            }
        } 
        else if (e instanceof RPMException) 
        {
            RPMException ex = (RPMException) e;
            throw ex;
        }

        ExceptionUtil.handleException(this, e, ErrorCodes.INVALID_SESSION_DB,
                new String[]{fullMessage});
    }

    public void rollbackConnection(Connection c) throws RPMException {
        if (c != null) {
            try {
                c.rollback();
            } catch (SQLException s2) {
                throw ManagerCaller.handleException(s2);
            }
        }
    }
    
    public void reloadContext(MessageContext messageContext, String sessionID, 
            String operationName)
            throws RPMException
    {
        Connection c = null;
        String resourceid = null;
        ResultSet resultSet = null;

        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);

            String selectStatement = "SELECT RESOURCE_ID,DSN,AUTOMATIC_CHECKOUT,WORKFLOW_ID FROM WEBAPI_SESSION_MONITOR WHERE SESSION_ID = ?";

            Object[] args = new Object[]{sessionID};
            resultSet = ManagerUtil.executeSqlQuery(messageContext, c, selectStatement, args);

            boolean isAutomaticCheckout = true;
            String workflowId = null;
            if (resultSet.next())
            {
                resourceid = resultSet.getString(1);
                //DSN is ignored now that we have only one datasource (RPMDATASOURCE)
                isAutomaticCheckout = resultSet.getBoolean(3);
                workflowId = resultSet.getString(4);
            }
            else
            {
                ExceptionUtil.handleRPMException(this,
                    ErrorCodes.INVALID_SESSION_ID, new String[]{operationName, sessionID});
            }

            Resource loadResource = new Resource();
            loadResource.setID(resourceid);

            messageContext.setAutomaticCheckout(isAutomaticCheckout);
            messageContext.setWorkflowId(workflowId);
            
            Resource retVal = (Resource) RPMManagerFactory.getInstance()
                .getRPMObjectManager("RESOURCE").load(loadResource, null,
                    messageContext); 

            if( retVal == null )
            {
                ExceptionUtil.handleRPMException(this,
                    ErrorCodes.INVALID_SESSION_ID, new String[]{operationName, sessionID});

            }
            
            //note: ressource should not stay in the cache in case the
            // user needs to load it with a scope in the same call
            messageContext.getCache().removeContainerFromCache(retVal);
 
            messageContext.addUser(retVal);
        }
        catch (Exception e)
        {
            handleSessionDBError(e, c);
        }
        finally
        {
            Manager.close(messageContext, resultSet);
            Manager.releaseConnection(messageContext, c);
        }
    }

    public void fillContext(MessageContext context, String sessionID, String operationName)
            throws RPMException
    {
        validateWebServicesEnabled(operationName);
        reloadContext(context, sessionID, operationName);
        updateSessionActivityTimer(context, sessionID );
    }

    public void validateWebServicesEnabledAndLog(String operationName)
            throws RPMException
    {
        internalValidateWebServicesEnabled(operationName, true);
    }

    private void internalValidateWebServicesEnabled(
            String operationName, boolean log) throws RPMException
    {
        if (!getOptions().isWebServicesEnabled())
        {
            RPMException e = new RPMException(
                ErrorCodes.WEB_SERVICES_DISABLED_ERROR, new String[]
                {
                    operationName
                });
            if (log)
            {
                e.log();
            }
            throw e;
        }
    }

    public void validateWebServicesEnabled(String operationName)
            throws RPMException
    {
        internalValidateWebServicesEnabled(operationName, false);
    }

    /**
     * @param sessionID
     */
    private void updateSessionActivityTimer(MessageContext messageContext, String sessionID)
            throws RPMException {
        Connection c = null;

        try {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);
            
            String updateStatement = "UPDATE WEBAPI_SESSION_MONITOR SET ACTIVITY = ? WHERE SESSION_ID = ?";

            Object[] arguments = new Object[]{new Timestamp(getDatabaseTimestamp(messageContext)), sessionID};


            ManagerUtil.executeSqlUpdate(messageContext, c, updateStatement, arguments);
        } catch (Exception e) {
            handleSessionDBError(e, c);
        } finally {
            Manager.releaseConnection(messageContext, c);
        }
    }

    public void start() 
    {
        synchronized( DefaultSessionUtility.class )
        {
            if( timeoutCheckThread == null )
            {
                timeoutCheckThread = new TimeoutCheckThread(this);
                sessionTimeoutThread = new Thread(timeoutCheckThread);
            }
            
            if (sessionTimeoutThread.isAlive() ) {
                // thread already started
                return;
            }
    
            checkVersions();
            getOptions().logInitialOptions();
            
            sessionTimeoutThread.start();
        }
    }
    
    /**
     * Logs a message that states the compatibility of the WS API with the
     * database.
     */
    private void checkVersions()
    {
        try
        {
            MessageContext context = new MessageContext(factory);
            setDatasourceIfNecessary(context);
            DatabaseCompatibility compatibility = new DatabaseCompatibility(
                    context);
            compatibility.log();
        }
        catch (Exception e)
        {
            logger.error(e.getMessage(), e);
        }
    }
    
    public void stop() {
        synchronized( DefaultSessionUtility.class )
        {
            if (sessionTimeoutThread == null || timeoutCheckThread == null
                    || !sessionTimeoutThread.isAlive())
            {
                // thread already stopped
                return;
            }
    
            timeoutCheckThread.setRunning(false);
            try {
                sessionTimeoutThread.join(1000);
            } catch (InterruptedException e) {
                ManagerCaller.handleException(e);
            }
        }
    }

    public static class TimeoutCheckThread implements Runnable {

        private boolean running = true;
        
        private DefaultSessionUtility sessionUtility;
        
        public TimeoutCheckThread(DefaultSessionUtility setSessionUtility)
        {
            sessionUtility = setSessionUtility;
        }

        public synchronized boolean isRunning() {
            return running;
        }

        public synchronized void setRunning(boolean setRunning) {
            running = setRunning;
            notifyAll();
        }

        public synchronized void waitForNextLoop() {
            try {
                wait(sessionUtility.getIdleLoopDelay());
            } catch (InterruptedException e) {
                // ignore errors
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Runnable#run()
         */
        public void run() {
            while (true) {
                waitForNextLoop();
                if (isRunning()) {
                    
                      try { 
                          sessionUtility.checkToTimeoutSessions(); 
                      } catch (RPMException e) {
                        ManagerCaller.handleException(e); 
                      }
                } else {
                    break;
                }
            }
        }

    }

    public RpmOptions getOptions()
    {
        return factory.getCurrentOptions();
    }

    private void setDatasourceIfNecessary(MessageContext messageContext)throws RPMException
    {
        if(messageContext.getDatasource()==null)
        {
            DataSource ds = getDataSource(RPM_DATASOURCE);
            messageContext.addDatasource(ds);
        }
    }

    
//    private void closeConnection(MessageContext messageContext, Connection c,
//            ResultSet resultSet) throws RPMException
//    {
//        try
//        {
//            Manager.close(messageContext, resultSet);
//        }
//        finally
//        {
//            Manager.commitAndCloseConnection3(messageContext, c);
//        }
//    }
    
//    private void commitAndCloseConnection(MessageContext messageContext) 
//    {
//        if(messageContext!=null && messageContext.getConnection()!=null)
//        {
//            Manager.commitAndCloseConnection3(messageContext, messageContext.getConnection());
//            messageContext.setConnection(null);                
//        }
//    }
}
