package com.ibm.rpm.interfaces.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Stack;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.communications.containers.Notification;
import com.ibm.rpm.communications.containers.StaffingRequest;
import com.ibm.rpm.communications.containers.WorkflowResponse;
import com.ibm.rpm.communications.managers.CommunicationManager;
import com.ibm.rpm.communications.managers.WorkflowResponseManager;
import com.ibm.rpm.communications.scope.GenericCommunicationScope;
import com.ibm.rpm.communications.scope.WorkflowResponseScope;
import com.ibm.rpm.document.containers.GenericDocument;
import com.ibm.rpm.document.managers.DocumentManager;
import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.framework.AlreadyCheckedOutResult;
import com.ibm.rpm.framework.LoadParams;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.Property;
import com.ibm.rpm.framework.PropertyArray;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.ReloadType;
import com.ibm.rpm.framework.RpmResult;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.constants.ValidationConstants;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.util.ArrayUtil;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.ExceptionUtil;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.framework.util.SystemInfo;
import com.ibm.rpm.framework.util.stats.StatsUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.resource.scope.ResourceScope;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.AbstractScope;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.managers.ScopeManager;
import com.ibm.rpm.scorecard.containers.AssignedScorecard;
import com.ibm.rpm.scorecard.containers.AssignedScorecardState;
import com.ibm.rpm.scorecard.containers.Scorecard;
import com.ibm.rpm.scorecard.managers.PublishedScorecardManager;
import com.ibm.rpm.scorecard.managers.ScorecardAssignmentManager;
import com.ibm.rpm.scorecard.scope.AbstractScorecardScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.ResourceTaskAssignmentManager;
import com.ibm.rpm.wbs.managers.TaskAssignmentManager;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.workflow.containers.RunningWorkflowProcess;
import com.ibm.rpm.workflow.containers.WorkflowProcess;
import com.ibm.rpm.workflow.managers.RunningWorkflowProcessManager;
import com.ibm.rpm.workflow.managers.WorkflowManager;
import com.ibm.rpm.workflow.scope.RunningWorkflowProcessScope;
import com.ibm.rpm.xpathparser.Table;
import com.ibm.rpm.xpathparser.XPathToStack;
import com.ibm.rpm.xpathparser.XPathUtil;

/** 
 * @author mcduffs
 */
public class ManagerCaller
{
    private static final String LOAD = "load";

    private static final Log logger = LogFactory.getLog(ManagerCaller.class);

    public static boolean PRINT_EXCEPTION_STACKTRACE = true;

    private ServerFactory factory;

    private static final String DATASOURCE_NAME = "jdbc/RPMDATASOURCE";

    public ManagerCaller(ServerFactory usefactory)
    {
        factory = usefactory;
    }

    private RPMManagerFactory managerFactory = RPMManagerFactory.getInstance();

    private RPMObjectManager getManager(RPMObject object) throws RPMException
    {
        if (object == null)
        {
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, new String[]
            {
                "Failed to get manager for null object"
            });
        }
        return getManager(object.getClass());
    }

    private RPMObjectManager getManager(Class managerClass) throws RPMException
    {
        RPMObjectManager manager = managerFactory
            .getRPMObjectManager(managerClass);
        if (manager == null)
        {
            throw new RPMException(ErrorCodes.MANAGER_NOT_IMPLEMENTED,
                new String[]
                {
                    StringUtil.getShortClassName(managerClass)
                }, managerClass.getName());
        }
        return manager;
    }

    public RPMManagerFactory getManagerFactory()
    {
        return managerFactory;
    }

    public void setManagerFactory(RPMManagerFactory newManagerFactory)
    {
        this.managerFactory = newManagerFactory;
    }

    //Package local accessor to avoid direct calls from managers.
    static RPMException handleException(Throwable e)
    {
        logger.error(e.getMessage(), e);

        if (PRINT_EXCEPTION_STACKTRACE)
        {
            e.printStackTrace();
        }

        if (e instanceof RPMException)
        {
            return (RPMException) e;
        }
        if (e instanceof SQLException)
        {
            SQLException sqlException = (SQLException) e;
            return new RPMException(sqlException.getErrorCode());
        }

        return new RPMException(e);
    }

    // ==================
    // API IMPLEMENTATION
    // ==================
    public RpmResult searchTimeCodes(String sessionID, String workElementID,
            String scopeElementID, boolean isTimeCode1)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            TimecodeManager manager = (TimecodeManager) getManager(TimeCode.class);
            fillSessionInformation(messageContext, sessionID, "searchTimeCodes");
            ArrayList objects = manager.searchTimeCodes(workElementID,
                scopeElementID, isTimeCode1, messageContext);
            objects = applyFiltering(messageContext, objects);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public PropertyArray getSystemInfo(String sessionID) throws RPMException
    {
        PropertyArray systemInfo = new PropertyArray();
        MessageContext messageContext = null;
        try
        {
            messageContext = new MessageContext(factory,
                sessionID, "getSystemInfo");
            ArrayList systemInfoList = SystemInfo.getSystemInfo(messageContext);
            Property[] systemInfoArray = (Property[]) ArrayUtil.listToArray(
                Property.class, systemInfoList);
            systemInfo.setProperties(systemInfoArray);
        }
        catch (Exception e)
        {
            throw handleException(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return systemInfo;
    }

    public RpmResult load(String sessionID, String xpath, RPMObjectScope scope,
            Integer idsType, String idsList, String attributesIdsList,
            Integer viewType, Integer inclusions, Integer startIndex,
            Integer endIndex, String extraWhere, String orderBy,
            boolean buildSecurity)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, LOAD);
            messageContext.setRootScope(scope);

            //stats
            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.LOADFROMXPATH_OPERATION, xpath);

            Stack stack = XPathToStack.parse(xpath);

            //Map field names to container types and validate the stack
            stack = XPathUtil.getinstance().validateAndPrepareTableStack(stack,
                stack.size() - 1);

            Table peek = (Table) stack.peek();
            RPMObjectManager objectManager = managerFactory
                .getRPMObjectManager(peek.getObjectName());

            //instantiate loadParams
            LoadParams loadParams = new LoadParams();
            loadParams.setIdsType(idsType);
            loadParams.setIdsList(idsList);
            loadParams.setAttributesIdsList(attributesIdsList);
            loadParams.setViewType(viewType);
            loadParams.setInclusions(inclusions);
            loadParams.setStartIndex(startIndex);
            loadParams.setEndIndex(endIndex);
            loadParams.setExtraWhere(extraWhere);
            loadParams.setOrderBy(orderBy);
            loadParams.setBuildSecurity(buildSecurity);
            messageContext.setLoadParams(loadParams);

            ArrayList objects = objectManager
                .load(stack, scope, messageContext);

            objects = applyFiltering(messageContext, objects);//Security
            messageContext.addObjects(objects);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);            
        }

        return messageContext;
    }

    public RpmResult load(String sessionID, String xpath, RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        //stats
        long operationStartTime = StatsUtil.initOperation(messageContext,
            StatsUtil.LOADFROMXPATH_OPERATION, xpath);
        try
        {
            fillSessionInformation(messageContext, sessionID, LOAD);
            messageContext.setRootScope(scope);

            Stack stack = XPathToStack.parse(xpath);

            //Map field names to container types and validate the stack
            stack = XPathUtil.getinstance().validateAndPrepareTableStack(stack,
                stack.size() - 1);

            Table peek = (Table) stack.peek();
            RPMObjectManager objectManager = managerFactory
                .getRPMObjectManager(peek.getObjectName());
            ArrayList objects = objectManager
                .load(stack, scope, messageContext);

            objects = applyFiltering(messageContext, objects);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        StatsUtil.endOperation(messageContext, operationStartTime, logger);

        return messageContext;
    }

    private MessageContext createMessageContext()
    {
        return new MessageContext(factory);
    }

    public RpmResult load(String sessionID, RPMObject object,
            RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        long operationStartTime = StatsUtil.initOperation(messageContext,
            StatsUtil.LOADFROMID_OPERATION, object);

        try
        {

            fillSessionInformation(messageContext, sessionID, LOAD);
            doLoad(messageContext, object, scope);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        StatsUtil.endOperation(messageContext, operationStartTime, logger);

        return messageContext;
    }

    public void fillSessionInformation(MessageContext messageContext,
            String sessionID, String operation) throws RPMException
    {
        messageContext.setSession(factory.getSession(), sessionID, operation);
    }

    private void doLoad(MessageContext messageContext, RPMObject object,
            RPMObjectScope scope) throws RPMException, SQLException,
            ParseException
    {
        messageContext.setRootObject(object);
        messageContext.setRootScope(scope);
        RPMObjectManager manager = getManager(object);
        RPMObject newObject = manager.load(object, scope, messageContext);

        ArrayList objects = applyFiltering(messageContext, newObject);//security
        messageContext.addObjects(objects);
    }

    public RpmResult load(String sessionID, RPMObject objects[],
            RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        long operationStartTime = StatsUtil.initOperation(messageContext,
            StatsUtil.LOADFROMARRAY_OPERATION, "");
        try
        {

            fillSessionInformation(messageContext, sessionID, LOAD);
            if (ArrayUtil.isEmpty(objects))
            {
                // no load required, just return
                return messageContext;
            }

            for (int i = 0; i < objects.length; i++)
            {
                try
                {
                    doLoad(messageContext, objects[i], scope);
                }
                catch (Exception e)
                {
                    messageContext.addException(e);
                }
            }
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        StatsUtil.endOperation(messageContext, operationStartTime, logger);

        return messageContext;
    }

    public RpmResult setAutomaticCheckout(String sessionID,
            boolean isAutomaticCheckout)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "setAutomaticCheckout");
            factory.getSession().setAutomaticCheckout(messageContext,
                sessionID, isAutomaticCheckout);

            if (isAutomaticCheckout)
            {
                logger.debug("Automatic checkout is enabled");
            }
            else
            {
                logger.debug("Automatic checkout is disabled");
            }
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult save(String sessionID, RPMObject object,
            RPMObjectScope scope, ReloadType reloadType)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "save");
            messageContext.addReloadType(reloadType);
            messageContext.setRootObject(object);
            messageContext.setRootScope(scope);

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.SAVE_OPERATION, object);

            RPMObjectManager manager = getManager(object);

            // Will not always reload, the ReloadType of the messageContext
            // will be checked internally...
            RPMObject reloaded = manager.save(object, scope, messageContext);

            //skip the result if reload type is none
            if (reloadType != ReloadType.None && reloaded != null)
            {
                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);
            }

            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);            
        }

        return messageContext;
    }

    public RpmResult delete(String sessionID, RPMObject object)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "delete");
            messageContext.setRootObject(object);

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.DELETE_OPERATION, object);

            RPMObjectManager manager = getManager(object);
            manager.delete(object, messageContext);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult checkIn(String sessionID, RPMObject rpmObject)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "checkIn");
            if (messageContext.isAutomaticCheckout())
            {
                RpmResult result = new RpmResult();
                String[] errorParams = new String[]
                {
                        "Application", "checkIn"
                };
                result.addExceptionNoThrow(new RPMException(
                    ErrorCodes.MANUAL_CHECKOUT_UNAVAILABLE, errorParams));
                return result;
            }

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.CHECKIN_OPERATION, rpmObject);
            RPMObjectManager manager = getManager(rpmObject);
            manager.internalCheckin(rpmObject, messageContext);
            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult checkOut(String sessionID, RPMObject rpmObject,
            boolean isTimeless)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "checkOut");
            if (messageContext.isAutomaticCheckout())
            {
                RpmResult result = new RpmResult();
                String[] errorParams = new String[]
                {
                        "Application", "checkOut"
                };
                result.addExceptionNoThrow(new RPMException(
                    ErrorCodes.MANUAL_CHECKOUT_UNAVAILABLE, errorParams));
                return result;
            }

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.CHECKOUT_OPERATION, rpmObject);
            RPMObjectManager manager = getManager(rpmObject);
            manager.internalCheckout(rpmObject, isTimeless, messageContext);
            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public AlreadyCheckedOutResult isCheckedOut(String sessionID,
            RPMObject rpmObject)
    {
        MessageContext messageContext = createMessageContext();
        boolean status = false;
        try
        {
            fillSessionInformation(messageContext, sessionID, "isCheckedOut");

            RPMObjectManager manager = getManager(rpmObject);
            status = manager.isCheckedOutBy(messageContext, rpmObject);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        AlreadyCheckedOutResult acr = new AlreadyCheckedOutResult();
        acr.setCheckedOut(status);
        acr.setCheckedOutBy(messageContext.getCheckOutRecUser());
        acr.setCheckedOutSince(messageContext.getCheckOutRecDateTime());
        return messageContext.toAlreadyCheckedOutResult(acr);
    }

    public RpmResult forceCheckinOnSessionsCheckOut(String sessionID,
            String targetSessionID)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "forceCheckinOnSessionsCheckOut");
            // Any manager will do

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.FORCE_CHECKIN_OPERATION, "RPMObject");
            RPMObjectManager manager = getManager(Resource.class);
            manager.forceCheckinOnSessionsCheckOut(targetSessionID,
                messageContext);
            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult forceCheckinOnUsersCheckOut(String sessionID, Resource user)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "forceCheckinOnUsersCheckOut");
            RPMObjectManager manager = getManager(user); //Any manager will do

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.FORCE_CHECKIN_OPERATION, "RPMObject");
            manager.forceCheckinOnUsersCheckOut(user, messageContext);
            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult forceCheckinOnRPMObjectCheckOut(String sessionID,
            RPMObject rpmObject)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "forceCheckinOnRPMObjectCheckOut");

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.FORCE_CHECKIN_OPERATION, rpmObject);
            RPMObjectManager manager = getManager(rpmObject);
            manager.forceCheckinOnRPMObjectCheckOut(rpmObject, messageContext);
            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    protected RpmResult notImplemented(String operationName)
    {
        RpmResult result = new RpmResult();
        String[] errorParams = new String[]
        {
                "Application", operationName
        };
        result.addExceptionNoThrow(new RPMException(ErrorCodes.NOT_IMPLEMENTED,
            errorParams));
        return result;
    }

    /**
     * The assigneScorecard method will assign a Scorecard object to an
     * RPMObject. There are three types of RPMObjects that are amenable for
     * scorecard assignment: 1) Asset; 2) GenericProject and its subobjects; 3)
     * AbstractScope and its subobjects. For more information see those objects
     * documentation. The assumption for the assignment is that the Scorecard is
     * already been created in the design phase and that the RPMObject is
     * already been saved. After the assignment, any change in the Scorecard
     * will not affect the AssignedScorecard.
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param objectToAssign -
     *            the object that you want to assign the scorecard.
     * @param scoreCardToAssign -
     *            the scorecard to assign
     * @return - the object that you wanted to assign with the scorecard
     *         assigned to it
     * @throws RPMException
     */
    public RpmResult assignScoreCard(String sessionID,
            RPMObject objectToAssign, RPMObjectScope objectToAssignScope,
            Scorecard scoreCardToAssign, ReloadType reloadType)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            PublishedScorecardManager manager = (PublishedScorecardManager) getManager(AssignedScorecard.class);
            fillSessionInformation(messageContext, sessionID, "assignScoreCard");
            messageContext.setRootObject(objectToAssign);
            messageContext.setRootScope(objectToAssignScope);

            RPMObject reloaded = manager.assignScoreCard(objectToAssign,
                objectToAssignScope, scoreCardToAssign, reloadType,
                messageContext);

            if (reloadType != ReloadType.None && reloaded != null)
            {
                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);
            }
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }
    

    /**
     * The assignScorecard method will assign a Scorecard object to a Resource.
     * Note that a Resource can have at most one scorecard per Project or Proposal.
     * In order for a scorecard to be assigned to her, the Resource also needs to 
     * have work assigned for that project.
     * 
     * @param sessionID -
     *            The session identifier obtained during the login call. If this
     *            parameter isnt valid, the method will fail with a RPM
     *            Exception
     * @param project
     *            Either an instance of Project, Proposal or WorkOrganization     
     * @param objectToAssign -
     *            the object that you want to assign the scorecard.
     * @param scoreCardToAssign -
     *            the scorecard to assign
     * @return - the object that you wanted to assign with the scorecard
     *         assigned to it
     */
    public RpmResult assignScoreCardToResource(String sessionID, GenericProject project, 
            Resource objectToAssign, ResourceScope objectToAssignScope, 
            Scorecard scoreCardToAssign, ReloadType reloadType)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            PublishedScorecardManager manager = (PublishedScorecardManager) getManager(AssignedScorecard.class);
            fillSessionInformation(messageContext, sessionID, "assignScoreCardToResource");
            messageContext.setRootObject(objectToAssign);
            messageContext.setRootScope(objectToAssignScope);

            RPMObject reloaded = manager.assignScoreCardToResource(objectToAssign, project,
                objectToAssignScope, scoreCardToAssign, reloadType,
                messageContext);

            if (reloadType != ReloadType.None && reloaded != null)
            {
                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);
            }
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult replaceProfileWithResource(String sessionID,
            ResourceTaskAssignment rta, ResourceTaskAssignmentScope rtaScope,
            Resource resource)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "replaceProfileWithResource");
            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) getManager(rta);
            ResourceTaskAssignment reloaded = manager
                .replaceProfileWithResource(rta, rtaScope, resource,
                    messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult replaceResourceWithProfile(String sessionID,
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "replaceResourceWithProfile");
            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) getManager(assignment);
            ResourceTaskAssignment reloaded = manager
                .replaceResourceWithProfile(assignment, rtaScope,
                    messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult startWorkflow(String sessionID, WorkflowProcess process,
            RPMObject object, RunningWorkflowProcessScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "startWorkflow");

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.WORKFLOW_START_OPERATION, process);

            WorkflowManager manager = (WorkflowManager) getManager(WorkflowProcess.class);
            RunningWorkflowProcess runningProcess = manager.startProcess(
                process, object, scope, messageContext);

            ArrayList objects = applyFiltering(messageContext, runningProcess);//security
            messageContext.addObjects(objects);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult associateWorkflow(String sessionID,
            WorkflowProcess process, GenericProject genericProject,
            boolean associate, boolean isDefault)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "associateWorkflow");

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.WORKFLOW_ASSOCIATE_OPERATION, process);

            WorkflowManager manager = (WorkflowManager) getManager(WorkflowProcess.class);
            manager.associateProcess(process, genericProject, associate,
                isDefault, messageContext);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult suspendWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "suspendWorkflow");

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.WORKFLOW_SUSPEND_OPERATION, runningProcess);

            RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) getManager(RunningWorkflowProcess.class);
            RunningWorkflowProcess reloadedProcess = manager.suspendProcess(
                runningProcess, scope, messageContext);

            ArrayList objects = applyFiltering(messageContext, reloadedProcess);//security
            messageContext.addObjects(objects);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult resumeWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess,
            RunningWorkflowProcessScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "resumeWorkflow");

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.WORKFLOW_RESUME_OPERATION, runningProcess);

            RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) getManager(RunningWorkflowProcess.class);
            RunningWorkflowProcess reloadedProcess = manager.resumeProcess(
                runningProcess, scope, messageContext);

            ArrayList objects = applyFiltering(messageContext, reloadedProcess);//security
            messageContext.addObjects(objects);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult cancelWorkflow(String sessionID,
            RunningWorkflowProcess runningProcess)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "cancelWorkflow");

            long operationStartTime = StatsUtil.initOperation(messageContext,
                StatsUtil.WORKFLOW_CANCEL_OPERATION, runningProcess);

            RunningWorkflowProcessManager manager = (RunningWorkflowProcessManager) getManager(RunningWorkflowProcess.class);
            manager.cancelProcess(runningProcess, messageContext);

            StatsUtil.endOperation(messageContext, operationStartTime, logger);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult sendNotification(String sessionID,
            GenericCommunicationScope scope, RPMObject attachment,
            Resource[] recipients, String subject, String comments)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "sendNotification");

            CommunicationManager manager = (CommunicationManager) getManager(Notification.class);
            manager.sendNotification(scope, attachment, recipients, subject,
                comments, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult sendStaffingRequest(String sessionID,
            GenericCommunicationScope scope, RPMObject attachment,
            Resource[] recipients, String subject, String comments,
            Calendar dueDate)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "sendStaffingRequest");

            CommunicationManager manager = (CommunicationManager) getManager(StaffingRequest.class);
            manager.sendStaffingRequest(scope, attachment, recipients, subject,
                comments, dueDate, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult saveWorkflowResponse(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "saveWorkflowResponse");

            WorkflowResponseManager manager = (WorkflowResponseManager) getManager(WorkflowResponse.class);
            manager.saveWorkflowResponse(response, scope, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult sendWorkflowResponse(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "sendWorkflowResponse");

            WorkflowResponseManager manager = (WorkflowResponseManager) getManager(WorkflowResponse.class);
            manager.sendWorkflowResponse(response, scope, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult updateReadAndFolder(String sessionID,
            WorkflowResponse response, WorkflowResponseScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "updateReadAndFolder");

            WorkflowResponseManager manager = (WorkflowResponseManager) getManager(WorkflowResponse.class);
            manager.updateReadAndFolder(response, scope, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult publishScorecard(String sessionID,
            AssignedScorecard assignedScorecard,
            AbstractScorecardScope scorecardScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            ScorecardAssignmentManager manager = (ScorecardAssignmentManager) getManager(AssignedScorecardState.class);
            fillSessionInformation(messageContext, sessionID,
                "publishScorecard");
            manager.publishScorecard(assignedScorecard, scorecardScope, messageContext);
            doLoad(messageContext, assignedScorecard, scorecardScope);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult acceptRejectTaskAssignmentChanges(String sessionID,
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean accept, boolean EETC, boolean forecastDates)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "acceptRejectTaskAssignmentChanges");
            messageContext.setRootObject(taskAssignment);
            messageContext.setRootScope(scope);
            TaskAssignmentManager manager = (TaskAssignmentManager) getManager(TaskAssignment.class);
            TaskAssignment reloaded = manager
                .acceptRejectTaskAssignmentChanges(taskAssignment, scope,
                    accept, EETC, forecastDates, messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult copyProposedToPlan(String sessionID,
            WorkElement workElement, WorkElementScope workElementScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "copyProposedToPlan");
            messageContext.setRootObject(workElement);
            messageContext.setRootScope(workElementScope);
            WbsManager manager = (WbsManager) getManager(Project.class);
            WorkElement reloaded = manager.copyProposedToPlan(workElement,
                workElementScope, messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult copyAssignmentProposedToPlan(String sessionID,
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "copyAssignmentProposedToPlan");
            messageContext.setRootObject(resourceTaskAssignment);
            messageContext.setRootScope(scope);
            ResourceTaskAssignmentManager manager = (ResourceTaskAssignmentManager) getManager(ResourceTaskAssignment.class);
            ResourceTaskAssignment reloaded = manager
                .copyAssignmentProposedToPlan(resourceTaskAssignment, scope,
                    messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult commitAffectedResources(String sessionID,
            GenericProject genericProject, WorkElementScope workElementScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "commitAffectedResources");
            messageContext.setRootObject(genericProject);
            messageContext.setRootScope(workElementScope);
            WbsManager manager = (WbsManager) getManager(Project.class);
            GenericProject reloaded = manager.commitAffectedResources(
                genericProject, workElementScope, messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult transferUnderWBS(String sessionID,
            WorkElement workElement, AbstractScope abstractScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            ScopeManager manager = (ScopeManager) getManager(abstractScope);
            fillSessionInformation(messageContext, sessionID,
                "transferUnderWBS");
            manager
                .transferUnderWBS(workElement, abstractScope, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult moveUnderWBS(String sessionID, WorkElement workElement,
            AbstractScope abstractScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            ScopeManager manager = (ScopeManager) getManager(abstractScope);
            fillSessionInformation(messageContext, sessionID, "moveUnderWBS");
            manager.moveUnderWBS(workElement, abstractScope, messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult moveAndTransferUnderWBS(String sessionID,
            WorkElement workElement, AbstractScope abstractScope)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            ScopeManager manager = (ScopeManager) getManager(abstractScope);
            fillSessionInformation(messageContext, sessionID,
                "moveAndTransferUnderWBS");
            manager.moveAndTransferUnderWBS(workElement, abstractScope,
                messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult transferUnderScope(String sessionID,
            AbstractAggregateScope scopeElement, WorkElement workElement)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID,
                "transferUnderScope");
            WbsManager manager = (WbsManager) getManager(WorkElement.class);
            manager.transferUnderScope(scopeElement, workElement,
                messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    public RpmResult convertContainer(String sessionID,
            RPMObject inputContainer, RPMObject outputContainer,
            RPMObjectScope rpmScope)
    {
        MessageContext messageContext = createMessageContext();

        if (inputContainer != null)
        {
            try
            {
                fillSessionInformation(messageContext, sessionID,
                    "convertContainer");
                RPMObject reloaded = null;

                if (inputContainer instanceof WorkElement)
                {
                    WbsManager manager = (WbsManager) getManager(GenericProject.class);
                    reloaded = manager.convertWorkElement(
                        (WorkElement) inputContainer, outputContainer,
                        (WorkElementScope) rpmScope, messageContext);
                }
                else if (inputContainer instanceof AbstractScope)
                {
                    ScopeManager manager = (ScopeManager) getManager(AbstractScope.class);
                    reloaded = manager.convertScopeElement(
                        (AbstractScope) inputContainer, outputContainer,
                        rpmScope, messageContext);
                }
                else
                {
                    String params[] =
                    {
                            StringUtil.getShortClassName(WorkElement.class)
                                    + " or "
                                    + StringUtil
                                        .getShortClassName(AbstractScope.class),
                            StringUtil.getShortClassName(inputContainer
                                .getClass())

                    };
                    ExceptionUtil.createAndHandleRPMException(inputContainer
                        .getClass(), ErrorCodes.INVALID_RPMOBJECT_TYPE, params);
                }
                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);

            }
            catch (Throwable e)
            {
                messageContext.addExceptionNoThrow(e);
            }
            finally
            {
                commitAndCloseConnection(messageContext);
            }
        }
        else
        {
            String params[] =
            {
                ValidationConstants.INPUT_CONTAINER_PARM
            };
            RPMException e = new RPMException(ErrorCodes.OBJECT_IS_NULL, params);
            messageContext.addExceptionNoThrow(e);
        }

        return messageContext;
    }

    public RpmResult createObjectFromTemplate(String sessionID,
            RPMObject inputObject, RPMObject template, RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        WbsManager manager = null;
        ScopeManager scopeMan = null;

        if (inputObject != null)
        {
            try
            {
                fillSessionInformation(messageContext, sessionID,
                    "createObjectFromTemplate");

                messageContext.setTemplate(template);

                if (inputObject instanceof WorkElement)
                {
                    manager = (WbsManager) getManager(GenericProject.class);
                    RPMObject reloaded = manager.createObjectFromTemplate(
                        inputObject, scope, messageContext);

                    ArrayList objects = applyFiltering(messageContext, reloaded);//security
                    messageContext.addObjects(objects);

                }
                else if (inputObject instanceof ScopeElement)
                {
                    scopeMan = (ScopeManager) getManager(AbstractScope.class);
                    RPMObject reloaded = scopeMan.createObjectFromTemplate(
                        inputObject, scope, messageContext);

                    ArrayList objects = applyFiltering(messageContext, reloaded);//security
                    messageContext.addObjects(objects);

                }

            }
            catch (Throwable e)
            {
                messageContext.addExceptionNoThrow(e);
            }
            finally
            {
                commitAndCloseConnection(messageContext);
            }
        }

        else
        {
            String params[] =
            {
                com.ibm.rpm.wbs.constants.ValidationConstants.TEMPLATE_INPUT_FIELD
            };
            RPMException e = new RPMException(ErrorCodes.OBJECT_IS_NULL, params);
            messageContext.addExceptionNoThrow(e);
        }

        return messageContext;

    }

    public RpmResult saveProjectBaseline(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        WbsManager manager = null;

        if (projectObject != null)
        {
            try
            {
                fillSessionInformation(messageContext, sessionID,
                    "saveProjectBaseline");

                manager = (WbsManager) getManager(GenericProject.class);
                RPMObject reloaded = manager.saveProjectBaseline(projectObject,
                    scope, messageContext);

                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);

            }
            catch (Throwable e)
            {
                messageContext.addExceptionNoThrow(e);
            }
            finally
            {
                commitAndCloseConnection(messageContext);
            }
        }

        else
        {
            String params[] =
            {
                com.ibm.rpm.wbs.constants.ValidationConstants.PROJECTS_FIELD
            };
            RPMException e = new RPMException(ErrorCodes.OBJECT_IS_NULL, params);
            messageContext.addExceptionNoThrow(e);
        }

        return messageContext;

    }

    public RpmResult rollupProjectEV(String sessionID, RPMObject projectObject,
            RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        WbsManager manager = null;

        if (projectObject != null)
        {
            try
            {
                fillSessionInformation(messageContext, sessionID,
                    "rollupProjectEV");

                manager = (WbsManager) getManager(GenericProject.class);
                RPMObject reloaded = manager.rollupProjectEV(projectObject,
                    scope, messageContext);

                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);

            }
            catch (Throwable e)
            {
                messageContext.addExceptionNoThrow(e);
            }
            finally
            {
                commitAndCloseConnection(messageContext);
            }
        }
        else
        {
            String params[] =
            {
                com.ibm.rpm.wbs.constants.ValidationConstants.PROJECTS_FIELD
            };
            RPMException e = new RPMException(ErrorCodes.OBJECT_IS_NULL, params);
            messageContext.addExceptionNoThrow(e);
        }

        return messageContext;

    }

    public RpmResult enableProjectEVCalculation(String sessionID,
            RPMObject projectObject, RPMObjectScope scope)
    {
        MessageContext messageContext = createMessageContext();
        WbsManager manager = null;

        if (projectObject != null)
        {
            try
            {
                fillSessionInformation(messageContext, sessionID,
                    "EnableProjectEVCalculation");
                manager = (WbsManager) getManager(GenericProject.class);
                RPMObject reloaded = manager.enableProjectEVCalculation(
                    projectObject, scope, messageContext);

                ArrayList objects = applyFiltering(messageContext, reloaded);//security
                messageContext.addObjects(objects);
            }
            catch (Throwable e)
            {
                messageContext.addExceptionNoThrow(e);
            }
            finally
            {
                commitAndCloseConnection(messageContext);
            }
        }
        else
        {
            String params[] =
            {
                com.ibm.rpm.wbs.constants.ValidationConstants.PROJECTS_FIELD
            };
            RPMException e = new RPMException(ErrorCodes.OBJECT_IS_NULL, params);
            messageContext.addExceptionNoThrow(e);
        }

        return messageContext;

    }

    public RpmResult activateEVWeekDelays(String sessionID, int evDelays)
    {
        MessageContext messageContext = createMessageContext();
        WbsManager manager = null;

        try
        {
            fillSessionInformation(messageContext, sessionID,
                "activateEVWeekDelays");
            manager = (WbsManager) getManager(GenericProject.class);

            messageContext = manager.activateEVWeekDelays(messageContext,
                evDelays);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }

        return messageContext;

    }

    public MessageContext moveBeforeSibling(String sessionID,
            ScopeElement objectToMove, ScopeElement nextSibling)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            ScopeManager manager = (ScopeManager) getManager(objectToMove);
            messageContext.setSession(factory.getSession(), sessionID,
                "moveBeforeSibling");
            manager.moveBeforeSibling(sessionID, objectToMove, nextSibling,
                messageContext);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    /**
     * Lock the Document
     * 
     * @param sessionID
     * @param documentToLock
     * @return
     */
    public MessageContext lockDocument(String sessionID,
            GenericDocument documentToLock)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            DocumentManager manager = (DocumentManager) getManager(documentToLock);
            messageContext.setSession(factory.getSession(), sessionID,
                "lockDocument");
            RPMObject reloaded = manager.lockDocument(documentToLock,
                messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    /**
     * Unlock the document
     * 
     * @param sessionID
     * @param documentToUnlock
     * @return
     */
    public MessageContext unLockDocument(String sessionID,
            GenericDocument documentToUnlock)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            DocumentManager manager = (DocumentManager) getManager(documentToUnlock);
            messageContext.setSession(factory.getSession(), sessionID,
                "unLockDocument");
            RPMObject reloaded = manager.unLockDocument(documentToUnlock,
                messageContext);
            ArrayList objects = applyFiltering(messageContext, reloaded);//security
            messageContext.addObjects(objects);

        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }

    /**
     * Return the database time
     * 
     * @param sessionID
     * @return a java.sql.Timestamp object wrapped into a MessageContext
     * @throws RPMException 
     */
    public Calendar getDatabaseTimestamp(String sessionID) throws RPMException
    {
        // validate the session ID first.
        MessageContext messageContext = createMessageContext();
        fillSessionInformation(messageContext, sessionID,
            "getDatabaseTimestamp");
        setDatasourceIfNecessary(messageContext);
        return getDatabaseTimestamp(messageContext, true);
    }

    //called internally only
    public Calendar getDatabaseTimestamp(MessageContext messageContext) throws RPMException
    {
        return getDatabaseTimestamp(messageContext, false);
    }
    
    public Calendar getDatabaseTimestamp(MessageContext messageContext, boolean forceCommitAndCloseConnection) throws RPMException
    {
        Timestamp dbTime = null;
        Connection c = null;
        ResultSet rs = null;
        Calendar ret = null;
        try
        {
            setDatasourceIfNecessary(messageContext);
            c = Manager.getConnection(messageContext);
            String query = null;
            if (Manager.isOracle(c))
            {
                query = "SELECT SYSTIMESTAMP FROM dual";
            }
            else
            {
                // DB2
                query = "SELECT current_timestamp FROM dual";
            }

            rs = ManagerUtil.executeSqlQuery(messageContext, c, query, null);
            if (rs.next())
            {
                dbTime = rs.getTimestamp(1);
            }

            ret = DateUtil.getCalendarInstance();
            ret.setTimeInMillis(dbTime.getTime());
        }
        catch (Throwable e)
        {
            throw new RPMException(ErrorCodes.INTERNAL_ERROR, new String[]
            {
                "Failed to fetch database timestamp."
            }, e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            if(forceCommitAndCloseConnection || !MessageContext.isConnectionCachingEnabled(messageContext))
            {
                Manager.commitAndCloseConnection(messageContext, c);                
            }
        }
        return ret;
    }    

    protected ArrayList applyFiltering(MessageContext messageContext,
            RPMObject rpmObject) throws RPMException
    {
        ArrayList objects = new ArrayList();
        objects.add(rpmObject);
        return applyFiltering(messageContext, objects);
    }

    protected ArrayList applyFiltering(MessageContext messageContext,
            ArrayList objects) throws RPMException
    {
        if (messageContext.isWebServicesSecurityEnabled())
        {
            return ISecurityController.INSTANCE.applyFiltering(messageContext,
                objects);
        }
        return objects;
    }
    
    public RpmResult executeProcedure(String sessionID, Class callerClass, String procedureName,
            Object[] args)
    {
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "executeProcedure");
            ManagerUtil.executeProcedure(callerClass, messageContext, procedureName,
                args);
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            commitAndCloseConnection(messageContext);
        }
        return messageContext;
    }
    
    public RpmResult executeSqlUpdate(String sessionID, String updateString,
            Object[] args)
    {
        Connection c = null;
        ResultSet rs = null;
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "executeSqlUpdate");
            c = Manager.getConnection(messageContext);
            ManagerUtil.executeSqlUpdate(messageContext, c, updateString, args);       
        }
        catch (Throwable e)
        {
            messageContext.addExceptionNoThrow(e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.commitAndCloseConnection(messageContext, c);
        }
        return messageContext;
    }
    
    public ArrayList executeSqlQuery(String sessionID, String query,
            Object[] args) throws RPMException
    {
        Connection c = null;
        ResultSet rs = null;
        MessageContext messageContext = createMessageContext();
        try
        {
            fillSessionInformation(messageContext, sessionID, "executeSqlQuery");
            c = Manager.getConnection(messageContext);
            rs = ManagerUtil.executeSqlQuery(messageContext, c, query, args);

            ResultSetMetaData rsmd = rs.getMetaData();
            int numberOfColumns = rsmd.getColumnCount();
            
            //Here we decode the resultSet into a list of lists
            //Each element of the list represents a row which
            //in turn contains a list of column values
            ArrayList rowsList = new ArrayList();
            ArrayList row = null;
            while(rs.next())
            {
                row = new ArrayList();
                for(int columnIndex=1;columnIndex<=numberOfColumns;columnIndex++)
                {
                    row.add(rs.getObject(columnIndex));
                }
                rowsList.add(row);
            }
            return rowsList;
        }
        catch (Throwable e)
        {
            throw new RPMException(e);
        }
        finally
        {
            Manager.close(messageContext, rs);
            Manager.commitAndCloseConnection(messageContext, c);
        }
    }
    
    private void commitAndCloseConnection(MessageContext messageContext) 
    {
        if(messageContext!=null && messageContext.getConnection()!=null)
        {
            Manager.commitAndCloseConnection(messageContext, messageContext.getConnection());
            messageContext.setConnection(null);                
        }
    }
    
    private void setDatasourceIfNecessary(MessageContext messageContext)throws RPMException
    {
        if(messageContext.getDatasource()==null)
        {
            DataSource ds = factory.getSession().getDataSource(DATASOURCE_NAME);
            messageContext.addDatasource(ds);
        }
    }
}
