package com.ibm.rpm.reporter;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.commons.io.FileUtils;

/*
 * Outputs c:\\junitdelta.timestamp reports about which new unit tests were broken yesterday, 
 * and also which were fixed.
 * 
 *  Uses c:\\lastrun to delay by n hours the time between copying the new JUnit reports
 *  
 *  Needs y:\ to be mapped to \\rpmdev07\builds\output\rpm\rpm-ws\7.1.0.0\test
 */
/*
 * Modernize unit tests

    * Question
          o what are the unit tests which passed yesterday but fail today
          o what are the unit tests which failed yesterday but passed today
    * benefits
          o trends
          o trace culprit easily
          o maintain a solid quality of unit tests
    * input
          o daily html files
                + date differs
                + keyword class="Error"
    * data required
          o today's output
          o yesterday's output
    * batch system
    * decomposition
          o system task
                + call data archival module with old and new paths
          o data archival module
                + every day
                      # move current to yesterday
                      # copy current from server
                      # make sure creation dates are sane
                      # call reporter process
          o reporter
                + algorithm
                      # create lists of failed unit tests
                            * today
                            * yesterday
                      # create list of successful unit tests
                            * today
                            * yesterday
                      # broken since yesterday
                            * for each passed yesterday
                            * if exists in failed today, add to list
                      # fixed since yesterday
                            * for each passed today
                            * if failed yesterday, add to list
                + quality attributes
                      # takes 60 seconds to run
                      # outputs linebreak separated output, good for email or anything else
                      # can't run if the data is not there
                      # why not Java
          o update report on server
    * dependencies
          o my system online
    * output
          o two lists
                + broken since yesterday
                + working since yesterday
 */
public class UnitTestReporter
{   
    private static final String SOURCE = "y:\\com";
    private static final String LASTRUN = "c:\\lastrun";

    private static final String OUTPUT_PREFIX = "c:\\junitdelta";
    
    /*
     * Don't run more often than once every n hours.
     */
    public UnitTestReporter(File present, File past) throws IOException
    {
//        process(present, past);
        boolean run = false;
        File source = new File(LASTRUN);
        
        if(source.exists()) {
            DataInputStream dis = new DataInputStream(new FileInputStream(source));
            long lastrun = 0L;
            
            try {
                lastrun = dis.readLong();
            } catch(EOFException e)
            {
                source.delete();
                run = true;
            }
            long now = System.currentTimeMillis();
            
           if((now - lastrun) > 6*60*60*1000) // 6 hours in ms
           {
               run = true;
           } else
           {
               process(present, past);
           }
           
           dis.close();
        } else {
            source.createNewFile();
            run = true;
        }
        
        if(run)
        {
            archive(present, past);
            process(present, past);
            
            DataOutputStream out = new DataOutputStream(new FileOutputStream(source));
            out.writeLong(System.currentTimeMillis());
            out.close();
        }
    }

    private void process(File present, File past) throws IOException
    {
        ArrayList failedNow = new ArrayList();
        ArrayList successfulNow = new ArrayList(); 
        grepContents(present, failedNow, successfulNow);

        ArrayList failedYesterday = new ArrayList();
        ArrayList successfulYesterday = new ArrayList(); 
        grepContents(past, failedYesterday, successfulYesterday);
        
        // Just broken
        ArrayList justBroken = new ArrayList();
        
        Iterator successfulYesIterator = successfulYesterday.iterator();
        while(successfulYesIterator.hasNext())
        {
            String passedYesterdayItem = (String) successfulYesIterator.next();
            
            Iterator failedTodayIterator = failedNow.iterator();
            while(failedTodayIterator.hasNext())
            {
                String failedTodayItem = (String) failedTodayIterator.next();
                
                if(passedYesterdayItem.equalsIgnoreCase(failedTodayItem))
                {
                    justBroken.add(failedTodayItem);
                    break;
                }
            }
        }
        
        // Just fixed
        ArrayList justFixed = new ArrayList();
        Iterator successfulNowIterator = successfulNow.iterator();
        while(successfulNowIterator.hasNext())
        {
            String passedTodayItem = (String) successfulNowIterator.next();
            
            Iterator failedYesterdayIterator = failedYesterday.iterator();
            while(failedYesterdayIterator.hasNext())
            {
                String failedYesterdayItem = (String) failedYesterdayIterator.next();
                
                if(passedTodayItem.equalsIgnoreCase(failedYesterdayItem))
                {
                    justFixed.add(passedTodayItem);
                    break;
                }
            }
        }
        
        System.out.println("Just broken\n" + justBroken);
        
        System.out.println("Just fixed\n" + justFixed);
        
        File outputFile = new File(OUTPUT_PREFIX + System.currentTimeMillis() + ".txt");
        
        FileOutputStream out = new FileOutputStream(outputFile);
        PrintWriter pw = new PrintWriter(out);
        pw.println("Broken today: ");
        pw.println(justBroken);
        
        pw.println("Fixed today: ");
        pw.println(justFixed);
        pw.flush();
        pw.close();
    }

    private void grepContents(File when, ArrayList failedList, ArrayList successfulList
            ) throws IOException
    {        
        String failedStr = "class=\"Error\"";
        String failedStr2 = "class=\"Failure\"";
        
        Iterator it = FileUtils.iterateFiles(when, new String[]{"html"}, true);
        
        while(it.hasNext())
        {
            File currentFile = (File) it.next();
            if(currentFile.getName().equalsIgnoreCase("package-summary.html") ||
                    currentFile.getName().equalsIgnoreCase("package-frame.html")  )
            {
                continue;
            }
            
            String currentFilename = currentFile.getCanonicalPath();
            currentFilename = currentFilename.substring(currentFilename.indexOf("\\com\\") +1);
            currentFilename = currentFilename.replace('\\', '.');
            currentFilename = currentFilename.substring(0, currentFilename.lastIndexOf(".html"));
            String currentContents = FileUtils.readFileToString(currentFile, null);
            if(currentContents.indexOf(failedStr) != -1 ||currentContents.indexOf(failedStr2) != -1)
            {
                failedList.add(currentFilename);
            } else 
            {
                successfulList.add(currentFilename);
            }
        }
    }

    private void archive(File today, File yesterday) throws IOException
    {
        
        File now, then;
        
        if(today.isDirectory() && yesterday.isDirectory() && today.canRead() && yesterday.canRead())
        {
            now = today;
            then = yesterday;
           
            FileUtils.deleteDirectory(then);
            FileUtils.copyDirectory(now, then);
            File source = new File(SOURCE);
            if(!source.isDirectory() && !source.canRead())
            {
                throw new IOException("can't read " + source);
            }
            
            FileUtils.copyDirectoryToDirectory(source, now);
        } else
        {
            System.err.println("Can't read input!");
            System.exit(1);
        }
    }

    public static void main(String args[]) throws IOException
    {
        if(args.length != 2)
        {
            System.err.println("Usage: <old> <new>");
            System.exit(2);
        }
        
        File present, past;
        present = new File(args[1]);
        past = new File(args[0]);
        
        UnitTestReporter startProcess = new UnitTestReporter(present, past);
    }
}
