/*
 * Created on Feb 28, 2006 
 * 
 * @author mmatwiy Utility methods for Resource specific validation 
 * (moved from GenericValidator)
 */
package com.ibm.rpm.resource.util;

import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.Manager;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Pool;
import com.ibm.rpm.resource.managers.PoolManager;
import com.ibm.rpm.resource.managers.ResourceManager;
import com.ibm.rpm.resource.managers.ResourceRoleAssignmentManager;
import com.ibm.rpm.security.managers.SecurityRoleManager;

public class ResourceValidator extends GenericValidator
{
    private static Log logger = LogFactory.getLog(ResourceValidator.class);

    final private static String COUNT_STAR = "count(*)";

    /**
     * @return Returns true if the resource exists and is capable of logging
     *         into the system. Otherwise, returns false.
     */
    private static boolean isExistingResource(MessageContext msgCtx,
            String columnName, String columnValue)
    {
        try
        {
            if (columnValue == null)
                return false;
            // Patch - We must know if we run on an Oracle database to format
            // the date part
            // of the query correctly
            String dateQuery = null;
            if (Manager.isOracle(msgCtx))
            {
                dateQuery = "AND COALESCE( "
                        + ResourceManager.NAME_EFFECTIVE_DATE
                        + ", CURRENT_DATE) <= CURRENT_DATE " + "AND COALESCE( "
                        + ResourceManager.NAME_DISCONTINUE_DATE
                        + ", CURRENT_DATE) >= CURRENT_DATE ";
            }
            else
            {
                dateQuery = " AND DATE( COALESCE( "
                        + ResourceManager.NAME_EFFECTIVE_DATE
                        + ", CURRENT DATE)) <= CURRENT DATE"
                        + " AND DATE( COALESCE( "
                        + ResourceManager.NAME_DISCONTINUE_DATE
                        + ", CURRENT DATE)) >= CURRENT DATE";
            }

            String selectStmt = "SELECT " + COUNT_STAR + " from "
                    + ResourceManager.TABLE_NAME + " WHERE " + columnName
                    + "= ?" + dateQuery + " AND "
                    + ResourceManager.NAME_CALENDAR_ID + " is not null AND "
                    + ResourceManager.NAME_ACTIVE + "='1' AND  "
                    + ResourceManager.NAME_REC_STATUS + " != 'D' AND "
                    + ResourceManager.NAME_REC_STATUS + " != 'L'";

            Object[] params = new Object[]
            {
                columnValue
            };

            return executeStatement(msgCtx, selectStmt, params);
        }
        catch (SQLException e)
        {
            logger.error(e);
            return false;
        }
    }

    public static boolean validateExistingResource(RPMObject object,
            String fieldName, String fieldValue, String resourceFieldName,
            MessageContext msgCtx)
    {
        String columnName = "RESOURCE_ID";

        if (resourceFieldName.equalsIgnoreCase("fullName"))
            columnName = "FULL_NAME";

        if (!isExistingResource(msgCtx, columnName, fieldValue))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()) + " "
                            + fieldName, "Resource",
                    getShortString(fieldValue), resourceFieldName
            };
            RPMException e = new RPMException(ErrorCodes.NOT_EXISTING_ID,
                errorParams, object.getClass().getName(), fieldName,
                object.getID());
            addException(e, msgCtx);
            return false;
        }
        return true;
    }

    /**
     * @return Returns true if the resource exists and is capable of logging
     *         into the system. - And is also a ResourceManager for any Pool
     *         that has not been deleted. Otherwise, returns false.
     */
    private static boolean isResourceManager(MessageContext msgCtx,
            String resourceID)
    {
        if (resourceID == null)
            return false;

        SqlBuffer securitySubSelect = new SqlBuffer();
        securitySubSelect.append(SecurityRoleManager.NAME_REC_STATUS
                + " != 'D' ");
        securitySubSelect.appendAnd();
        securitySubSelect.append(SecurityRoleManager.NAME_ELEMENT_ID
                + " != 'SCDUMMYGROUP' ");
        securitySubSelect.appendAnd();
        securitySubSelect.appendOpenParenthesis();
        securitySubSelect.appendIsNull(SecurityRoleManager.NAME_PARENT_ID);
        securitySubSelect.appendOr();
        securitySubSelect.append(SecurityRoleManager.NAME_PARENT_ID
                + " != 'SCDUMMYGROUP' ");
        securitySubSelect.appendCloseParenthesis();
        securitySubSelect.appendAnd();
        securitySubSelect.append(SecurityRoleManager.NAME_LEVEL_ID + " >= 1 ");
        securitySubSelect.appendEqual(SecurityRoleManager.NAME_ELEMENT_ID,
            "SCRESOURCEMANAGER");

        SqlBuffer subSubSelectCondition = new SqlBuffer();
        subSubSelectCondition.append(ResourceManager.NAME_REC_STATUS
                + " != 'D'");
        subSubSelectCondition.appendEqual(ResourceManager.NAME_RESOURCE_ID,
            resourceID);

        SqlBuffer resourceSubSelectCondition = new SqlBuffer();
        resourceSubSelectCondition.appendSubSelect(
            ResourceRoleAssignmentManager.NAME_RESOURCE_ID,
            ResourceManager.NAME_RESOURCE_ID, ResourceManager.TABLE_NAME,
            subSubSelectCondition.toString());

        SqlBuffer selectCondition = new SqlBuffer();
        selectCondition.append(PoolManager.NAME_REC_STATUS + " != 'D'");
        selectCondition.appendEqual(PoolManager.NAME_TYPE_ID, Pool.TYPE_ID);
        selectCondition.appendAnd();
        selectCondition.appendOpenParenthesis();
        selectCondition.appendSubSelect(PoolManager.NAME_ELEMENT_ID,
            ResourceRoleAssignmentManager.NAME_ELEMENT_ID,
            ResourceRoleAssignmentManager.TABLE_NAME,
            resourceSubSelectCondition.toString());
        selectCondition.appendSubSelect(
            ResourceRoleAssignmentManager.NAME_SECURITY_ID,
            SecurityRoleManager.NAME_ELEMENT_ID,
            SecurityRoleManager.TABLE_NAME, securitySubSelect.toString());
        selectCondition.appendCloseParenthesis();

        SqlBuffer selectStmt = new SqlBuffer();
        selectStmt.appendSelect(COUNT_STAR);
        selectStmt.appendFrom(PoolManager.TABLE_NAME);
        selectStmt.appendWhere(selectCondition);
        selectStmt.appendCloseParenthesis();
        selectStmt.appendCloseParenthesis();
        selectStmt.appendCloseParenthesis();

        return executeStatement(msgCtx, selectStmt.toString(), null);
    }

    public static boolean validateResourceManager(RPMObject object,
            String fieldName, String fieldValue, String resourceFieldName,
            MessageContext msgCtx)
    {
        if (validateExistingResource(object, fieldName, fieldValue,
            resourceFieldName, msgCtx))
        {
            if (!isResourceManager(msgCtx, fieldValue))
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(object.getClass()) + " "
                                + fieldName, getShortString(fieldValue)
                };
                RPMException e = new RPMException(
                    com.ibm.rpm.resource.constants.ErrorCodes.NOT_RESOURCE_MANAGER,
                    errorParams, object.getClass().getName(), fieldName,
                    object.getID());
                addException(e, msgCtx);
                return false;
            }
        }

        return true;
    }

}
