/*
 * Created on Jul 26, 2006
 *
 */
package com.ibm.rpm.security.managers;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.security.SecurityFlags;
import com.ibm.rpm.security.containers.AbstractSecurityGroup;
import com.ibm.rpm.security.containers.SecurityRole;

/**
 * @author eturcot, Yasser Lulu
 * 
 */
public class SecurityUtil
{

    public static int SECURITY_GROUP_LEVELS_COUNT = 7;

    public static int SECURITY_ROLE_LEVELS_COUNT = 26;

    private static final Class[] BOOL_ARRAY_PARAM =
    {
        Boolean[].class
    };

    private static final Class[] BOOL_PARAM =
    {
        Boolean.class
    };

    public static abstract class BaseFieldDescr
    {
        public abstract boolean isArray();

        public String name;

        public Method getter;

        public Method setter;

        public Method delta;

        public Method test;
    }

    public static class FieldDescr extends BaseFieldDescr
    {
        public final boolean isArray()
        {
            return false;
        }

        public int clearanceLevelId;

        public int mask;
    }

    public static class ArrayFieldDescr extends BaseFieldDescr
    {
        public final boolean isArray()
        {
            return true;
        }

        public int[] clearanceLevelIds;

        public int[] masks;

    }

    private static Log logger = LogFactory.getLog(SecurityUtil.class);

    private static Map FIELD_DESCR_MAP = new HashMap();

    private static void doGetClearanceLevels(RPMObject securityObject,
            int[] clearanceLevels, Map fields) throws RPMException
    {
        Iterator it = fields.values().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr baseDescr = (BaseFieldDescr) it.next();
            if (!baseDescr.isArray())
            {
                FieldDescr descr = (FieldDescr) baseDescr;
                Boolean b = (Boolean) invokeMethod(securityObject,
                    descr.getter, null);
                if ((b != null) && b.booleanValue())
                {
                    clearanceLevels[descr.clearanceLevelId] |= descr.mask;
                }
                else
                {
                    clearanceLevels[descr.clearanceLevelId] &= ~descr.mask;
                }
            }
            else
            {
                ArrayFieldDescr arrayDescr = (ArrayFieldDescr) baseDescr;
                Boolean[] flags = (Boolean[]) invokeMethod(securityObject,
                    arrayDescr.getter, null);
                
                if (flags != null)
                {
                	if(arrayDescr.clearanceLevelIds.length < flags.length)
                    {
                    	throw new RPMException("Invalid number of flags -'" + flags.length + "' of type '" +arrayDescr.getter.getName() +"' set to the SecurityObject - " + securityObject.getClass().getName()); 
                    }
                    for (int i = 0; i < flags.length; ++i)
                    {
                        if ((flags[i] != null) && flags[i].booleanValue())
                        {
                            clearanceLevels[arrayDescr.clearanceLevelIds[i]] |= arrayDescr.masks[i];
                        }
                        else
                        {
                            clearanceLevels[arrayDescr.clearanceLevelIds[i]] &= ~arrayDescr.masks[i];
                        }
                    }
                }
            }
        }

    }

    public static int[] getClearanceLevels(AbstractSecurityGroup rights,
            int[] clearanceLevels) throws RPMException
    {
        validateClearanceLevelsArrayLength(clearanceLevels,
            SECURITY_GROUP_LEVELS_COUNT);
        doGetClearanceLevels(rights, clearanceLevels, getFieldDescrsMap(rights
            .getClass()));

        // Bit 10 of clearance level 1 is always set
        //TODO Yasser we should update the metamodel to make default true and read-only: Always 
        clearanceLevels[0] |= SecurityMask.MASK_BIT10;
        return clearanceLevels;
    }

    private static void validateClearanceLevelsArrayLength(
            int[] clearanceLevels, int expectedLength)
    {
        if (clearanceLevels.length != expectedLength)
        {
            throw new IllegalArgumentException(
                "The clearance levels array should be of length "
                        + expectedLength);
        }
    }

    public static void setClearanceLevels(int[] clearanceLevels,
            AbstractSecurityGroup rights) throws Exception
    {
        validateClearanceLevelsArrayLength(clearanceLevels,
            SECURITY_GROUP_LEVELS_COUNT);
        doSetClearanceLevel(clearanceLevels, rights, getFieldDescrsMap(rights
            .getClass()));
    }

    private static void performDeltaClearanceLevels(int[] clearanceLevels,
            RPMObject securityObject, Map fields) throws RPMException
    {
        Iterator it = fields.values().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr baseDescr = (BaseFieldDescr) it.next();
            if (!baseDescr.isArray())
            {
                FieldDescr descr = (FieldDescr) baseDescr;
                Boolean b = Boolean
                    .valueOf((clearanceLevels[descr.clearanceLevelId] & descr.mask) == descr.mask);
                if (null == invokeMethod(securityObject, descr.getter, null))
                {
                    // use the value from the result set if it was not
                    // set by the user.
                    //This also means there is no need to invoke the delta method 
                    invokeMethod(securityObject, descr.setter, b);
                }
                else
                {
                    invokeMethod(securityObject, descr.delta, b);
                }
            }
            else
            {

                ArrayFieldDescr arrayDescr = (ArrayFieldDescr) baseDescr;
               
                int length = arrayDescr.clearanceLevelIds.length;
                Boolean[] flags = new Boolean[length];
                for (int i = 0; i < length; ++i)
                {
                    flags[i] = Boolean
                        .valueOf((clearanceLevels[arrayDescr.clearanceLevelIds[i]] & arrayDescr.masks[i]) == arrayDescr.masks[i]);
                }

                Boolean[] currentflags = (Boolean[]) invokeMethod(
                    securityObject, arrayDescr.getter, null);
                if (currentflags == null)
                {
                    invokeMethod(securityObject, arrayDescr.setter, flags);
                }
                else
                {
                    if(length < currentflags.length)
                    {
                    	throw new RPMException("Invalid number of flags -'" + currentflags.length + "' of type '" +arrayDescr.getter.getName() +"' set to the SecurityObject - " + securityObject.getClass().getName()); 
                    }
                    for (int i = 0; i < currentflags.length; ++i)
                    {
                        if (currentflags[i] == null)
                        {
                            currentflags[i] = flags[i];
                        }
                    }
                    invokeMethod(securityObject, arrayDescr.delta, flags);
                }
            }
        }

    }

    public static void deltaClearanceLevels(int[] clearanceLevels,
            AbstractSecurityGroup rights) throws RPMException
    {
        validateClearanceLevelsArrayLength(clearanceLevels,
            SECURITY_GROUP_LEVELS_COUNT);
        performDeltaClearanceLevels(clearanceLevels, rights,
            getFieldDescrsMap(rights.getClass()));
    }

    private static boolean isAnyClearanceLevelModified(
            RPMObject securityObject, Map fields)
    {
        try
        {
            Iterator it = fields.values().iterator();
            while (it.hasNext())
            {
                BaseFieldDescr baseDescr = (BaseFieldDescr) it.next();
                Boolean b = (Boolean) invokeMethod(securityObject,
                    baseDescr.test, null);
                if ((b != null) && b.booleanValue())
                {
                    return true;
                }
            }
        }
        catch (Exception e)
        {
            logger.error(e);
        }

        return false;
    }

    public static boolean isAnyClearanceLevelModified(
            AbstractSecurityGroup rights)
    {
        if (rights != null)
        {
            return isAnyClearanceLevelModified(rights, getFieldDescrsMap(rights
                .getClass()));
        }
        return false;
    }

    public static boolean isAnyClearanceLevelModified(SecurityRole securityRole)
    {
        if (securityRole != null)
        {
            return isAnyClearanceLevelModified(securityRole,
                getFieldDescrsMap(securityRole.getClass()));
        }
        return false;
    }

    public static void deltaClearanceLevel(int[] clearanceLevels,
            SecurityRole securityRole) throws RPMException
    {
        validateClearanceLevelsArrayLength(clearanceLevels,
            SECURITY_ROLE_LEVELS_COUNT);
        performDeltaClearanceLevels(clearanceLevels, securityRole,
            getFieldDescrsMap(securityRole.getClass()));
    }

    public static int[] getClearanceLevels(SecurityRole securityRole,
            int[] clearanceLevels) throws RPMException
    {
        validateClearanceLevelsArrayLength(clearanceLevels,
            SECURITY_ROLE_LEVELS_COUNT);
        doGetClearanceLevels(securityRole, clearanceLevels,
            getFieldDescrsMap(securityRole.getClass()));
        return clearanceLevels;
    }

    private static Map buildFieldDescrsMap(Class securityObjectClass,
            Map securityObjectFieldToIntegerMap)
    {
        Map filedsMap = new HashMap();
        Method[] methods = securityObjectClass.getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            String methodName = methods[i].getName();
            if (methodName.startsWith("getCan")
                    || methodName.startsWith("getAll"))
            {
                Class returnType = methods[i].getReturnType();
                if (Boolean.class == returnType)
                {
                    String fieldName = methodName.substring(3);
                    FieldDescr descr = new FieldDescr();
                    descr.name = StringUtils.uncapitalize(fieldName);// so that it will match the field name
                    descr.getter = methods[i];
                    try
                    {
                        descr.delta = securityObjectClass.getMethod("delta"
                                + fieldName, BOOL_PARAM);
                        descr.setter = securityObjectClass.getMethod("set"
                                + fieldName, BOOL_PARAM);
                        descr.test = securityObjectClass.getMethod("test"
                                + fieldName + "Modified", null);
                    }
                    catch (SecurityException e)
                    {
                        logger.error(e);
                        continue;// something is wrong
                    }
                    catch (NoSuchMethodException e)
                    {
                        logger.error(e);
                        continue;// something is wrong
                    }
                    Integer hashInteger = (Integer) securityObjectFieldToIntegerMap
                        .get(descr.name);
                    if (hashInteger != null)
                    {
                        // Get hashvalue
                        int hashValue = hashInteger.intValue();
                        // extract clearance level and bit mask
                        descr.clearanceLevelId = hashValue / 32;
                        descr.mask = SecurityMask.MASK_ARRAY[hashValue % 32];
                        filedsMap.put(descr.name, descr);
                    }
                }
                else if (returnType.isArray()
                        && returnType.getComponentType() == Boolean.class)
                {
                    String fieldName = methodName.substring(3);
                    ArrayFieldDescr arrayDescr = new ArrayFieldDescr();
                    arrayDescr.name = StringUtils.uncapitalize(fieldName);// so that it will match the field name
                    arrayDescr.getter = methods[i];
                    try
                    {
                        arrayDescr.delta = securityObjectClass.getMethod(
                            "delta" + fieldName, BOOL_ARRAY_PARAM);
                        arrayDescr.setter = securityObjectClass.getMethod("set"
                                + fieldName, BOOL_ARRAY_PARAM);
                        arrayDescr.test = securityObjectClass.getMethod("test"
                                + fieldName + "Modified", null);
                    }
                    catch (SecurityException e)
                    {
                        logger.error(e);
                        continue;// something is wrong
                    }
                    catch (NoSuchMethodException e)
                    {
                        logger.error(e);
                        continue;// something is wrong
                    }

                    List hashIntegers = new ArrayList();
                    int index = 1;
                    Integer hashInteger = (Integer) securityObjectFieldToIntegerMap
                        .get(arrayDescr.name + index++);
                    while (hashInteger != null)
                    {
                        hashIntegers.add(hashInteger);
                        hashInteger = (Integer) securityObjectFieldToIntegerMap
                            .get(arrayDescr.name + index++);
                    }

                    if (!hashIntegers.isEmpty())
                    {
                        int size = hashIntegers.size();
                        int[] clearanceLevelIds = new int[size];
                        int[] masks = new int[size];
                        for (int j = 0; j < size; ++j)
                        {
                            hashInteger = (Integer) hashIntegers.get(j);
                            int hashValue = hashInteger.intValue();
                            clearanceLevelIds[j] = hashValue / 32;
                            masks[j] = SecurityMask.MASK_ARRAY[hashValue % 32];
                        }
                        arrayDescr.clearanceLevelIds = clearanceLevelIds;
                        arrayDescr.masks = masks;
                        filedsMap.put(arrayDescr.name, arrayDescr);
                    }
                }
            }
        }
        return filedsMap;
    }

    public static Map getFieldDescrsMap(Class securityObjectClass)
    {
        Map filedsMap = (Map) FIELD_DESCR_MAP.get(securityObjectClass);
        if (filedsMap == null)
        {
            Map securityFieldToIntegerMap = null;
            if (AbstractSecurityGroup.class
                .isAssignableFrom(securityObjectClass))
            {
                securityFieldToIntegerMap = SecurityFlags
                    .getResourceSecurityFieldToIntegerMap();
            }
            else if (SecurityRole.class.isAssignableFrom(securityObjectClass))
            {
                securityFieldToIntegerMap = SecurityFlags
                    .getRoleSecurityFieldToIntegerMap();
            }
            if (securityFieldToIntegerMap == null)
            {
                //                throw new IllegalArgumentException(
                //                    "Invalid security Object Class: " + securityObjectClass);
                return Collections.EMPTY_MAP;
            }
            filedsMap = buildFieldDescrsMap(securityObjectClass,
                securityFieldToIntegerMap);
            FIELD_DESCR_MAP.put(securityObjectClass, filedsMap);
        }
        return filedsMap;
    }

    private static Object invokeMethod(Object securityObject, Method method,
            Object paramObj)
    {
        Object results = null;
        try
        {
            if (paramObj == null)
            {
                results = method.invoke(securityObject, null);
            }
            else
            {
                Object[] objects =
                {
                    paramObj
                };
                results = method.invoke(securityObject, objects);
            }
        }
        catch (IllegalArgumentException e)
        {
            logger.error(e);
        }
        catch (IllegalAccessException e)
        {
            logger.error(e);
        }
        catch (InvocationTargetException e)
        {
            logger.error(e);
        }
        return results;
    }

    private static void doSetClearanceLevel(int[] clearanceLevels,
            RPMObject securityObject, Map fields)
    {
        Iterator it = fields.values().iterator();
        while (it.hasNext())
        {
            BaseFieldDescr baseDescr = (BaseFieldDescr) it.next();
            if (!baseDescr.isArray())
            {
                FieldDescr descr = (FieldDescr) baseDescr;
                Boolean b = Boolean
                    .valueOf((clearanceLevels[descr.clearanceLevelId] & descr.mask) == descr.mask);
                invokeMethod(securityObject, descr.setter, b);
            }
            else
            {
                ArrayFieldDescr arrayDescr = (ArrayFieldDescr) baseDescr;
                int length = arrayDescr.clearanceLevelIds.length;
                Boolean[] flags = new Boolean[length];
                for (int i = 0; i < length; ++i)
                {
                    flags[i] = Boolean
                        .valueOf((clearanceLevels[arrayDescr.clearanceLevelIds[i]] & arrayDescr.masks[i]) == arrayDescr.masks[i]);
                }
                invokeMethod(securityObject, arrayDescr.setter, flags);
            }
        }
    }

    public static void setClearanceLevel(int[] clearanceLevels,
            SecurityRole securityRole)
    {
        validateClearanceLevelsArrayLength(clearanceLevels,
            SECURITY_ROLE_LEVELS_COUNT);
        doSetClearanceLevel(clearanceLevels, securityRole,
            getFieldDescrsMap(securityRole.getClass()));
    }

}