package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.managers.StepManager;
import com.ibm.rpm.timesheet.managers.StepTimesheetManager;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.util.TimesheetValidator;
import com.ibm.rpm.wbs.containers.TaskAssignment;

public class StepCheckpoint extends AbstractStepCheckpoint
{
    protected static StepCheckpoint instance = new StepCheckpoint();

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {

        if (!GenericValidator.validateInstance(rpmObject, Step.class, msgCtx))
            return;

        super.validateSave(rpmObject, rpmScope, msgCtx);

        if (!msgCtx.isSuccessful())
        {
            return;
        }

        // Initialize
        Step step = (Step) rpmObject;
        StepScope scope = (StepScope) rpmScope;
        
        validateName(step, msgCtx);

        Object[] loadedValues = new Object[3];
        if(step.testBillableModified() || step.testETCChangedModified() || step.testWeightModified())
        {
            loadedValues = loadStepDBValues(step, msgCtx);            
        }
        Boolean  billable = (Boolean)loadedValues[0];
        Boolean  ETCChanged = (Boolean)loadedValues[1];
        Double  weight = (Double)loadedValues[2];
        
        if (step.testBillableModified())
        {
            GenericValidator.validateReadOnly(step,
                ValidationConstants.BILLABLE_FIELD, msgCtx);

            // reset the field to its previous value
            step.setBillable(billable);
        }

        if (step.testEETCModified())
        {
            GenericValidator.validateGreaterThanOrEqual(step,
                ValidationConstants.EETC_FIELD, step.getEETC(),
                ValidationConstants.EETC_MIN, msgCtx);

            GenericValidator.validateWorkingHour(step,
                ValidationConstants.EETC_FIELD, step.getEETC(), msgCtx);
        }

        if (step.testETCChangedModified())
        {
            GenericValidator.validateReadOnly(step,
                ValidationConstants.ETC_CHANGED_FIELD, msgCtx);

            // reset the field to its previous value
            step.setETCChanged(ETCChanged);
        }

        if (step.testAlwaysAvailableModified())
        {
            GenericValidator.validateDefaultValueUpdate(step,
                ValidationConstants.STEP_ALWAYS_AVAILABLE_FIELD, step
                    .getAlwaysAvailable(), msgCtx);
        }

        if (step.testWeightModified())
        {
            boolean valid = GenericValidator.validateRange(step,
                ValidationConstants.WEIGHT_FIELD, step.getWeight(),
                ValidationConstants.PERCENTAGE_MIN,
                ValidationConstants.PERCENTAGE_MAX, msgCtx);

            GenericValidator.validateDecimalDigits(step,
                ValidationConstants.WEIGHT_FIELD, step.getWeight(), msgCtx);

            if (valid)
            {
                Double previousWeight = weight;
                validateWeightAcrossSteps(step, previousWeight, msgCtx);
            }
        }
        
        TimesheetValidator.validateStepTimeCodes(step, scope, msgCtx);
        validateTimesheets(step, scope, msgCtx);
    }
    
    private void validateTimesheets(Step step, StepScope scope,
            MessageContext msgCtx)
    {
        boolean valid = false;

        if (scope != null && scope.isTimesheets())
        {
            valid = step.getStepTimesheets() != null;
        }
        else
        {
            if (step.getID() == null)
            {
                valid = false;
            }
            else
            {
                // Check in the database that REALLY there
                // is no timesheet for the step
                SqlBuffer stmt = new SqlBuffer();
                stmt.appendSelect(StepTimesheetManager.NAME_ACTIVITY_ID);
                stmt.appendFrom(StepTimesheetManager.TABLE_NAME);
                stmt.appendWhere();
                stmt
                    .appendEqualQuestionMark(StepTimesheetManager.NAME_ACTIVITY_ID);

                List result = null;
                try
                {
                    result = ManagerUtil.select(null,
                        step, ValidationConstants.STEP_TIMESHEETS_FIELD,
                        stmt, new Object[]
                        {
                            step.getID()
                        }, msgCtx);
                }
                catch (RPMException e)
                {
                    addException(e, msgCtx);
                    valid = false;
                }

                valid = result != null && result.size() > 0;
            }
        }

        if (!valid)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(step.getClass()) + " "
                        + ValidationConstants.STEP_TIMESHEETS_FIELD
            };
            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, step.getClass().getName(),
                ValidationConstants.STEP_TIMESHEETS_FIELD, step.getID());
            addException(e, msgCtx);
        }

    }

    /**
     * The summation of weight for all steps cannot exceed 100.
     * 
     * @param step
     * @param msgCtx
     * @return
     */
    private void validateWeightAcrossSteps(Step step, Double previousWeight,
            MessageContext msgCtx)
    {
        Step[] steps = null;
        TaskAssignment taskAssignment = step.getTaskAssignment();

        // look in the db for the steps
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect(StepManager.NAME_EDPERCENT);
        stmt.appendFrom(StepManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(StepManager.NAME_TSK_ASSIGNMENT_ID);

        List result = null;
        try
        {
            result = ManagerUtil.select(new Class[]
            {
                Integer.class
            }, null, null, stmt, new Object[]
            {
                taskAssignment.getID()
            }, msgCtx);
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
        }

        if (result != null)
        {
            steps = new Step[result.size()];
            for (int i = 0; i < result.size(); i++)
            {
                ArrayList row = (ArrayList) result.get(i);
                Step act = new Step();

                double weight = 0;
                Integer number = (Integer) row.get(0);
                if (number != null)
                {
                    weight = number.doubleValue();
                }
                act.setWeight(new Double (weight/100.0));
                steps[i] = act;
            }
        }

        if (steps == null)
            return;

        double weight = 0;
        
        if(step.getWeight() != null){
            weight = step.getWeight().doubleValue();
        }
        
        if(previousWeight != null){
            weight -= previousWeight.doubleValue();
        }

        for (int i = 0; i < steps.length; i++)
        {
            if (steps[i].getWeight() != null)
                weight += steps[i].getWeight().doubleValue();
        }

        if (weight > 100)
        {
            RPMException e = new RPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.WEIGHT_GREATER_THAN_100,
                step.getClass().getName(), ValidationConstants.STEPS_FIELD,
                step.getID());
            addException(e, msgCtx);
        }
    }

    private Object[] loadStepDBValues(Step step, MessageContext msgCtx)
    {
        Object[] loadedValues = new Object[3];
        
        if (step.getID() != null)
        {
            StepManager stepManager = new StepManager();
            
            SqlBuffer sql = new SqlBuffer();
            sql.appendSelect(StepManager.NAME_BILLABLE);
            sql.appendComma();
            sql.append(StepManager.NAME_ETC_MODIFIED);
            sql.appendComma();
            sql.append(StepManager.NAME_EDPERCENT);
            sql.appendFrom(StepManager.TABLE_NAME);
            sql.appendWhere();
            sql.appendEqual(StepManager.NAME_ELEMENT_ID, step.getID());
            
            try
            {
                Class[] columnTypes = new Class[]{Integer.class, String.class, Integer.class};
                List dbRows = ManagerUtil.select(columnTypes, null, null, sql, null, msgCtx);

                List dbValues = (List)dbRows.get(0); 
                loadedValues[0] = stepManager.decodeBillable(((Integer)dbValues.get(0)).shortValue());
                loadedValues[1] = stepManager.decodeETCChanged((String)(dbValues.get(1)));
                loadedValues[2] = stepManager.decodeWeight(((Integer)dbValues.get(2)).intValue());
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                addException(rpmException, msgCtx);
            }
        }
        
        return loadedValues;
    }
}