package com.ibm.rpm.timesheet.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.ValidateSerializer;
import com.ibm.rpm.timesheet.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.StepTimesheet;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.managers.StepTimesheetManager;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.timesheet.util.TimesheetManagerUtil;

/**
 * @author shahla
 */
public class StepTimesheetCheckpoint extends AbstractTimesheetCheckpoint
{
    private static final List TIMESHEET_PARENT_LIST = new ArrayList();

    protected static StepTimesheetCheckpoint instance = new StepTimesheetCheckpoint();

    static
    {
        TIMESHEET_PARENT_LIST.add(Step.class);
    }

    public static StepTimesheetCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {

        // validate the parent
        if (!super.isValidParent(rpmObject, messageContext, true, true,
            TIMESHEET_PARENT_LIST))
        {
            return;
        }

        super.validateSave(rpmObject, messageContext);
        if (!messageContext.isSuccessful())
            return;

        if (!GenericValidator.validateInstance(rpmObject, StepTimesheet.class,
            messageContext))
        {
            return;
        }

        StepTimesheet stepTimesheet = (StepTimesheet) rpmObject;

        Step step = (Step) stepTimesheet.getParent();
        
        // weekOf is needed before doing more validation
        if (!validWeekOf(stepTimesheet, messageContext))
            return;
        
        //replace weekof with a date that matches the beginning of the week
        TimesheetManagerUtil.setValidWeekOf(stepTimesheet, messageContext);

        ValidateSerializer serializer = messageContext.getObjectsToValidate();
        if (!serializer.before(step, stepTimesheet))
        {
            GenericValidator.validateMandatoryID(stepTimesheet,
                ValidationConstants.PARENT_FIELD, stepTimesheet.getParent(),
                messageContext);
            
        }
        
        if(stepTimesheet.getParent().getID() != null && stepTimesheet.getID() == null){
            checkForExistingStepTimesheet(stepTimesheet, messageContext);
        }
        
        SummaryTimesheet summaryTimesheet = loadSummaryTimesheet(stepTimesheet,
            messageContext);
        
        if (summaryTimesheet == null)
        {

            String[] params = new String[]
            {
                DateUtil.convertToString(stepTimesheet.getWeekOf())
            };

            RPMException e = new RPMException(
                ErrorCodes.NO_SUMMARY_TIMESHEET_FOR_WEEKOF, params,
                stepTimesheet.getClass().getName());
            addException(e, messageContext);
            return;
        }
        
        if (summaryTimesheet.getTimeCode1() != null
                || summaryTimesheet.getTimeCode2() != null)
        {
            // can't create step when SummaryTimesheet has Timecode
            String[] params = new String[]
            {
                summaryTimesheet.getID()
            };

            RPMException e = new RPMException(
                ErrorCodes.TIMECODE_ASSIGNED_TO_SUMMARY_TIMESHEET, params,
                stepTimesheet.getClass().getName());
            addException(e, messageContext);
            return;
        }
        
        if (!validApprovalStatus(summaryTimesheet, messageContext))
        {
            return;
        }

        if (stepTimesheet.testBillableModified())
        {
            GenericValidator.validateReadOnly(stepTimesheet,
                ValidationConstants.BILLABLE_FIELD, messageContext);

            Boolean billable = loadTimesheetBillable(stepTimesheet, messageContext);
            
            // reset read-only field value
            stepTimesheet.setBillable(billable);
        }
    }

    private Boolean loadTimesheetBillable(StepTimesheet timesheet,
            MessageContext msgCtx)
    {
        Boolean billable = null;
        
        if (timesheet.getID() != null)
        {
            try
            {
                StepTimesheetManager stepTimesheetManager = new StepTimesheetManager();

                String[] primaryKeyArray = stepTimesheetManager.getPrimaryKeyArray(timesheet);
                Object[] primaryKeyValues = stepTimesheetManager.getPrimaryKeyValues(timesheet);

                SqlBuffer sql = new SqlBuffer();
                for(int i=0;i<primaryKeyArray.length;i++)
                {
                    sql.appendEqualQuestionMark(primaryKeyArray[i]);
                }
                
                Integer iBillable = (Integer) ManagerUtil.selectColumnValue(
                    Integer.class, null, null,
                    StepTimesheetManager.NAME_BILLABLE,
                    StepTimesheetManager.TABLE_NAME, sql, primaryKeyValues,
                    msgCtx);
                billable = stepTimesheetManager.decodeBillable(iBillable.shortValue());
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                rpmException.assignContainer(timesheet);
                addException(rpmException, msgCtx);
            }
        }
        return billable;
    }

    private static void checkForExistingStepTimesheet(
            StepTimesheet stepTimesheet, MessageContext messageContext)            
    {
       
        SqlBuffer stmt = new SqlBuffer();
        stmt.appendSelect();
        stmt.append(StepTimesheetManager.NAME_ACTIVITY_ID);
        stmt.appendFrom(StepTimesheetManager.TABLE_NAME);
        stmt.appendWhere();
        stmt.appendEqualQuestionMark(StepTimesheetManager.NAME_ACTIVITY_ID);
        stmt.appendEqualQuestionMark(StepTimesheetManager.NAME_WEEK_OF);
        List result = null;
    
        try
        {
            result = ManagerUtil.select(null, null, null, stmt, new Object[]
            {
                    stepTimesheet.getParent().getID(),
                    DateUtil.convertToString(stepTimesheet.getWeekOf())
            }, messageContext);

        }
        catch (RPMException e)
        {
            addException(e, messageContext);
        }
    
        //result should not be null or empty at this point
        if (result != null && result.size() > 0)
        {
            
            String[] params = new String[]
            {
                stepTimesheet.getParent().getID(), DateUtil.convertToString(stepTimesheet.getWeekOf())
            };

            RPMException e = new RPMException(
                ErrorCodes.STEP_TIMESHEET_ALREADY_EXIST, params,
                stepTimesheet.getClass().getName());
            addException(e, messageContext);
        }
    }

    private static SummaryTimesheet loadSummaryTimesheet(StepTimesheet timesheet,
            MessageContext messageContext)
    {
        SummaryTimesheet summaryTimesheet = new SummaryTimesheet();

        try
        {
            Step step = (Step) timesheet.getParent();

            String taskAssignmentId = null;
            if (step.getTaskAssignment() != null)
            {
                taskAssignmentId = step.getTaskAssignment().getID();
            }
            else
            {
                taskAssignmentId = TimesheetManagerUtil
                    .retrieveStep_TaskAssignmentId(step.getID(), messageContext);
            }

            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(SummaryTimesheet.class);
            summaryTimesheet.setID(taskAssignmentId);
            summaryTimesheet.setWeekOf(timesheet.getWeekOf());
            
            SummaryTimesheetScope summaryTimesheetScope = new SummaryTimesheetScope();
            summaryTimesheetScope.setTimeCodes(true);

            summaryTimesheet = (SummaryTimesheet) manager.loadByPrimaryKey(
                summaryTimesheet, summaryTimesheetScope, messageContext, false);
        }
        catch (Exception e)
        {
            RPMException rpmException = new RPMException(e);
            rpmException.assignContainer(timesheet);
            addException(rpmException, messageContext);

        }
        return summaryTimesheet;
    }

}