/*
 * Created on Feb 27, 2006
 * @author mmatwiy
 * Utility methods for Timesheet specific validation
 * (moved from GenericValidator)
 */
package com.ibm.rpm.timesheet.util;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.financial.containers.TimeCode;
import com.ibm.rpm.financial.containers.TimeCodeAssignment;
import com.ibm.rpm.financial.managers.TimecodeManager;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.timesheet.constants.ErrorCodes;
import com.ibm.rpm.timesheet.constants.ValidationConstants;
import com.ibm.rpm.timesheet.containers.GenericTaskAssignment;
import com.ibm.rpm.timesheet.containers.Step;
import com.ibm.rpm.timesheet.containers.SummaryTimesheet;
import com.ibm.rpm.timesheet.containers.TimesheetApprovalStatus;
import com.ibm.rpm.timesheet.scope.StepScope;
import com.ibm.rpm.timesheet.scope.SummaryTimesheetScope;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.types.ProjectPublishedType;


public class TimesheetValidator extends GenericValidator
{
    
    public static boolean validateProjectInPlannedStage(RPMObject object,
            WorkElement workElement, String fieldName,
            MessageContext messageContext)
    {
        WbsScheduleDate plannedDate = workElement.getPlanDate();
        boolean valid = (plannedDate != null
                && plannedDate.getStartDate() != null && plannedDate
            .getFinishDate() != null);

        if (!valid)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()),
                    StringUtil.getShortClassName(workElement.getClass()),
                    workElement.getName()
            };
            RPMException e = new RPMException(
                ErrorCodes.WORK_ELEMENT_STILL_PROPOSED, errorParams,
                object.getClass().getName(), fieldName, object.getID());
            addException(e, messageContext);
        }

        return valid;
    }
    
    public static boolean validateProjectPublishedToTeam(SummaryTimesheet object,
            GenericProject project,
            String fieldName, 
            MessageContext messageContext)
    {
        
        boolean published = false;

        if (project.getPublished() != null)
            published = project.getPublished().equals(ProjectPublishedType.ToTeam);

        if (!published)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(object.getClass()),
                    StringUtil.getShortClassName(project.getClass()),
                    project.getName()
            };
            RPMException e = new RPMException(
                ErrorCodes.PROJECT_NOT_PUBLISHED_TO_TEAM, errorParams, 
                object.getClass().getName(), fieldName, object.getID());
            addException(e, messageContext);
            return false;
        }

        return published;
    }
    
    public static boolean validateTimeCodeAssignedToProject(RPMObject object, 
            TimeCode timecode, GenericProject project, String fieldName,  
            MessageContext msgCtx) 
    {
        boolean valid = true;
        
        TimeCodeAssignment[] projectTimeCodes = null;
        if (project != null)
            projectTimeCodes = project.getTimeCodeAssignments();
        
        if (projectTimeCodes != null)
        {
            for (int i = 0; i < projectTimeCodes.length && !valid; i++)
                valid = projectTimeCodes[i].getTimeCode().equals(timecode);
        } 
        else
        {
            // search for the TimeCodeAssignmnet in the database
            SqlBuffer stmt = new SqlBuffer();
            stmt.appendSelect();
            stmt.append(TimecodeManager.NAME_ELEMENT_ID);
            stmt.appendFrom(TimecodeManager.TABLE_NAME);
            stmt.appendWhere();
            stmt.appendEqualQuestionMark(TimecodeManager.NAME_ELEMENT_ID);
            stmt.appendEqualQuestionMark(TimecodeManager.NAME_PARENT_ID);
            List result = null;
            try
            {
                result = ManagerUtil.select(null, null, null, stmt, new Object[]
                {
                        project.getID(), timecode.getID()
                }, msgCtx);
            }
            catch (RPMException e)
            {
                valid = false;
            }
            if (result == null || result.size() == 0)
                valid = false;
        }
        
        if (!valid)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(object.getClass()) + " " +
                fieldName, timecode.getName()
            };
            RPMException e = new RPMException(
                ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT, errorParams, 
                object.getClass().getName(), fieldName, object.getID());
            addException(e, msgCtx);
        }
        
        return valid;
    }
    
    public static boolean validateUserCanSubmit(
            TimesheetApprovalStatus timesheetApprovalStatus,
            MessageContext messageContext)
    {

        boolean valid = false;

        if (messageContext
                .getUser().getID().equals(timesheetApprovalStatus.getResource().getID()))
        {
            valid = true;
        }

        if (!valid)
        {
            String[] errorParams =
            {
                timesheetApprovalStatus.getApprovalStatus().toString()
            };
            RPMException e = new RPMException(
                ErrorCodes.USER_NOT_ALLOWED_TO_CHANGE_STATUS, errorParams,
                timesheetApprovalStatus.getClass().getName(),
                ValidationConstants.APPROVAL_STATUS_FIELD);
            addException(e, messageContext);
        }

        return valid;

    }
    
    public static boolean validateSummaryTimesheetTimeCodes(SummaryTimesheet summaryTimesheet,
            SummaryTimesheet loadedTimesheet,
            int activtiesCount, SummaryTimesheetScope scope,
            MessageContext msgCtx)
    {
        boolean valid = true;
        if (scope != null && scope.isTimeCodes())
        {
            // validate TimeCode1
            valid = validateSummaryTimesheetTimeCode(summaryTimesheet, loadedTimesheet,
                activtiesCount, true, msgCtx);
            // validate TimeCode2
            valid = validateSummaryTimesheetTimeCode(summaryTimesheet, loadedTimesheet,
                activtiesCount, false, msgCtx) && valid;
        }
        
        return valid;
    }
    
    private static boolean validateSummaryTimesheetTimeCode(SummaryTimesheet summaryTimesheet,
            SummaryTimesheet loadedTimesheet,
            int activtiesCount, boolean isTimeCode1, MessageContext msgCtx)
    {
        boolean valid = true;
        boolean testTimeCodeModified;
        String timeCodeFieldName;
        TimeCode timeCode = null;
        if (isTimeCode1)
        {
            testTimeCodeModified = summaryTimesheet.testTimeCode1Modified();
            timeCodeFieldName = ValidationConstants.TIMECODE1_FIELD;
            timeCode = summaryTimesheet.getTimeCode1();
        }
        else
        {
            testTimeCodeModified = summaryTimesheet.testTimeCode2Modified();
            timeCodeFieldName = ValidationConstants.TIMECODE2_FIELD;
            timeCode = summaryTimesheet.getTimeCode2();
        }

        // Field validation: timecodes have to be assigned to the project
        if (testTimeCodeModified)
        {
            if (activtiesCount > 0)
            {
                valid = GenericValidator.validateReadOnlyConditionExists(
                    summaryTimesheet, timeCodeFieldName, StringUtil
                        .getShortClassName(TaskAssignment.class.getName())
                            + " " + ValidationConstants.STEPS_FIELD, msgCtx) && valid;

                // reset the read-only field value
                if (isTimeCode1)
                {
                    summaryTimesheet.setTimeCode1(loadedTimesheet
                        .getTimeCode1());
                }
                else
                {
                    summaryTimesheet.setTimeCode2(loadedTimesheet
                        .getTimeCode2());
                }
            }
            else
            {
                valid = validateTimeCode(summaryTimesheet, summaryTimesheet
                    .getTaskAssignment().getID(), timeCode, isTimeCode1,
                    timeCodeFieldName, msgCtx);
            }
        }
        
        return valid;
    }
    
    public static boolean validateStepTimeCodes(Step step, StepScope scope, MessageContext msgCtx)
    {
        boolean valid = true;
        
        if (scope != null && scope.isTimeCodes())
        {
            // valid TimeCode1
            valid = validateStepTimeCode(step, true, msgCtx);
            
            // valid TimeCode2
            valid = validateStepTimeCode(step, false, msgCtx) && valid;
        }
        
        return valid;
    }
    
    private static boolean validateStepTimeCode(Step step, boolean isTimeCode1,
            MessageContext msgCtx)
    {
        boolean valid = true;
        boolean testTimeCodeModified;
        String timeCodeFieldName;
        TimeCode timeCode = null;
        if (isTimeCode1)
        {
            testTimeCodeModified = step.testTimeCode1Modified();
            timeCodeFieldName = ValidationConstants.TIMECODE1_FIELD;
            timeCode = step.getTimeCode1();
        }
        else
        {
            testTimeCodeModified = step.testTimeCode2Modified();
            timeCodeFieldName = ValidationConstants.TIMECODE2_FIELD;
            timeCode = step.getTimeCode2();
        }
        
        if (testTimeCodeModified)
        {
            try
            {
                valid = validateTimeCode(step,
                    step.getTaskAssignment().getID(), timeCode, isTimeCode1,
                    timeCodeFieldName, msgCtx);
            }
            catch (Exception e)
            {
                addException(new RPMException(e), msgCtx);
            }
        }
        
        return valid;
    }

    // TODO after implement DefaultStep
//    public static boolean validateDefaultStepTimeCodes(DefaultStep defaultStep, DefaultStepScope defaultStepScope, MessageContext msgCtx)
//    {
//        boolean valid = true;
//        
//        if (defaultStepScope != null && defaultStepScope.isTimeCodes())
//        {
//            // valid TimeCode1
//            valid = validateDefaultStepTimeCode(defaultStep, true, msgCtx);
//            
//            // valid TimeCode2
//            valid = validateDefaultStepTimeCode(defaultStep, false, msgCtx) && valid;
//        }
//        
//        return valid;
//    }
    
//    public static boolean validateDefaultStepTimeCode(DefaultStep defaultStep, boolean isTimeCode1, MessageContext msgCtx)
//    {
//        
//    }

    private static boolean validateTimeCode(RPMObject object,
            String taskAssignmentId, TimeCode timeCode, boolean isTimeCode1,
            String fieldName, MessageContext msgCtx)
    {
        if (timeCode == null)
        {
            return true;
        }
        
        ArrayList allowedTimeCode = null;
        try
        {
            String workElementId = TimesheetManagerUtil
                .retrieveTaskAssignment_TaskId(taskAssignmentId, msgCtx);
            TimecodeManager manager = (TimecodeManager) RPMManagerFactory
                .getInstance().getRPMObjectManager(timeCode);
            allowedTimeCode = manager.searchTimeCodes(workElementId, null,
                isTimeCode1, msgCtx);
        }
        catch (RPMException e)
        {
            addException(e, msgCtx);
        }
        catch (Exception e)
        {
            addException(new RPMException(e), msgCtx);
        }

        if (allowedTimeCode != null)
        {
            for (int i = 0; i != allowedTimeCode.size(); ++i)
            {
                if (((RPMObject) allowedTimeCode.get(i)).getID().trim().equals(
                    timeCode.getID().trim()))
                {
                    return true;
                }
            }
        }

        String[] errorParams =
        {
            StringUtil.getShortClassName(object.getClass()) + " " +
            fieldName, timeCode.getName()
        };
        RPMException e = new RPMException(
            ErrorCodes.TIMECODE_NOT_ASSIGNED_TO_WORKELEMENT, errorParams, 
            object.getClass().getName(), fieldName, object.getID());
        addException(e, msgCtx);

        return false;
    }
    
    public static GenericProject loadProject(GenericTaskAssignment taskAssignment,
            MessageContext messageContext)
    {
        GenericProject project = new GenericProject();

        // retrieve the projectId in task_assignments table
        String projectId = null;
        try
        {
            projectId = TimesheetManagerUtil.retrieveTaskAssignment_ProjectId(
                taskAssignment.getID(), messageContext);
        }
        catch (RPMException e1)
        {
            addException(e1, messageContext);
            return null;
        }

        if (projectId == null)
        {
            return null;
        }

        // check if id exist in tmt_wbs and load the name
        try
        {
            RPMObjectManager manager = RPMManagerFactory.getInstance()
                .getRPMObjectManager(GenericProject.class);

            project.setID(projectId);

            messageContext.getCache().removeContainerFromCache(project);

            project = (GenericProject) manager.loadByPrimaryKey(project, null,
                messageContext, false);
        }
        catch (Exception e2)
        {
            RPMException e = new RPMException(e2);
            addException(e, messageContext);
            return null;
        }

        if (project == null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(GenericProject.class),
                    projectId
            };
            RPMException e = new RPMException(
                com.ibm.rpm.timesheet.constants.ErrorCodes.OBJECT_DOES_NOT_EXIST,
                errorParams);
            addException(e, messageContext);
        }

        return project;
    }
}
