/*
 * Created on Nov 24, 2005 @author mmatwiy Checkpoint validation for container
 * Profile.
 * Checkpoint validation for container Profile. 
 * Note that the Profile grandparent validation is done in ResourceTaskAssignmentCheckpoint.
 * This checkpoint is called from ResourceTaskAssignmentCheckpoint because a single
 * Profile can have multiple parents; the normal parental hierarchy does not apply.
 */
package com.ibm.rpm.wbs.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.ResourceAttributeCompetencyProficiencyLevel;
import com.ibm.rpm.resource.containers.ResourceAttributeSkillProficiencyLevel;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Profile;
import com.ibm.rpm.wbs.containers.ProfileCompetencyAssignment;
import com.ibm.rpm.wbs.containers.ProfileSkillAssignment;
import com.ibm.rpm.wbs.managers.ProfileManager;
import com.ibm.rpm.wbs.scope.ProfileScope;

public class ProfileCheckpoint extends AbstractCheckpoint
{
    protected static ProfileCheckpoint instance = new ProfileCheckpoint();

    public static ProfileCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        if (!GenericValidator
            .validateInstance(rpmObject, Profile.class, msgCtx))
            return;

        Profile profile = (Profile) rpmObject;

        // a Profile's parent must be null; there are no valid parents
        if (profile.getParent() != null)
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(rpmObject.getClass()),
                    StringUtil
                        .getShortClassName(profile.getParent().getClass()),
                    "null"
            };
            RPMException e = new RPMException(
                com.ibm.rpm.framework.constants.ErrorCodes.WRONG_PARENT_TYPE,
                errorParams, rpmObject.getClass().getName(),
                ValidationConstants.PARENT, rpmObject.getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
        }

        // profileName field is read-only
        if (profile.testProfileNameModified())
        {
            GenericValidator.validateReadOnly(profile,
                ValidationConstants.PROFILE_NAME_FIELD, msgCtx);
            
//            //TODO replace loading by a direct sql (profileName)
//            Profile loadedProfile = loadProfile(profile, scope, msgCtx);
//            
//            // reset read-only field value
//            profile.setProfileName(loadedProfile.getProfileName());
            
            String profileName = loadProfileName(profile, msgCtx);
            profile.setProfileName(profileName);
        }

        // profileCompetencyAssignments is a mandatory field
        GenericValidator.validateMandatory(profile,
            ValidationConstants.PROFILE_COMPETENCY_ASSIGNMENTS_FIELD, profile
                .getProfileCompetencyAssignments(), msgCtx);
        
        if(profile.getID() == null)
        {
     	   validateCreate(profile,(ProfileScope)rpmScope,msgCtx);
        }
    }

    private void validateCreate(Profile profile, ProfileScope profileScope, MessageContext msgCtx)
    {
    	ProfileCompetencyAssignment pca[] = profile.getProfileCompetencyAssignments();
    	if(pca != null && pca.length > 0)
    	{
    		for(int i=0; i<pca.length; i++)
    		{
    			if(pca[i] != null && !(pca[i].getProficiencyLevel() instanceof ResourceAttributeCompetencyProficiencyLevel))
    			{
    				String[] errorParams =
    				{
    						StringUtil
    						.getShortClassName(ProfileCompetencyAssignment.class)
    				};
    				RPMException e = new RPMException(
    						com.ibm.rpm.resource.constants.ErrorCodes.INVALID_COMPTENCY_PROFICIENCY_LEVEL,
    						errorParams, pca[i].getClass().getName(),
    						ValidationConstants.PROFILE_PROFICIENCY_LEVEL_FIELD, pca[i].getID());
    				addException(e, msgCtx);
    			}
    		}
    	}
    	ProfileSkillAssignment[] psa = profile.getProfileSkillAssignments();
    	if(psa != null && psa.length > 0)
    	{
    		for(int i=0; i<psa.length; i++)
    		{
    			if(psa[i] != null && !(psa[i].getProficiencyLevel() instanceof ResourceAttributeSkillProficiencyLevel))
    			{
    				String[] errorParams =
    	            {
    	                    StringUtil
    	                        .getShortClassName(psa[i].getClass())
    	            };
    				RPMException e = new RPMException(
    						com.ibm.rpm.resource.constants.ErrorCodes.INVALID_SKILL_PROFICIENCY_LEVEL,
    		                errorParams, psa[i].getClass().getName(),
    		                ValidationConstants.PROFILE_PROFICIENCY_LEVEL_FIELD, psa[i].getID());
    		            addException(e, msgCtx);
    			}
    		}
    	}
    }
    
    private String loadProfileName(Profile profile, MessageContext msgCtx)
    {
        String profileName = null;

        if (profile.getID() != null)
        {
            SqlBuffer sql = new SqlBuffer();
            sql.appendEqualQuestionMark(ProfileManager.NAME_PROFILE_ID);
            
            Object[] arguments = new Object[]
            {
                profile.getID()
            };
            
            try
            {
                profileName = (String) ManagerUtil.selectColumnValue(
                    String.class, null, null, ProfileManager.NAME_PROFILE_NAME,
                    ProfileManager.TABLE_NAME, sql, arguments, msgCtx);
            }
            catch (RPMException e)
            {
                RPMException rpmException = new RPMException(e);
                addException(rpmException, msgCtx);
            }
        }

        return profileName;
    }
}