/*
 * Created on Nov 24, 2005 
 * @author mmatwiy 
 * Checkpoint validation for WBS container ResourceTaskAssignment.
 * Note that the Profile grandparent validation is done in this checkpoint.
 * ProfileCheckpoint is called from ResourceTaskAssignmentCheckpoint because a single
 * Profile can have multiple parents; the normal parental hierarchy does not apply.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ibm.rpm.applicationadministration.containers.AssignmentStatus;
import com.ibm.rpm.applicationadministration.managers.RPMCalendarManager;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.resource.containers.Resource;
import com.ibm.rpm.scopemanagement.containers.Action;
import com.ibm.rpm.scopemanagement.containers.AggregateScope;
import com.ibm.rpm.scopemanagement.containers.ChangeRequest;
import com.ibm.rpm.scopemanagement.containers.Defect;
import com.ibm.rpm.scopemanagement.containers.Issue;
import com.ibm.rpm.scopemanagement.containers.Requirement;
import com.ibm.rpm.scopemanagement.containers.Risk;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeElement;
import com.ibm.rpm.scopemanagement.containers.ScopeFolder;
import com.ibm.rpm.scopemanagement.containers.ScopeMilestone;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.scopemanagement.containers.ScopeTask;
import com.ibm.rpm.scopemanagement.containers.ServiceRequest;
import com.ibm.rpm.security.containers.ActionItemSecurityRole;
import com.ibm.rpm.security.containers.ChangeRequestSecurityRole;
import com.ibm.rpm.security.containers.DefectSecurityRole;
import com.ibm.rpm.security.containers.DeliverableSecurityRole;
import com.ibm.rpm.security.containers.FolderSecurityRole;
import com.ibm.rpm.security.containers.IssueSecurityRole;
import com.ibm.rpm.security.containers.ProjectSecurityRole;
import com.ibm.rpm.security.containers.RequirementSecurityRole;
import com.ibm.rpm.security.containers.RiskSecurityRole;
import com.ibm.rpm.security.containers.SecurityRole;
import com.ibm.rpm.security.containers.ServiceRequestSecurityRole;
import com.ibm.rpm.security.containers.TaskSecurityRole;
import com.ibm.rpm.security.managers.SecurityRoleManager;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.Contour;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.Project;
import com.ibm.rpm.wbs.containers.Proposal;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkOrganization;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.managers.ResourceTaskAssignmentManager;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.AssignmentType;
import com.ibm.rpm.wbs.types.ContourType;

public class ResourceTaskAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final List RESOURCE_TASK_ASSIGNMENT_PARENT_LIST = new ArrayList();

    private static final Map SECURITY_ROLE_CLASS_MAP = new HashMap();

    protected static ResourceTaskAssignmentCheckpoint instance = new ResourceTaskAssignmentCheckpoint();

    static
    {
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Proposal.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Project.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(WorkOrganization.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Deliverable.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(WorkProduct.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(SummaryTask.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Milestone.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Task.class);

        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(ChangeRequest.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Defect.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Requirement.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Risk.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Issue.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(ScopeTask.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(ServiceRequest.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(Action.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(ScopeDeliverable.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(ScopeSummaryTask.class);
        RESOURCE_TASK_ASSIGNMENT_PARENT_LIST.add(ScopeMilestone.class);
    }

    static
    {
        SECURITY_ROLE_CLASS_MAP.put(Project.class, ProjectSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Proposal.class, ProjectSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(WorkOrganization.class,
            ProjectSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Deliverable.class,
            DeliverableSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(WorkProduct.class,
            DeliverableSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Milestone.class, TaskSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Task.class, TaskSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Action.class, ActionItemSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(ChangeRequest.class,
            ChangeRequestSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Defect.class, DefectSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Issue.class, IssueSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Requirement.class,
            RequirementSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(Risk.class, RiskSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(ScopeDeliverable.class,
            DeliverableSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(ScopeMilestone.class,
            TaskSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(ScopeSummaryTask.class,
            DeliverableSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(ScopeTask.class, TaskSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(SummaryTask.class, DeliverableSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP.put(ServiceRequest.class,
            ServiceRequestSecurityRole.class);
        SECURITY_ROLE_CLASS_MAP
            .put(ScopeFolder.class, FolderSecurityRole.class);
    }

    public static ResourceTaskAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // the parent must have an ID
        if (!super.isValidParent(rpmObject, msgCtx, true, true,
            RESOURCE_TASK_ASSIGNMENT_PARENT_LIST))
        {
            return;
        }
        
        if (!GenericValidator.validateInstance(rpmObject,
            ResourceTaskAssignment.class, msgCtx))
        {
            return;
        }

        ResourceTaskAssignment resourceTaskAssignment = (ResourceTaskAssignment) rpmObject;
        ResourceTaskAssignmentScope rtaScope = (ResourceTaskAssignmentScope) rpmScope;
        
        /*
         * 61102 verify the type of the object we're assigning. Since here, 
         * the valid-type depends on the type of the element we're associating, 
         * it can't be hardcoded into the metadata
         */
        if (resourceTaskAssignment.getSecurityRole() != null
                && resourceTaskAssignment.getParent() != null)
        {
            RPMObject parent = resourceTaskAssignment.getParent(); // always in scope
            SecurityRole userSecurityRole = resourceTaskAssignment
                .getSecurityRole();

            if (userSecurityRole instanceof SecurityRole
                    && userSecurityRole.getID() != null)
            {
                try
                {
                    // can't do a direct sql as we need to validate
                    // securityRole type
                    SecurityRoleManager mgr = (SecurityRoleManager) msgCtx
                        .getFactory().getManagerCaller().getManagerFactory()
                        .getRPMObjectManager(userSecurityRole);
                    userSecurityRole = (SecurityRole) mgr.loadByPrimaryKey(
                        userSecurityRole, new RPMObjectScope(), msgCtx, false);

                    if (userSecurityRole == null)
                    {
                        throw new Exception(
                            "Can't load the security role specified!");
                    }
                }
                catch (Exception e)
                {
                    RPMException rpmException = new RPMException(e);
                    rpmException.setSeverity(SeverityLevel.CriticalError);
                    rpmException.assignContainer(resourceTaskAssignment);
                    addException(rpmException, msgCtx);
                }
            }

            Class validSecurityRoleClass = (Class) SECURITY_ROLE_CLASS_MAP
                .get(parent.getClass());

            if (validSecurityRoleClass != null)
            {
                if (validSecurityRoleClass.isAssignableFrom(userSecurityRole
                    .getClass()))
                {
                    resourceTaskAssignment.setSecurityRole(userSecurityRole);
                    rpmObject = resourceTaskAssignment;
                }
                else
                {
                    RPMException e = new RPMException(
                        com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_SECURITY_ROLE_OBJECT_TYPE,
                        new String[]
                        {
                            validSecurityRoleClass.getName()
                        }, resourceTaskAssignment.getClass().getName(),
                        ValidationConstants.SECURITY_ROLE_FIELD);
                    addException(e, msgCtx);
                }
            }
            else
            {
                RPMException e = new RPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_FIND_VALID_SECURITYROLE_PARENT,
                    resourceTaskAssignment.getClass().getName(),
                    ValidationConstants.SECURITY_ROLE_FIELD);
                addException(e, msgCtx);
            }
        }

        
        // assignmentStatus field is read-only
        if (rtaScope != null && rtaScope.isAssignmentStatus()
                && resourceTaskAssignment.testAssignmentStatusModified())
        {
            GenericValidator.validateReadOnly(resourceTaskAssignment,
                ValidationConstants.ASSIGNMENT_STATUS_FIELD, msgCtx);

            AssignmentStatus assignmentStatus = loadAssignmentStatus(resourceTaskAssignment, msgCtx);
            
            // reset read-only field value
            resourceTaskAssignment.setAssignmentStatus(assignmentStatus);
        }

        // resourceType field is read-only
        if (resourceTaskAssignment.testResourceTypeModified())
        {
            GenericValidator.validateReadOnly(resourceTaskAssignment,
                ValidationConstants.RESOURCE_TYPE_FIELD, msgCtx);
        }

        // securityRole field is mandatory on create.
        if (resourceTaskAssignment.getID() == null)
        {
            GenericValidator.validateMandatory(resourceTaskAssignment,
                ValidationConstants.SECURITY_ROLE_FIELD, resourceTaskAssignment
                    .getSecurityRole(), msgCtx);
        }

        // on insert only, the resource OR profile field is mandatory
        if (resourceTaskAssignment.getID() == null
                && resourceTaskAssignment.getResource() == null
                && resourceTaskAssignment.getProfile() == null)
        {
            String[] errorParams =
            {
                StringUtil.getShortClassName(resourceTaskAssignment.getClass())
                        + " " + ValidationConstants.RESOURCE_FIELD + " or "
                        + ValidationConstants.PROFILE_FIELD
            };

            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                errorParams, resourceTaskAssignment.getClass().getName(),
                ValidationConstants.RESOURCE_FIELD + " or "
                        + ValidationConstants.PROFILE_FIELD,
                resourceTaskAssignment.getID());
            addException(e, msgCtx);

        }

        if (resourceTaskAssignment.getProfile() != null)
        {
            // if profile contains a value, taskAssignment cannot be null
            GenericValidator.validateConditionalMandatory(
                resourceTaskAssignment,
                ValidationConstants.TASK_ASSIGNMENT_FIELD,
                resourceTaskAssignment.getTaskAssignment(),
                ValidationConstants.PROFILE_FIELD + " contains a value.",
                msgCtx);

            // if profile contains a value, resource must be null
            if (resourceTaskAssignment.getResource() != null)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(resourceTaskAssignment
                            .getClass()), ValidationConstants.PROFILE_FIELD,
                        ValidationConstants.RESOURCE_FIELD
                };
                RPMException e = new RPMException(
                    ErrorCodes.NULL_REQUIRED_WHEN_FIELD_VALUE_EXISTS,
                    errorParams, resourceTaskAssignment.getClass().getName(),
                    ValidationConstants.PROFILE_FIELD, resourceTaskAssignment
                        .getID());
                addException(e, msgCtx);
            }

            // The containing profile must be validated from here 
            // Only validate the profile if the enclosing ResourceTaskAssignment is valid
            if (msgCtx.isSuccessful() && rtaScope != null
                    && rtaScope.getProfile() != null)
            {
                RPMObject profileGrandparent = resourceTaskAssignment
                    .getParent();

                GenericProject grandparentProject = (profileGrandparent == null)
                        ? null
                        : (profileGrandparent instanceof WorkElement
                                ? (GenericProject) ((WorkElement) profileGrandparent)
                                    .getContainingProject()
                                : (GenericProject) ((ScopeElement) profileGrandparent)
                                    .getProject());

                // the Profile's grandparent must contain a project with an ID
                if (profileGrandparent != null)
                {
                    GenericValidator.validateMandatoryID(resourceTaskAssignment
                        .getProfile(),
                        ValidationConstants.PROFILE_GRANDPARENT_PROJECT,
                        grandparentProject, msgCtx);
                }

                // if the Profile's grandparent is not of type GenericProject,
                // the grandparent's project assignmentType must be 'Work'
                if (profileGrandparent != null && grandparentProject != null)
                {
                    if ((grandparentProject.getAssignmentType() == null)
                            || ((grandparentProject.getAssignmentType()
                                .getValue().equals(AssignmentType._Duration)) && !(profileGrandparent instanceof GenericProject)))
                    {
                        RPMException e = new RPMException(
                            com.ibm.rpm.wbs.constants.ErrorCodes.PROFILE_GRANDPARENT_ASSIGNMENT_TYPE_INVALID,
                            resourceTaskAssignment.getProfile().getClass()
                                .getName());
                        addException(e, msgCtx);
                    }
                }

                resourceTaskAssignment.getProfile().setModified();

                ProfileCheckpoint.getInstance().validateSave(
                    resourceTaskAssignment.getProfile(), rtaScope.getProfile(),
                    msgCtx);
            }
        }
        // if a resource is being assigned, the calendar and the parent's
        // containing project must exist
        else if (resourceTaskAssignment.getResource() != null
                && resourceTaskAssignment.getTaskAssignment() != null)
        {
            /*
             * 60180 - We raise an exception if the TaskAssignment is being
             * attached to a duration-based project, but only for Tasks
             * 
             * 
             */
            RPMObject resourceTaskAssignmentParentObject = resourceTaskAssignment
                .getParent();
            if (resourceTaskAssignmentParentObject instanceof Task)
            {
                Task resourceTaskAssignmentParent = (Task) resourceTaskAssignmentParentObject;
                GenericProject containingProject = (GenericProject) resourceTaskAssignmentParent
                    .getContainingProject();
                /*
                 * Load the containingProject if the user did not specify it.
                 */
                if (containingProject == null)
                {
                    try
                    {
                        RPMObjectManager taskObjectManager = msgCtx
                            .getFactory().getManagerCaller()
                            .getManagerFactory()
                            .getRPMObjectManager(Task.class);
                        WorkElementScope taskScope = new WorkElementScope();
                        taskScope.setContainingProject(new WorkElementScope());

                        resourceTaskAssignmentParent = (Task) taskObjectManager
                            .load(resourceTaskAssignmentParent, taskScope,
                                msgCtx);
                        containingProject = (GenericProject) resourceTaskAssignmentParent
                            .getContainingProject();
                    }
                    catch (Exception e)
                    {
                        RPMException rpmException = new RPMException(e);
                        rpmException
                            .assignContainer(resourceTaskAssignmentParent);
                        addException(rpmException, msgCtx);
                    }
                }

                if (containingProject != null
                        && containingProject.getAssignmentType() == AssignmentType.Duration)
                {
                    RPMException taskAssignmentShouldBeNull = new RPMException(
                        com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_CREATE_TASKASSIGNMENT_UNDER_DURATION_BASED_PROJECTS,
                        ResourceTaskAssignment.class.getName());

                    addException(taskAssignmentShouldBeNull, msgCtx);
                }
            }

            if (!isResourceWithCalendar(resourceTaskAssignment.getResource(),
                msgCtx))
            {
                // make sure the resource has a calendar
                String[] errorParams =
                {
                        StringUtil.getShortClassName(resourceTaskAssignment
                            .getClass())
                                + ": "
                                + ValidationConstants.RESOURCE_CALENDAR_FIELD,
                        "the ResourceTaskAssigment's resource field is not null"
                };
                RPMException e = new RPMException(
                    ErrorCodes.CONDITIONAL_MANDATORY, errorParams, StringUtil
                        .getShortClassName(resourceTaskAssignment.getClass()),
                    ValidationConstants.RESOURCE_CALENDAR_FIELD,
                    resourceTaskAssignment.getID());
                addException(e, msgCtx);
            }

            // also need to make sure the parent has a containing project
            if (resourceTaskAssignment.getParent() instanceof WorkElement
                    && ((WorkElement) resourceTaskAssignment.getParent())
                        .getContainingProject() == null)
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(resourceTaskAssignment
                            .getClass())
                                + ": "
                                + ValidationConstants.PARENT_CONTAINING_PROJECT_FIELD,
                        "the ResourceTaskAssigment's resource field is not null"
                };
                RPMException e = new RPMException(
                    ErrorCodes.CONDITIONAL_MANDATORY, errorParams, StringUtil
                        .getShortClassName(resourceTaskAssignment.getClass()),
                    ValidationConstants.PARENT_CONTAINING_PROJECT_FIELD,
                    resourceTaskAssignment.getID());
                addException(e, msgCtx);
            }

            //The ScopeElement parent cannot have ScopeElement children
            //if the resourceTaskAssignment is assigned as work and not
            //participant.           
            if (resourceTaskAssignment.getParent() instanceof AggregateScope)
            {
                AggregateScope aggScope = (AggregateScope) resourceTaskAssignment
                    .getParent();
                if (aggScope.getChildren() != null
                        && aggScope.getChildren().length > 0)
                {
                    String[] errorParams =
                    {
                            StringUtil.getShortClassName(resourceTaskAssignment
                                .getParent().getClass()),
                            resourceTaskAssignment.getParent().getID()
                    };

                    RPMException e = new RPMException(
                        ErrorCodes.CANNOT_SAVE_RTA_WITH_TA, errorParams,
                        StringUtil.getShortClassName(resourceTaskAssignment
                            .getClass()), resourceTaskAssignment.getID());
                    addException(e, msgCtx);
                }
            }
        }
        if(resourceTaskAssignment.getContour() != null)
        {
        	validateContours(resourceTaskAssignment, msgCtx);
        }
    }

    private Integer loadAssignmentStatusId(ResourceTaskAssignment resourceTaskAssignment, MessageContext msgCtx)
    {
        Integer assignmentStatusId = null;
        SqlBuffer sql = new SqlBuffer();
        sql.appendEqualQuestionMark(ResourceTaskAssignmentManager.NAME_RES_ASSIGNMENT_ID);
        
        Object[] arguments = new Object[]
        {
            resourceTaskAssignment.getID()
        };
        
        try
        {
            assignmentStatusId = (Integer) ManagerUtil.selectColumnValue(
                Integer.class, null, null,
                ResourceTaskAssignmentManager.NAME_ASSIGN_STATUS,
                ResourceTaskAssignmentManager.TABLE_NAME, sql, arguments,
                msgCtx);
        }
        catch (RPMException e)
        {
            RPMException rpmException = new RPMException(e);
            addException(rpmException, msgCtx);
        }
        return assignmentStatusId;
    }
    
    private AssignmentStatus loadAssignmentStatus(ResourceTaskAssignment resourceTaskAssignment, MessageContext msgCtx)
    {
        AssignmentStatus assignmentStatus = null;
        if (resourceTaskAssignment.getID() != null)
        {
            try
            {
                Integer assignmentStatusId = loadAssignmentStatusId(resourceTaskAssignment, msgCtx);
                
                if(assignmentStatusId!=null && assignmentStatusId.intValue()!=0)
                {
                    assignmentStatus = new AssignmentStatus();
                    assignmentStatus.setID(assignmentStatusId.toString());
                    RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(AssignmentStatus.class);
                    assignmentStatus = (AssignmentStatus) manager
                    .loadByPrimaryKey(assignmentStatus, null, msgCtx, false);                    
                }                
            }
            catch (Exception e)
            {
                assignmentStatus = null;
                RPMException rpmException = new RPMException(e);
                addException(rpmException, msgCtx);
            }
        } 
        return assignmentStatus;
    }
   
    public static boolean isResourceWithCalendar(Resource resource,
            MessageContext messageContext)
    {
        SqlBuffer sql = new SqlBuffer();
        sql.appendEqualQuestionMark(RPMCalendarManager.NAME_ELEMENT_ID);
        
        Object[] arguments = new Object[]
        {
            resource.getID()
        };
        
        String elementId = null;
        try
        {
            elementId = (String) ManagerUtil.selectColumnValue(String.class,
                null, null, RPMCalendarManager.NAME_ELEMENT_ID,
                RPMCalendarManager.TABLE_NAME, sql, arguments, messageContext);
        }
        catch (RPMException e)
        {
            RPMException rpmException = new RPMException(e);
            addException(rpmException, messageContext);
        }
        
        return (elementId!=null);
    }

    public final boolean canReplaceProfileWithResource(
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope, Resource resource,
            MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canReplaceProfileWithResource(
                messageContext, assignment, rtaScope, resource).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

    public final boolean canReplaceResourceWithProfile(
            ResourceTaskAssignment assignment,
            ResourceTaskAssignmentScope rtaScope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canReplaceResourceWithProfile(
                messageContext, assignment, rtaScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    private void validateContours(ResourceTaskAssignment resourceTaskAssignment, MessageContext msgCtx) {
    	/**
    	 * Rules to be validated for Contours (copied from re-docs for SP_SEARCH_2)
    	 * 
    	 * RULE1: cannot assign more then once a resource to a element. ---> Not validated.. Directly handled in the SP Call 
    	 * RULE2: cannot assign a resource to a project if the resource is already assigned to a task whitin the project. ---> Not validated.. Directly handled in the SP Call
    	 * RULE3: cannot assign to a task from project reserve more effort then remaining effort at the project. ---> validated here by executing search
    	 * RULE4: if bitflag = 9 then assignment without effort, otherwise assignment with effort.  ---> validated here
N/R      * RULE5: if bitflag between 1 and 8 the assignment is contoured: contour > 0 and work > 0 ---> Not replicated in Client.
    	 * RULE6: if search by name then partially ignore the search criteria: org_id, geo_id, pool_id, work_loc, and_roles, and_skills, shw_roles, shw_skills
    	 * RULE7: if effort = 0 and bitflag < 9 then both start & finish dates are required ---> validated here
N/R		 * RULE8: if assign units <> 100%, then bitflag < 9 and effort > 0 and both start & finish dates are required --> validated here
    	 * RULE9: cannot assign a resource if is not active, effective date is in the future or discontinue date is in the past ---> second part of the rule is validated here
    	 */
    	Contour contour = resourceTaskAssignment.getContour();

    	if(resourceTaskAssignment.getID()==null)         // While creating validate Contour
    	{
    		if(isValidContour(contour.getContourType()))
    		{	
    			if(contour.getContourType().equals(ContourType.ASAP))
    			{
    				if(contour.getWork()!=null && contour.getWork().doubleValue() > 0)
    				{
    					RPMException e = new RPMException(
    							com.ibm.rpm.wbs.constants.ErrorCodes.WORK_MUST_BE_NULL_WHEN_CONTOUR_TYPE_IS_ASAP,
    							resourceTaskAssignment.getClass().getName(),
    							ValidationConstants.CONTOUR_FIELD);
    					addException(e, msgCtx);
    				}
    				if(contour.getOccurrences() != null && contour.getOccurrences().intValue() > 0 )
    				{
    					RPMException e = new RPMException(
    							com.ibm.rpm.wbs.constants.ErrorCodes.OCCURRENCE_MUST_BE_ZERO_FOR_ASAP,
    							resourceTaskAssignment.getClass().getName(),
    							ValidationConstants.CONTOUR_FIELD);
    					addException(e, msgCtx);       				
    				}
    				// Validating that Effort and FinishDate are mutually exclusive
    				if(contour.getEffortInMinutes() == null || contour.getEffortInMinutes().intValue() <= 0)
    				{
    					if(contour.getFinishDate() == null)
    					{
    						RPMException e = new RPMException(
    								com.ibm.rpm.wbs.constants.ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_AND_FINISH_DATE,
    								resourceTaskAssignment.getClass().getName(),
    								ValidationConstants.CONTOUR_FIELD);
    						addException(e, msgCtx);       				
    					}
    				}else{
    					if(contour.getFinishDate() != null)
    					{
    						RPMException e = new RPMException(
    								com.ibm.rpm.wbs.constants.ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_AND_FINISH_DATE,
    								resourceTaskAssignment.getClass().getName(),
    								ValidationConstants.CONTOUR_FIELD);
    						addException(e, msgCtx);       				        					
    					}
    				}

    			}else // if ContourType != ASAP
    			{
    				boolean effort = (contour.getEffortInMinutes() !=null && contour.getEffortInMinutes().intValue() > 0);
    				boolean finishDate =  (contour.getFinishDate()!=null);
    				boolean occurrences = (contour.getOccurrences()!=null && contour.getOccurrences().intValue() > 0);
    				if((effort && (finishDate||occurrences)) || (finishDate && (effort||occurrences)) || (occurrences && (finishDate||effort)))
    				{
    					RPMException e = new RPMException(
    							com.ibm.rpm.wbs.constants.ErrorCodes.MUTUALLY_EXCLUSIVE_EFFORT_FINISH_DATE_AND_OCCURRENCES,
    							resourceTaskAssignment.getClass().getName(),
    							ValidationConstants.CONTOUR_FIELD);
    					addException(e, msgCtx);       				        					
    				}
    			}

    			// Validations regardless of ContourType
    			if(contour.getStartDate()==null) // This is mandatory
    			{
    				RPMException e = new RPMException(
    						com.ibm.rpm.wbs.constants.ErrorCodes.START_DATE_MISSING_FOR_CONTOUR,
    						resourceTaskAssignment.getClass().getName(),
    						ValidationConstants.MANDATORy_FIELD);
    				addException(e, msgCtx);        			
    			}

    			if(contour.getFinishDate()!=null)
    			{
    				if(contour.getFinishDate().before(contour.getStartDate())) //finish date is before start date throw exception
    				{
    					RPMException e = new RPMException(
    							com.ibm.rpm.wbs.constants.ErrorCodes.FINISH_DATE_BEFORE_START_DATE,
    							resourceTaskAssignment.getClass().getName(),
    							ValidationConstants.CONTOUR_FIELD);
    					addException(e, msgCtx);        			        				
    				}
    			}

    			if (resourceTaskAssignment.getTaskAssignment()== null)
    			{
    				RPMException e = new RPMException(
    						com.ibm.rpm.wbs.constants.ErrorCodes.CONTOUR_APPLICABLE_ONLY_FOR_EFFORT_ASSIGNMENT,
    						resourceTaskAssignment.getClass().getName(),
    						ValidationConstants.CONTOUR_FIELD);
    				addException(e, msgCtx);

    			}

    			if(resourceTaskAssignment.getParent() instanceof ScopeElement)
    			{
    				GenericProject project = ((ScopeElement)resourceTaskAssignment.getParent()).getProject();
    				GenericValidator.validateMandatory(resourceTaskAssignment.getParent(),
    						com.ibm.rpm.scopemanagement.constants.ValidationConstants.NAME_PROJECT , project, msgCtx);
    			}

    			if(msgCtx.isSuccessful())
    			{
    				// This validation is not done for Profiles because
    				// Profiles can be created only with ResourceTaskAssignments
    				// Hence we cannot search for the profiles before creating them.
    				if (resourceTaskAssignment.getResource() != null)
    				{
    					String ID = resourceTaskAssignment.getResource().getID();
    					try {
    						ResourceTaskAssignmentManager rtaManager = (ResourceTaskAssignmentManager)msgCtx.getFactory().getManagerCaller()
    						.getManagerFactory().getRPMObjectManager(ResourceTaskAssignment.class);
    						String[] searchIDs = rtaManager.executeSearch(resourceTaskAssignment, msgCtx);
    						if(searchIDs != null)
    						{
    							boolean found = false;
    							for(int i=0; i<searchIDs.length && !found; i++)
    							{
    								if(ID.equals(searchIDs[i]))
    								{
    									found = true;
    								}
    							}
    							if(!found)
    							{
    								RPMException e = new RPMException(
    										com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_CONTOUR,
    										resourceTaskAssignment.getClass().getName(),
    										ValidationConstants.CONTOUR_FIELD);
    								addException(e, msgCtx);
    							}
    						}
    						else
    						{
    							RPMException e = new RPMException(
    									com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_CONTOUR,
    									resourceTaskAssignment.getClass().getName(),
    									ValidationConstants.CONTOUR_FIELD);
    							addException(e, msgCtx);
    						}
    					} catch (RPMException e) {
    						RPMException rpmEx = new RPMException(
    								com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_CONTOUR,
    								resourceTaskAssignment.getClass().getName(),
    								ValidationConstants.CONTOUR_FIELD);
    						rpmEx.setMessage(e.getMessage());
    						addException(rpmEx, msgCtx);
    					}
    					catch (SQLException e) {
    						RPMException rpmEx = new RPMException(
    								com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_CONTOUR,
    								resourceTaskAssignment.getClass().getName(),
    								ValidationConstants.CONTOUR_FIELD);
    						rpmEx.setMessage(e.getMessage());
    						addException(rpmEx, msgCtx);
    					}
    				}
    			}
    		}
    		else
    		{
    			RPMException e = new RPMException(
    					com.ibm.rpm.wbs.constants.ErrorCodes.UNSUPPORTED_CONTOUR,
    					resourceTaskAssignment.getClass().getName(),
    					ValidationConstants.CONTOUR_FIELD);
    			addException(e, msgCtx);
    		}
    	}
    	else  // During Update
    	{
    		//cant use contour while update
    		RPMException e = new RPMException(
    				com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_ASSIGN_CONTOUR_DURING_UPDATE_OF_TASK_ASSIGNMENTS,
    				resourceTaskAssignment.getClass().getName(),
    				ValidationConstants.CONTOUR_FIELD);
    		addException(e, msgCtx);        			
    	}

    }
    
    private boolean isValidContour(ContourType contourType)
    {
    	boolean isValidContour = false;
    	if(contourType.equals(ContourType.ASAP) || contourType.equals(ContourType.HoursPerDay)
    			|| contourType.equals(ContourType.PercentilePerDay))
    	{
    		isValidContour = true;
    	}
    	return isValidContour;
    }
}