/*
 * Created on Nov 24, 2005 
 * @author mmatwiy 
 * Checkpoint validation for WBS container TaskAssignment.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectManager;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.managers.WbsUtil;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;

public class TaskAssignmentCheckpoint extends AbstractCheckpoint
{
    private static final List TASK_ASSIGNMENT_PARENT_LIST = new ArrayList();

    protected static TaskAssignmentCheckpoint instance = new TaskAssignmentCheckpoint();

    static
    {
        TASK_ASSIGNMENT_PARENT_LIST.add(ResourceTaskAssignment.class);
    }

    public static TaskAssignmentCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // Tasks can be either duration or time-based. 
        // Three fields are always read-only for TaskAssignments attached to duration-based SimpleNodes

        
        super.isValidParent(rpmObject, msgCtx, true,
            TASK_ASSIGNMENT_PARENT_LIST);

        TaskAssignment taskAssignment = (TaskAssignment) rpmObject;
        TaskAssignmentScope scope = (TaskAssignmentScope) rpmScope;
        
        // we need the parent to check validation of EETC and the grandparent to distinguish between the types of tasks
        if (scope == null || scope.getParent() == null)
        {
            if (scope == null)
            {
                scope = new TaskAssignmentScope();
            }
            ResourceTaskAssignmentScope grandParentScope = new ResourceTaskAssignmentScope();
            grandParentScope.setParent(new RPMObjectScope());
            
            scope.setParent(grandParentScope);
        }

        //TODO replace loading by a direct sql (check if it's possible)
        TaskAssignment loadedAssignment = loadTaskAssignment(taskAssignment,
            scope, msgCtx);
        
        if (taskAssignment.testCompleteModified())
        {
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.COMPLETE_FIELD, msgCtx);

            // reset the read-only field value
            taskAssignment.setComplete(loadedAssignment.getComplete());
        }
        
        // the actualWork, percentageComplete, and percentageWork fields must be 
        // modified and saved individually because they affect each other.
        // In the case where more than one of these is modified:
        // -if actualWork is modified, it will be saved
        // -if actualWork is not modified then the two others and therefore
        // percentage work will be saved.
        // For duration-based tasks, only actualWork can be updated
        if (((taskAssignment.testActualWorkModified() && 
                (taskAssignment.testPercentageCompleteModified() || taskAssignment
            .testPercentageWorkModified())) || (taskAssignment.testPercentageCompleteModified() && 
                        ( taskAssignment.testActualWorkModified() || taskAssignment.testPercentageWorkModified()))
                || (taskAssignment.testPercentageWorkModified() && (taskAssignment.testPercentageCompleteModified() || taskAssignment
                    .testActualWorkModified()))))
        {            
            RPMException e = null;
            if (taskAssignment.testActualWorkModified())
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(taskAssignment.getClass()),
                        ValidationConstants.ACTUAL_WORK_FIELD,
                        ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                        ValidationConstants.PERCENTAGE_WORK_FIELD
                };

                e = new RPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.INDIVIDUAL_MODIFICATION_REQUIRED,
                    errorParams, taskAssignment.getClass().getName());
            }
            else
            {
                String[] errorParams =
                {
                        StringUtil.getShortClassName(taskAssignment.getClass()),
                        ValidationConstants.PERCENTAGE_WORK_FIELD,
                        ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                        ValidationConstants.ACTUAL_WORK_FIELD
                };

                e = new RPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.INDIVIDUAL_MODIFICATION_REQUIRED,
                    errorParams, taskAssignment.getClass().getName());
            }
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
        }

        if (taskAssignment.testActualFinishDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.ACTUAL_FINISH_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setActualFinishDate(loadedAssignment
                .getActualFinishDate());
        }

        if (taskAssignment.testActualStartDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.ACTUAL_START_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setActualStartDate(loadedAssignment
                .getActualStartDate());
        }

        if (taskAssignment.testActualWorkModified())
        {
            // Read-only if there's no actualStartDate and no 
            // plannedDurationInMinutes(from copy proposed to plan on the RTA)
            if (taskAssignment.getActualStartDate() == null
                    && taskAssignment.getPlannedDurationHours() == null)
            {
                GenericValidator.validateReadOnlyConditionMissing(
                    taskAssignment, ValidationConstants.ACTUAL_WORK_FIELD,
                    ValidationConstants.ACTUAL_START_DATE_FIELD + " or "
                            + ValidationConstants.PLANNED_DURATION_HOURS_FIELD, null,
                    msgCtx);
                
                // reset the field to its original value
                taskAssignment.setActualWork(loadedAssignment.getActualWork());
            }
            else
            {
                // Check that actualWork is in the correct range of values
                GenericValidator.validateRange(taskAssignment,
                    ValidationConstants.ACTUAL_WORK_FIELD, taskAssignment
                        .getActualWork(), ValidationConstants.ACTUAL_WORK_MIN,
                    ValidationConstants.ACTUAL_WORK_MAX, msgCtx);

                GenericValidator.validateDefaultValueUpdate(taskAssignment,
                    ValidationConstants.ACTUAL_WORK_FIELD, taskAssignment
                        .getActualWork(), msgCtx);
            }
        }

        if (taskAssignment.testAssignmentStartDateModified())
        {
            // Cannot be modified if actualStartDate has a value
            if (!GenericValidator.validateReadOnlyConditionExists(
                taskAssignment,
                ValidationConstants.ASSIGNMENT_START_DATE_FIELD,
                ValidationConstants.ACTUAL_START_DATE_FIELD, taskAssignment
                    .getActualStartDate(), msgCtx))
            {
                // reset the field to its original value
                taskAssignment.setAssignmentStartDate(loadedAssignment
                    .getAssignmentStartDate());

            }
            else
            {
                GenericValidator.validateDefaultValueUpdate(taskAssignment,
                    ValidationConstants.ASSIGNMENT_START_DATE_FIELD,
                    taskAssignment.getAssignmentStartDate(), msgCtx);
            }
        }

        if (taskAssignment.testBaselineFinishDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.BASELINE_FINISH_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setBaselineFinishDate(loadedAssignment
                .getBaselineFinishDate());
        }

        if (taskAssignment.testBaselineStartDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.BASELINE_START_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setBaselineStartDate(loadedAssignment
                .getBaselineStartDate());

        }

        if (taskAssignment.testBaselineWorkModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.BASELINE_WORK_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setBaselineWork(loadedAssignment.getBaselineWork());
        }

        if (taskAssignment.testBillableModified())
        {
            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.BILLABLE_FIELD, taskAssignment
                    .getBillable(), msgCtx);
        }

        if (taskAssignment.testCalculatedLeveledFinishDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.CALCULATED_LEVELED_FINISH_DATE_FIELD,
                msgCtx);

            // reset the field to its original value
            taskAssignment.setCalculatedLeveledFinishDate(loadedAssignment
                .getCalculatedLeveledFinishDate());

        }

        if (taskAssignment.testCalculatedLeveledStartDateModified())
        {
            // read-only field
            GenericValidator
                .validateReadOnly(taskAssignment,
                    ValidationConstants.CALCULATED_LEVELED_START_DATE_FIELD,
                    msgCtx);

            // reset the field to its original value
            taskAssignment.setCalculatedLeveledStartDate(loadedAssignment
                .getCalculatedLeveledStartDate());
        }

        if (taskAssignment.testCalculatedLeveledWorkModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.CALCULATED_LEVELED_WORK_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setCalculatedLeveledWork(loadedAssignment
                .getCalculatedLeveledWork());
        }

        if (taskAssignment.testCostPerHourModified())
        {
            // Check that costPerHour is in the correct range of values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.COST_PER_HOUR_FIELD, taskAssignment
                    .getCostPerHour(), ValidationConstants.COST_PER_HOUR_MIN,
                ValidationConstants.COST_PER_HOUR_MAX, msgCtx);

            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.COST_PER_HOUR_FIELD, taskAssignment
                    .getCostPerHour(), msgCtx);
        }

        if (taskAssignment.testDaysOfDelayModified())
        {
            // Cannot be modified if actualStartDate has a value
            if (!GenericValidator.validateReadOnlyConditionExists(
                taskAssignment, ValidationConstants.DAYS_OF_DELAY_FIELD,
                ValidationConstants.ACTUAL_START_DATE_FIELD, taskAssignment
                    .getActualStartDate(), msgCtx))
            {
                // reset the field to its original value
                taskAssignment
                    .setDaysOfDelay(loadedAssignment.getDaysOfDelay());
            }
            else
            {
                // Check that daysOfDelay is in the correct range of values
                GenericValidator.validateRange(taskAssignment,
                    ValidationConstants.DAYS_OF_DELAY_FIELD, taskAssignment
                        .getDaysOfDelay(),
                    ValidationConstants.DAYS_OF_DELAY_MIN,
                    ValidationConstants.DAYS_OF_DELAY_MAX, msgCtx);

                GenericValidator.validateDefaultValueUpdate(taskAssignment,
                    ValidationConstants.DAYS_OF_DELAY_FIELD, taskAssignment
                        .getDaysOfDelay(), msgCtx);
            }
        }

        if (taskAssignment.testDefaultLaborModified())
        {
            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.DEFAULT_LABOR_FIELD, taskAssignment
                    .getDefaultLabor(), msgCtx);
        }

        if (taskAssignment.testEETCHoursModified())
        {
            if (taskAssignment.getEETCHours() != null)
            {
                GenericValidator.validateGreaterThanOrEqual(taskAssignment,
                    ValidationConstants.EETC_HOURS_FIELD, taskAssignment
                        .getEETCHours(), ValidationConstants.EETC_HOURS_MIN,
                    msgCtx);
                GenericValidator.validateWorkingHour(taskAssignment,
                    ValidationConstants.EETC_HOURS_FIELD, taskAssignment
                        .getEETCHours(), msgCtx);
                
                if (taskAssignment.getEETCHours() == new Double(0))
                {
                    // EETCHours cannot be 0 if there is no actualStartDate
                    GenericValidator.validateConditionMissing(taskAssignment,
                        ValidationConstants.EETC_HOURS_FIELD, taskAssignment
                            .getEETCHours().toString(),
                        ValidationConstants.ACTUAL_START_DATE_FIELD, taskAssignment
                            .getActualStartDate(), msgCtx);
                }
            }
            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.EETC_HOURS_FIELD, taskAssignment
                    .getEETCHours(), msgCtx);

            // EETCHours must be null for a project assignment 
            // (the ResourceTaskAssignment's parent is a GenericProject)
            ResourceTaskAssignment parent = (ResourceTaskAssignment) taskAssignment
                .getParent();
            RPMObject grandparent = parent == null ? null : parent.getParent();

            if (grandparent == null)
            {
                // if the parent is null (valid on update), we'll use the loaded one 
                // to retrieve the parent type
                grandparent = WbsUtil.loadResourceTaskAssignmentParentType(
                    msgCtx, (ResourceTaskAssignment) loadedAssignment
                        .getParent());
            }

            if (grandparent instanceof GenericProject)
            {
                GenericValidator
                    .validateMustBeNull(
                        taskAssignment,
                        ValidationConstants.EETC_HOURS_FIELD,
                        taskAssignment.getEETCHours(),
                        "the ResourceTaskAssignment's parent is a GenericProject",
                        msgCtx);
            }
        }

        if (taskAssignment.testAssignmentFinishDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.ASSIGNMENT_FINISH_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setAssignmentFinishDate(loadedAssignment
                .getAssignmentFinishDate());
        }

        if (taskAssignment.testExpectedWorkModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.EXPECTED_WORK_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setExpectedWork(loadedAssignment.getExpectedWork());
        }

        if (taskAssignment.testExpectedFinishDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.EXPECTED_FINISH_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setExpectedFinishDate(loadedAssignment
                .getExpectedFinishDate());

        }

        if (taskAssignment.testExpectedStartDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.EXPECTED_START_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setExpectedStartDate(loadedAssignment
                .getExpectedStartDate());
        }
        
        if (taskAssignment.testForecastFinishDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.FORECAST_FINISH_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setForecastFinishDate(loadedAssignment
                .getForecastFinishDate());
        }
        
        if (taskAssignment.testForecastStartDateModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.FORECAST_START_DATE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setForecastStartDate(loadedAssignment
                .getForecastStartDate());
        }

        if (taskAssignment.testLastScheduleWorkModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.LAST_SCHEDULE_WORK_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setLastScheduleWork(loadedAssignment
                .getLastScheduleWork());
        }

        if (taskAssignment.testPercentageCompleteModified())
        {
            // the field is read-only before copyProposedToPlan
            if (!WbsUtil.isTaskAssignmentInPlannedStage(taskAssignment, msgCtx))
            {
                GenericValidator.validateReadOnlyConditionMissing(
                    taskAssignment, ValidationConstants.PERCENTAGE_COMPLETE_FIELD,
                    ValidationConstants.SCHEDULE_START_FIELD, null,
                    msgCtx);
            }
            
            // Check that percentageComplete is in the correct range of
            // values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.PERCENTAGE_COMPLETE_FIELD, taskAssignment
                    .getPercentageComplete(),
                ValidationConstants.PERCENTAGE_COMPLETE_MIN,
                ValidationConstants.PERCENTAGE_COMPLETE_MAX, msgCtx);
            GenericValidator.validateDecimalDigits(taskAssignment,
                ValidationConstants.PERCENTAGE_COMPLETE_FIELD, taskAssignment
                    .getPercentageComplete(), msgCtx);
        }

        if (taskAssignment.testPercentageDurationCompleteModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.PERCENTAGE_DURATION_COMPLETE_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setPercentageDurationComplete(loadedAssignment
                .getPercentageDurationComplete());
        }

        if (taskAssignment.testPercentageRateAdjustmentModified())
        {
            // Check that percentageRateAdjustment is in the correct range of
            // values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_FIELD,
                taskAssignment.getPercentageRateAdjustment(),
                ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_MIN,
                ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_MAX, msgCtx);

            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.PERCENTAGE_RATE_ADJUSTMENT_FIELD,
                taskAssignment.getPercentageRateAdjustment(), msgCtx);
        }

        if (taskAssignment.testPercentageWorkModified())
        {   
            // Read-only if there's no actualStartDate and no 
            // plannedDurationInMinutes(from copy proposed to plan on the RTA)
            if (taskAssignment.getActualStartDate() == null
                    && taskAssignment.getPlannedDurationHours() == null)
            {
                GenericValidator.validateReadOnlyConditionMissing(
                    taskAssignment, ValidationConstants.PERCENTAGE_WORK_FIELD,
                    ValidationConstants.ACTUAL_START_DATE_FIELD + " or "
                            + ValidationConstants.PLANNED_DURATION_HOURS_FIELD, null,
                    msgCtx);
           
                // reset the field to its original value
                taskAssignment.setPercentageWork(loadedAssignment
                    .getPercentageWork());
            } 
            else 
            {
                // Check that percentageWork is in the correct range of values
                GenericValidator.validateRange(taskAssignment,
                    ValidationConstants.PERCENTAGE_WORK_FIELD, taskAssignment
                        .getPercentageWork(),
                    ValidationConstants.PERCENTAGE_WORK_MIN,
                    ValidationConstants.PERCENTAGE_WORK_MAX, msgCtx);
            }
        }

        if (taskAssignment.testRemainingHoursModified())
        {
            // read-only field
            GenericValidator.validateReadOnly(taskAssignment,
                ValidationConstants.REMAINING_HOURS_FIELD, msgCtx);

            // reset the field to its original value
            taskAssignment.setRemainingHours(loadedAssignment
                .getRemainingHours());
        }

        if (taskAssignment.testSellingRatePerHourModified())
        {
            // Check that sellingRatePerHour is in the correct range of values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.SELLING_RATE_PER_HOUR_FIELD, taskAssignment
                    .getSellingRatePerHour(),
                ValidationConstants.SELLING_RATE_PER_HOUR_MIN,
                ValidationConstants.SELLING_RATE_PER_HOUR_MAX, msgCtx);

            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.SELLING_RATE_PER_HOUR_FIELD, taskAssignment
                    .getSellingRatePerHour(), msgCtx);
        }

        if (taskAssignment.testSpecialCostPerHourModified())
        {
            // Check that specialCostPerHour is in the correct range of values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.SPECIAL_COST_PER_HOUR_FIELD, taskAssignment
                    .getSpecialCostPerHour(),
                ValidationConstants.SPECIAL_COST_PER_HOUR_MIN,
                ValidationConstants.SPECIAL_COST_PER_HOUR_MAX, msgCtx);

            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.SPECIAL_COST_PER_HOUR_FIELD, taskAssignment
                    .getSpecialCostPerHour(), msgCtx);
        }

        if (taskAssignment.testSpecialSellingRatePerHourModified())
        {
            // Check that specialSellingRatePerHour is in the correct range of
            // values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_FIELD,
                taskAssignment.getSpecialSellingRatePerHour(),
                ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_MIN,
                ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_MAX, msgCtx);

            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.SPECIAL_SELLING_RATE_PER_HOUR_FIELD,
                taskAssignment.getSpecialSellingRatePerHour(), msgCtx);
        }

        if (taskAssignment.testUnitsPercentageModified())
        {
            // Check that unitsPercentage is in the correct range of values
            GenericValidator.validateRange(taskAssignment,
                ValidationConstants.UNITS_PERCENTAGE_FIELD, taskAssignment
                    .getUnitsPercentage(),
                ValidationConstants.UNITS_PERCENTAGE_MIN,
                ValidationConstants.UNITS_PERCENTAGE_MAX, msgCtx);

            GenericValidator.validateDefaultValueUpdate(taskAssignment,
                ValidationConstants.UNITS_PERCENTAGE_FIELD, taskAssignment
                    .getUnitsPercentage(), msgCtx);
        }

    }

    private TaskAssignment loadTaskAssignment(TaskAssignment taskAssignment,
            TaskAssignmentScope scope, MessageContext msgCtx)
    {
        TaskAssignment loadedAssignment = null;

        if (taskAssignment.getID() != null)
        {
            try
            {
                RPMObjectManager manager = RPMManagerFactory.getInstance()
                    .getRPMObjectManager(TaskAssignment.class);
                loadedAssignment = (TaskAssignment) manager.loadByPrimaryKey(
                    taskAssignment, scope, msgCtx, false);
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                rpmException.assignContainer(taskAssignment);
                addException(rpmException, msgCtx);
            }
        }
        if (loadedAssignment == null)
        {
            loadedAssignment = new TaskAssignment();
        }

        return loadedAssignment;
    }
    
}
