/*
 * Created on 31-May-2006
 * @author mmatwiy
 * Checkpoint validation for application services methods for WBS.
 */
package com.ibm.rpm.wbs.checkpoints;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.checkpoints.ScopeCheckpoint;
import com.ibm.rpm.framework.constants.ErrorCodes;
import com.ibm.rpm.framework.security.controller.ISecurityController;
import com.ibm.rpm.framework.security.controller.SecurityValidationResult;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.scopemanagement.containers.AbstractAggregateScope;
import com.ibm.rpm.scopemanagement.containers.ScopeDeliverable;
import com.ibm.rpm.scopemanagement.containers.ScopeSummaryTask;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.AggregateNode;
import com.ibm.rpm.wbs.containers.Deliverable;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.containers.Milestone;
import com.ibm.rpm.wbs.containers.ResourceTaskAssignment;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.SummaryTask;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.TaskAssignment;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.containers.WorkProduct;
import com.ibm.rpm.wbs.managers.WbsUtil;
import com.ibm.rpm.wbs.scope.ResourceTaskAssignmentScope;
import com.ibm.rpm.wbs.scope.TaskAssignmentScope;
import com.ibm.rpm.wbs.scope.WorkElementScope;

public class WbsMethodCheckpoint extends AbstractCheckpoint
{
    protected static WbsMethodCheckpoint instance = new WbsMethodCheckpoint();

    public static WbsMethodCheckpoint getInstance()
    {
        return instance;
    }

    public void validateCopyProposedToPlan(WorkElement workElement,
            MessageContext msgCtx)
    {
        // WorkElement must exist and contain an ID
        GenericValidator
            .validateMandatoryID("WorkElement", workElement, msgCtx);
    }

    public void validateCopyAssignmentProposedToPlan(
            ResourceTaskAssignment resourceTaskAssignment, MessageContext msgCtx)
    {
        // ResourceTaskAssignment must exist and contain an ID
        GenericValidator.validateMandatoryID("ResourceTaskAssignment",
            resourceTaskAssignment, msgCtx);

        if (resourceTaskAssignment != null)
        {
            // Resource must exist and contain an ID        
            GenericValidator.validateMandatoryID(resourceTaskAssignment,
                ValidationConstants.RESOURCE_FIELD, resourceTaskAssignment
                    .getResource(), msgCtx);

            // TaskAssignment must exist and contain an ID        
            GenericValidator.validateMandatoryID(resourceTaskAssignment,
                ValidationConstants.TASK_ASSIGNMENT_FIELD,
                resourceTaskAssignment.getTaskAssignment(), msgCtx);

            if (resourceTaskAssignment.getTaskAssignment() != null)
            {
                // Get actualStartDate must be null; plannedDurationHours must not be populated
                if ((resourceTaskAssignment.getTaskAssignment()
                    .getActualStartDate() != null)
                        || (!(resourceTaskAssignment.getTaskAssignment()
                            .getPlannedDurationHours() == null || resourceTaskAssignment
                            .getTaskAssignment().getPlannedDurationHours()
                            .doubleValue() == 0)))
                {
                    String[] errorParams =
                    {
                        StringUtil.getShortClassName(resourceTaskAssignment
                            .getClass())
                    };

                    RPMException e = new RPMException(
                        com.ibm.rpm.wbs.constants.ErrorCodes.RTA_COPY_PROPOSED_TO_PLAN_INVALID,
                        errorParams, resourceTaskAssignment.getClass()
                            .getName());
                    addException(e, msgCtx);
                }
            }
        }

    }

    public final boolean canConvertWorkElement(WorkElement inputWorkElement,
            RPMObject outputWorkElement, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        try
        {
            return (ISecurityController.INSTANCE.canConvertContainer(msgCtx,
                inputWorkElement, outputWorkElement, rpmScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e), msgCtx);
            return false;
        }
    }

    public final boolean canCopyAssignmentProposedToPlan(
            ResourceTaskAssignment resourceTaskAssignment,
            ResourceTaskAssignmentScope scope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE
                .canCopyAssignmentProposedToPlan(messageContext,
                    resourceTaskAssignment, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

    public final boolean canCopyProposedToPlan(WorkElement workElement,
            WorkElementScope scope, MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canCopyProposedToPlan(
                messageContext, workElement, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

    public final boolean canAcceptRejectTaskAssignmentChanges(
            TaskAssignment taskAssignment, TaskAssignmentScope scope,
            boolean accept, boolean acceptEETC, boolean acceptForecastDates,
            MessageContext msgCtx)
    {
        try
        {
            if (accept)
            {
                return (ISecurityController.INSTANCE
                    .canAcceptTaskAssignmentChanges(msgCtx, taskAssignment,
                        scope, acceptEETC, acceptForecastDates).type != SecurityValidationResult.FALSE);
            }
            else
            {
                return (ISecurityController.INSTANCE
                    .canRejectTaskAssignmentChanges(msgCtx, taskAssignment,
                        scope, acceptEETC, acceptForecastDates).type != SecurityValidationResult.FALSE);
            }
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e), msgCtx);
            return false;
        }
    }

    public void validateConvertWorkElement(WorkElement inputWorkElement,
            RPMObject outputWorkElement, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // the inputWorkElement must be previously saved
        GenericValidator.validateMandatoryID("inputWorkElement",
            inputWorkElement, msgCtx);

        // make sure the scope is of the correct type
        ScopeCheckpoint.getInstance().isValid(inputWorkElement, rpmScope,
            msgCtx);

        boolean validOutputObject = false;

        if (outputWorkElement instanceof WorkElement)
        {
            if (inputWorkElement instanceof AggregateNode)
            {
                if (inputWorkElement instanceof GenericProject)
                {
                    // GenericProject can be converted to GenericProject only
                    if (outputWorkElement instanceof GenericProject)
                    {
                        validOutputObject = true;
                    }
                }
                else
                {
                    // AggregateNode can be converted to AggregateNode excluding GenericProject
                    if (outputWorkElement instanceof WorkProduct
                            || outputWorkElement instanceof SummaryTask
                            || outputWorkElement instanceof Deliverable)
                    {
                        validOutputObject = true;
                    }
                    // AggregateNode can also be converted to SimpleNode if it has no children
                    else if (outputWorkElement instanceof SimpleNode)
                    {
                        validOutputObject = true;

                        if (((AggregateNode) inputWorkElement).getChildren() != null)
                        {
                            // invalid output type when inputWorkElement has children
                            String[] errorParams =
                            {
                                    StringUtil
                                        .getShortClassName(inputWorkElement
                                            .getClass().getName()),
                                    StringUtil
                                        .getShortClassName(outputWorkElement
                                            .getClass().getName()),
                            };

                            RPMException e = new RPMException(
                                ErrorCodes.CONVERSION_FROM_TO_TYPE_INVALID_WITH_CHILDREN,
                                errorParams, inputWorkElement.getClass()
                                    .getName());
                            addException(e, msgCtx);
                        }
                    }
                }
            }
            else if (inputWorkElement instanceof SimpleNode)
            {
                // SimpleNode can be converted to SimpleNode and to 
                // AggregateNode excluding GenericProject
                if (outputWorkElement instanceof WorkProduct
                        || outputWorkElement instanceof SummaryTask
                        || outputWorkElement instanceof Deliverable
                        || outputWorkElement instanceof SimpleNode)
                {
                    validOutputObject = true;
                }
            }
        }

        if (!validOutputObject)
        {
            GenericValidator.validateConversionType(inputWorkElement,
                outputWorkElement, msgCtx);
        }

    }

    public boolean validateAcceptRejectArguments(TaskAssignment taskAssignment,
            boolean isAccept, boolean isEETC, boolean isForecastDates,
            MessageContext messageContext)
    {
        // validate that taskAssignment is not null
        if (taskAssignment == null)
        {
            StringBuffer errorParam = new StringBuffer();
            if (isAccept)
            {
                errorParam
                    .append(ValidationConstants.ACCEPT_TASK_ASSIGNMENT_METHOD);
            }
            else
            {
                errorParam
                    .append(ValidationConstants.REJECT_TASK_ASSIGNMENT_METHOD);
            }
            errorParam.append("(): ");
            errorParam.append(StringUtil
                .getShortClassName(TaskAssignment.class));

            RPMException e = new RPMException(ErrorCodes.MANDATORY,
                new String[]
                {
                    errorParam.toString()
                }, TaskAssignment.class.getName(), null, null);
            addException(e, messageContext);
        }

        // validate that either eetc or forecast is true
        boolean valid = (isEETC || isForecastDates);
        if (!valid)
        {
            String[] errorParams = new String[4];
            if (isAccept)
            {
                errorParams[0] = ValidationConstants.ACCEPT_TASK_ASSIGNMENT_METHOD;
                errorParams[1] = ValidationConstants.ACCEPT_TASK_ASSIGNMENT;
                errorParams[2] = ValidationConstants.ACCEPT_EETC_ARG;
                errorParams[3] = ValidationConstants.ACCEPT_FORECAST_DATES_ARG;
            }
            else
            {
                errorParams[0] = ValidationConstants.REJECT_TASK_ASSIGNMENT_METHOD;
                errorParams[1] = ValidationConstants.REJECT_TASK_ASSIGNMENT;
                errorParams[2] = ValidationConstants.REJECT_EETC_ARG;
                errorParams[3] = ValidationConstants.REJECT_FORECAST_DATES_ARG;
            }

            RPMException e = new RPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.ERROR_TASK_ASSIGNMENT_APPROVE_ARGS,
                errorParams, TaskAssignment.class.getName(), null, null);
            addException(e, messageContext);
        }

        return taskAssignment != null && valid;
    }

    public boolean canApproveRejectForecastDate(TaskAssignment taskAssignment,
            boolean isAccept, MessageContext messageContext)
    {
        boolean datesChanged = (taskAssignment.getForecastFinishDate() != null || taskAssignment
            .getForecastStartDate() != null);

        if (!datesChanged)
        {
            String method = null;
            if (isAccept)
                method = ValidationConstants.ACCEPT_TASK_ASSIGNMENT_METHOD;
            else
                method = ValidationConstants.REJECT_TASK_ASSIGNMENT_METHOD;

            String[] errorParams =
            {
                    method, ValidationConstants.FORECAST_DATE_ARG
            };
            RPMException e = new RPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_APPROVE_REJECT_TASK_ASSIGNMENT,
                errorParams, taskAssignment.getClass().getName(), null,
                taskAssignment.getID());
            addException(e, messageContext);
        }

        return datesChanged;
    }

    public boolean canApproveRejectEETC(TaskAssignment taskAssignment,
            boolean isAccept, MessageContext messageContext)
    {
        boolean eetcChanged = true;

        if (taskAssignment.getEETCHours() != null
                && taskAssignment.getExpectedWork() != null
                && taskAssignment.getActualWork() != null)
        {
            double eetc = taskAssignment.getEETCHours().doubleValue();
            double expectedWork = taskAssignment.getExpectedWork()
                .doubleValue();
            double actualWork = taskAssignment.getActualWork().doubleValue();

            eetcChanged = (expectedWork != eetc + actualWork);

            if (!eetcChanged)
            {
                String method = null;
                if (isAccept)
                    method = ValidationConstants.ACCEPT_TASK_ASSIGNMENT_METHOD;
                else
                    method = ValidationConstants.REJECT_TASK_ASSIGNMENT_METHOD;

                String[] errorParams =
                {
                        method, ValidationConstants.EETC_HOURS_FIELD
                };
                RPMException e = new RPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_APPROVE_REJECT_TASK_ASSIGNMENT,
                    errorParams, taskAssignment.getClass().getName(), null,
                    taskAssignment.getID());
                addException(e, messageContext);
            }
        }

        return eetcChanged;
    }

    public void validateCreateProjectWithTemplate(GenericProject inputProject,
            MessageContext msgCtx)
    {

        GenericValidator.validateMustBeNull(inputProject,
            ValidationConstants.ID_FIELD, inputProject.getID(),
            "creating a project with a template", msgCtx);

        GenericValidator.validateMandatoryID(inputProject,
            ValidationConstants.TEMPLATE_FIELD, msgCtx.getTemplate(), msgCtx);
    }

    public void validateTransferUnderScope(AbstractAggregateScope scopeElement,
            WorkElement workElement, MessageContext msgCtx) throws RPMException
    {
        // the scopeElement and the workElement must be previously saved
        GenericValidator.validateMandatoryID("ScopeElement", scopeElement,
            msgCtx);
        GenericValidator
            .validateMandatoryID("WorkElement", workElement, msgCtx);

        if (workElement instanceof GenericProject)
        {
            // error: can't transfer a GenericProject 
            String[] errorParams =
            {
                StringUtil.getShortClassName(workElement.getClass())
            };
            RPMException e = new RPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_TRANSFER_UNDER_SCOPE,
                errorParams, workElement.getClass().getName());
            addException(e, msgCtx);
        }
        else if ((workElement instanceof Deliverable)
                && (scopeElement instanceof ScopeDeliverable || scopeElement instanceof ScopeSummaryTask))
        {
            // error: can't transfer a Deliverable to a ScopeDeliverable or ScopeSummaryTask
            String[] errorParams =
            {
                    StringUtil.getShortClassName(workElement.getClass()),
                    StringUtil.getShortClassName(scopeElement.getClass())
            };
            RPMException e = new RPMException(
                com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_TRANSFER_UNDER_SPECIFIC_SCOPE_ELEMENT,
                errorParams, workElement.getClass().getName());
            addException(e, msgCtx);
        }
        else if (scopeElement != null && scopeElement.getProject() != null
                && scopeElement.getProject().getID() != null)
        {
            String projectId = WbsUtil.getWorkElementProjectID(workElement,
                msgCtx);

            if (!(projectId.equals(scopeElement.getProject().getID())))
            {
                // error: can't cross transfer a WorkElement under Scope
                RPMException e = new RPMException(
                    com.ibm.rpm.wbs.constants.ErrorCodes.CANNOT_CROSS_TRASNFER_UNDER_SCOPE,
                    workElement.getClass().getName());
                addException(e, msgCtx);
            }
        }

    }

    private boolean isGenericProject(RPMObject rpmObject)
    {
        return rpmObject instanceof GenericProject;
    }

    /**
     * @param inputObject
     * @param msgCtx
     * @return
     */
    public boolean isTemplateAndInputAvailable(RPMObject inputObject,
            MessageContext msgCtx)
    {
        return GenericValidator.validateNotNull(inputObject,
            ValidationConstants.TEMPLATE_INPUT_FIELD, inputObject, msgCtx)
                && GenericValidator.validateNotNull(inputObject,
                    ValidationConstants.TEMPLATE_FIELD, msgCtx.getTemplate(),
                    msgCtx);
    }

    public boolean isProjectAvailable(RPMObject inputObject,
            MessageContext msgCtx)
    {
        return GenericValidator.validateNotNull(inputObject,
            ValidationConstants.PROJECTS_FIELD, inputObject, msgCtx);
    }

    public void validateEVWeekDelays(MessageContext msgCtx, int evValue)
    {
        if (evValue < ValidationConstants.EV_DELAY_DAYS_MIN
                || evValue > ValidationConstants.EV_DELAY_DAYS_MAX)
        {
            String[] errorParams =
            {
                    "EV Delay (weeks)",
                    String.valueOf(ValidationConstants.EV_DELAY_DAYS_MIN),
                    String.valueOf(ValidationConstants.EV_DELAY_DAYS_MAX),
                    String.valueOf(evValue)
            };
            RPMException e = new RPMException(ErrorCodes.OUT_OF_RANGE,
                errorParams);
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
        }

    }

    public void validateProjectInput(RPMObject inputObject,
            Class[] expectedClassList, MessageContext msgCtx)
    {
        // ID is Manadatory for input project
        GenericValidator.validateMandatoryID(inputObject,
            ValidationConstants.PROJECTS_FIELD, inputObject, msgCtx);

        GenericValidator.validateType(inputObject, expectedClassList,
            ValidationConstants.PROJECTS_FIELD, inputObject, msgCtx);
    }

    /**
     * @param inputObject
     * @param msgCtx
     * @param expectedClassList
     */
    public void validateTemplateAndInput(RPMObject inputObject,
            MessageContext msgCtx)
    {
        Class[] expectedClassList =
        {
                Deliverable.class, Milestone.class, SummaryTask.class,
                Task.class, WorkProduct.class
        };

        //-----validations for all cases

        //ID is Manadatory for template
        GenericValidator.validateMandatoryID(inputObject,
            ValidationConstants.TEMPLATE_FIELD, msgCtx.getTemplate(), msgCtx);

        //ID must be null for inputObject
        GenericValidator.validateMustBeNull(inputObject,
            ValidationConstants.ID_FIELD, inputObject.getID(),
            "creating a project with template", msgCtx);

        //Parent should be present except for GP
        if (!isGenericProject(inputObject))
        {
            GenericValidator.validateMandatory(inputObject,
                ValidationConstants.PARENT, inputObject.getParent(), msgCtx);
        }

        //case 2: Create WorkElement From Template
        if (!(inputObject instanceof GenericProject
                && msgCtx.getTemplate() != null && msgCtx.getTemplate() instanceof GenericProject))
        {

            //Validate that input is same instance as template
            GenericValidator.validateExactType(inputObject, inputObject
                .getClass(), ValidationConstants.TEMPLATE_INPUT_FIELD, msgCtx
                .getTemplate(), msgCtx);

            GenericValidator.validateType(inputObject, expectedClassList,
                ValidationConstants.TEMPLATE_INPUT_FIELD, inputObject, msgCtx);

            GenericValidator.validateType(msgCtx.getTemplate(),
                expectedClassList, ValidationConstants.TEMPLATE_FIELD, msgCtx
                    .getTemplate(), msgCtx);
        }
    }

    public final boolean canRollupProjectEV(RPMObject workElementObject,
            RPMObjectScope scope, MessageContext msgCtx)
    {
        try
        {
            return (ISecurityController.INSTANCE.canRollupProjectEV(msgCtx,
                workElementObject, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e), msgCtx);
            return false;
        }
    }

    public final boolean canSaveProjectBaseline(RPMObject workElementObject,
            RPMObjectScope scope, MessageContext msgCtx)
    {
        try
        {
            return (ISecurityController.INSTANCE.canSaveProjectBaseline(msgCtx,
                workElementObject, scope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e), msgCtx);
            return false;
        }
    }

    public final boolean canTransferUnderScope(
            AbstractAggregateScope scopeElement, WorkElement workElement,
            MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canTransferUnderScope(
                messageContext, scopeElement, workElement).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }
    
    public final boolean canCreateFromTemplate(RPMObject rpmObject,
            RPMObject template, RPMObjectScope rpmScope,
            MessageContext messageContext)
    {
        try
        {
            return (ISecurityController.INSTANCE.canCreateObjectFromTemplate(
                messageContext, rpmObject, template, rpmScope).type != SecurityValidationResult.FALSE);
        }
        catch (RPMException e)
        {
            addException(SecurityValidationResult.makeException(e),
                messageContext);
            return false;
        }
    }

}