/*
 * Created on Nov 15, 2005 @author mmatwiy Validation of fields common to all
 * work elements.
 */
package com.ibm.rpm.wbs.checkpoints;

import java.sql.Timestamp;
import java.util.Calendar;
import java.util.List;

import com.ibm.rpm.financial.util.FinancialUtil;
import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.ManagerUtil;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.types.SeverityLevel;
import com.ibm.rpm.framework.util.DateUtil;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.framework.util.SqlBuffer;
import com.ibm.rpm.framework.util.StringUtil;
import com.ibm.rpm.wbs.constants.ErrorCodes;
import com.ibm.rpm.wbs.constants.ValidationConstants;
import com.ibm.rpm.wbs.containers.SimpleNode;
import com.ibm.rpm.wbs.containers.Task;
import com.ibm.rpm.wbs.containers.WbsScheduleDate;
import com.ibm.rpm.wbs.containers.WorkElement;
import com.ibm.rpm.wbs.managers.WbsScheduleDateManager;
import com.ibm.rpm.wbs.managers.WbsUtil;
import com.ibm.rpm.wbs.scope.WorkElementScope;
import com.ibm.rpm.wbs.types.ProjectPublishedType;


public class WorkElementCheckpoint extends AbstractCheckpoint
{
    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx, boolean parentRequired, List validParentTypes)
    {
        super
            .isValidParent(rpmObject, msgCtx, parentRequired, validParentTypes);

        WorkElement workElement = (WorkElement) rpmObject;
        WorkElementScope scope = (WorkElementScope) rpmScope;

        // Check that the mandatory name exists
        GenericValidator.validateMandatory(workElement,
            ValidationConstants.NAME_FIELD, workElement.getName(), msgCtx);

        if (workElement.testLastModifiedDateModified())
        {
            GenericValidator.validateReadOnly(workElement,
                ValidationConstants.LAST_MODIFIED_DATE_FIELD, msgCtx);
        }
        
        //EarnedValue is ReadOnly
        if (scope != null
                && scope.isEarnedValues()
                && (workElement.testEarnedValuesModified()
                || FinancialUtil.isEarnedValuesModified(workElement
                    .getEarnedValues())))
        {
            GenericValidator.validateReadOnlyError(workElement,
                ValidationConstants.EARNED_VALUE_FIELD, msgCtx);
        }
        
        if (workElement.testNameModified())
        {
            GenericValidator.validateMinLength(workElement,
                ValidationConstants.NAME_FIELD, workElement.getName(),
                ValidationConstants.WORK_ELEMENT_NAME_MIN, msgCtx);
        }

        if (workElement.testPriorityModified())
        {
            // Check that the priority is in the correct range of values
            GenericValidator.validateRange(workElement,
                ValidationConstants.PRIORITY_FIELD, workElement.getPriority(),
                ValidationConstants.PRIORITY_MIN, ValidationConstants.PRIORITY_MAX,
                msgCtx);
        }

        if (workElement.testEstimatedStartDateModified()
                || workElement.testEstimatedFinishDateModified())
        {
            // Check that the estimated start date is prior to the estimated
            // finish date
            GenericValidator.validateDates(workElement,
                ValidationConstants.ESTIMATED_START_DATE_FIELD, workElement
                    .getEstimatedStartDate(),
                ValidationConstants.ESTIMATED_FINISH_DATE_FIELD, workElement
                    .getEstimatedFinishDate(), msgCtx);
        }

        if (workElement.testExternalIDModified())
        {
            // Check that the external id doesn't exceed its max length
            GenericValidator.validateMaxLength(workElement,
                ValidationConstants.EXTERNAL_ID_FIELD, workElement
                    .getExternalID(),
                ValidationConstants.EXTERNAL_ID_CHARACTER_LIMIT, msgCtx);
        }
        
        // populated by the stored proc
        if (workElement.testReferenceIDModified())
        {
            GenericValidator.validateReadOnly(workElement,
                ValidationConstants.REFERENCE_ID_FIELD, msgCtx);
        }
        
        if (scope != null)
        {
            if (scope.isTimeCodes() && workElement.testTimeCode1Modified())
            {
                GenericValidator.validateTimeCode1(workElement, workElement
                    .getTimeCode1(), ValidationConstants.TIME_CODE1_FIELD,
                    msgCtx);
            }

            if (scope.isTimeCodes() && workElement.testTimeCode2Modified())
            {
                GenericValidator.validateTimeCode2(workElement, workElement
                    .getTimeCode2(), ValidationConstants.TIME_CODE2_FIELD,
                    msgCtx);
            }

            // On update, the assetCode, estimatedStartDate, priority,
            // fields cannot be cleared
            if (scope.isAssetCode())
            {
                GenericValidator.validateDefaultValueUpdate(workElement,
                    ValidationConstants.ASSET_CODE_FIELD, workElement
                        .getAssetCode(), msgCtx);
            }
        }

        GenericValidator.validateDefaultValueUpdate(workElement,
            ValidationConstants.ESTIMATED_START_DATE_FIELD, workElement
                .getEstimatedStartDate(), msgCtx);

        GenericValidator.validateDefaultValueUpdate(workElement,
            ValidationConstants.PRIORITY_FIELD, workElement.getPriority(), msgCtx);
    }

    protected boolean validateReadOnlyWorkAssigned(WorkElement workElement,
            String fieldName, MessageContext msgCtx)
    {
        boolean valid = true;

        if (WbsUtil.isWorkAssigned(workElement))
        {
            String[] errorParams =
            {
                    StringUtil.getShortClassName(workElement.getClass()) + " "
                            + fieldName, fieldName,
                    StringUtil.getShortClassName(workElement.getClass())
            };
            RPMException e = new RPMException(
                ErrorCodes.READ_ONLY_ONCE_WORK_ASSIGNED, errorParams,
                workElement.getClass().getName(), fieldName, workElement
                    .getID());
            e.setSeverity(SeverityLevel.Warning);
            addException(e, msgCtx);
            valid = false;
        }
        return valid;
    }

    public void validateWorkElementTemplateCreation(WorkElement inputProject,
            MessageContext msgCtx)
    {    	
	        //validate template is in fact a template
	        if (((WorkElement) msgCtx.getTemplate()).getPublished() == null
				|| !(ProjectPublishedType.ToTemplate.getValue()
						.equals(((WorkElement) msgCtx.getTemplate())
								.getPublished().getValue()))) {
			String[] errorParams = { StringUtil.getShortClassName(inputProject
					.getClass())
					+ " " + ValidationConstants.TEMPLATE_FIELD

			};
			//invalid project template
			RPMException e = new RPMException(
					com.ibm.rpm.wbs.constants.ErrorCodes.INVALID_OBJECT_TEMPLATE_TYPE,
					errorParams, inputProject.getClass().getName(),
					ValidationConstants.TEMPLATE_FIELD, inputProject.getID());
			addException(e, msgCtx);
		} 
	        else {
			validateWorkElementReadOnlyFields(inputProject, msgCtx);

		}
    }

    
    /**
	 * @param inputProject
	 * @param msgCtx
	 */
	protected void validateWorkElementReadOnlyFields(WorkElement inputProject, MessageContext msgCtx) {

		//SimpleNodes specific fields validation
		if(inputProject instanceof SimpleNode)
		{
			SimpleNode snode = (SimpleNode)inputProject;
			
			//Field validation: ConstraintDate
			if (snode.testConstraintDateModified())
			{
			    GenericValidator.validateReadOnly(snode,
			        ValidationConstants.CONSTRAINT_DATE_FIELD, msgCtx);
			}
			
			//Field validation: ConstraintType
			if (snode.testConstraintTypeModified())
			{
			    GenericValidator.validateReadOnly(snode,
			        ValidationConstants.CONSTRAINT_TYPE_FIELD, msgCtx);
			}
		}
		
		
		//Task specific fields validation
		if(inputProject instanceof Task)
		{
			Task task = (Task)inputProject;
			
			//Field validation: PercentageDurationComplete
			if (task.testPercentageDurationCompletedModified())
			{
			    GenericValidator.validateReadOnly(task,
			        ValidationConstants.PERCENT_DURATION_COMPLETED_FIELD, msgCtx);
			}
		}
		
		
		//-----------Common WorkElement fields validation
		
		//Field validation: AssetCode
		if (inputProject.testAssetCodeModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.ASSET_CODE_FIELD, msgCtx);
		}

		//    	Field validation: Asset
		if (inputProject.testAssetModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.ASSET_FIELD, msgCtx);
		}

		//    	Field validation: BaselineDate
		if (inputProject.testBaselineDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.BASELINE_DATE_FIELD, msgCtx);
		}

		//    	Field validation: ContainingProject
		if (inputProject.testContainingProjectModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.CONTAINING_PROJECT_FIELD, msgCtx);
		}

		//    	Field validation: DocumentFolder
		if (inputProject.testDocumentFolderModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.DOCUMENT_FOLDER_FIELD, msgCtx);
		}

		//    	Field validation: EstimatedFinishDate
		if (inputProject.testEstimatedFinishDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.ESTIMATED_FINISH_DATE_FIELD, msgCtx);
		}

		//    	Field validation: EstimatedStartDate
		if (inputProject.testEstimatedStartDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.ESTIMATED_START_DATE_FIELD, msgCtx);
		}

		//    	Field validation: ExpectedDate
		if (inputProject.testExpectedDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.EXPECTED_DATE_FIELD, msgCtx);
		}

		//    	Field validation: ExternalID
		if (inputProject.testExternalIDModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.EXTERNAL_ID_FIELD, msgCtx);
		}

		//    	Field validation: InitialBaselineDate
		if (inputProject.testInitialBaselineDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.INITIAL_BASELINE_DATE_FIELD, msgCtx);
		}

		//    	Field validation: LastModifiedDate
		if (inputProject.testLastModifiedDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.LAST_MODIFIED_DATE_FIELD, msgCtx);
		}

		//    	Field validation: MinutesPerDay
		if (inputProject.testMinutesPerDayModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.MINUTES_PER_DAY_FIELD, msgCtx);
		}

		//    	Field validation: Name
		if (inputProject.testNameModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.NAME_FIELD, msgCtx);
		}

		//    	Field validation: PreviousBaselineDate
		if (inputProject.testPreviousBaselineDateModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.PREVIOUS_BASELINE_DATE_FIELD, msgCtx);
		}

		//    	Field validation: Published
		if (inputProject.testPublishedModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.PUBLISHED_FIELD, msgCtx);
		}

		//    	Field validation: Priority
		if (inputProject.testPriorityModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.PRIORITY_FIELD, msgCtx);
		}

		//    	Field validation: ReferenceNumber
		if (inputProject.testReferenceNumberModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.REFERENCE_NUMBER_FIELD, msgCtx);
		}

		//    	Field validation: ScopeElement
		if (inputProject.testScopeElementModified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.SCOPE_ELEMENT_FIELD, msgCtx);
		}

		//    	Field validation: TimeCode1
		if (inputProject.testTimeCode1Modified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.TIME_CODE1_FIELD, msgCtx);
		}

		//    	Field validation: TimeCode2
		if (inputProject.testTimeCode2Modified())
		{
		    GenericValidator.validateReadOnly(inputProject,
		        ValidationConstants.TIME_CODE2_FIELD, msgCtx);
		}
	}

    //This method is used to load the following wbsScheduleDates
    //(Actual, Forecast, Plan, Proposed)
    protected Object[] loadWbsScheduleDatesFromDB(WorkElement workElement, MessageContext msgCtx)
    {
        Object[] loadedValues = new Object[4];
        
        if (workElement.getID() != null)
        {
            String[] columns = new String[]{
                    WbsScheduleDateManager.NAME_ACTUAL_START,
                    WbsScheduleDateManager.NAME_ACTUAL_FINISH,
                    WbsScheduleDateManager.NAME_ACTUAL_DURATION,
                    WbsScheduleDateManager.NAME_WORK_TODATE,
                    WbsScheduleDateManager.NAME_FORECAST_START,
                    WbsScheduleDateManager.NAME_FORECAST_FINISH,
                    WbsScheduleDateManager.NAME_FORECAST_DURATION,
                    WbsScheduleDateManager.NAME_FORECAST_EFFORT,
                    WbsScheduleDateManager.NAME_SCHED_START,
                    WbsScheduleDateManager.NAME_SCHED_FINISH,
                    WbsScheduleDateManager.NAME_SCHED_DURATION,
                    WbsScheduleDateManager.NAME_SCHED_EFFORT,
                    WbsScheduleDateManager.NAME_PLANNED_START,
                    WbsScheduleDateManager.NAME_PLANNED_FINISH,
                    WbsScheduleDateManager.NAME_PLANNED_DURATION,
                    WbsScheduleDateManager.NAME_PLANNED_EFFORT,
                    WbsScheduleDateManager.NAME_MINUTE_PER_DAY
            };
            String tableName = WbsScheduleDateManager.TABLE_NAME;

            try
            {
                StringBuffer columnList = constructSelectColumnList(columns); 
                
                SqlBuffer sql = new SqlBuffer();
                sql.appendSelect();
                sql.append(columnList);
                sql.appendFrom(tableName);
                sql.appendWhere();
                sql.appendEqual(WbsScheduleDateManager.NAME_ELEMENT_ID, workElement.getID());
                
                Class[] columnTypes = new Class[]
                {
                        Timestamp.class, Timestamp.class, Integer.class, Integer.class, 
                        Timestamp.class, Timestamp.class, Integer.class, Integer.class,
                        Timestamp.class, Timestamp.class, Integer.class, Integer.class,
                        Timestamp.class, Timestamp.class, Integer.class, Integer.class,
                        Integer.class
                };
                
                List dbRows = ManagerUtil.select(columnTypes, null, null, sql, null, msgCtx);

                List dbValues = (List)dbRows.get(0); 

                String id = workElement.getID();
                
                WbsScheduleDate actualDate = constructWbsScheduleDate(0,id,dbValues);
                WbsScheduleDate forecastDate = constructWbsScheduleDate(4,id,dbValues);
                WbsScheduleDate planDate = constructWbsScheduleDate(8,id,dbValues);
                WbsScheduleDate proposedDate = constructWbsScheduleDate(12,id,dbValues);
                
                loadedValues[0] = actualDate;
                loadedValues[1] = forecastDate;
                loadedValues[2] = planDate;
                loadedValues[3] = proposedDate;
            }
            catch (Exception e)
            {
                RPMException rpmException = new RPMException(e);
                addException(rpmException, msgCtx);
            }
        }
        
        return loadedValues;
    }
    
    private WbsScheduleDate constructWbsScheduleDate(int startIndex, String id,
            List dbValues) throws Exception
    {
        WbsScheduleDate wbsScheduleDate = new WbsScheduleDate();
        wbsScheduleDate.setID(id);

        Calendar startDate = DateUtil.convertObjectToCalendar(dbValues
            .get(startIndex));
        Calendar finishDate = DateUtil.convertObjectToCalendar(dbValues
            .get(startIndex + 1));
        int duration = ((Integer) dbValues.get(startIndex + 2)).intValue();
        int effort = ((Integer) dbValues.get(startIndex + 3)).intValue();

        int minutesPerDay = ((Integer) dbValues.get(16)).intValue();

        // This logic is taken from WbsScheduleDateManager.decodeRow
        double days = (double) duration / minutesPerDay;
        int hours = (duration % minutesPerDay) / 60;
        double workEffort = (double) effort / 60.0;

        wbsScheduleDate.setStartDate(startDate);
        wbsScheduleDate.setFinishDate(finishDate);
        wbsScheduleDate.setDurationInMinutes(new Integer(duration));
        wbsScheduleDate.setDays(new Double(days));
        wbsScheduleDate.setHours(new Double(hours));
        wbsScheduleDate.setWorkHours(new Double(workEffort));

        return wbsScheduleDate;
    } 

    private StringBuffer constructSelectColumnList(String[] columns)
    {
        StringBuffer selectColumnList = new StringBuffer();
        for(int i=0;i<columns.length;i++)
        {
            selectColumnList.append(columns[i]);
            if(i+1<columns.length)
            {
                selectColumnList.append(",");                
            }
        }
        return selectColumnList;
    }
}