/*
 * Created on Oct 17, 2006 
 * @author mmatwiy 
 * Checkpoint validation of the WorkflowRoleMapping container.
 * 
 * Note that a WorkflowRoleMapping with no ID is a mapping that doesn't contain
 * a Resource, ContactGroup, or SecurityRole.
 * 
 * Once one or more of these three are added and the WorkflowRoleMapping is saved,
 * the container will have an ID (the ID of the associated WorkflowRole).
 * 
 */
package com.ibm.rpm.workflow.checkpoints;

import java.util.ArrayList;
import java.util.List;

import com.ibm.rpm.framework.AbstractCheckpoint;
import com.ibm.rpm.framework.MessageContext;
import com.ibm.rpm.framework.RPMException;
import com.ibm.rpm.framework.RPMManagerFactory;
import com.ibm.rpm.framework.RPMObject;
import com.ibm.rpm.framework.RPMObjectScope;
import com.ibm.rpm.framework.util.GenericValidator;
import com.ibm.rpm.wbs.checkpoints.MilestoneCheckpoint;
import com.ibm.rpm.wbs.containers.GenericProject;
import com.ibm.rpm.wbs.managers.WbsManager;
import com.ibm.rpm.workflow.constants.ErrorCodes;
import com.ibm.rpm.workflow.constants.ValidationConstants;
import com.ibm.rpm.workflow.containers.WorkflowRoleMapping;
import com.ibm.rpm.workflow.scope.WorkflowRoleMappingScope;

public class WorkflowRoleMappingCheckpoint extends AbstractCheckpoint
{
    private static final List MAPPING_PARENT_LIST = new ArrayList();

    protected static MilestoneCheckpoint instance = new MilestoneCheckpoint();

    static
    {
        MAPPING_PARENT_LIST.add(GenericProject.class);
    }

    public static MilestoneCheckpoint getInstance()
    {
        return instance;
    }

    public void validateSave(RPMObject rpmObject, RPMObjectScope rpmScope,
            MessageContext msgCtx)
    {
        // the parent and its ID must exist on insert
        super.isValidParent(rpmObject, msgCtx, true, true, MAPPING_PARENT_LIST);
        
        WorkflowRoleMapping mapping = (WorkflowRoleMapping) rpmObject;
        WorkflowRoleMappingScope scope = (WorkflowRoleMappingScope) rpmScope;
        
        if (mapping.getID() != null)
        {
            // parent is mandatory on update as well as insert
            GenericValidator.validateMandatoryID(mapping,
                ValidationConstants.PARENT_FIELD, mapping.getParent(),
                msgCtx);
        }

        GenericValidator.validateMandatoryID(mapping,
            ValidationConstants.WORKFLOW_ROLE_FIELD, mapping.getWorkflowRole(),
            msgCtx);

        // cannot modify the workflowRole on update
        if (mapping.getID() != null)
        {
            if (mapping.getWorkflowRole() != null
                    && mapping.getWorkflowRole().testNameModified())
            {
                GenericValidator.validateReadOnlyError(mapping,
                    ValidationConstants.WORKFLOW_ROLE_FIELD, msgCtx);
            }
        }
        else if (msgCtx.isSuccessful())
        {
            // The user should not be modifying the workflowRole on 
            // insert either but the modified flags will be set because the WorkflowRoleMapping
            // ID is null. So, check that the mapping is valid.
            roleMappingIsValid(mapping, msgCtx);
        }

        if (scope != null)
        {
            if (mapping.testSecurityRoleModified()
                    && scope.getSecurityRole() != null)
            {
                GenericValidator.validateMandatoryID(mapping,
                    ValidationConstants.SECURITY_ROLE_FIELD, mapping
                        .getSecurityRole(), msgCtx);
            }
            
            if (mapping.testResourceModified()
                    && scope.getResource() != null)
            {
                GenericValidator.validateMandatoryID(mapping,
                    ValidationConstants.RESOURCE_FIELD, mapping
                        .getResource(), msgCtx);
            }
            
            if (mapping.testContactGroupModified()
                    && scope.getContactGroup() != null)
            {
                GenericValidator.validateMandatoryID(mapping,
                    ValidationConstants.CONTACT_GROUP_FIELD, mapping
                        .getContactGroup(), msgCtx);
            }
        }
    }

    /*
     * Get a list of existing roles that can be mapped to the GenericProject
     * and make sure this mapping contains one of them
     */
    private void roleMappingIsValid(WorkflowRoleMapping mapping,
            MessageContext msgCtx)
    {
        WorkflowRoleMapping[] mappings = null;

        try
        {
            WbsManager manager = (WbsManager) RPMManagerFactory.getInstance()
                .getRPMObjectManager(GenericProject.class.getName());

            // get a list of existing roles that can be mapped to the GenericProject
            mappings = manager.SP_S_PRJ_WKF_ROLE((GenericProject) mapping
                .getParent(), null, msgCtx);
        }
        catch (Exception e)
        {
            msgCtx.addExceptionNoThrow(e);
        }

        boolean found = false;

        if (mappings != null)
        {
            // make sure the mapping being updated is in the list
            for (int i = 0; i < mappings.length; i++)
            {
                WorkflowRoleMapping loadedMapping = mappings[i];

                if (loadedMapping.getWorkflowRole().getName()
                            .equals(mapping.getWorkflowRole().getName()))
                {
                    found = true;
                    break;
                }

            }
        }

        if (!found)
        {
            RPMException e = new RPMException(
                ErrorCodes.INVALID_WORKFLOW_ROLE_MAPPING, mapping.getClass()
                    .getName());
            addException(e, msgCtx);
        }
    }
}