package com.ibm.rpm.xpathparser;

import java.lang.reflect.Field;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xpath.compiler.OpCodes;

/**
 * reflexion loaded operation codes from Xalan. Used to be compatible with multiple
 * versions of the JDK which don't use the same constants.
 */
public class ReflectOpCodes
{
    
    private static Class OP_CODE_CLASS = OpCodes.class;

    /**
     * <meta name="usage" content="advanced"/> [ENDOP] Some operators may like
     * to have a terminator.
     */
    public static final int ENDOP = loadConstant("ENDOP");

    /**
     * [EMPTY] Empty slot to indicate NULL.
     */
    public static final int EMPTY = loadConstant("EMPTY");

    /**
     * <meta name="usage" content="advanced"/> [ELEMWILDCARD] Means ELEMWILDCARD
     * ("*"), used instead of string index in some places.
     */
    public static final int ELEMWILDCARD = loadConstant("ELEMWILDCARD");

    /**
     * <meta name="usage" content="advanced"/> [OP_XPATH] [length] {expression}
     * 
     * returns: XNodeSet XNumber XString XBoolean XRTree XObject
     */
    public static final int OP_XPATH = loadConstant("OP_XPATH");

    /**
     * <meta name="usage" content="advanced"/> [OP_OR] [length] {boolean
     * expression} {boolean expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_OR = loadConstant("OP_OR");

    /**
     * <meta name="usage" content="advanced"/> [OP_AND] [length] {boolean
     * expression} {boolean expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_AND = loadConstant("OP_AND");

    /**
     * <meta name="usage" content="advanced"/> [OP_NOTEQUALS] [length]
     * {expression} {expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_NOTEQUALS = loadConstant("OP_NOTEQUALS");

    /**
     * <meta name="usage" content="advanced"/> [OP_EQUALS] [length] {expression}
     * {expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_EQUALS  = loadConstant("OP_EQUALS");

    /**
     * <meta name="usage" content="advanced"/> [OP_LTE] (less-than-or-equals)
     * [length] {number expression} {number expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_LTE  = loadConstant("OP_LTE");

    /**
     * <meta name="usage" content="advanced"/> [OP_LT] (less-than) [length]
     * {number expression} {number expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_LT  = loadConstant("OP_LT");

    /**
     * <meta name="usage" content="advanced"/> [OP_GTE] (greater-than-or-equals)
     * [length] {number expression} {number expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_GTE = loadConstant("OP_GTE");

    /**
     * <meta name="usage" content="advanced"/> [OP_GT] (greater-than) [length]
     * {number expression} {number expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_GT = loadConstant("OP_GT");

    /**
     * <meta name="usage" content="advanced"/> [OP_PLUS] [length] {number
     * expression} {number expression}
     * 
     * returns: XNumber
     */
    public static final int OP_PLUS = loadConstant("OP_PLUS");

    /**
     * <meta name="usage" content="advanced"/> [OP_MINUS] [length] {number
     * expression} {number expression}
     * 
     * returns: XNumber
     */
    public static final int OP_MINUS = loadConstant("OP_MINUS");

    /**
     * <meta name="usage" content="advanced"/> [OP_MULT] [length] {number
     * expression} {number expression}
     * 
     * returns: XNumber
     */
    public static final int OP_MULT = loadConstant("OP_MULT");

    /**
     * <meta name="usage" content="advanced"/> [OP_DIV] [length] {number
     * expression} {number expression}
     * 
     * returns: XNumber
     */
    public static final int OP_DIV = loadConstant("OP_DIV");

    /**
     * <meta name="usage" content="advanced"/> [OP_MOD] [length] {number
     * expression} {number expression}
     * 
     * returns: XNumber
     */
    public static final int OP_MOD  = loadConstant("OP_MOD");

    /**
     * <meta name="usage" content="advanced"/> [OP_QUO] [length] {number
     * expression} {number expression}
     * 
     * returns: XNumber
     */
    public static final int OP_QUO = loadConstant("OP_QUO");

    /**
     * <meta name="usage" content="advanced"/> [OP_NEG] [length] {number
     * expression}
     * 
     * returns: XNumber
     */
    public static final int OP_NEG = loadConstant("OP_NEG");

    /**
     * <meta name="usage" content="advanced"/> [OP_STRING] (cast operation)
     * [length] {expression}
     * 
     * returns: XString
     */
    public static final int OP_STRING = loadConstant("OP_STRING");

    /**
     * <meta name="usage" content="advanced"/> [OP_BOOL] (cast operation)
     * [length] {expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_BOOL = loadConstant("OP_BOOL");

    /**
     * <meta name="usage" content="advanced"/> [OP_NUMBER] (cast operation)
     * [length] {expression}
     * 
     * returns: XBoolean
     */
    public static final int OP_NUMBER = loadConstant("OP_NUMBER");

    /**
     * <meta name="usage" content="advanced"/> [OP_UNION] [length] {PathExpr}+
     * 
     * returns: XNodeSet
     */
    public static final int OP_UNION = loadConstant("OP_UNION");

    /**
     * <meta name="usage" content="advanced"/> [OP_LITERAL] [3] [index to token]
     * 
     * returns: XString
     */
    public static final int OP_LITERAL = loadConstant("OP_LITERAL");

    /**
     * The low opcode for nodesets, needed by getFirstPredicateOpPos and
     * getNextStepPos.
     */
    // Field not supported by Sun JDK 1.4
    //static final int FIRST_NODESET_OP = loadConstant("FIRST_NODESET_OP");

    /**
     * <meta name="usage" content="advanced"/> [OP_VARIABLE] [4] [index to
     * namespace token, or EMPTY] [index to function name token]
     * 
     * returns: XString
     */
    public static final int OP_VARIABLE = loadConstant("OP_VARIABLE");

    /**
     * <meta name="usage" content="advanced"/> [OP_GROUP] [length] {expression}
     * 
     * returns: XNodeSet XNumber XString XBoolean XRTree XObject
     */
    public static final int OP_GROUP = loadConstant("OP_GROUP");

    /**
     * <meta name="usage" content="advanced"/> [OP_EXTFUNCTION] (Extension
     * function.) [length] [index to namespace token] [index to function name
     * token] {OP_ARGUMENT}
     * 
     * returns: XNodeSet XNumber XString XBoolean XRTree XObject
     */
    public static final int OP_EXTFUNCTION = loadConstant("OP_EXTFUNCTION");

    /**
     * <meta name="usage" content="advanced"/> [OP_FUNCTION] [length]
     * [FUNC_name] {OP_ARGUMENT} [ENDOP]
     * 
     * returns: XNodeSet XNumber XString XBoolean XRTree XObject
     */
    public static final int OP_FUNCTION = loadConstant("OP_FUNCTION");

    /** The last opcode for stuff that can be a nodeset. */
    // Field not supported by Sun JDK 1.4
    // static final int LAST_NODESET_OP = loadConstant("LAST_NODESET_OP");

    /**
     * <meta name="usage" content="advanced"/> [OP_ARGUMENT] (Function
     * argument.) [length] {expression}
     * 
     * returns: XNodeSet XNumber XString XBoolean XRTree XObject
     */
    public static final int OP_ARGUMENT = loadConstant("OP_ARGUMENT");

    /**
     * <meta name="usage" content="advanced"/> [OP_NUMBERLIT] (Number literal.)
     * [3] [index to token]
     * 
     * returns: XString
     */
    public static final int OP_NUMBERLIT = loadConstant("OP_NUMBERLIT");

    /**
     * <meta name="usage" content="advanced"/> [OP_LOCATIONPATH] [length]
     * {FROM_stepType} | {function} {predicate} [ENDOP]
     * 
     * (Note that element and attribute namespaces and names can be wildcarded
     * '*'.)
     * 
     * returns: XNodeSet
     */
    public static final int OP_LOCATIONPATH = loadConstant("OP_LOCATIONPATH");

    // public static final int LOCATIONPATHEX_MASK = 0x0000FFFF;
    // public static final int LOCATIONPATHEX_ISSIMPLE = 0x00010000;
    // public static final int OP_LOCATIONPATH_EX = (28 | 0x00010000);

    /**
     * <meta name="usage" content="advanced"/> [OP_PREDICATE] [length]
     * {expression} [ENDOP] (For safety)
     * 
     * returns: XBoolean or XNumber
     */
    public static final int OP_PREDICATE = loadConstant("OP_PREDICATE");

    /**
     * <meta name="usage" content="advanced"/> [OP_MATCHPATTERN] [length]
     * {PathExpr}+
     * 
     * returns: XNodeSet
     */
    public static final int OP_MATCHPATTERN = loadConstant("OP_MATCHPATTERN");

    /**
     * <meta name="usage" content="advanced"/> [OP_LOCATIONPATHPATTERN] [length]
     * {FROM_stepType} | {function}{predicate} [ENDOP] returns: XNodeSet
     */
    public static final int OP_LOCATIONPATHPATTERN = loadConstant("OP_LOCATIONPATHPATTERN");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_COMMENT] No size or
     * arguments. Note: must not overlap function OP number!
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_COMMENT = loadConstant("NODETYPE_COMMENT");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_TEXT] No size or
     * arguments. Note: must not overlap function OP number!
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_TEXT = loadConstant("NODETYPE_TEXT");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_PI] [index to token]
     * Note: must not overlap function OP number!
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_PI = loadConstant("NODETYPE_PI");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_NODE] No size or
     * arguments. Note: must not overlap function OP number!
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_NODE = loadConstant("NODETYPE_NODE");

    /**
     * <meta name="usage" content="advanced"/> [NODENAME] [index to ns token or
     * EMPTY] [index to name token]
     * 
     * returns: XBoolean
     */
    public static final int NODENAME = loadConstant("NODENAME");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_ROOT] No size or
     * arguments.
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_ROOT = loadConstant("NODETYPE_ROOT");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_ANY] No size or
     * arguments.
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_ANYELEMENT = loadConstant("NODETYPE_ANYELEMENT");

    /**
     * <meta name="usage" content="advanced"/> [NODETYPE_ANY] No size or
     * arguments.
     * 
     * returns: XBoolean
     */
    public static final int NODETYPE_FUNCTEST = loadConstant("NODETYPE_FUNCTEST");

    /**
     * <meta name="usage" content="advanced"/> [FROM_stepType] [length,
     * including predicates] [length of just the step, without the predicates]
     * {node test} {predicates}?
     * 
     * returns: XBoolean
     */
    public static final int AXES_START_TYPES = loadConstant("AXES_START_TYPES");

    /** ancestor axes opcode. */
    public static final int FROM_ANCESTORS = loadConstant("FROM_ANCESTORS");

    /** ancestor-or-self axes opcode. */
    public static final int FROM_ANCESTORS_OR_SELF = loadConstant("FROM_ANCESTORS_OR_SELF");

    /** attribute axes opcode. */
    public static final int FROM_ATTRIBUTES = loadConstant("FROM_ATTRIBUTES");

    /** children axes opcode. */
    public static final int FROM_CHILDREN = loadConstant("FROM_CHILDREN");

    /** descendants axes opcode. */
    public static final int FROM_DESCENDANTS = loadConstant("FROM_DESCENDANTS");

    /** descendants-of-self axes opcode. */
    public static final int FROM_DESCENDANTS_OR_SELF = loadConstant("FROM_DESCENDANTS_OR_SELF");

    /** following axes opcode. */
    public static final int FROM_FOLLOWING = loadConstant("FROM_FOLLOWING");

    /** following-siblings axes opcode. */
    public static final int FROM_FOLLOWING_SIBLINGS = loadConstant("FROM_FOLLOWING_SIBLINGS");

    /** parent axes opcode. */
    public static final int FROM_PARENT = loadConstant("FROM_PARENT");

    /** preceding axes opcode. */
    public static final int FROM_PRECEDING = loadConstant("FROM_PRECEDING");

    /** preceding-sibling axes opcode. */
    public static final int FROM_PRECEDING_SIBLINGS = loadConstant("FROM_PRECEDING_SIBLINGS");

    /** self axes opcode. */
    public static final int FROM_SELF = loadConstant("FROM_SELF");

    /** namespace axes opcode. */
    public static final int FROM_NAMESPACE = loadConstant("FROM_NAMESPACE");

    /** '/' axes opcode. */
    public static final int FROM_ROOT = loadConstant("FROM_ROOT");

    /**
     * <meta name="usage" content="advanced"/> For match patterns.
     */
    public static final int MATCH_ATTRIBUTE = loadConstant("MATCH_ATTRIBUTE");

    /**
     * <meta name="usage" content="advanced"/> For match patterns.
     */
    public static final int MATCH_ANY_ANCESTOR = loadConstant("MATCH_ANY_ANCESTOR");

    /**
     * <meta name="usage" content="advanced"/> For match patterns.
     */
    public static final int MATCH_IMMEDIATE_ANCESTOR = loadConstant("MATCH_IMMEDIATE_ANCESTOR");

    /** The end of the axes types. */
    public static final int AXES_END_TYPES = loadConstant("AXES_END_TYPES");

    /** The next free ID. Please keep this up to date. */
    // Field not supported by Sun JDK 1.4
    // private static final int NEXT_FREE_ID = loadConstant("NEXT_FREE_ID");
    
    

    private static int loadConstant(String string)
    {
        int retVal = 0;
        try
        {
            Field field = OP_CODE_CLASS.getField(string);
            Integer value = (Integer) field.get(null);
            retVal = value.intValue();
        }
        catch (Exception e)
        {
            //Can only happen if class is not in sync with OpCodes class.
            Log logger = LogFactory.getLog(ReflectOpCodes.class);
            logger.error(e);
            e.printStackTrace();
        }

        return retVal;
    }

}
